\ AnaClock.fth - Analogue clock demo

((
Copyright (c) 1998, 2011, 2022
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       901 313 4312 (North American access number to UK office)


This code provided as example of a "Petzold" Windows program.

To do
=====

Change history
==============
20220201 MPE002 32/64 bit compatibility changes.
20110311 MPE001 Overhaul
First issue: September 1998
))


\ ==========================================================================

cellbits #32 = [if]
  include %lib%\x86\ndp387		\ floating point pack
[else]
  include %lib%\x64\ndpx64
[then]

Extern: HDC BeginPaint( HWND hwnd, void * lpPaint );
Extern: BOOL EndPaint( HWND hWnd, void * lpPaint );
Extern: BOOL SetWindowExtEx(
  HDC hdc, int nXExtent, int nYExtent, void * lpSize
);
Extern: BOOL BitBlt(
  HDC hdcDest, int nXDest, int nYDest, int nWidth, int nHeight, HDC hdcSrc, int nXSrc,
  int nYSrc, DWORD dwRop
);
Extern: HBRUSH CreateHatchBrush( int fnStyle, COLORREF clrref );
Extern: int SetMapMode( HDC hdc, int fnMapMode  );
Extern: BOOL Rectangle(
  HDC hdc, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect
);
Extern: BOOL Ellipse(
  HDC hdc, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect
);
Extern: BOOL Polyline( HDC hdc, void * lppt, int cPoints );
Extern: ATOM RegisterClassEx( void * lpwcx );
Extern: VOID PostQuitMessage( int nExitCode );
Extern: BOOL SetViewportExtEx(
  HDC hdc, int nXExtent, int nYExtent, void * lpSize
);
Extern: BOOL SetViewportOrgEx(
  HDC hdc, int X, int Y, LPPOINT lpPoint
);

[undefined] l+! [if]
: l+!		\ x addr --
  tuck l@ + swap l!
;
[then]


\ ==========================================================================

: SetIsotropic { hdc cxClient cyClient | -- }

  hdc MM_ISOTROPIC SetMapMode drop
  hdc #1000 #1000 NULL SetWindowExtEx drop
  hdc  cxClient 2/  cyClient 2/ negate  NULL  SetViewportExtEx drop
  hdc  cxClient 2/  cyClient 2/         NULL  SetViewportOrgEx drop
;

fvariable cosTemp
fvariable sinTemp

: RotatePoint { *pt iNum iAngle | &pt[i] ptTemp[ POINT ] -- }

  2.0e0 %pi f* iAngle s>f f* 360.0e0 f/ fcos cosTemp f!
  2.0e0 %pi f* iAngle s>f f* 360.0e0 f/ fsin sinTemp f!

  iNum 0 ?do

     *pt POINT i * + -> &pt[i]

     &pt[i] POINT.x l@ s>f cosTemp f@ f*
     &pt[i] POINT.y l@ s>f sinTemp f@ f*
     f+ f>s
     ptTemp[ POINT.x l!

     &pt[i] POINT.y l@ s>f cosTemp f@ f*
     &pt[i] POINT.x l@ s>f sinTemp f@ f*
     f- f>s
     ptTemp[ POINT.y l!

     ptTemp[ &pt[i] POINT cmove

  loop
;

\ ==========================================================================

: DrawClock { hdc | &pt[0] &pt[1] &pt[2] pt[ POINT 3 * ] }

  pt[            -> &pt[0]
  pt[ POINT +    -> &pt[1]
  pt[ POINT 2* + -> &pt[2]

  #360 0 do

    0    &pt[0] POINT.x l!
    #900 &pt[0] POINT.y l!

    &pt[0] 1 i RotatePoint

    i 5 mod if #33 else #100 endif dup
    &pt[2] POINT.x l!
    &pt[2] POINT.y l!

    &pt[2] POINT.x l@ 2/ negate &pt[0] POINT.x l+!
    &pt[2] POINT.y l@ 2/ negate &pt[0] POINT.y l+!

    &pt[0] POINT.x l@ &pt[2] POINT.x l@ + &pt[1] POINT.x l!
    &pt[0] POINT.y l@ &pt[2] POINT.y l@ + &pt[1] POINT.y l!

    hdc BLACK_BRUSH GetStockObject SelectObject drop

    hdc
    &pt[0] POINT.x l@ &pt[0] POINT.y l@
    &pt[1] POINT.x l@ &pt[1] POINT.y l@
    Ellipse drop

  #6 +loop
;

\ ==========================================================================

create ptx
  0 l, -150 l, 100 l, 0 l, 0 l, 600 l, -100 l, 0 l, 0 l, -150 l,
  0 l, -200 l,  50 l, 0 l, 0 l, 800 l,  -50 l, 0 l, 0 l, -200 l,
  0 l,    0 l,   0 l, 0 l, 0 l,   0 l,    0 l, 0 l, 0 l,  800 l,

: DrawHands { hdc *st bChange | &ptTemp[i] iAngle[ 3 cells ] ptTemp[ POINT #15 * ] }

  *st SYSTEMTIME.wHour   w@ #30 * #360 mod
  *st SYSTEMTIME.wMinute w@ 2/
  + iAngle[ !

  *st SYSTEMTIME.wMinute w@ 6 * iAngle[ cell+ !

  *st SYSTEMTIME.wSecond w@ 6 * iAngle[ 2 cells + !

  ptx ptTemp[ POINT #15 * cmove

  3  bChange if 0 else 2 endif
  do

    ptTemp[ POINT 5 * i * + -> &ptTemp[i]

    &ptTemp[i]  5  iAngle[ i cells + @  RotatePoint

    hdc  &ptTemp[i]  5  Polyline

  loop
;

\ ==========================================================================

variable cxClient 0 cxClient !
variable cyClient 0 cyClient !

SYSTEMTIME buffer: dtPrevious
SYSTEMTIME buffer: dtNow

\ ==========================================================================

: (WndProc) { hwnd iMsg wParam lParam | bChange hdc ps[ PAINTSTRUCT ] -- lResult }

  case iMsg

    WM_CREATE of

      dtPrevious GetLocalTime

      0
    endof

    WM_SIZE of

      lParam LOWORD cxClient !
      lParam HIWORD cyClient !

      0
    endof

    WM_TIMER of

      dtNow GetLocalTime

      dtNow SYSTEMTIME.wHour   w@ dtPrevious SYSTEMTIME.wHour   w@ <>
      dtNow SYSTEMTIME.wMinute w@ dtPrevious SYSTEMTIME.wMinute w@ <>
      or -> bChange

      hwnd GetDC -> hdc

      hdc cxClient @ cyClient @ SetIsotropic

      hdc WHITE_PEN GetStockObject SelectObject drop
      hdc dtPrevious bChange DrawHands

      hdc BLACK_PEN GetStockObject SelectObject drop
      hdc dtNow TRUE DrawHands

      hwnd hdc ReleaseDC drop

      dtNow dtPrevious SYSTEMTIME cmove

      0
    endof

    WM_PAINT of

      hwnd ps[ BeginPaint -> hdc

      hdc cxClient @ cyClient @ SetIsotropic
      hdc DrawClock
      hdc dtPrevious TRUE DrawHands

      hwnd ps[ EndPaint drop

      0
    endof

    WM_DESTROY of

      hwnd 1 KillTimer drop

      0 PostQuitMessage

      0
    endof

    hwnd iMsg wParam lParam DefWindowProc

  end-case
;
4 1 Callback: WndProc
  ' (WndProc) to-callback Wndproc

\ ==========================================================================

: WinMain { hInstance hPrevInstance lpszCmdLine nCmdShow | hwnd msg[ MSG ] wndclas[ WNDCLASSEX ] -- res }

  WNDCLASSEX                    wndclas[ WNDCLASSEX.cbSize l!
  CS_HREDRAW CS_VREDRAW or      wndclas[ WNDCLASSEX.style l!
  WndProc                       wndclas[ WNDCLASSEX.lpfnWndProc !
  0                             wndclas[ WNDCLASSEX.cbClsExtra l!
  0                             wndclas[ WNDCLASSEX.cbWndExtra l!
  hInstance                     wndclas[ WNDCLASSEX.hInstance !
  NULL IDI_APPLICATION LoadIcon wndclas[ WNDCLASSEX.hIcon !
  NULL IDC_ARROW LoadCursor     wndclas[ WNDCLASSEX.hCursor !
  WHITE_BRUSH GetStockObject    wndclas[ WNDCLASSEX.hbrBackground !
  NULL                          wndclas[ WNDCLASSEX.lpszMenuName !
  z" AnaClock"                  wndclas[ WNDCLASSEX.lpszClassName !
  NULL IDI_APPLICATION LoadIcon wndclas[ WNDCLASSEX.hIconSm !

  wndclas[ RegisterClassEx drop

  NULL
  z" AnaClock"
  z" Analogue Clock"
  WS_OVERLAPPEDWINDOW
  100 100 400 400		\ x y w h
  NULL
  NULL
  hInstance
  NULL
  CreateWindowEx -> hwnd

  hwnd #1 #1000 NULL SetTimer
  0= if
    hwnd
    z" Too many clocks or timers"
    z" AnaClock"
    MB_ICONEXCLAMATION MB_OK or
    MessageBox drop
    FALSE exit
  endif

  hwnd nCmdShow ShowWindow drop

  hwnd UpdateWindow drop

  begin
    msg[ NULL 0 0 GetMessage
  while
    msg[ TranslateMessage drop
    msg[ DispatchMessage drop
  repeat

  msg[ MSG.wParam @
;

\ ==========================================================================

assign winmain to-do entrypoint

cr .( Type )
cr .(  save AnaClock )
cr .( to save application, then run it )

