\ Bounce.fth - Bouncing ball demo

((
Copyright (c) 1998, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       901 313 4312 (North American access number to UK office)


This code provided as example of a "Petzold" Windows program.

To do
=====

Change history
==============
20110311 MPE001 Overhaul
First issue: September 1998
))

Extern: BOOL Rectangle(
  HDC hdc, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect
);
Extern HBRUSH CreateHatchBrush( int fnStyle, COLORREF clrref );
Extern: BOOL Ellipse(
  HDC hdc, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect
);
Extern: BOOL BitBlt(
  HDC hdcDest, int nXDest, int nYDest, int nWidth, int nHeight, HDC hdcSrc, int nXSrc,
  int nYSrc, DWORD dwRop
);
Extern: ATOM RegisterClassEx( void * lpwcx );
Extern: VOID PostQuitMessage( int nExitCode );

$00CC:0020 constant SRCCOPY

\ ==========================================================================

variable hBitmap  0 hBitmap !
variable cxClient 0 cxClient !
variable cyClient 0 cyClient !
variable xCentre  0 xCentre !
variable yCentre  0 yCentre !
variable cxTotal  0 cxTotal !
variable cyTotal  0 cyTotal !
variable cxRadius 0 cxRadius !
variable cyRadius 0 cyRadius !
variable cxMove   0 cxMove !
variable cyMove   0 cyMove !
variable xPixel   0 xPixel !
variable yPixel   0 yPixel !

\ ==========================================================================

: (WndProc) { hwnd iMsg wParam lParam | hBrush hdc hdcMem iScale -- lResult }

  case iMsg

    WM_CREATE of

      hwnd GetDC -> hdc
      hdc ASPECTX GetDeviceCaps xPixel !
      hdc ASPECTY GetDeviceCaps yPixel !
      hwnd hdc ReleaseDC drop

      0
    endof

    WM_SIZE of

      lParam LOWORD cxClient !
      lParam HIWORD cyClient !

      cxClient @ 2/ xCentre !
      cyClient @ 2/ yCentre !

      cxClient @ xPixel @ *  cyClient @ yPixel @ *  min  16 / -> iScale

      iScale xPixel @ / cxRadius !
      iScale yPixel @ / cyRadius !

      1  cxRadius @ 2/  max cxMove !
      1  cyRadius @ 2/  max cyMove !

      cxRadius @ cxMove @ + 2* cxTotal !
      cyRadius @ cyMove @ + 2* cyTotal !

      hBitmap @ 0<> if hBitmap @ DeleteObject drop endif

      hwnd GetDC -> hdc
      hdc CreateCompatibleDC -> hdcMem
      hdc cxTotal @ cyTotal @ CreateCompatibleBitmap hBitmap !
      hwnd hdc ReleaseDC drop

      hdcMem hBitmap @ SelectObject drop
      hdcMem -1 -1 cxTotal @ 1+ cyTotal @ 1+ Rectangle drop
      HS_DIAGCROSS 0 CreateHatchBrush -> hBrush
      hdcMem hBrush SelectObject drop
      hdcMem 0xFF00FF SetBkColor drop

      hdcMem cxMove @ cyMove @
      cxTotal @ cxMove @ -
      cyTotal @ cyMove @ -
      Ellipse drop

      hdcMem DeleteDC drop
      hBrush DeleteObject drop

      0
    endof

    WM_TIMER of

      hBitmap @ 0=
      if
        hwnd iMsg wParam lParam DefWindowProc
        exit
      endif

      hwnd GetDC -> hdc
      hdc CreateCompatibleDC -> hdcMem
      hdcMem hBitmap @ SelectObject drop

      hdc
      xCentre @ cxTotal @ 2/ -
      yCentre @ cyTotal @ 2/ -
      cxTotal @ cyTotal @ hdcMem 0 0 SRCCOPY BitBlt drop

      hwnd hdc ReleaseDC drop
      hdcMem DeleteDC drop

      cxMove @ xCentre +!
      cyMove @ yCentre +!

      xCentre @ cxRadius @ +  cxClient @ >=
      xCentre @ cxRadius @ -  0          <=
      or if
        cxMove @ negate cxMove !
      endif

      yCentre @ cyRadius @ +  cyClient @ >=
      yCentre @ cyRadius @ -  0          <=
      or if
        cyMove @ negate cyMove !
      endif

      0
    endof

    WM_DESTROY of

      hBitmap @ 0<> if hBitmap @ DeleteObject drop endif

      hwnd 1 KillTimer drop

      0 PostQuitMessage

      0
    endof

    hwnd iMsg wParam lParam DefWindowProc

  end-case
;
4 1 Callback: WndProc
  ' (WndProc) to-callback Wndproc

\ ==========================================================================

: WinMain { hInstance hPrevInstance lpszCmdLine nCmdShow | hwnd msg[ MSG ] wndclas[ WNDCLASSEX ] -- res }

  WNDCLASSEX                    wndclas[ WNDCLASSEX.cbSize !
  CS_HREDRAW CS_VREDRAW or      wndclas[ WNDCLASSEX.style !
  WndProc                       wndclas[ WNDCLASSEX.lpfnWndProc !
  0                             wndclas[ WNDCLASSEX.cbClsExtra !
  0                             wndclas[ WNDCLASSEX.cbWndExtra !
  hInstance                     wndclas[ WNDCLASSEX.hInstance !
  NULL IDI_APPLICATION LoadIcon wndclas[ WNDCLASSEX.hIcon !
  NULL IDC_ARROW LoadCursor     wndclas[ WNDCLASSEX.hCursor !
  WHITE_BRUSH GetStockObject    wndclas[ WNDCLASSEX.hbrBackground !
  NULL                          wndclas[ WNDCLASSEX.lpszMenuName !
  z" Bounce"                   wndclas[ WNDCLASSEX.lpszClassName !
  NULL IDI_APPLICATION LoadIcon wndclas[ WNDCLASSEX.hIconSm !

  wndclas[ RegisterClassEx drop

  NULL
  z" Bounce"
  z" Bouncing Ball"
  WS_OVERLAPPEDWINDOW
  100 100 400 400		\ x y w h
  NULL
  NULL
  hInstance
  NULL
  CreateWindowEx -> hwnd

  hwnd #1 #50 NULL SetTimer
  0= if
    hwnd
    z" Too many clocks or timers"
    z" Bounce"
    MB_ICONEXCLAMATION MB_OK or
    MessageBox drop
    FALSE exit
  endif

  hwnd nCmdShow ShowWindow drop

  hwnd UpdateWindow drop

  begin
    msg[ NULL 0 0 GetMessage
  while
    msg[ TranslateMessage drop
    msg[ DispatchMessage drop
  repeat

  msg[ MSG.wParam @
;

\ ==========================================================================

assign winmain to-do entrypoint

cr .( Type )
cr .(  save Bounce )
cr .( to save application, then run it )

