(*
 * LANGUAGE    : Forth; ***** ONLY iForth and VFX Forth are supported *****
 * PROJECT     : Forth Tool
 * DESCRIPTION : Audio processing client using JACK
 * CATEGORY    : Realtime Audio tools
 * AUTHOR      : Hanno Schwalm
 * LAST CHANGE : 2009-08-25 21:05:32; Hanno Schwalm hanno@schwalm-bremen.de
		 2009-12-19 18:11:00; additions for overlay generation
		 2011-08-11 22:33:04; fixed for 1.10
		 2012-07-08 16:11:31; using threads.frt - 1.17
		 2012-07-10 23:26:20; added midi support as it works now
		 2012-07-16 17:46:42; semaphores needed for midi handling
		 2012-08-22 21:32:55; instruments support
		 2012-09-06 19:27:32; version 2.00
		 2015-01-28 19:57:55; 2.04 the instruments made turnkeys impossible, now fixed
		 2016-04-28 10:45:24; 2.05 Using the new logging system for iforth, fixing vfx port done 
		 2017-04-16 08:18:39; 2.06 Version made vfx ready again
		 2017-06-02 07:32:09; 2.90 switch to sfloats for a) performance and b) making ARM possible
		 2017-06-08 12:34:27; 2.91 Mostly finished ARM workup 
*)

FORTH DEFINITIONS

\ =======================================================================================================================================
\ TRUE CONSTANT JACK-EMBEDDED?
\ FALSE CONSTANT WAV-SUPPORT?
\ FALSE CONSTANT IO-MODULES?
\ FALSE CONSTANT JACK-MIDI?
\ FALSE CONSTANT MIDI-INSTRUMENTS?
\ TRUE CONSTANT MIDI-DEMO?
\ TRUE CONSTANT JACK-DEMOS?
\ TRUE CONSTANT JACK-FORTH-FIR?

\ =======================================================================================================================================
0 CONSTANT MAKE-JACK-DOCS	\ When TRUE and on vfx generate the docs file. 
\ =======================================================================================================================================

[DEFINED] IFORTH!
[IF]	NEEDS -terminal
\	NEEDS -assemble		\ Only required to recode the optimized filter snippets
	NEEDS -debugopt
	NEEDS -threads
	NEEDS -extlocals
	NEEDS -dynlibs

	FORTH DEFINITIONS
	REVISION -fjack "--- fJACK audio client  Version 3.02 ---"

	FALSE CONSTANT ARMLINUX?
[THEN]

\ =======================================================================================================================================
[DEFINED] VFXFORTH
[IF]	[UNDEFINED] CreateThread	[DEFINED] Target_386_Windows AND	[IF] INCLUDE %Lib%/Win32/MultiWin32.fth [THEN]
	[UNDEFINED] semaphore		[DEFINED] Target_386_Linux AND		[IF] INCLUDE %Lib%/Lin32/MultiLin32.fth [THEN]
	[UNDEFINED] semaphore		[DEFINED] Target_ARM_Linux AND		[IF] INCLUDE %Lib%/Lin32/MultiLin32.fth [THEN]
	[UNDEFINED] semaphore		[DEFINED] Target_386_OSX AND		[IF] INCLUDE %Lib%/Osx32/MultiOsx32.fth [THEN]
	[UNDEFINED] textbuff:							[IF] INCLUDE %Lib%/Genio/Buffer.fth	[THEN]

	[DEFINED] Target_ARM_Linux
	[IF]	FpSystem 0 =	[IF] INCLUDE %Lib%/ARM/VFP64S.fth [THEN]

		FORTH DEFINITIONS
		ANEW -fjack
		FALSE CONSTANT WINDOWS?
		FALSE CONSTANT DARWIN?
		TRUE CONSTANT ARMLINUX?
		TRUE CONSTANT LINUX?

	[ELSE]
		FpSystem 0 =	[IF] INCLUDE %Lib%/x86/Ndp387.fth [THEN]
		FpSystem 2 <>	[IF] CR .( fJACK wants the Ndp FP module!) ABORT [THEN]

		FORTH DEFINITIONS
		ANEW -fjack
		MAKE-JACK-DOCS
		[IF]	FORTH DEFINITIONS	\ make sure the docs are written
			[UNDEFINED] +docgen	[IF] INCLUDE %Lib%/DocGen.fth [THEN]
			+docgen docgen_latex
			+TOC
		[THEN]
		FALSE CONSTANT ARMLINUX?

		[DEFINED] Target_386_Windows	CONSTANT WINDOWS?
		[DEFINED] Target_386_Linux	CONSTANT LINUX?
		[DEFINED] Target_386_OSX	CONSTANT DARWIN?
	[THEN]
[THEN]

[UNDEFINED] -fjack [IF] CR .( Your Forth system is not supported) ABORT [THEN]

: (jack-version)	S" 3.02" ;
INCLUDE fjack/main.frt

WAV-SUPPORT? [DEFINED] MAKE-JACK-DOCS OR 
[IF] 	INCLUDE fjack/wavsupport.frt
[THEN]

IO-MODULES? [DEFINED] MAKE-JACK-DOCS OR 
[IF]	INCLUDE fjack/iosupport.frt
[THEN]

JACK-MIDI? [DEFINED] MAKE-JACK-DOCS OR
[IF]	INCLUDE fjack/midi.frt
[THEN]

MIDI-INSTRUMENTS? JACK-MIDI? AND IO-MODULES? AND MAKE-JACK-DOCS OR
JACK-EMBEDDED? 0= AND
[IF]	INCLUDE fjack/instrument.frt
	INCLUDE fjack/synth.frt
	INCLUDE fjack/karplus.frt
[THEN]

MIDI-DEMO? MIDI-INSTRUMENTS? AND JACK-MIDI? AND IO-MODULES? AND MAKE-JACK-DOCS OR
JACK-EMBEDDED? 0= AND
[IF]	: CHORD	( channel -- )	\ guess who is the master of symphonic words?
		LOCALS| #chan |
		#70	#50	#chan 0.20e0	NOTE-ON
		#50	#51	#chan 1.20e0	NOTE-ON
		#60	#48	#chan 1.80e0	NOTE-ON
		#60	#47	#chan 2.40e0	NOTE-ON
			#50	#chan 1.20e0	NOTE-OFF
			#51	#chan 1.80e0	NOTE-OFF
			#48	#chan 2.40e0	NOTE-OFF
			#47	#chan 4.40e0	NOTE-OFF ;
	:NONAME	\ Connect the fjack MIDI output to its input.
		TRUE MIDI-TO-MYSELF
		\ Define the midiport to whome the default synth is listening to.
		\ This also tells the instrument mode, you could replace DEFAULT-SYNTH by
		\ SIMPLE-SYNTH or your own definitions.
		2 DEFAULT-SYNTH
		\ The synth listeners are plugged in the two main channels.
		SYNTH-0 4 0 SET-JACK-PLUGIN		TRUE SYNTH-0 SET-PLUGIN-BYPASS
		SYNTH-1 4 1 SET-JACK-PLUGIN		TRUE SYNTH-1 SET-PLUGIN-BYPASS
		\ Set up the volumes
		1e0 SYNTH-0 SET-OBJ-LEVEL		1e0 SYNTH-1 SET-OBJ-LEVEL
		\ Now do the same for karplus
		[ JACK-EMBEDDED? ]
		[IF]	1 SIMPLE-KARPLUS
		[ELSE]	1 DEFAULT-KARPLUS
		[THEN]
		KARPLUS-PLUCK-0 5 0 SET-JACK-PLUGIN	TRUE KARPLUS-PLUCK-0 SET-PLUGIN-BYPASS
		KARPLUS-PLUCK-1 5 1 SET-JACK-PLUGIN	TRUE KARPLUS-PLUCK-1 SET-PLUGIN-BYPASS
		1e0 KARPLUS-PLUCK-0 SET-OBJ-LEVEL	1e0 KARPLUS-PLUCK-1 SET-OBJ-LEVEL
		\ Play the chord via karplus!
		1 CHORD ; AT-START-JACK-CLIENT
[THEN]

MAKE-JACK-DOCS JACK-DEMOS? OR
[IF]	INCLUDE fjack/examples/tests.frt
[THEN]

:NONAME	START-JACK-CLIENT ; DUP EXECUTE AtCold

IS-VFX? [DEFINED] -docgen AND
[IF] -docgen [THEN]

