(*
 * LANGUAGE    : Forth
 * PROJECT     : Working on MPE and iForth
 * DESCRIPTION : VFX Forth OS dependent functions for the jack sound interface
 * CATEGORY    : Realtime Audio tool
 * AUTHOR      : Hanno Schwalm
 * LAST CHANGE : 2012-07-08 17:37:01 ported FORTH-THREAD to vfx
 *)

FORTH DEFINITIONS
FALSE CONSTANT 64BIT?
: ??CR	OUT @ IF CR THEN ;
: HTAB	OUT @ - 1 MAX SPACES ;


JACKAUDIO DEFINITIONS
CREATE $FJACKSYSFILE $100 ALLOT
WINDOWS? [IF]
: START-BACK-THREAD	( xt parameters *threadid -- )
	>R >R >R
	0	\ security attributes
	$20000	\ stack size
	R>	\ thread function
	R>	\ arguments
	$0	\ creation flags
	R>	\ pointer to receive id
	CreateThread DROP ;
[THEN]


LINUX? [IF]
\ The stack oprder when calling this is like this ( xt arg 'thread -- )
: START-BACK-THREAD	( xt parameters threadid -- )
	-ROT 0 -ROT pthread_create DROP ;
[THEN]

DARWIN? [IF]
\ The stack oprder when calling this is like this ( xt arg 'thread -- )
: START-BACK-THREAD	( xt parameters threadid -- )
	-ROT 0 -ROT pthread_create DROP ;
[THEN]

Alias: 'OF	ADDR
Alias: S16B@	w@s
Alias: 16B!	w!
Alias: PLACE+	APPEND
Alias: 32B!	!	( n addr -- )
Alias: 32B@	@	( addr -- n )
Alias: S32B@	@	( addr -- n )
Alias: H.	.LWORD
Alias: ?MS	ticks ( -- n )
Alias: PI	%pi
Alias: PI/2	%pi/2
Alias: $CR	eol$	( -- addr )
Alias: SEMAPHORE semaphore
Alias: GETSEMAPHORE LockSem
Alias: FREESEMAPHORE UnlockSem
: INITSEMAPHORE DROP InitSem ;

[UNDEFINED] FVALUE
[IF]
: fvalCompChild	\ xt --
\ Given the xt of an FVALUE, compile the required action.
  >body postpone literal
  case  OperatorType @off
    0 of  postpone f@  endof		\ Default, fetch
    1 of  postpone f!  endof		\ store, ->, to
    2 of  ( noop )  endof		\ addr
\					\ inc
\					\ dec
    5 of  postpone f+!  endof		\ add
\					\ zero
    7 of  postpone f-!  endof		\ sub
    8 of  postpone drop  postpone FPCELL  endof	\ sizeof
\					\ set
      bad-method
  endcase
; doNotSin

: FVALUE	\ F: f -- ; ??? -- ???
  create
    f,  ['] fvalCompChild set-compiler
  interp>
    case  OperatorType @off
      0 of  f@  endof			\ default, fetch
      1 of  f!  endof			\ store, ->, to
      2 of  ( noop )  endof		\ addr
\                         		\ inc
\					\ dec
      5 of  f+!  endof			\ add
\                        		\ zero
      7 of  f-!  endof			\ sub
      8 of  drop FPCELL  endof		\ sizeof
\					\ set
         bad-method
    endcase
;

[THEN]

: (#.R)		( u r -- addr n )	BASE @ >R DECIMAL (U.R) R> BASE ! ;
: ($.R)		( u r -- addr n )	BASE @ >R HEX (U.R) R> BASE ! ;
: @+		( a -- a' n )		DUP @ SWAP CELL+ SWAP ;
: 0<=		( n -- flag )		0> 0= ;
: 32B!+		( n addr -- addr' )	TUCK ! 1 CELLS + ;
: 32B@+		( addr -- addr' n )	DUP @ SWAP CELL+ SWAP ;
: C@+		( addr -- addr' c )	DUP C@ SWAP CHAR+ SWAP ;
: FSQR		( F1: f1 -- f2 )	FDUP F* ;
: DEC.		( n -- )		BASE @ DECIMAL SWAP . BASE ! ;
: 2+!		( d a -- )		DUP >R 2@ D+ R> 2! ;
: REDUCE.2PI	( F n1 -- n2 )		FDUP %pi %pi F+ F> IF %pi F- %pi F- THEN ;
: ?FILE					THROW ;
: ?ALLOCATE				THROW ;
: @LATEST-NAME	( -- addr )		LATEST CTRL>NFA ;
: .NAMEHEADER	( addr -- )		.NAME ;
: top-mem	( -- addr )		HERE ;
: (H.)		( n -- addr len )	BASE @ >R HEX S>D <# #S [CHAR] $ HOLD #> R> BASE ! ;
: ALIGNED16	( n -- n' )		#15 + $FFFFFFF0 AND ;
: ALIGNED1024	( n -- n' )		#1023 + $FFFFFC00 AND ;
: FRANDOM	( -- f: r )		RANDOM $7FFFFFFF AND S>F  2147483647E0 F/ ;
: .LEVEL	( f1 -- )		BASE @ DECIMAL PRECISION  4 SET-PRECISION  F. SET-PRECISION BASE ! ;
: CUROFF	( -- )			;
: CURON		( -- )			;

[UNDEFINED] fromc [IF] : fromc ; [THEN]
: (THREADER) ( arg -- f )
	@+ SWAP @+ SWAP @+ SWAP @+ SWAP LOCALS| *data *free cnt *id xt |
	*data cnt 0 ?DO @+ SWAP LOOP DROP
	xt EXECUTE
	*free ?DUP IF FREE DROP THEN ;

fromc 1 1 CALLBACK: DO-THREAD
	ASSIGN (THREADER) to-callback DO-THREAD

: ALLOC-THREAD ( struct -- )
	DO-THREAD SWAP DUP 1 CELLS + @ START-BACK-THREAD ;

\ A FORTH-THREAD is started as a thread using the standard ALLOC-THREAD.
\ You may pass integer parameters to it
: FORTH-THREAD	( par-1 par-2 .... par-n number-of-int-parameters  xt -- )
	OVER ( parameters ) $10 + CELLS ALLOCATE ?ALLOCATE LOCALS| buff xt pars |
	xt buff 1 CELLS + !
	pars buff 3 CELLS + !
	buff  DUP buff 2 CELLS + !  buff 4 CELLS + !
	pars 0 ?DO pars 1- I - #5 + CELLS buff + ! LOOP
	buff 1 CELLS + ALLOC-THREAD ;
: SET-TASKNAME	2DROP ;
0 VALUE debugbuff

: USE-DEBUG-I//O	debugbuff SetIO ;

: DEBUG>>	POSTPONE IO] ; IMMEDIATE
: <<DEBUG	POSTPONE [IO POSTPONE USE-DEBUG-I//O ; IMMEDIATE
: SHOW-LOG	( -- )
	CR
	BEGIN	[IO debugbuff SetIO KEY? IF KEY ELSE 0 THEN IO] ?DUP
	WHILE	EMIT
	REPEAT ;
: BOLD ;

:NONAME	( -- )	\ reads the current filesystem name to $FJACKSYSFILE;
	CurrSourceName $FJACKSYSFILE PLACE  $FJACKSYSFILE DUP C@ S" V_osdeps.frt" NIP - SWAP C!
	S" sysfiles/" $FJACKSYSFILE PLACE+
	[ WINDOWS? ]
	[IF]	$FJACKSYSFILE COUNT BOUNDS ?DO I C@ [CHAR] / = IF [CHAR] \ I C! THEN LOOP
	[ELSE]	$FJACKSYSFILE COUNT BOUNDS ?DO I C@ [CHAR] \ = IF [CHAR] / I C! THEN LOOP
	[THEN]
	$10000 SizedTextBuff DUP TO debugbuff 0= ABORT" Can't allocate the debugging device" ; DUP EXECUTE AtCold

ALSO TYPES DEFINITIONS
	Alias: jack_client_t		INT
	Alias: jack_nframes_t		INT
	Alias: jack_port_t		INT
	Alias: jack_transport_state_t	INT
	Alias: jack_position_t		INT
	Alias: jack_midi_event_t	INT
	Alias: jack_midi_data_t		INT
	Alias: fftwf_plan		INT
	Alias: jack_latency_range_t	INT
	Alias: jack_session_event_t	INT
	Alias: fftwf_complex		double
	Alias: jack_time		LongLong


	[UNDEFINED] size_t [IF] Alias: size_t	INT [THEN]
PREVIOUS DEFINITIONS

