
\ generic forth high-level fir filter routines, can be replaced by optimized
\ sse2/neon code.
\ All filters used in fJACK >=2.90 assume
\ 1) we have 32bit floats as data and as filter parameters
\ 2) filters always have x+4+1 taps

\ calc-fir-val		\ ( muls data size -- )	(F -- val )
\ This points to the valid function. It has the current data in the array, does the multiply&add and
\ returns the value.
\ Please note that size is always n*4+1

\ calc-adp-muls		\ ( muls data cnt -- )	(F corr -- )
\ This points to the valid function. It has the current data in the array, does the calculation of the coefficients.
\ The correction val is consumed from the floating stack.
\ Please note that size is always n*4

JACKAUDIO DEFINITIONS



IS-IFORTH? 64BIT? AND
[IF]	[DEFINED] -assemble
  [IF]	: DUMP-ASM	( addr len -- )
		BASE @ >R HEX 0
		?DO	I #15 AND 0= IF CR THEN
			DUP I + C@ S>D <# # # [CHAR] $ HOLD #> TYPE ."  C, "
		LOOP	DROP R> BASE ! ;

	CODE calc-fir-val	rax	pop,
				rcx	pop,	\ data
				rdx	pop,	\ multipliers
	4 b#		->	rax	sub,
	xmm0		->	xmm0	subps,
  @@1:	[rcx]		->	xmm1	movups,
	[rcx $10 +]	->	rcx	lea,
	[rdx]		->	xmm1	mulps,
	[rdx $10 +]	->	rdx	lea,
	xmm1		->	xmm0	addps,
	4 b#		->	rax	sub,
			@@1 byte-offset	jg,
	[rcx]		->	xmm1	movss,
	[rdx]		->	xmm1	mulss,
	xmm1		->	xmm0	addss,
	xmm0		->	xmm0	haddps,
	xmm0		->	xmm0	haddps,
	[rbp #-8 +]	->	rbp	lea,
	xmm0		->	[rbp]	movss,
	[rbp]			dword fld,
				fpush,
	[rbp #8 +]	->	rbp	lea,
				rbx jmp, END-CODE

	CODE calc-adp-muls		fpop,
	[rbp $-10 +]	->	rbp	lea,
	[rbp]			dword	fstp,
	[rbp]		->	xmm0	movss,
	[rbp $10 +]	->	rbp	lea,
	0 :: xmm0	->	xmm0	shufps,		\ should fill a singles by the loaded sfloat
				rax	pop,
	4 b#		->	rax	sub,
				rcx	pop,	\ data
				rdx	pop,	\ multipliers
  @@1:	[rcx]		->	xmm1	movups,
	[rcx $10 +]	->	rcx	lea,
	xmm0		->	xmm1	mulps,
	[rdx]		->	xmm1	addps,
	xmm1		->	[rdx]	movaps,
	[rdx $10 +]	->	rdx	lea,
	4 b#		->	rax	sub,
			@@1 byte-offset	jg,
				rbx	jmp, END-CODE
  [ELSE]
	CODE calc-fir-val
	$58 C, $59 C, $5A C, $48 C, $83 C, $E8 C, $04 C, $0F C, $5C C, $C0 C, $0F C, $10 C, $09 C, $48 C, $8D C, 
	$49 C, $10 C, $0F C, $59 C, $0A C, $48 C, $8D C, $52 C, $10 C, $0F C, $58 C, $C1 C, $48 C, $83 C, $E8 C, $04 C, 
	$7F C, $E9 C, $F3 C, $0F C, $10 C, $09 C, $F3 C, $0F C, $59 C, $0A C, $F3 C, $0F C, $58 C, $C1 C, $F2 C, $0F C, 
	$7C C, $C0 C, $F2 C, $0F C, $7C C, $C0 C, $48 C, $8D C, $6D C, $F8 C, $F3 C, $0F C, $11 C, $45 C, $00 C, $D9 C, 
	$45 C, $00 C, $4D C, $8D C, $6D C, $F0 C, $D9 C, $C9 C, $41 C, $DB C, $7D C, $00 C, $48 C, $8D C, $6D C, $08 C, 
	$FF C, $E3 C,	END-CODE
	CODE calc-adp-muls
	$41 C, $DB C, $6D C, $00 C, $D9 C, $C9 C, $4D C, $8D C, $6D C, $10 C, $48 C, $8D C, $6D C, $F0 C, $D9 C, 
	$5D C, $00 C, $F3 C, $0F C, $10 C, $45 C, $00 C, $48 C, $8D C, $6D C, $10 C, $0F C, $C6 C, $C0 C, $00 C, $58 C, 
	$48 C, $83 C, $E8 C, $04 C, $59 C, $5A C, $0F C, $10 C, $09 C, $48 C, $8D C, $49 C, $10 C, $0F C, $59 C, $C8 C, 
	$0F C, $58 C, $0A C, $0F C, $29 C, $0A C, $48 C, $8D C, $52 C, $10 C, $48 C, $83 C, $E8 C, $04 C, $7F C, $E6 C, 
	$FF C, $E3 C,	END-CODE
  [THEN]
[THEN]

IS-VFX? ARMLINUX? 0= AND		\ Intel vfx
[IF]	\ This is the source used by iForth64 assembler for the optimized vfx code
\	CODE test1
\		[rbp]		->	eax	mov,	\ cnt
\		[rbp 4 +]	->	ecx	mov,	\ data
\		[rbp 8 +]	->	edx	mov,	\ mul
\		4 b#		->	eax	sub,
\		xmm0		->	xmm0	subps,
\	  @@1:	[rcx]		->	xmm1	movups,
\		[rcx $10 +]	->	rcx	dword lea,
\		[rdx]		->	xmm1	mulps,
\		[rdx $10 +]	->	rdx	dword lea,
\		xmm1		->	xmm0	addps,
\		4 b#		->	eax	sub,
\				@@1 byte-offset	jg,
\		[rcx]		->	xmm1	movss,
\		[rdx]		->	xmm1	mulss,
\		xmm1		->	xmm0	addss,
\		xmm0		->	xmm0	haddps,
\		xmm0		->	xmm0	haddps,
\		xmm0		->	[rbx]	movss,
\		[rbp #12 +]	->	rbp	dword lea,
	CODE (calc-fir-val)
	$8B C, $45 C, $00 C, $8B C, $4D C, $04 C, $8B C, $55 C, $08 C, $83 C, $E8 C, $04 C, $0F C, $5C C, $C0 C, $0F C, 
	$10 C, $09 C, $8D C, $49 C, $10 C, $0F C, $59 C, $0A C, $8D C, $52 C, $10 C, $0F C, $58 C, $C1 C, $83 C, $E8 C, 
	$04 C, $7F C, $EC C, $F3 C, $0F C, $10 C, $09 C, $F3 C, $0F C, $59 C, $0A C, $F3 C, $0F C, $58 C, $C1 C, $F2 C, 
	$0F C, $7C C, $C0 C, $F2 C, $0F C, $7C C, $C0 C, $F3 C, $0F C, $11 C, $03 C, $8D C, $6D C, $0C C, $C3 C, $FF C, 
	END-CODE
	2 +USER dummy
	: calc-fir-val	dummy (calc-fir-val) SF@ ;
[THEN]


[UNDEFINED] calc-fir-val JACK-FORTH-FIR? OR
[IF] : calc-fir-val
	4 / 0 0e0
	DO	SF@+ SWAP SF@+ F* F+
		SF@+ SWAP SF@+ F* F+		
		SF@+ SWAP SF@+ F* F+		
		SF@+ SWAP SF@+ F* F+		
	LOOP	SF@+ SWAP SF@+ F* F+ 2DROP ;
[THEN]

[UNDEFINED] calc-adp-muls JACK-FORTH-FIR? OR
[IF] : calc-adp-muls
	4 / 0
	DO	FDUP SF@+ F* OVER SF@ F+	SWAP SF!+ SWAP
		FDUP SF@+ F* OVER SF@ F+	SWAP SF!+ SWAP
		FDUP SF@+ F* OVER SF@ F+	SWAP SF!+ SWAP
		FDUP SF@+ F* OVER SF@ F+	SWAP SF!+ SWAP
	LOOP	2DROP FDROP ;
[THEN]

