(*
 * LANGUAGE    : ANS Forth
 * PROJECT     : Forth Environments
 * DESCRIPTION : A generic processing plugin
 * CATEGORY    : Sound processing
 * AUTHOR      : Hanno Schwalm, BGDegazio
 * LAST CHANGE : 2012-09-02 17:25:57
		2015-01-25 18:39:21 bypassing fixed and a better performance
		2015-02-28 11:54:37 proper stopping of instruments with timeout
 *)
[DEFINED] -fjack 0=		[IF] INCLUDE fjack.frt [THEN]
[DEFINED] -fjack/iosupport 0=	[IF] INCLUDE fjack/iosupport.frt [THEN]
[DEFINED] -fjack/midi 0=	[IF] INCLUDE fjack/midi.frt [THEN]

IS-IFORTH?
[IF]	REVISION -fjack/instrument "--- MIDI Instruments    Version 0.15 ---"
[ELSE]	ANEW -fjack/instrument
[THEN]

ALSO JACKAUDIO ALSO DEFINITIONS
INCLUDE fjack/tonetable.frt

IS-IFORTH?	[IF]	USER VOICE-OBJECT
			USER INSTR-OBJECT
		[THEN]
IS-VFX?		[IF]	1 CELLS +USER VOICE-OBJECT
			1 CELLS +USER INSTR-OBJECT
		[THEN]

: -->VOICE	( voice -- )	VOICE-OBJECT ! ;
: @VOICE	( -- voice )	VOICE-OBJECT @ ;
: -->INSTRUMENT	( instr -- )	INSTR-OBJECT ! ;
: @INSTRUMENT	( -- instr )	INSTR-OBJECT @ ;

: COMP-VOICE-OBJ,	POSTPONE VOICE-OBJECT POSTPONE @  @ POSTPONE LITERAL  POSTPONE + ;
: COMP-INS-OBJ,		POSTPONE INSTR-OBJECT POSTPONE @  @ POSTPONE LITERAL  POSTPONE + ;

IS-IFORTH?
[IF]	: VOICE-VALUE	( n -- n' )		\ name
		CREATE	IMMEDIATE COMPILE-ONLY ALIGNED DUP , 1 CELLS +
		DOES>	COMP-VOICE-OBJ, %VAR @ %VAR OFF COMP-VALUE ;
	: VOICE-FVALUE	( n -- n' )	\ name
		CREATE	IMMEDIATE COMPILE-ONLY SFALIGNED DUP , 1 SFLOATS +
		DOES>	COMP-VOICE-OBJ, %VAR @ %VAR OFF COMP-FVALUE ;
	: INSTR-VALUE	( n -- n' )		\ name
		CREATE	IMMEDIATE COMPILE-ONLY ALIGNED DUP , 1 CELLS +
		DOES>	COMP-INS-OBJ, %VAR @ %VAR OFF COMP-VALUE ;
	: INSTR-FVALUE	( n -- n' )	\ name
		CREATE	IMMEDIATE COMPILE-ONLY SFALIGNED DUP , 1 SFLOATS +
		DOES>	COMP-INS-OBJ, %VAR @ %VAR OFF COMP-FVALUE ;
[THEN]


IS-VFX?
[IF]	: VOICE-VAL,	( xt -- )	>BODY COMP-VOICE-OBJ, @iFORTH-OPS COMP-VALUE ;
	: VOICE-FVAL,	( xt -- )	>BODY COMP-VOICE-OBJ, @iFORTH-OPS COMP-FVALUE ;
	: INS-VAL,	( xt -- )	>BODY COMP-INS-OBJ, @iFORTH-OPS COMP-VALUE ;
	: INS-FVAL,	( xt -- )	>BODY COMP-INS-OBJ, @iFORTH-OPS COMP-FVALUE ;
	: VOICE-VALUE
		CREATE	ALIGNED DUP , 1 CELLS +
		['] VOICE-VAL, SET-COMPILER
		INTERP> NoInterp ;
	: VOICE-FVALUE
		CREATE	SFALIGNED DUP , 1 SFLOATS +
		['] VOICE-FVAL, SET-COMPILER
		INTERP> NoInterp ;
	: INSTR-VALUE
		CREATE	ALIGNED DUP , 1 CELLS +
		['] INS-VAL, SET-COMPILER
		INTERP> NoInterp ;
	: INSTR-FVALUE
		CREATE	SFALIGNED DUP , 1 SFLOATS +
		['] INS-FVAL, SET-COMPILER
		INTERP> NoInterp ;
[THEN]

#256 CONSTANT #max-midi-voices
HERE ," instrument stream" CONSTANT #instrument-id
#PRIVATE-DSP
	DSP-VALUE instrument-struct	\ the data structure
	CONSTANT #instr-plugin-size

1 CELLS						\ holds Instrument-Link
DUP	CONSTANT __first_midi_voice	#max-midi-voices CELLS +		\ pointers to the instrument voice structs
DUP	CONSTANT __instrument_name	#64 +
DUP	CONSTANT __instrument_class	#64 +
	INSTR-VALUE #instrument-voices
	INSTR-VALUE MIDIChannel			\ a bitmask for the MIDI commands channels the instrument listenes to
	INSTR-VALUE MIDIInterpreter		\ xt of the instruments midi command interprteter
	INSTR-VALUE CalculateAction		\ holds the xt to calculate the next val for this instrument
	INSTR-VALUE *accumulator		\ holds allocated buffer for sfloats data
	INSTR-VALUE #latest-voice		\ last used voice for this instrument
	INSTR-VALUE #running-voices		\ counts the currently processing voices
	INSTR-VALUE #do-bypassing?		\ a flag; when true the listening plugs will be set to bypassing state
	INSTR-VALUE #voice-struct-size		\ must be set before allocating an instrument struct
	INSTR-VALUE #voice-dynmem-size		\ each voice of an instrument has this size memory allocated at VoiceDynmem
	INSTR-VALUE #instr-dynmem-size		\ each instruments has also its private allocated mem at InstrumentDynmem
	INSTR-VALUE InstrumentDynmem

	INSTR-VALUE voice-init-xt		\ each voice of an instrument might be setup via this xt
	INSTR-VALUE instrument-init-xt		\ also the instrument might be initialised via this xt
						\ this allows turnkey applications

	INSTR-FVALUE VolumeCorrection		\ Correct the loadness of a tone by this value
	CONSTANT #instrument-common-size	\ THIS should be used by the specific instrument definitions

0	VOICE-VALUE StartNewNote?		\ flag is true when a voice is to be started
	VOICE-VALUE TimeoutToRelease		\ A voice in attack state will be forced to release when this sample counter is zero
	VOICE-VALUE TimeoutToAttack		\ The voice will be started when this counter is zero, used for precise starting times
	VOICE-VALUE MIDIVelocity		\ The velocity of this voice when started
	VOICE-VALUE MIDIPitch			\ the MIDI note number
	VOICE-VALUE MIDIJacktime		\ the exact frame when this note had started
	VOICE-VALUE AttackState?		\ A running voice is either in Attack or ReleaseState
	VOICE-VALUE ReleaseState?		\ Envelope
	VOICE-VALUE VoiceDynmem			\ when installing instruments each voice has a specifiewd aount of allocated memory

	VOICE-FVALUE VoiceVolume		\ master volume; can be set when starting the tone or later modified
	VOICE-FVALUE EnvelopeVolume		\ this volume will be changed at runtime to make the volume-envelope
	VOICE-FVALUE VoiceFrequency		\ As it says :-)
 	VOICE-FVALUE AttackStep			\ AR Envelope
 	VOICE-FVALUE ReleaseStep		\ AR Envelope
	VOICE-FVALUE VoiceDuration		\ precalculated length of this tone in seconds
	CONSTANT #voice-common-size


: is-bypassed?	( -- flag )	AttackState? ReleaseState? OR 0= ;
: mod-voice	( vce -- vce' )	#instrument-voices MOD ;
: *instr-voice	( n -- addr )	#instrument-voices MOD CELLS __first_midi_voice + @INSTRUMENT + ;
: set-voice	( n -- )	*instr-voice @ -->VOICE ;
: check-voices	( n -- n' )	DUP 1 #max-midi-voices 1+ WITHIN 0= ABORT" number of voices outside 1-maxvoices range" ;
: instrument-name ( -- addr )	@INSTRUMENT __instrument_name + ;
: instrument-class ( -- addr )	@INSTRUMENT __instrument_class + ;

: set-bypassing	( flag instrument -- )
	@DSP LOCALS| plug struct flag |
	ACTIVE-PLUGINS
	BEGIN	DUP @
	WHILE	DUP @ -->DSP #instrument-id plug_type =
		IF	instrument-struct struct = IF flag TO plug_bypassed THEN
		THEN
		1 CELLS +
	REPEAT DROP plug -->DSP ;
: basic-voice-volume		(  F: -- val )
	1e0 16e0 F/ ;
: init-voice-basics	( -- )
	FALSE		TO StartNewNote?
	FALSE		TO AttackState?
	FALSE		TO ReleaseState?
	0e0		TO VoiceVolume ;
: SetAttackTime		( f: time -- )		\ sets this for each voice of the current instrument
	1e0 JACK-SAMPLE-F-RATE F/ FSWAP F/
	#instrument-voices 0 DO I set-voice FDUP TO AttackStep LOOP FDROP ;
: SetReleaseTime	( f: time -- )
	1e0 JACK-SAMPLE-F-RATE F/ FSWAP F/
	#instrument-voices 0 DO I set-voice FDUP TO ReleaseStep LOOP FDROP ;
: SetVoiceDuration	( F: val -- )
	#instrument-voices 0 DO I set-voice FDUP TO VoiceDuration LOOP FDROP ;

: plus-voice	( time )
	DUP TO TimeoutToAttack
	JACK-POSITION + TO MIDIJacktime
	TRUE TO StartNewNote?					\ flag to fJack to start playing this note
	TRUE TO AttackState?
	FALSE TO ReleaseState?
	0e0 TO EnvelopeVolume
	1 #running-voices + DUP TO #running-voices
	1 = IF FALSE @INSTRUMENT set-bypassing THEN ;
: minus-voice
	FALSE TO ReleaseState?
	-1 #running-voices + DUP TO #running-voices
	0= IF TRUE TO #do-bypassing? THEN ;

TRUE	\ tools for debugging; might be false in release version
[IF]	: .FLAG	IF ."   +  " ELSE ."      " THEN ;
	: (SHOW.KSVOICE)
		??CR 6 .R
		MIDIJacktime 14 U.R
		MIDIPitch 4 .R
		MIDIVelocity 4 .R
		@VOICE 16 U.R
		VoiceVolume .LEVEL
		EnvelopeVolume .LEVEL
		TimeoutToRelease #1000 JACK-SAMPLE-RATE */ 99999 MIN 0 MAX 6 .R
		AttackState?	.FLAG
		ReleaseState?	.FLAG #80 HTAB ;
	: .vchead
		??CR  instrument-class COUNT TYPE ." /" instrument-name COUNT TYPE  #running-voices 6 .R ." /" #instrument-voices 1 .R ."  voices" #80 HTAB
		??CR ."   #vce ----TIME----- nte# vel --------DSP---   Level   ArLev -Time ATTK RLSE " #80 HTAB ;
	FORTH DEFINITIONS
	: SHOW-INSTRUMENT  ( -- )
		.vchead #instrument-voices 0 DO I set-voice I (SHOW.KSVOICE) LOOP ;
	: WATCH-VOICES
		CUROFF CLS
		BEGIN	0 0 AT-XY JACK-INFO #80 HTAB
			top-instrument
			BEGIN	?DUP
			WHILE	DUP @ SWAP -->INSTRUMENT SHOW-INSTRUMENT
			REPEAT
			#100 MS KEY?
		UNTIL	KEY DROP CURON CR ;
[THEN]
JACKAUDIO DEFINITIONS

\ *****************************************************************************************
\ following words are used with the instrument already set; voice management
: find-oldest-voice	( -- #vce )
	#latest-voice 1+ #FJACK-SAMPLE LOCALS| oldest vce |
	#latest-voice 1+ #instrument-voices BOUNDS
	DO	I set-voice
		MIDIJacktime oldest <
		IF I mod-voice TO vce  MIDIJacktime TO oldest THEN
	LOOP
	vce ;
: get-free-voice ( -- #vce )
	-1 #latest-voice 1+ #instrument-voices BOUNDS
	DO	I set-voice is-bypassed?
		IF DROP I mod-voice LEAVE THEN
	LOOP	DUP -1 =	\ fallback - use the oldest voice
	IF	DROP -1 +TO #running-voices
		find-oldest-voice
	THEN
	DUP TO #latest-voice ;
: current-voice-off	( samples -- )	\ the voice is set to release state after some samples
	AttackState?
	IF	1 MAX TO TimeoutToRelease
		TRUE TO ReleaseState?
		FALSE TO AttackState?
	ELSE	DROP
	THEN	;
: voice-off	( samples vce# -- )
	set-voice current-voice-off ;
: find-playing-note	( note# -- vce# )
	-1 LOCALS| #vce #note |
	#latest-voice 1+ #instrument-voices BOUNDS
	DO	I set-voice
		#note MIDIPitch =  AttackState? StartNewNote? OR AND	\ forget about notes in release state
		IF I mod-voice TO #vce LEAVE THEN
	LOOP #vce ;
: SetVoiceVelocity		( velocity -- )
	DUP TO MIDIVelocity
	S>F 127e0 F/  PI/2 F* FSIN TO VoiceVolume ;		\ FSIN gives nicer shape to vel response

: fully-attacked?	( -- flag )
	EnvelopeVolume AttackStep F+ FDUP TO EnvelopeVolume 1e0 F> ;
: fully-released?	( -- flag )
	EnvelopeVolume ReleaseStep F- FDUP TO EnvelopeVolume F0< ;
\ *************************************************************************************
\ instrument management
: free-instrument	( instrument -- )	\ free the allocated memory of an instrument; used for cleaning up
	-->INSTRUMENT
	TRUE @INSTRUMENT set-bypassing
	#instrument-voices 0 DO I *instr-voice @ FREE ?ALLOCATE LOOP
	*accumulator FREE ?ALLOCATE
	InstrumentDynmem FREE ?ALLOCATE
	@INSTRUMENT @ top-instrument MIN TO top-instrument ;

: allocate-instrument-mem	( -- )
	\ instrument is the pointer to the instrument struct
	\ #struct is the size of a voice struct used by this instrument
	\ #dynmem; allocated memory for each voice - VoiceDynmem
	#max-frame-size SFLOATS ALLOCATE ?ALLOCATE TO *accumulator
	#instr-dynmem-size ALLOCATE ?ALLOCATE TO InstrumentDynmem
	0 TO #running-voices
	0 TO #latest-voice
	instrument-init-xt ?DUP IF EXECUTE THEN
	#instrument-voices 0
	DO	#voice-dynmem-size #voice-struct-size + $200 + ALLOCATE ?ALLOCATE -->VOICE
		@VOICE I *instr-voice !
		@VOICE #voice-struct-size + ALIGNED16 TO VoiceDynmem
		init-voice-basics
		voice-init-xt ?DUP IF EXECUTE THEN
	LOOP
	TRUE TO #do-bypassing? ;
: link-instrument	( -- )
	top-instrument @INSTRUMENT !
	@INSTRUMENT TO top-instrument ;


\ =======================================================================================
\                               JACK MIDI INPUT HANDLER
\ =======================================================================================
: Instrument-MIDI-receiver	( midi-command-struct -- )
\ This receives all MIDI commands from DO-PROCESS-MIDI-IN-EVENT and presents them only to those
\ instruments with a corresponding bit-mask set - MIDIChannel
	MIDI-IN-EVENT-DATA JACK-POSITION - 0 MAX JACK-CHKSIZE 1- MIN NIP SWAP INTERPRET-MIDI-MESSAGE
	LOCALS| #data_2 #data_1 #midichannel #midicommand #time |
	top-instrument
	BEGIN	?DUP
	WHILE	DUP @ SWAP -->INSTRUMENT
		1 #midichannel LSHIFT MIDIChannel AND
		IF #time #midicommand #data_1 #data_2 MIDIInterpreter EXECUTE THEN
	REPEAT ;

\ =========================================================================================
\                                PLUGIN to stream the instrument into the channel
\ =========================================================================================
\ process the instrument for the whole bunch of the chunksize
: CalculateInstrument	\ ( instrument -- )
	-->INSTRUMENT
	#do-bypassing? #running-voices 0= AND IF TRUE @INSTRUMENT set-bypassing THEN
	FALSE TO #do-bypassing?
	#running-voices 0= IF EXIT THEN
	*accumulator  JACK-CHKSIZE 0 DO CalculateAction EXECUTE SF!+ LOOP DROP ;


: get-instrument-data	( val -- 'val )
	instrument-struct -->INSTRUMENT
	#FJACK-SAMPLE JACK-CHKSIZE 1- AND SFLOATS *accumulator + SF@
	plug_level F*
	F+ ;
: cnt-voices	( -- n )
	0 top-instrument
	BEGIN ?DUP WHILE DUP @ SWAP -->INSTRUMENT #running-voices ROT + SWAP REPEAT ;	

: finish-instruments
	#500 0 DO cnt-voices 0= IF LEAVE THEN #20 MS LOOP
	0 TO instrument-action
	['] DROP ['] DO-PROCESS-MIDI-IN-EVENT DEFER! ; ' finish-instruments AT-STOP-JACK-CLIENT

: midi-listener	( channel -- )	\ used when the instrument is set up
	1 SWAP LSHIFT TO MIDIChannel ;

FORTH DEFINITIONS
\ *N Instruments
\ *P An fJACKs "instrument" implements many things needed by real-time synthesizers, instrument and voices objects,
\ ** allocation of dynamic memory, managing the voices of an instruments listening to midi commands, management
\ ** of received MIDI commands and dispatching those commands to the available instruments depending on the midi channel.
\ *P The fJACK plugin system is fully supported, available are plugins 'listening' to the instruments voices.
\ *P This can be understood as a synthesizer middleware, currently there are two instruments coming with fjack.
\ ** There is the wonderful Karplus-Pluck string-like instrument (fjack/karplus.frt). It was implemented by
\ ** Bruno Degazio <lifemusic@sympatico.ca> and we worked hard to make it as performant as possible.
\ ** He also implemented & inspired important parts of the instruments internals and kept me debugging.
\ *P There is also the fjack/synth.frt module; it's a much simpler system and should be read as a 'make-your-own-synth'
\ ** addon.
\ *P Instruments are pretty complex, they produce signals on the fly and have to listen to MIDI commands. When developing
\ ** karplus-pluck we ran into many problems, if you dig into this, i suggest to use MIDI debugging :-) 

: MAKE-INSTRUMENT-STRUCT		\ name ( instrument-struct-size -- )
\ *G Instruments are structs - as you might have expected. Internals are mostly hidden and should
\ ** be used with great care. This defines an instrument struct with a given name and struct size.
\ ** After creating this instrument you will have to fill in some data to lets the instrument know
\ ** what to do and what data it should use. Look into the karplus.frt and synth.frt sources to get a better
\ ** idea. If you want to make a fjack instruments, just ask for suport.
	CREATE	HERE -->INSTRUMENT DUP ALLOT @INSTRUMENT SWAP ERASE
		1e0 TO VolumeCorrection
		@LATEST-NAME COUNT #30 MIN instrument-name PLACE
IS-IFORTH?
[IF]	FORGET>	free-instrument
[THEN]
	DOES>	; ( -- instrument )

: MAKE-INSTRUMENT-LISTENER	( instrument -- plugin )
\ *G Make a listener plugin. This behaves like most other plugins, it adds the instruments sound
\ ** to the audio stream. It is 'hard-wired' to the instrument, this can't be changed later.
\ ** You can set the volume for this plugin by *\forth{SET-OBJ-LEVEL}
	-->INSTRUMENT
	['] NOOP 0 #instr-plugin-size MAKE&ALLOCATE-PLUGIN-OBJECT
	#instrument-id TO plug_type  INSTRUMENT-PLUGIN TO plug_filtype
 	['] get-instrument-data TO plug_xt
 	['] NOOP TO plug_xt_tune
	@INSTRUMENT TO instrument-struct
	@DSP ;
:NONAME	['] Instrument-MIDI-receiver ['] DO-PROCESS-MIDI-IN-EVENT DEFER!
	['] CalculateInstrument TO instrument-action ; AT-START-JACK-CLIENT


:NONAME	top-instrument BEGIN ?DUP WHILE DUP @ SWAP -->INSTRUMENT allocate-instrument-mem REPEAT ; AtCold

IS-IFORTH?
[IF]	:NONAME DROP finish-instruments ; IS-FORGET MAKE-INSTRUMENT-LISTENER
	:ABOUT ??CR ." Generic MIDI instrument objects" ;
[THEN]
IS-VFX?
[IF]	prune: finish-instruments ;
[THEN]
PREVIOUS PREVIOUS DEFINITIONS
