(*
 * LANGUAGE    : Forth
 * PROJECT     : Working on MPE and iForth
 * DESCRIPTION : Interface for the libjack
 * CATEGORY    : Realtime Audio tools
 * AUTHOR      : Hanno Schwalm
 * LAST CHANGE  : 2009-05-04 01:24:36
		: 2011-07-24 23:10:21 added functions for session and latency support
		: 2017-10-25 07:28:46 simplyfied paths
 *)

JACKAUDIO DEFINITIONS
WINDOWS? [IF]
 64BIT?	[IF]	Library: libjack64.dll
	[ELSE]	Library: libjack.dll
	[THEN]
	Library: libfftw3f-3.dll
[THEN]

LINUX?	[IF]
	Library: libjack.so.0
	Library: libfftw3f.so.3
[THEN]

DARWIN? [IF]
	Library: libjack.0.dylib
	Library: libfftw3f.3.dylib
[THEN]

\ enum JackPortFlags
$001 CONSTANT JackPortIsInput
$002 CONSTANT JackPortIsOutput
$004 CONSTANT JackPortIsPhysical
$008 CONSTANT JackPortCanMonitor
$010 CONSTANT JackPortIsTerminal

\ enum JackOptions
$000 CONSTANT JackNullOption
$001 CONSTANT JackNoStartServer
$002 CONSTANT JackUseExactName
$004 CONSTANT JackServerName
$008 CONSTANT JackLoadName
$010 CONSTANT JackLoadInit
$020 CONSTANT JackSessionID \ currently the fjack doesn't support it 

\ enum JackStatus
 $001 CONSTANT JackFailure
 $002 CONSTANT JackInvalidOption
 $004 CONSTANT JackNameNotUnique
 $008 CONSTANT JackServerStarted
 $010 CONSTANT JackServerFailed
 $020 CONSTANT JackServerError
 $040 CONSTANT JackNoSuchClient
 $080 CONSTANT JackLoadFailure
 $100 CONSTANT JackInitFailure
 $200 CONSTANT JackShmFailure
 $400 CONSTANT JackVersionError
 $800 CONSTANT JackBackendError
$1000 CONSTANT JackClientZombie

$001 CONSTANT JackSessionSave
$002 CONSTANT JackSessionSaveAndQuit
$003 CONSTANT JackSessionSaveTemplate
$001 CONSTANT JackSessionSaveError
$002 CONSTANT JackSessionNeedTerminal

0 CONSTANT JackCaptureLatency
1 CONSTANT JackPlaybackLatency

\ enum jack_transport_state_t
$000 CONSTANT JackTransportStopped
$001 CONSTANT JackTransportRolling
$002 CONSTANT JackTransportLooping
$003 CONSTANT JackTransportStarting
$004 CONSTANT JackNetStarting
#-2  CONSTANT JackZombie

Extern: int "C" jack_get_version_string			( void );
Extern: int "C" jack_port_name_size			( void );
Extern: int "C"	jack_client_name_size 			( void );
Extern: jack_client_t "C" jack_client_open		( const char * client_name, int options, void * status, void * args );
Extern: int "C" jack_client_close			( jack_client_t * client);
Extern: int "C" jack_activate				( jack_client_t * client);
Extern: int "C" jack_deactivate				( jack_client_t * client);
Extern: int "C" jack_get_client_name			( jack_client_t * client);
Extern: int "C" jack_is_realtime			( jack_client_t * client);
Extern: jack_nframes_t "C" jack_get_sample_rate		( jack_client_t * client);
Extern: float "C" jack_cpu_load				( jack_client_t * client);
Extern: jack_nframes_t "C" jack_get_buffer_size		( jack_client_t * client);

Extern: void "C" jack_port_get_latency_range		( jack_port_t * port, int mode, jack_latency_range_t * range);
Extern: void "C" jack_port_set_latency_range		( jack_port_t * port, int mode, jack_latency_range_t * range);
Extern: int "C" jack_recompute_total_latencies		( jack_client_t * client );

Extern: int "C" jack_set_session_callback		( jack_client_t * client, void * session_callback, void * arg);
Extern: int "C" jack_session_reply			( jack_client_t * client, jack_session_event_t * event);
Extern: void "C" jack_session_event_free		( jack_session_event_t * event);

\ Extern: int "C" jack_client_get_uuid			( jack_client_t * client);
Extern: int "C" jack_get_uuid_for_client_name		( jack_client_t * client, const char * client_name);
Extern: int "C" jack_reserve_client_name		( jack_client_t * client, const char * name, const char * uuid);

Extern: jack_port_t "C" jack_port_register		( jack_client_t * client, const char * port_name, const char * port_type, int flags, int buffer_size);
Extern: int "C" jack_port_unregister			( jack_client_t * client, jack_port_t * id);
Extern: int "C" jack_port_flags				( jack_port_t * port);
Extern: int "C" jack_port_get_buffer			( jack_port_t * port, jack_nframes_t size );
Extern: int "C" jack_port_connected			( jack_port_t * port);
Extern: int "C" jack_port_name				( jack_port_t * port);
Extern: int "C" jack_port_disconnect			( jack_client_t * client, jack_port_t * port);
Extern: int "C" jack_connect				( jack_client_t * client, const char * source_port, const char * destination_port);
Extern: int "C" jack_disconnect				( jack_client_t * client, const char * source_port, const char * destination_port);
Extern: int "C" jack_port_get_connections		( jack_port_t * port);
Extern: int "C" jack_port_by_name			( jack_client_t * client, jack_port_t * port);

Extern: jack_time "C" jack_get_time			( void);
Extern: void "C" jack_on_shutdown			( jack_client_t * client, void * Shutdowncallback, void * arg );
Extern: int "C" jack_set_process_callback		( jack_client_t * client, void * process_callback, void * arg);
Extern: int "C" jack_set_sample_rate_callback		( jack_client_t * client, void * srate_callback, void * arg);
Extern: int "C" jack_set_buffer_size_callback		( jack_client_t * client, void * bufsize_callback, void * arg);
Extern: int "C" jack_set_client_registration_callback	( jack_client_t * client, void * registration_callback, void * arg);
Extern: int "C" jack_set_port_registration_callback	( jack_client_t * client, void * registration_callback, void * arg);
Extern: int "C" jack_set_port_connect_callback		( jack_client_t * client, void * connect_callback, void * arg);
Extern: int "C" jack_set_graph_order_callback		( jack_client_t * client, void * graph_callback, void * arg);
Extern: int "C" jack_set_xrun_callback			( jack_client_t * client, void * xrun_callback, void * arg);
Extern: int "C" jack_set_sync_callback			( jack_client_t * client, void * sync_callback, void * arg);
Extern: int "C" jack_set_latency_callback		( jack_client_t * client, void * latency_callback, void * arg-fjack);

Extern: void "C" jack_set_error_function		( void * writeerror);
Extern: void "C" jack_set_info_function			( void * function);

Extern: jack_transport_state_t "C" jack_transport_query	( jack_client_t * client, jack_position_t * pos);
Extern: void "C" jack_transport_start			( jack_client_t * client);
Extern: void "C" jack_transport_stop			( jack_client_t * client);

Extern: void "C" jack_free				( void * ptr );

Extern: void "C" jack_midi_clear_buffer			( void * port_buffer);
Extern: int "C" jack_midi_event_get			( jack_midi_event_t * event, void * port_buffer, jack_nframes_t event_index);
Extern: jack_midi_data_t "C" jack_midi_event_reserve	( void * port_buffer, jack_nframes_t  time, int data_size);
Extern: int "C" jack_midi_event_write			( void * port_buffer, jack_nframes_t time, const jack_midi_data_t * data, int data_size);
Extern: jack_nframes_t "C" jack_midi_get_event_count	( void * port_buffer);
Extern: jack_nframes_t "C" jack_midi_get_lost_event_count ( void * port_buffer);
Extern: int "C" jack_midi_max_event_size		( void * port_buffer);



: JACK-LIB?	['] jack_set_process_callback FUNC-LOADED? 0<> ;
: JACK-SESSION?	['] jack_set_session_callback FUNC-LOADED? 0<> ;
: JACK-LATENCY?	['] jack_set_latency_callback FUNC-LOADED? 0<> ;
: JACK-LIB+?	['] jack_set_info_function FUNC-LOADED? 0<> ;

