\ ======================================================================================
\  Karplus_Pluck_JACK
\  Created by BGDegazio on 2012-06-21.
\  Copyright (c) 2012. All rights reserved.
\ ======================================================================================
\ Changes:
\ 2012-07-09 16:25:10 jhs;  integrated in iforth forgetting sheme
\               Using At-START-JACK-CLIENT and AT-STOP-JACK-CLIENT
\ Last Revision: Thu Aug  9 16:16:36 EDT 2012 - bgd
\ jhs: major rewrite with implemented fjack/instrument and factored
\ lots of code to there to make life easier for other instruments
\ jhs 2012-09-02 17:26:18

[DEFINED] -fjack/instrument 0=	[IF] INCLUDE fjack/instrument.frt [THEN]

IS-IFORTH?
[IF]	REVISION -fjack/karplus "--- Karplus-Pluck       Version 0.18 ---"
[ELSE]	ANEW -fjack/karplus
[THEN]

DECIMAL ALSO JACKAUDIO ALSO DEFINITIONS

TRUE CONSTANT EXPERIMENTAL? IMMEDIATE
\ ----------------------------------------------------------------------------------------
\       ------------------------- BASIC JACK SETUP ----------------------
\ -------------------------------------------------------------------------------------
$4000 CONSTANT #pluck-size			\ this should be enough for 96kHz sample rate with all tones

#voice-common-size
	VOICE-VALUE KSLegatoMode		\ nyi
	VOICE-VALUE PreviousKarplusSample#	\ pointer into wavelength table

	VOICE-FVALUE KSRandFreq			\ frequency detune randomness

	VOICE-FVALUE KarplusWavelength		\ basic tuning parameter
	VOICE-FVALUE KarplusWavelengthInc	\ basic tuning parameter - smoothing
	VOICE-FVALUE KarplusWavelengthGoal	\ basic tuning parameter- smoothing

	VOICE-FVALUE KarplusAmplitudeInc	\ for smoothing
	VOICE-FVALUE KarplusAmplitudeGoal	\ for smoothing

	VOICE-FVALUE KSReleaseLength		\ envelope rate for note release
	VOICE-FVALUE KSSmoothTime		\ for sommthing
	VOICE-FVALUE MyLastSample		\ single sample delay line

	\ parameters for Moore-Chamberlin tuning filter
	VOICE-FVALUE KarplusShrinkFactor	\ used for releases only, otherwise at 1.0e	\ = Moore's rho
	VOICE-FVALUE KarplusStretchFactor	\ always leave at 1.0e  (not 0.5)		\ = Moore's 'S'
	VOICE-FVALUE KarplusTuningValue		\ used by Moore tuning allpass			\ = Moore's 'C'
	VOICE-FVALUE MooreLastF1		\ used by Moore tuning allpass
	VOICE-FVALUE MooreLastF2		\ used by Moore tuning allpass
	VOICE-FVALUE KSAccumulator		\ used by Chamberlin LPF
	VOICE-FVALUE KSLPF'K'			\ used by Chamberlin LPF
	VOICE-FVALUE KSLPF'L'			\ these two must always be inverse of one another
	VOICE-FVALUE KSLPF'K'-INC		\ experimental -  dynamic change of LPF to shorten overly-long notes
	VOICE-FVALUE KSLPF'K'-INIT		\ experimental - initial value used by LPF

	\ NYI
	VOICE-FVALUE KS_LFO_PHASE
	VOICE-FVALUE KS_LFO_INCRM		\ 1/JACK-SAMPLE-F-RATE
	VOICE-FVALUE KS_LFO_FM			\ in Hz
	VOICE-FVALUE KS_LFO_AM
	CONSTANT #karplus-voice-size

#instrument-common-size
	INSTR-VALUE KarplusFiller-xt
	INSTR-VALUE MyKarplus-Filter-xt
	INSTR-VALUE NormalizeInput?
	INSTR-VALUE RemoveDC?
	INSTR-VALUE SmoothInput?
	INSTR-VALUE HPFInput?
	INSTR-VALUE ApplyWindow?
	INSTR-VALUE AddChamberlinLPF?
	CONSTANT #karplus-instrument-size

\   ---------- initialise voice ----------
: init-karplus-voice	( voice -- )
	init-voice-basics	\ basic karplus parameters and now karplus specific
	220e0		TO VoiceFrequency
	0e0		TO KSRandFreq
	512e0		TO KarplusWavelength		\ basic tuning parameter
	0e0		TO KarplusWavelengthInc		\ basic tuning parameter - smoothing
	512e0		TO KarplusWavelengthGoal	\ basic tuning parameter- smoothing

	0		TO PreviousKarplusSample#	\ pointer into wavelength table
	0e0		TO MyLastSample			\ single sample delay line
	0.9e0		TO KSReleaseLength		\ controls speed of note release
	0.1e0		TO KSSmoothTime			\ for smoothing paramter changes
	0e0		TO KarplusAmplitudeInc		\ for smoothing
	0e0		TO KarplusAmplitudeGoal		\ for smoothing
	2e0		TO VoiceDuration
	FALSE		TO KSLegatoMode			\ testing

	\ parameters for Moore-Chamberlin tuning filter
	1e0		TO KarplusShrinkFactor		\ used for releases only, otherwise at 1.0e
	1e0		TO KarplusStretchFactor		\ always leave at 1.0e  (not 0.5)
	0e0		TO KarplusTuningValue		\ used by Moore tuning allpass
	0e0		TO MooreLastF1			\ used by Moore tuning allpass
	0e0		TO MooreLastF2			\ used by Moore tuning allpass
	0e0		TO KSAccumulator		\ used by Chamberlin LPF
	0.8e0		TO KSLPF'K'			\ used by Chamberlin LPF
	1e0 KSLPF'K' F- TO KSLPF'L'			\ these two must always be inverse of one another
	0e0		TO KSLPF'K'-INC			\ experimental - dynamic change of LPF to shorten overly-long notes
	0.8e0		TO KSLPF'K'-INIT		\ experimental - dynamic change of LPF to shorten overly-long notes

	\ LFO PARAMETERS    - NYI
	PI F2* FRANDOM F*				TO KS_LFO_PHASE
	PI F2* JACK-SAMPLE-F-RATE F/ 5e0 FRANDOM F+ F*	TO KS_LFO_INCRM		\ 5-6 Hz LFO rate
	0e0		TO KS_LFO_FM
	0e0		TO KS_LFO_AM ;

(*
\ **********************************************************************************
\ *************************  KARPLUS_STRONG PLUCKED STRING   ***********************
\ **********************************************************************************
*)

(*   --------------------------------------------------------------------------

WAVETABLE INITIALIZATION
    \ OPTIONS:  a) classic random number wavetable - very thin and tinny
                b) variation on classic version - random walk - fatter sounding
                c) single-cycle sine-wave + randomwalk deviance - boosts fundamental freq.
                d) live-input
    --------------------------------------------------------------------------  *)

\ --------------------------------------------------
\ a) classic version - fill buffer with random numbers
\ very thin and tinny
\ --------------------------------------------------
: RandomKarplusFiller ( #frames -- )
	KarplusWavelength F>S 1+ MAX VoiceDynmem SWAP
	0 DO FRANDOM F2* 1e0 F- SF!+ LOOP DROP ;

\ --------------------------------------------------
\ b) fills with sine wave plus randomwalk
\ much fuller sounding than classic version above
\ --------------------------------------------------
\ : SineWaveKarplusFiller	( #frames )
\ 	KarplusWavelength F>S 1+ MAX DUP S>F 1e0 FSWAP F/ PI F2* F/
\ 	0e0 {: #frames f: corrector f: prevrandom -- :}
\ 	VoiceDynmem  #frames 0
\ 	DO	FRANDOM F2* 1e0 F-
\ 		VoiceVolume F* prevrandom F+ TO prevrandom
\ 		I S>F  corrector F* FSIN
\ 		prevrandom F+
\ 		SF!+
\ 	LOOP DROP ;
: SineWaveKarplusFiller	( #frames )
	KarplusWavelength F>S 1+ MAX DUP S>F 1e0 FSWAP F/ PI F2* F/  0e0	\ corrector prevrandom
	VoiceDynmem SWAP 0
	DO	FRANDOM F2* 1e0 F-
		VoiceVolume F* F+			\ corrector prevrandom
		FOVER I S>F  F* FSIN
		FOVER F+
		SF!+
	LOOP DROP FDROP FDROP ;


(* --------------------------------------------------------------------------
\ -----------------------------------------------------------------------------
    WAVETABLE UTILITES

    a) SUBTRACT.AVERAGE - removes DC from wavetable
    b) SMOOTH.INPUT     - simple LPF from wavetable
    c) HPF.INPUT        - simple High Pass Filter wavetable
    d) GAIN.INPUT       - adjust level of wavetable
    e) NORMALIZE        - adjust level to max 1.0
    f) SinCWindow       - apply a 1/2 sinewave window to wavetable
    --------------------------------------------------------------------------
*)

\ --------------------------------------------------------------------------
\ a) removes DC from wavetable
\ --------------------------------------------------------------------------
: SUBTRACT.AVERAGE.KS		( VoiceDynm size -- )
	2DUP 0e0
	0 DO SF@+ F+ LOOP DROP	\ float has avarage
	DUP S>F F/
	0 DO DUP SF@ FOVER F- SF!+ LOOP DROP FDROP ;

\ --------------------------------------------------------------------------
\ b) pre-apply simpleLPF to wavetable
\ --------------------------------------------------------------------------
0.1e0 FVALUE SmoothK
0.9e0 FVALUE SmoothL
: SMOOTH.INPUT	( VoiceDynm size -- )
	0e0 0
	DO			SmoothL F*
		DUP SF@		SmoothK F*
		F+ FDUP SF!+
	LOOP DROP FDROP ;

\ --------------------------------------------------------------------------
\ c) apply simple HPF TO input
\ --------------------------------------------------------------------------
-0.5e0 FVALUE MOORE.HPF.'K'
: HPF.INPUT		( VoiceDynm size -- )
	0e0
	0 DO	MOORE.HPF.'K' F*
		DUP SF@ F+ FDUP SF!+
	LOOP DROP FDROP ;

\ --------------------------------------------------------------------------
\ d) adjust amplitude of input
\ --------------------------------------------------------------------------
: GAIN.INPUT		( VoiceDynmem size -- ) ( F: gain -- )
	0 DO FDUP DUP SF@  F* SF!+ LOOP DROP FDROP ;

\ --------------------------------------------------------------------------
\ e) normalize input and adjust amplitude
\ --------------------------------------------------------------------------
10e0 FCONSTANT NormalizeLimit
: FIND.NORMALIZE-FACTOR		( VoiceDynmem size -- ) ( F: --val )
	0e0 0 DO SF@+ FABS FMAX LOOP DROP 1e0 FSWAP F/ ;

: NORMALIZE.SFLOAT.DATA		( VoiceDynmem size -- )
	2DUP FIND.NORMALIZE-FACTOR
	NormalizeLimit FMIN
	0 DO DUP SF@ FOVER F* SF!+ LOOP DROP FDROP ;

\ ----------------------------------------------------------------------------
\ f) 1/2 sin wave window
\ works well for live input
\ ----------------------------------------------------------------------------
: SinCKarplusWindow	( addr size )
 	DUP S>F PI F/ 0 DO I S>F FOVER F/  DUP SF@ F* SF!+ LOOP DROP FDROP ;


: FillKSWaveTable	( #frames -- )
	1+ {: size -- :}
	size 1 < IF EXIT THEN
 	size KarplusFiller-xt EXECUTE		\ use one of the above wavetable filling routines
	RemoveDC?	IF VoiceDynmem	size SUBTRACT.AVERAGE.KS	THEN
	NormalizeInput?	IF VoiceDynmem	size NORMALIZE.SFLOAT.DATA	THEN
	HPFInput?	IF VoiceDynmem	size HPF.INPUT			THEN
	SmoothInput?	IF VoiceDynmem	size SMOOTH.INPUT		THEN
	ApplyWindow?	IF VoiceDynmem	size SinCKarplusWindow		THEN
	;

\ ---------------------------------------------------------
\   ---------- perform DSP calculation per voice ----------
\   ----------  this is the heart of the algorithm --------
\ ---------------------------------------------------------
: calc-karplus-voice
	StartNewNote?
	IF	FALSE TO StartNewNote?
		KSLegatoMode 0=						\ POLY MODE
		IF	VoiceDynmem #pluck-size SFLOATS ERASE
			KarplusWavelength F>S FillKSWaveTable		\ new note, so fill wavetable random, optionally normalize, etc.
			0 TO PreviousKarplusSample#
 			VoiceDynmem SF@
				FDUP TO MyLastSample
 				FDUP TO MooreLastF1
 				TO MooreLastF2
 			0e0 TO KSAccumulator
		THEN
	THEN
	TimeoutToAttack 0> IF -1 +TO TimeoutToAttack 0e0 EXIT THEN

 	\ ** CORE PLUCK takes place here ***
 	VoiceDynmem PreviousKarplusSample# SFLOATS + SF@  FDUP
 	MyKarplus-Filter-xt EXECUTE							\ does the heavy work here
  	VoiceDynmem PreviousKarplusSample# SFLOATS + SF!
  	PreviousKarplusSample# 1+ KarplusWavelength F>S MOD TO PreviousKarplusSample#	\ increment for next time

 	\ increment attack-release envelope
	AttackState?
	IF	fully-attacked?
		IF	1e0 TO EnvelopeVolume TimeoutToRelease 0>
			IF -1 +TO TimeoutToRelease ELSE FALSE TO AttackState? TRUE TO ReleaseState? THEN
		THEN
	THEN
	ReleaseState?
	IF	0 TO TimeoutToRelease  fully-released?
		IF 0e0 TO EnvelopeVolume minus-voice THEN
	THEN
	EnvelopeVolume F*			\ apply AR envelope
	VoiceVolume F* ;			\ apply per-note amplitude

: calc-karplus-instrument	( -- ) ( F: -- 'val )
	0e0 #instrument-voices 0
	DO	I set-voice
		is-bypassed? 0= IF calc-karplus-voice F+ THEN
	LOOP
	VolumeCorrection F* ;



(*  --------------------------------------------------------------------------

    INTERNAL DELAY LINE LOOP FILTER
    This is the heart of the KS pluck algorithm

    a) ClassicKarplusFilter - simple adjacent sample average
    b) MooreKarplusFilter   - with tuning allpass, stretch and shrink, + Chamberlin LPF from Moore, p. 291

    --------------------------------------------------------------------------
*)

\ ---------------------------------------------------------------
\   a) ClassicKarplusFilter - simple adjacent sample average
\ ---------------------------------------------------------------
: ClassicKarplusFilter	( F: sample -- 'sample )
	FDUP MyLastSample F+ 0.5e0 F* FSWAP TO MyLastSample ;

\ --------------------------------------------------------------
\ ---------- b.  with tuning and stretch/shrink as per Moore ---
\ --------------  and additional LPF from Chamberlin -----------
\ --------------------------------------------------------------
: MooreKarplusFilter	( F: val -- val' )	\ stretch and shrink filter
	\ This is the same algorithm as above but increasing performance by ~30%
	1e0 KarplusStretchFactor F- FOVER F*
	KarplusStretchFactor MyLastSample F*
	F+
	KarplusShrinkFactor F* \ sample mooref1

	FSWAP TO MyLastSample		\ mooref1

	\ tuning allpass filter     -
	FDUP KarplusTuningValue F*
	MooreLastF1 F+
	MooreLastF2 KarplusTuningValue F* F-	\ mooref1 mooref2
	FDUP TO MooreLastF2 FSWAP TO MooreLastF1	\ mooref2

	AddChamberlinLPF?
	IF	\  LPF as per Chamberlin
		KSLPF'K' F*
		KSAccumulator KSLPF'L' F*
		F+ FDUP TO KSAccumulator
	THEN ;


\ =============================================================================
\           -------------- STRETCH AND SHRINK CALCULATION --------------
                \           from Moore p. 289
\ =============================================================================

-40.0e0 FVALUE 'KSQ'      \ desired loss in dB

\ Gf is the loop gain required for Q dB loss over dur seconds
: CALC-KS'GF'	( F: frequency dur -- gain )
	F* 20e F* 'KSQ' FSWAP F/     10.0e FSWAP F** ;

\ G is the loop gain witout adjustments to Rho and 'S'
: CALC-KS'G'	( F: frequency -- gain )
	PI F* JACK-SAMPLE-F-RATE F/ FCOS ;

: CALC-STRETCH.FACTOR {:  f: freq  f: 'Gf' | f: cosf1  f: 'a'  f: 'b'  f: 'c'  f: 'D'  f: 'a2'  f: 'S1'  f: 'S2' -- f: 'S'  :}
	freq JACK-SAMPLE-F-RATE F/ PI F2* FDEPTH . F* F2*		TO cosf1

	2e0 cosf1 F-				TO 'a'
	cosf1 2e0 F-				TO 'b'

	1e0 'Gf' FDUP F* F-			TO 'c'
	'b' 'b' F*  'a' 'c' F*  4e0 F* F- FSQRT	TO 'D'
	'a' 2e0 F*				TO 'a2'
	'D' 'b' F- 'a2' F/			TO 'S1'
	'D' FNEGATE 'b' F-  'a2' F/		TO 'S2'

	\ use value of S that stretches rather than flattens partials
	0e0 'S1' F>  0.5e0 'S1' F< AND
	IF	'S1'
	ELSE	'S2'
	THEN  ;

\ rho is duration shrink factor
\ 'S' is duration stretch factor
: CALC-SHRINK&STRETCH	{: f: dur f: freq |  f: rho  f: 'S'  f: 'Gf'  f: 'G' -- f: rho  f: S :}
	1e0	TO rho
	0.5e0	TO 'S'
	0e0	TO 'Gf'
	0e0	TO  'G'

	freq dur CALC-KS'GF'	TO 'Gf'
	freq     CALC-KS'G'	TO 'G'
	'Gf' 'G' F<=
	IF	'Gf' 'G' F/ TO rho
	ELSE	freq 'Gf' CALC-STRETCH.FACTOR TO 'S'
	THEN
	rho 'S' ;

\ =============================================================================
\               ----------- TUNING ALGORITHM from Moore p.289 --------------
\ =============================================================================
(*  loop = JACK-SAMPLE-F-RATE/freq
    'P' = integer part of loop
    'S' = stretch factor calcualted previously
    'D' = temp
    'C' = tuning allpass factor
*)
: TUNE.KS	{:  f: freq  f: 'S'  f: 'L' |  f: theloop  f: thedelay  f: 'P'  f: 'D'  f: 'C' -- f: 'P' f: 'C' :}
	\ P	# of sample in loop table
	\ C =	tuning all-pass filter
	\ 'L'	extra Chamberlin filter
	\ 'S'	stretch factor calculated previously
	\ freq	desired frequency
	0e0 TO theloop
	0e0 TO thedelay
	0e0 TO 'P'			\ wavetable integer size (as float)
	0e0 TO 'D'			\ temp
	0e0 TO 'C'			\ tuning allpass factor

	JACK-SAMPLE-F-RATE freq F/		TO theloop
	theloop FTRUNC		TO 'P'
	'P' 'S' F+ 'L' F+	TO thedelay

	thedelay theloop F>
	IF	-1e0 'P' F+ TO 'P'
		'P' 'S' F+ 'L' F+ TO thedelay
	THEN
	theloop thedelay F-	TO 'D'
	1e 'D' F-  1e 'D' F+ F/ TO 'C'

	'P' 'C' ;

: start-karplus-note	{: vce# note# vel time# -- :}
	vel 0=
	IF	note# find-playing-note DUP -1 <> IF time# SWAP voice-off ELSE DROP THEN EXIT
	ELSE	vce# set-voice
		time# plus-voice
		note#       TO MIDIPitch
		vel SetVoiceVelocity
		\ set duration factors
		note# Note#>Frequency SF@ FDUP TO VoiceFrequency
		VoiceDuration
		FOVER 261e0 FSWAP F/ F*
		FDUP JACK-SAMPLE-F-RATE F* F>S TO TimeoutToRelease	\ scale duration by frequency, middle-C = 1.0
		CALC-SHRINK&STRETCH
		TO KarplusStretchFactor
		TO KarplusShrinkFactor

		VoiceFrequency  FRANDOM 0.5e0 F- KSRandFreq F*		\ detune amount - OK at 100 Hz
		FOVER 261e F/ F* F+					\ scale detuning according to basic freq - exact at middle -C
		KarplusStretchFactor KSLPF'L'
		TUNE.KS
		TO KarplusTuningValue TO KarplusWavelength

		0e0 TO KarplusAmplitudeInc
		EXPERIMENTAL?
		[IF]	note# S>F 56e0 F/ 0.2e0 F+
			VoiceVolume F* TO VoiceVolume

		[THEN]
	THEN  ;

: karplus-midi-receiver	{: #time #midi #note #vel -- :}
	#midi
	CASE	$80 OF	#note find-playing-note DUP -1 <>
			IF	#time SWAP voice-off
			ELSE	DROP
			THEN
		ENDOF
		$90 OF	get-free-voice DUP -1 <>
			IF	#note #vel #time start-karplus-note
			ELSE	DROP
			THEN
		ENDOF
    	ENDCASE ;

\ =======================================================================================
\                               UTILITIES
\ =======================================================================================

: (SetKSChamberlinFilter)	(  f: K --)
	FDUP TO KSLPF'K'  FDUP TO KSLPF'K'-INIT  1e0 FSWAP F- TO KSLPF'L' ;

: SetKSChamberlinFilter		( f:val -- )	\ .5e SetKSChamberlinFilter
	0.5e0 FMIN 1e0 FMAX		\ blows up outisde this range
	#instrument-voices 0 DO I set-voice FDUP (SetKSChamberlinFilter) LOOP FDROP ;

: SetKSRandFreq			( f: val --)		\ 4e SetKSRandFreq
	#instrument-voices 0 DO I set-voice FDUP TO KSRandFreq LOOP FDROP ;

: PolyMode			( -- )
	#instrument-voices 0 DO I set-voice FALSE TO KSLegatoMode LOOP ;

\ =======================================================================================
\                               INSTALL AND PLAY
\ =======================================================================================

FORTH DEFINITIONS

\ ==============================================================================
: MAKE-KARPLUS-INSTRUMENT	\ name ( voices -- )
\ *G This defines a named karplus-pluck instrument structure, you must pass the number of
\ ** voices this instrument has. The new word will return the
\ ** address of the structure. Code for allocating & deallocating dynamic memory is managed automatically.
	check-voices #karplus-instrument-size MAKE-INSTRUMENT-STRUCT
	#karplus-voice-size TO #voice-struct-size
	#pluck-size SFLOATS TO #voice-dynmem-size
	TO #instrument-voices
	0 TO #instr-dynmem-size
	['] init-karplus-voice TO voice-init-xt
	0 TO instrument-init-xt
	S" Karplus_Pluck" instrument-class PLACE
	allocate-instrument-mem
	\ The next two MUST be set after allocation
	['] karplus-midi-receiver TO MIDIInterpreter
	['] calc-karplus-instrument TO CalculateAction
	link-instrument	;

#37 MAKE-KARPLUS-INSTRUMENT STANDARD-KARPLUS

: DEFAULT-KARPLUS		( midi-channel -- )		\ you may define other pluck instruments like this
\ *G This uses the existing *\forth{STANDARD-KARPLUS}instrument and defines how it should sound ...
	STANDARD-KARPLUS -->INSTRUMENT midi-listener
	\ --------------------------------
	\ --- basic plucked instrument ---
	\ --------------------------------
	['] SineWaveKarplusFiller TO KarplusFiller-xt
	['] MooreKarplusFilter TO MyKarplus-Filter-xt
	PolyMode
	FALSE TO NormalizeInput?	\ on only for live input
	 TRUE TO RemoveDC?
	FALSE TO SmoothInput?
	FALSE TO HPFInput?
	 TRUE TO ApplyWindow?
 	 TRUE TO AddChamberlinLPF?
 	0.5e0 SetKSChamberlinFilter	\ equivalent to classic averaging filter
 	  4e0 SetKSRandFreq		\ moderate detuning
	  5e0 SetVoiceDuration		\ 5 seconds pluck
 	0.001e0 SetAttackTime		\ very sharp attack
 	1.25e0 SetReleaseTime		\ medium release
	basic-voice-volume 0.1e0 F* TO VolumeCorrection ;
: SIMPLE-KARPLUS		( midi-channel -- )		\ you may define other pluck instruments like this
\ *G This uses the existing *\forth{STANDARD-KARPLUS}instrument and defines how it should sound ...
	STANDARD-KARPLUS -->INSTRUMENT midi-listener
	\ --------------------------------
	\ --- basic plucked instrument ---
	\ --------------------------------
	['] RandomKarplusFiller TO KarplusFiller-xt
	['] ClassicKarplusFilter TO MyKarplus-Filter-xt
	PolyMode
	FALSE TO NormalizeInput?	\ on only for live input
	 TRUE TO RemoveDC?
	 TRUE TO SmoothInput?
	FALSE TO HPFInput?
	 TRUE TO ApplyWindow?
 	 TRUE TO AddChamberlinLPF?
 	0.5e0 SetKSChamberlinFilter	\ equivalent to classic averaging filter
 	  4e0 SetKSRandFreq		\ moderate detuning
	  5e0 SetVoiceDuration		\ 5 seconds pluck
 	0.001e0 SetAttackTime		\ very sharp attack
 	1.25e0 SetReleaseTime		\ medium release
	basic-voice-volume 0.1e0 F* TO VolumeCorrection ;

STANDARD-KARPLUS MAKE-INSTRUMENT-LISTENER CONSTANT KARPLUS-PLUCK-0
STANDARD-KARPLUS MAKE-INSTRUMENT-LISTENER CONSTANT KARPLUS-PLUCK-1


IS-IFORTH?
[IF]	:ABOUT ??CR ." The fJACK Karplus-Pluck synthesizer based on Bruno Degazios code" ;
	NESTING @ 1 = [IF] .ABOUT -fjack/karplus [THEN]
[THEN]


PREVIOUS PREVIOUS DEFINITIONS
