(*
 * LANGUAGE    : iForth
 * PROJECT     : Audio processing
 * DESCRIPTION : Fast&Simple multitone synthesizer
 * CATEGORY    : fjack audio processing addon
 * AUTHOR      : Hanno Schwalm
 * LAST CHANGE : 2012-07-19 11:52:47; fixed to work with all 32bit versions too.
		 2012-08-05 12:42:53; stopping and restarting a tone is done in the state machine
		 2012-09-02 22:40:35; using instrument
 *)

[DEFINED] -fjack/instrument 0=	[IF] INCLUDE fjack/instrument.frt [THEN]

IS-IFORTH?
[IF]	REVISION -fjack/synth "--- Simple Synthesizer  Version 0.88 ---"
[ELSE]	ANEW -fjack/synth
[THEN]

ALSO JACKAUDIO ALSO JACKAUDIO DEFINITIONS

$10000	CONSTANT #sintab 	\ size of sinus array for x->2pi
#14	CONSTANT #fractbits	\ #extra bits in phases/adders are fraction

#sintab #fractbits LSHIFT S>F FCONSTANT fp_scaler
: >sintab	( idx -- addr )		#fractbits RSHIFT #sintab 1- AND SFLOATS InstrumentDynmem + ;
: @sinus	( idx -- ) ( F: -- v )	>sintab SF@ ;
: frequ->step	( -- n ) ( F: f -- )	JACK-SAMPLE-F-RATE F/ fp_scaler F* F>S ;

#voice-common-size
	VOICE-VALUE VoicePhase
	VOICE-VALUE VoicePhase-add
	VOICE-VALUE VoicePhase-corrector
	VOICE-VALUE LesliePhase
	VOICE-VALUE LesliePhase-add
	VOICE-FVALUE FadeoutReduction
	VOICE-FVALUE VoiceLeslieShift
	CONSTANT #synth-voice-size

#instrument-common-size
	INSTR-FVALUE SynthLeslieShift
	INSTR-FVALUE SynthLeslieFrequency
	INSTR-FVALUE SynthOverTone
	CONSTANT #synth-instrument-size

: (calc-synth-voice)	( -- )		( F: -- val )
	LesliePhase-add +TO LesliePhase
	LesliePhase @sinus VoiceLeslieShift F* F>S TO VoicePhase-corrector
	VoicePhase-add VoicePhase-corrector + +TO VoicePhase
	VoicePhase @sinus ;

: calc-synth-voice	( -- ) ( F: -- val )
	StartNewNote?
	IF	FALSE TO StartNewNote? 0e0 TO EnvelopeVolume
	THEN
	TimeoutToAttack 0> IF -1 +TO TimeoutToAttack 0e0 EXIT THEN
	(calc-synth-voice)
	AttackState?
	IF	fully-attacked?
		IF	1e0 TO EnvelopeVolume  TimeoutToRelease
			IF	-1 +TO TimeoutToRelease
				VoiceVolume FadeoutReduction F- 0e0 FMAX TO VoiceVolume
			ELSE	FALSE TO AttackState? TRUE TO ReleaseState?
			THEN
		THEN
	THEN
	ReleaseState?
	IF	0 TO TimeoutToRelease
		fully-released?
		IF 0e0 TO EnvelopeVolume minus-voice THEN
	THEN
	EnvelopeVolume F*
	VoiceVolume F* ;

: calc-synth-instrument	( -- ) ( F: -- 'val )
	0e0 #instrument-voices 0
	DO	I set-voice
		is-bypassed? 0= IF calc-synth-voice F+ THEN
	LOOP
	VolumeCorrection F* ;

: fill-sinus-tab
	#sintab 0
	DO	I S>F #sintab S>F F/ 2e0 F* PI F* FSIN FDUP F0<
 		IF FNEGATE SynthOverTone F** FNEGATE ELSE SynthOverTone F** THEN
		I SFLOATS InstrumentDynmem + SF!
	LOOP ;

: start-synth-note	{: vce# note# vel time# -- :}
	vel 0=
	IF	note# find-playing-note DUP -1 <> IF time# SWAP voice-off ELSE DROP THEN EXIT
	ELSE	vce# set-voice
		time# plus-voice
		note# TO MIDIPitch
		vel SetVoiceVelocity
		\ set SynthDuration factors
		VoiceDuration JACK-SAMPLE-F-RATE F* F>S TO TimeoutToRelease
		note# Note#>Frequency SF@ FDUP TO VoiceFrequency
		frequ->step TO VoicePhase-add
		0 TO VoicePhase 0 TO LesliePhase
		VoiceVolume VoiceDuration F/ JACK-SAMPLE-F-RATE F/ TO FadeoutReduction

		SynthLeslieFrequency frequ->step TO LesliePhase-add
		1 #fractbits LSHIFT S>F SynthLeslieShift F* VoiceFrequency F*  TO VoiceLeslieShift
	THEN ;
: synth-midi-receiver	{: #time #midi #note #vel -- :}
	#midi
	CASE	$80 OF	#note find-playing-note DUP -1 <>
			IF	#time SWAP voice-off
			ELSE	DROP
			THEN
		ENDOF
		$90 OF	get-free-voice DUP -1 <>
			IF	#note #vel #time start-synth-note
			ELSE	DROP
			THEN
		ENDOF
    	ENDCASE ;

FORTH DEFINITIONS
\ ==============================================================================
\ This defines an instrument structure plus handling autoremoving/allocating ...
\ It is dependent on the instrument module; don't change without exact understanding
\ Defines an instrument struct 'name'; it will simply return it's address
: MAKE-SYNTH-INSTRUMENT		\ name ( voices -- )
\ *G This defines a named synth instrument structure, you must pass the number of
\ ** voices this instrument has. The new word will return the
\ ** address of the structure. Code for allocating & deallocating dynamic memory is managed automatically.
	check-voices #synth-instrument-size MAKE-INSTRUMENT-STRUCT
	#synth-voice-size TO #voice-struct-size
	TO #instrument-voices
	0 TO #voice-dynmem-size
	#sintab SFLOATS TO #instr-dynmem-size
	0 TO voice-init-xt
	['] fill-sinus-tab TO instrument-init-xt
	1e0 TO SynthOverTone
	S" Simple Synthesizer" instrument-class PLACE
	allocate-instrument-mem
	\ The next two MUST be set after allocation
	['] synth-midi-receiver TO MIDIInterpreter
	['] calc-synth-instrument TO CalculateAction
	link-instrument ;

#15 MAKE-SYNTH-INSTRUMENT STANDARD-SYNTH

: DEFAULT-SYNTH	( midichannel -- )
\ *G This uses the existing *\forth{STANDARD-SYNTH}instrument and defines how it should sound ...
	STANDARD-SYNTH -->INSTRUMENT midi-listener
	0.95e0 TO SynthOverTone fill-sinus-tab
	8e0 TO SynthLeslieFrequency
	0.03e0 TO SynthLeslieShift
	5e0	SetVoiceDuration
	0.02e0	SetAttackTime		\ very sharp attack
	0.2e0	SetReleaseTime		\ medium release
	basic-voice-volume 4e0 F* TO VolumeCorrection ;
: SIMPLE-SYNTH	( midichannel -- )
\ *G This uses the existing *\forth{STANDARD-SYNTH}instrument and defines how it should sound ...
	STANDARD-SYNTH -->INSTRUMENT midi-listener
	0.99e0 TO SynthOverTone fill-sinus-tab
	0e0 TO SynthLeslieFrequency
	0e0 TO SynthLeslieShift
	10e4	SetVoiceDuration
	0.02e0	SetAttackTime		\ very sharp attack
	0.1e0	SetReleaseTime		\ medium release
	basic-voice-volume 2e0 F* TO VolumeCorrection ;

STANDARD-SYNTH MAKE-INSTRUMENT-LISTENER CONSTANT SYNTH-0
STANDARD-SYNTH MAKE-INSTRUMENT-LISTENER CONSTANT SYNTH-1


IS-IFORTH?	[IF]	:ABOUT ??CR ." A simple fJACK synthesizer using instruments" ; [THEN]

PREVIOUS PREVIOUS DEFINITIONS
