\ BenchMove.fth - for Vfx64

((
Copyright (c) 2021
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441


To do
=====

Change history
==============
))

code cmove1	\ src dest len --
\ load parameters
	mov	r8, rsi
	mov	r9, rdi
	mov	rsi, cell [rbp]		\ src
	mov	rdi, 0 [rbp]		\ dest
	mov	rcx, rbx		\ len
	jrcxz	l$1
	rep movsb
l$1:
	mov	rdi, r9
	mov	rsi, r8
	mov	rbx, 2 cells [rbp]
	lea	rbp, 3 cells [rbp]
	next,
end-code

code cmove1>	\ src dest len --
\ load parameters
	mov	r8, rsi
	mov	r9, rdi
	mov	rsi, cell [rbp]		\ src
	mov	rdi, 0 [rbp]		\ dest
	mov	rcx, rbx		\ len
	jrcxz	l$2
	lea	rsi, -1 [rsi] [rcx]
	lea	rdi, -1 [rdi] [rcx]
	std
	rep movsb
	cld
l$2:
	mov	rdi, r9
	mov	rsi, r8
	mov	rbx, 2 cells [rbp]
	lea	rbp, 3 cells [rbp]
	next,
end-code

code cmove2	\ src dest len --
\ load parameters
  mov    rdx, 0 [rbp]	\ edx=dest
  mov    rcx, cell [rbp]	\ ecx=src

\ optimised move - 8 bytes at a time
l$3:
  push   rbx		\ ebx=len
  shr    rbx, # 3	\ number of cells to move
  nz, if,
    begin,		\ copy cells
      mov    rax, 0 [rcx]
      lea    rcx, 8 [rcx]
      mov    0 [rdx], rax
      sub    rbx, # 1
      lea    rdx, 8 [rdx]
    z, until,
  then,
  pop    rbx
  and    rbx, # 7	\ number of bytes to move
  nz, if,
    begin,		\ copy bytes
      mov    al, 0 [rcx]
      lea    rcx, 1 [rcx]
      mov    0 [rdx], al
      sub    rbx, # 1
      lea    rdx, 1 [rdx]
    z, until,
  then,
  mov       rbx, 2 cells [rbp]
  lea       rbp, 3 cells [rbp]
  next,
end-code

code cmove2>	\ src dest len --
\ load parameters
  mov    rcx, cell [rbp]	\ ecx=src
  mov    rdx, 0 [rbp]	\ edx=dest

\ optimised move - four bytes at a time
l$5:
  push   rbx		\ ebx=len
  lea    rcx, -8 [rcx] [rbx]	\ point to last cells
  lea    rdx, -8 [rdx] [rbx]
  shr    rbx, # 3	\ number of cells to move
  nz, if,
    begin,
      mov    rax, 0 [rcx]
      sub    rbx, # 1
      lea    rcx, -8 [rcx]
      mov    0 [rdx], rax
      lea    rdx, -8 [rdx]
    z, until,				\ points to NEXT cells
  then,
  pop    rbx		\ EBX=len
  and    rbx, # 7	\ number of bytes to move
  lea    rcx, 7 [rcx]			\ adjust cell to byte addresses
  lea    rdx, 7 [rdx]
  nz, if,
    begin,
      mov    al, 0 [rcx]
      sub    rbx, # 1
      lea    rcx, -1 [rcx]
      mov    0 [rdx], al
      lea    rdx, -1 [rdx]
    z, until,
  then,
  mov       rbx, 2 cells [rbp]
  lea       rbp, 3 cells [rbp]
  next,
end-code

code cmove3	\ src dest len --
\ load parameters
	mov	r8, rsi
	mov	r9, rdi

	mov	rsi, cell [rbp]		\ src
	mov	rdi, 0 [rbp]		\ dest
	mov	rcx, rbx		\ len
	and     rcx, # -8		\ clear bottom bits
	jrcxz	l$1
	rep movsq
l$1:
	mov	rcx, rbx		\ len
	and	rcx, # 7
	jrcxz	l$2
	rep movsb
l$2:

	mov	rdi, r9
	mov	rsi, r8

	mov	rbx, 2 cells [rbp]
	lea	rbp, 3 cells [rbp]
	next,
end-code

code cmove3>	\ src dest len --
\ load parameters
	mov	r8, rsi
	mov	r9, rdi

	mov	rsi, cell [rbp]		\ src
	mov	rdi, 0 [rbp]		\ dest
	mov	rcx, rbx		\ len

	std
	lea	rsi, -8 [rsi] [rcx]
	lea	rdi, -8 [rdi] [rcx]

	and	rcx, # -8		\ QWORDs only
	jrcxz	l$1
	rep movsq
l$1:
	mov	rcx, rbx
	and	rcx, # 7 		\ bytes
	jrcxz   l$2
	lea	rsi, 7 [rsi]
	lea	rdi, 7 [rdi]
	rep movsb
l$2:			\
	cld

	mov	rdi, r9
	mov	rsi, r8

	mov	rbx, 2 cells [rbp]
	lea	rbp, 3 cells [rbp]
	next,
end-code

: MOVE1          \ src dest len --                              6.1.1900
\ *G An intelligent memory move which avoids memory overlap problems.
\ ** Note that as VFX Forth characters are 8 bit units, there is
\ ** an implicit connection between a byte and a character.
  2 pick over +			\ -- src dest len src+len
  2 pick u< if			\ src+len < dest
    cmove
  else
    2dup +  3 pick u< if	\ dest+len < src
      cmove
    else
      cmove>
    then
  then
;

1024 value usize
16 value ulength

: testitO	{: xt -- :}	\ xt of CMOVE or CMOVE>
\ This the test for mostly unaligned moves.
  cr  xt >name .name 16 >pos
  usize ulength + 4096 + 2* allocate throw	\ -- addr
  ulength 0 ?do 'a' i + over i + c! loop
  ticks >r
  10000000 0 ?do			\ -- addr ; R: -- ms
    dup dup ulength + usize  xt execute
  loop
  ticks r> - abs .
  free throw
;

: benchO	\ --
  1024 to usize  0 to ulength
  16 0 do
    key? ?leave  i to ulength
    ['] cmove1 testitO  i .
  loop
  16 0 do
    key? ?leave  i to ulength
    ['] cmove1> testitO  i .
  loop
  16 0 do
    key? ?leave  i to ulength
    ['] cmove2 testitO  i .
  loop
  16 0 do
    key? ?leave  i to ulength
    ['] cmove2> testitO  i .
  loop
  16 0 do
    key? ?leave  i to ulength
    ['] cmove3 testitO  i .
  loop
  16 0 do
    key? ?leave  i to ulength
    ['] cmove3> testitO  i .
  loop
;

: testitA	{: xt -- :}	\ xt of CMOVE or CMOVE>
\ This the test for mostly aligned moves.
  cr  xt >name .name 16 >pos
  usize ulength + 4096 + 2* allocate throw	\ -- addr
  ulength 0 ?do 'a' i + over i + c! loop
  ticks >r
  10000000 0 ?do			\ -- addr ; R: -- ms
    dup dup usize 2* + usize  xt execute
  loop
  ticks r> - abs .
  free throw
;

: benchA		\ --
  1024 to usize  0 to ulength
  ['] cmove1 testitA
  ['] cmove1> testitA
  ['] cmove2 testitA
  ['] cmove2> testitA
  ['] cmove3 testitA
  ['] cmove3> testitA
;

0 value osrc
0 value odest
0 value xt

: passUA	\ addr -- addr
  ticks >r
  1000000 0 ?do			\ -- addr ; R: -- ms
    dup osrc +				\ source
    over 4096 + odest +			\ dest
    usize  xt execute
  loop
  ticks r> - abs .
;

: u>hd		\ u --
  base @ >r hex 0 .r r> base !
;

: testitUA	\ xt --	; xt of CMOVE or CMOVE>
\ This the test for mostly unaligned moves.
  to xt
  cr  xt >name .name 16 >pos
  usize 4096 + 2* allocate throw	\ -- addr
  ulength 0 ?do 'a' i + over i + c! loop
  16 0 do
    cr  16 0 do
      i 8 = if space then
      i to osrc  j to odest
      passUA  i u>hd  j u>hd space
    loop
  loop
  free throw
;

: benchUA		\ --
  1024 to usize  0 to ulength
  ['] cmove1 testitUA
  ['] cmove1> testitUA
  ['] cmove2 testitUA
  ['] cmove2> testitUA
  ['] cmove3 testitUA
  ['] cmove3> testitUA
;

