\ PostAsm.fth - Forth cross assembler epilogue

((
Copyright (c) 2002, 2006, 2020
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441


To do
=====

Change history
==============
20200325 SFP002 Updated for x64
20060220 SFP001 Added ENDPROC to match some target assemblers.
))


only forth  also asm-core  also asm-access  also forth definitions

decimal

target-only
: Proc		\ "name" --
  ?exec !csp  create
    initfwd  asm_start asm-on
;  immediate

: lbl:		\ "name" --
  ?exec !csp  create
    asm_start  asm-on
;  immediate

: code		\ "name" --
  ?exec !csp
\  create  0 >body negate allot
  bl word count header code-align	\ sort of CREATE ; SFP012
    initfwd  asm_start asm-on
    here last-colon-xt !
;  immediate

: asmcode		\ "name" --
  ?exec !csp
  initfwd  asm_start asm-on
;  immediate
host&target

: ;code		\ --
  ?comp ?csp
  discard-sinline			\ SFP004
  drop-locals                           \ compiles in stack cleanup ; SFP009
  cleanup-locals                        \ cleans up local dictionary etc. ; SFP009
  compile (;code)
  state off
  !csp
  initfwd  asm_start asm-on
  r> drop				\ exit caller
;  immediate

: end-code	\ --
  asm_end  asm-off
  unres-refs? #-257 ?throw
  ?exec ?csp  reveal
;  immediate


\ ******************************
\ *S Macros and Assembler access
\ ******************************

( *P Because of the performance of the VFX optimiser, use of assembler )
( ** is only necessary when defining new compilation structures. Otherwise )
( ** the use of assembler code should be avoided. )

( *P Assembler macros are defined as follows: )
( *E MASM: <name>
( **   <assembler code goes here>
( ** ;MASM )
( ** e.g. the following macro pops the top of the NDP stack to the external )
( ** floating point stack. )
( ** MASM: popFPU	\ -- ; pops FTOS to float stack              )
( **   mov   eax, FSP-OFFSET [esi]   \ get FP stack pointer          )
( **   lea   eax, -FPCELL [eax]      \ update stack pointer          )
( **   fstp  fword 0 [eax]           \ store and pop                 )
( **   mov   FSP-OFFSET [esi], eax   \ restore FP stack pointer      )
( ** ;MASM )

( *P In line assembler code may be compiled into the middle of a colon )
( ** definition by using the phrase: )
( *E : <name>      \ just another Forth word )
( **   ... )
( **   [ASM  <insert assembler here>  ASM] )
( **   ... )
( ** ; )

( *P A fragment of assembler for compilation when the containing )
( ** word is executed can be defined by using the following: )
( *E : a-compiler  \ will compile some assembler )
( **   ... )
( **   a[ <fragment to be compiled> ]a
( **   ... )
( ** ; )
( *P The following example compiles an in-line floating point literal. )
( *E : o_flit,	\ F: f -- ; F: -- f ; compile floating point literal   )
( **   a[ popFPU                        \ references a previous macro  )
( **      jmp   here 2+  FPCELL +       \ skip inline literal          )
( **   ]a                                                              )
( **   f,                                                              )
( **   a[ fld   fword ptr here FPCELL - ]a                             )
( ** ;                                                                 )

\ *P When using in-line code generators such as *\fo{[ASM ...ASM]}
\ ** you should flush the code generator contents with *\fo{[O/F]}.
( *E [O/F] [ASM ... ASM]	)
\ *P After *\fo{[ASM} the top of the data stack will be in EBX
\ ** with all other stack items pointed to by EBP. The code
\ ** generator expects this same state to exist after *\fo{ASM]}.

also asm-core
: MASM:		\ --
  ?exec
  also asm-access definitions
  :
  postpone asm_end  postpone asm_start
; immediate

: ;MASM		\ --
  ?comp
  previous definitions
  postpone asm_end  postpone ;
; immediate

: [ASM		\ -- ; start assembler sequence
  ?comp  asm_end asm_start
  also asm-access  state off
; immediate

: ASM]		\ -- ; end assembler sequence
  asm_end  ]  previous			\ SFP010
;

: a[            \ -- ; start assembler sequence to be compiled later, A[ ... ]A
  also asm-access
  postpone asm-init
;  immediate

: ]a            \ -- ; end of assembler sequence
  postpone asm_end
  previous  ;  immediate
previous

