\ PreAsm.fth - Forth assembler preamble

((
Copyright (c) 2002, 2010, 2017, 2020
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441


To do
=====

Change history
==============
20200324 SFP003 x64 host update
20170704 SFP002 64 bit update
20100212 SFP001 Corrected ?UNCONS.
))


only forth definitions  decimal

vocabulary asm-core	\ -- ; holds guts of aaembler
vocabulary asm-access	\ -- ; holds

also asm-core definitions

compiler
: cell(t)	cell  ;

: c@c(t)	c@  ;
: w@c(t)        w@  ;
: l@c(t)        l@  ;
: x@c(t)	@  ;
: @c(t)		@  ;

: c!c(t)        c!  ;
: w!c(t)        w!  ;
: l!c(t)	l!  ;
: x!c(t)         !  ;
: !c(t)         !  ;

: chere(t)	here  ;
: callot(t)     allot  ;

: c,c(t)	c,  ;
: w,c(t)	w,  ;
: l,c(t)	l,  ;
: x,c(t)	,  ;
: ,(t)		,  ;
target

((
: .uncons	\ symbol -- ; SFP001
  cr ." At target address " cell+ @(s) .dword
  err_uncons throw
;

: ?UNCONS
  REF1 @(m) ?dup
  IF  .uncons  REF1 off(m)  ENDIF	\ SFP001
  REF2 @(m) ?dup
  IF  .uncons  REF2 off(m)  ENDIF	\ SFP001
;
))


\ ***********************************************************************
\ load these to redefine C,(T) W,(T) and L,(T) to display compiled values
\ at assembly time for debugging
\ ***********************************************************************

variable showing	\ -- addr ; true if displaying code
  showing off

also forth definitions
: +show-code    \ -- ; code will be displayed
  showing on  ;

: -show-code    \ -- ; code will not be displayed
  showing off  ;
previous definitions

: pcr	cr  ;

: ?cr           \ --
  out @ #72 u>
  if  pcr  endif  ;

: c,c(T)         \ b --
  showing @
  if  ?cr  DUP .BYTE  endif
  c,c(T)  ;

: w,c(T)         \ u --
  showing @
  if  ?cr  DUP .WORD  endif
  w,c(T)  ;

: l,c(T)         \ u --
  showing @
  if  ?cr  DUP .LWORD  endif
  l,c(T)  ;

: x,c(T)         \ u --
  showing @
  if  ?cr  DUP .XWORD  endif
  x,c(T)  ;


\ ***************************
\ general assembler variables
\ ***************************

variable <prefix>               \ prefix/postfix mode
  <prefix> on                   \ default to prefix
create aprior  0 , 0 ,          \ see ?prefix
create last-pos 0 , 0 ,         \ holds BLK & >IN
variable pos-#tib               \ -- addr ; holds previous #TIB
$0100 equ /pos-buff	\ -- len size of buffer
/pos-buff buffer: pos-buff	\ -- addr ; holds previous TIB


\ *************************************
\ Cross compiler extensions for ?PREFIX
\ *************************************

host previous				\ stop cross-compiling

also compiler definitions c-c		\ new compiler words

: ?prefix	\ --
  target-sym (?PREFIX) compile-ref
  chere(t)  0 ,c(t)
\ 386/x64 needs no entry code for :NONAME
  chere(t) swap !c(t) ?csp
;

previous definitions

target

only forth also asm-core definitions


\ ************************************
\ PREFIX and POSTFIX notation handling
\ ************************************

\ This function is used to allow opcodes to be before (conventional)
\ or after (old Forth style) the operands. If the <PREFIX> flag is set
\ a previously stored routine is executed, thus effectively putting off
\ the execution of an opcode's function until the next opcode is encountered.
\
\ The basis of this code was presented by Bob Smith at the
\ November 1987 FORML Conference - thanks.
\ Rewritten to avoid return stack issues Feb 2002

: prefix?       \ -- t/f ; true if in prefix mode
  <prefix> @  ;

(( Original Bob Smith version
: ?prefix       \ struct -- struct' ; executes previous opcode
  prefix? if
    r>                                  \ struct ret-addr --
    aprior 2@  2swap  aprior 2!         \ exchange with contents of APRIOR
    >r                                  \ will use previous return address
  endif
;
))

also forth definitions	\ for required:
: (?prefix)	\ struct -- struct' ; executes previous opcode
\ This word assumes that the return address is on the top of the
\ return stack and that an xt is inline. It exits the caller.
  r> @					\ -- struct xt
  prefix? if
    aprior 2@  2swap  aprior 2!         \ exchange with contents of APRIOR
  endif
  execute				\ use previous xt
;
previous definitions

: ?prefix	\ -- ; finish and start nameless word
\ This word assumes that the return address is on the top of the
\ return stack and that it can compile an xt inline.
  postpone (?prefix)  here 0 ,		\ (?PREFIX) gets xt inline
  state off  smudge
  :noname swap !  !csp
; immediate

[defined] disassembler [if]
also disassembler
[+switch dasm-switch
  ' (?prefix) run:
    disinit @xlong disclose
    s" ?PREFIX" >disLine
    set-Disaddr  ;
switch]
previous
[then]

also forth definitions
: prefix	\ -- ; select prefix notation
  <prefix> on  ;
: postfix	\ -- ; select postfix notation
  <prefix> off  ;
: prefix?	\ -- t/f ; true if in prefix mode
  <prefix> @  ;
defer a-reset	\ -- ; reset assembler
previous definitions

: null-op       \ -- ; force execution of previous opcode
  0 ?prefix drop  a-reset  ;

: kickoff       \ n -- ; dummy operator
  drop  a-reset  ;

: set-null      \ -- ; initialise APRIOR
  0  ['] kickoff aprior 2!  ;


\ **************
\ Error handling
\ **************

: save-pos      \ -- ; preserves opcode position in text
  blk @  >in @  last-pos 2!
  blk @ 0= if				\ if not in screen file
    tib pos-buff #tib @ /pos-buff min cmove	\ save TIB
    #tib @ /pos-buff min pos-#tib !		\ save #TIB
  endif  ;

: restore-pos   \ -- ; restore text position
  last-pos 2@  >in !  dup blk !  0=     \ if in text file
  if  pos-buff tib pos-#tib @ /pos-buff min cmove \ restore TIB
      pos-#tib @ #tib !
  endif  ;

: a-error       \ n -- ; error exit
  prefix?
  if  restore-pos  endif
  throw  ;

: ?a-error      \ t/f n -- ; error n if t/f true
  swap
  if  a-error  else  drop  endif  ;


\ ******************************
\ Assembler startup and shutdown
\ ******************************

: ASM_START     \ -- ; initialise assembler
  set-null  save-pos  a-reset  ;

: ASM_END       \ -- ; reset assembler
  null-op  ;


\ ***********
\ File output
\ ***********

#32 constant text-start         \ -- n ; start position of text in listing

[-short-branches
: .show-code    \ -- ; adds text to code laid by instruction
  showing @                             \ debugging info needed?
  if  text-start out @ - 0 max spaces   \ tab to start of text display
      prefix?                           \ source of text is different
      if  pos-buff pos-#tib @ type      \  if in prefix
      else  tib #tib @ type             \  or in postfix
      endif
      pcr chere(t) .XWORD space		\ show program counter location
  endif
;
short-branches]


\ *******************************************************
\ Macros - because of the prefix notation, some assembler
\ state needs to be preserved across macros, especially
\ what is to be done when the macro finishes. See the
\ earlier section on PREFIX notation
\ *******************************************************

decimal

32 constant #mb-entries                 \ macro nesting permitted
3 cells constant /mb-entry              \ 3 cells per entry
#mb-entries /mb-entry *                 \ size of macro buffer
  constant mb-size

mb-size buffer: mb	\ -- addr ; macro buffer

: push-macro    \ -- ; save APRIOR and STATE(T), put null into APRIOR
  mb mb mb-size /mb-entry /string cmove>  \ copy buffer
  aprior 2@ mb 2!                       \ save APRIOR
  state @ mb 2 cells+ !			\ save STATE
  set-null  ;                           \ insert null into assembler

: pop-macro     \ -- ; restore APRIOR and STATE(T)
  mb 2 cells+ @ state !			\ restore STATE
  mb 2@ aprior 2!                       \ restore APRIOR
  mb /mb-entry +  mb                    \ source and dest
  mb-size /mb-entry -  cmove            \ slide buffer back
;

: (start-macro) \ -- ; compiles prologue code
  postpone 0  postpone ?prefix  postpone drop      \ 0 ?prefix drop
  postpone push-macro
;

: (end-macro)   \ -- ; compiles epilogue code
  postpone null-op  postpone pop-macro	\ end of macro
;

: (macro:)      \ -- sys ; builds a macro header
  : (start-macro)			\ : is not IMMEDIATE in VFX
;

: (;macro)      \ sys --
  postpone null-op  postpone pop-macro    \ end of macro
  postpone ;
;

also forth definitions

: macro:        \ -- sys
  also asm-access definitions		\ word is in assembler
  (macro:)
;  immediate

((
: mc_gen:       \ -- sys ; builds micro code generator in C-C
  also c-c definitions
  (macro:)
;  immediate
))

: start-macro   \ -- ; used by code generators
  postpone null-op  postpone push-macro
  also asm-access
;  immediate

: end-macro     \ -- ; used by code generators
  (end-macro) previous  ;  immediate

also asm-access definitions
: ;m		\ sys --
  (;macro)				\ end of macro
  previous definitions			\ restore search order
;  immediate

: $		\ -- addr
  chere(t)
;
previous definitions


\ ********
\ all done
\ ********

only forth definitions
decimal

