\ hdasmSSE - disassembler for: prefix [REX] 0F instructions

((
Copyright (c) 2020, 2022
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone numbers are:
       011 44 23 8063 1441
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20221025 SFP001 Added ENDBR64 and ENDBR32
))

only forth  also disassembler definitions
decimal

create regsXMM	\ -- addr ; table of XMM register names
  ", XMM0"  ", XMM1"  ", XMM2"  ", XMM3"  ", XMM4"  ", XMM5"  ", XMM6"  ", XMM7"
  ", XMM8"  ", XMM9"  ", XMM10" ", XMM11" ", XMM12" ", XMM13" ", XMM14" ", XMM15"

: xreg>dis	\ reg# -- ; output XMMn
  regsXMM table$ >disline
;

[-short-branches
: XR/M+DISP>DIS	\ -- ; process XR/M, maybe a sib, maybe a displacement, or xreg
  \ 32bit?= -1  32bit data
  \ mod= 0, 1 or 2 and r/m= 4 then sib byte present
  \ modr/m = 00 reg# 100
  r/m 4 =  modifier 3 < AND
  IF  MODRM32-2BYTE  EXIT  THEN		\ mod r/m + sib
  \ 00 101 mod= 0 & r/m= 5 data 32
  modifier 0= r/m 5 = NOT AND
  IF  MEM(32bits)  EXIT  THEN
  modifier 3 < IF
    modifier 0= r/m 5 = AND IF
      disp+rip>dis
    ELSE
      [>DIS
      r/m +rex>base areg>dis
      modifier 1 = IF			\ byte offset
        @BYTE ?DUP
        IF  +>DIS SEXT>DIS  THEN
      THEN
      modifier 2 = IF			\ 32 bit offset
        @LONG ?DUP
        IF  +>DIS LONG>DIS  THEN
      THEN
      ]>DIS
    THEN
  ELSE
    r/m +rex>base xreg>dis
  THEN
;
short-branches]
((
: MODRM32  ( -- )
  \ 32bit?= -1  32bit data
  \ mod= 0, 1 or 2 and r/m= 4 then sib byte present
  \ modr/m = 00 reg# 100
  r/m 4 =  modifier 3 < AND
  IF  MODRM32-2BYTE  EXIT  THEN		\ mod r/m + sib
  \ 00 101 mod= 0 & r/m= 5 data 32
  modifier 0= r/m 5 <> AND
  IF  MEM(32bits)  EXIT  THEN
  modifier 3 < IF
    modifier 0= r/m 5 = AND IF
      disp+rip>dis
    ELSE
      [>DIS
      r/m +rex>base areg>dis
      modifier 1 = IF			\ byte offset
        @BYTE ?DUP
        IF  +>DIS SEXT>DIS  THEN
      THEN
      modifier 2 = IF			\ 32 bit offset
        @LONG ?DUP
        IF  +>DIS LONG>DIS  THEN
      THEN
      ]>DIS
    THEN
  ELSE
    r/m +rex>base dreg>dis
  THEN
;
))
: dis-Imm8	\ --
  s" , # " >disline  @byte sext>dis
;

: dis-dreg	\ --
  reg# +rex>reg dreg>dis
;

: dis-xreg	\ --
  reg# +rex>reg xreg>dis
;

: dis-xreg,xmem	\ --
\ disassemble xreg, XR/M, maybe a sib, maybe a displacement, or xreg
  @MODRM  dis-xreg  ,>DIS  XR/M+DISP>DIS
;

: op$-xreg,xmem	\ caddr len --
  op$>dis    @MODRM  dis-xreg  ,>DIS  XR/M+DISP>DIS
;

: op$-xmem,xreg	\ caddr len --
  op$>dis  @modrm  XR/M+DISP>DIS  ,>dis  dis-xreg
;

: op$-xreg,xmem,imm8	\ caddr len --
  op$>dis  dis-xreg,xmem  dis-Imm8
;

: op$-memsz	\ caddr len --
  op$>dis  @modrm  XR/M+DISP>DIS
;

: op$-xreg,imm8	\ caddr len --
  op$>dis  @modrm  XR/M+DISP>DIS  dis-Imm8
;

: op$-xreg,r/m	\ caddr len --
  op$>dis  @modrm  dis-xreg  ,>dis  R/M+DISP>DIS
;

: op$-r/m,xreg	\ caddr len --
  op$>dis  @modrm  R/M+DISP>DIS  ,>dis  dis-xreg
;

: op$-xreg,r32/m16,imm8	\ caddr len
  op$>dis  @modrm  dis-xreg  ,>dis  R/M+DISP>DIS  dis-Imm8
;

: op$-reg,xreg	\ caddr len --
  op$>dis  @modrm  dis-dreg  ,>dis  XR/M+DISP>DIS
;

: op$-reg,xmem	\ caddr len --
  op$>dis  @modrm  dis-dreg  ,>dis  XR/M+DISP>DIS
;

create xconds	\ --
  ", EQ"	", LT"		", LE"		", UNORD"
  ", NEQ"	", NLT"		", NLE"		", ORD"
\  --
  ", EQ_UQ"	", NGE"		", NGT"		", FALSE"
  ", NEQ_OQ"	", GE"		", GT"		", TRUE"
\  --
  ", EQ_OS"	", LT_OQ"	", LE_OQ"	", UNORD_S"
  ", NEQ_OS"	", NLT_UQ"	", NLE_UQ"	", ORD_S"
\  --
  ", EQ_US"	", NGE_UQ"	", NGT_UQ"	", FALSE_OS"
  ", NEQ_OS"	", GE_OQ"	", GT_OQ"	", TRUE_US"

: comp-SSE	\ --
  @byte $01F and xconds table$ >disline
;

: op$-xreg,xmem,comp8	\ caddr len --
  op$>dis  @modrm  dis-xreg  ,>dis  XR/M+DISP>DIS  ,>dis  comp-SSE
;


: op-0F12	\ --
  @modrm  modifier 3 = if
    s" MOVHLPS" op$>dis  dis-xreg  ,>dis  XR/M+DISP>DIS
  else
    s" MOVHPS" op$>dis  dis-xreg  ,>dis  XR/M+DISP>DIS
  then
;

: op-0F16	\ --
  @modrm  modifier 3 = if
    s" MOVLHPS" op$>dis  dis-xreg  ,>dis  XR/M+DISP>DIS
  else
    s" MOVLPS" op$>dis  dis-xreg  ,>dis  XR/M+DISP>DIS
  then
;

((
alt (for modern CPUs, same for Intel & AMD):

 1 90
 2 6690
 3 0f1f00                          nopl (%[re]ax)
 4 0f1f4000                        nopl 0(%[re]ax)
 5 0f1f440000                      nopl 0(%[re]ax,%[re]ax,1)
 6 660f1f440000                    nopw 0(%[re]ax,%[re]ax,1)
 7 0f1f8000000000                  nopl 0L(%[re]ax)
 8 0f1f840000000000                nopl 0L(%[re]ax,%[re]ax,1)
 9 660f1f840000000000              nopw 0L(%[re]ax,%[re]ax,1)
10 662e0f1f840000000000            nopw %cs:0L(%[re]ax,%[re]ax,1)
))
: op-660F1Fxx	\ --
  case @byte
    $44 of  @word $0000 =
            if  s" NOP6"  else  s" ???"  then
            op$>dis
        endof
    $84 of  @byte 0=  @dword 0= and
            if  s" NOP9"  else  s" ???"  then
            op$>dis
        endof
        s" ???" op$>dis
  endcase
;

: op-0F1Fxx	\ -- ; multibyte NOPs
  @byte dup $00 =
  if  drop s" NOP3" op$>dis  exit  then
  dup $40 = if
    drop  @byte $00 =
    if  s" NOP4" op$>dis
    else  s" ???" op$>dis
    then  exit
  then
  dup $44 = if
    drop  @word $0000 =
    if  s" NOP5" op$>dis
    else  s" ???" op$>dis
    then  exit
  then
  dup $80 = if
    drop  @dword $0000:0000 =
    if  s" NOP7" op$>dis
    else  s" ???" op$>dis
    then  exit
  then
  dup $84 = if
    drop  @byte 0=  @dword $0000:0000 = and
    if  s" NOP8" op$>dis
    else  s" ???" op$>dis
    then  exit
  then
  drop  s" ???" op$>dis
;

: op-0FA0	\ --
  opcDsize @ 64 =
  if  s" FXSAVE64"  else  s" FXSAVE"  then
  op$-memsz
;

: op-0FAE	\ --
  @modrm
  case  modrm-byte @
    $E8 of  s" LFENCE" op$>dis  exit  endof
    $F0 of  s" MFENCE" op$>dis  exit  endof
  endcase
  case  reg#
    2 of  s" LDMXCSR" op$>dis  32 opcDsize !  R/M+DISP>DIS  endof
    3 of  s" STMXCSR" op$>dis  32 opcDsize !  R/M+DISP>DIS  endof
    7 of  s" CLFLUSH" op$>dis  8 opcDsize !  R/M+DISP>DIS  endof
      illegal
  endcase
;


\  -1 $0F $54 xreg,xmem ANDPS
\  -1 $0F $55 xreg,xmem ANDNPS
\  -1 $0F $C2 0 cc-xreg,xmem CMPxxxPS
\  -1 $0F $2F xreg,xmem COMISS
\  -1 $0F $5B xreg,xmem CVTDQ2PS
\  -1 $0F $5A xreg,xmem CVTPS2PD
\  -1 $0F $5E xreg,xmem DIVPS
\  -1 $0F $A0 0 #32 op3-memsz FXSAVE
\  -1 $0F $A0 0 #64 op3-memsz FXSAVE64
\  -1 $0F $5F xreg,xmem MAXPS
\  -1 $0F $5D xreg,xmem MINPS
\  -1 $0F $58 xreg,xmem ADDPS
\  -1 $0F $28 $29 xmem,xmem MOVAPS
\  -1 $0F $16 $17 xr/r/m,xr/r/m MOVHPS  $16 op2 <> reg, $17 op1 <> reg
\  -1 $0F $16 xreg,xreg MOVLHPS		op2 = reg only
\  -1 $0F $12 $13 xr/r/m,xr/r/m MOVLPS
\  -1 $0F $12 xreg,xreg MOVHLPS
\  -1 $0F $50 reg,xreg MOVMSKPS
\  -1 $0F $E7 mem,xreg MOVNTPS
\  -1 $0F $7E $D6 xmem,xmem MOVQ-XM
\  -1 $0F $10 $11 xmem,xmem MOVUPS
\  -1 $0F $59 xreg,xmem MULPS
\  -1 $0F $f6 xreg,xmem ORPS
\  -1 $0F $53 xreg,xmem RCPPS
\  -1 $0F $52 xreg,xmem RSQRTPS
\  -1 $0F $C6 xreg,xmem,imm8 SHUFPS
\  -1 $0F $51 xreg,xmem SQRTPS
\  -1 $0F $AE 2 #32 op3-memsz LDMXCSR
\  -1 $0F $AE 3 #32 op3-memsz STMXCSR
\  -1 $0F $5C xreg,xmem SUBPS
\  -1 $0F $57 xreg,xmem XORPS
\  -1 $0F $2E xreg,xmem UCOMISS
\  -1 $0F $14 xreg,xmem UNPCKLPS
\  -1 $0F $15 xreg,xmem UNPCKHPS
[+switch switch0Fxx	\ byte --
\ DisAddr points past xx byte, REX has been processed
  $54 run:  s" ANDPS"    op$-xreg,xmem  ;
  $55 run:  s" ANDNPS"   op$-xreg,xmem  ;
  $C2 run:  s" CMPPS"    op$-xreg,xmem,comp8  ;
  $58 run:  s" ADDPS"    op$-xreg,xmem  ;
  $2F run:  s" COMISS"   op$-xreg,xmem  ;
  $5B run:  s" CVTDQ2PS" op$-xreg,xmem  ;
  $5A run:  s" CVTPS2PD" op$-xreg,xmem  ;
  $5E run:  s" DIVPS"    op$-xreg,xmem  ;
  $A0 run:  op-0FA0  ;
  $5F run:  s" MAXPS"    op$-xreg,xmem  ;
  $5D run:  s" MINPS"    op$-xreg,xmem  ;
  $28 run:  s" MOVAPS"   op$-xreg,xmem  ;
  $29 run:  s" MOVAPS"   op$-xmem,xreg  ;
  $16 run:  op-0F16  ;
  $17 run:  s" MOVHPS"   op$-xmem,xreg  ;
  $1F run:  op-0F1Fxx  ;
  $12 run:  op-0F12  ;
  $13 run:  s" MOVLPS"   op$-xmem,xreg  ;
  $50 run:  s" MOVMSKPS" op$-reg,xreg   ;
  $E7 run:  s" MOVNTPS"  op$-xmem,xreg  ;
  $7E run:  s" MOVQ"     op$-xreg,xmem  ;
  $D6 run:  s" MOVQ"     op$-xmem,xreg  ;
  $10 run:  s" MOVUPS"   op$-xreg,xmem  ;
  $11 run:  s" MOVUPS"   op$-xmem,xreg  ;
  $59 run:  s" MULPS"    op$-xreg,xmem  ;
  $F6 run:  s" ORPS"     op$-xreg,xmem  ;
  $53 run:  s" RCPPS"    op$-xreg,xmem  ;
  $52 run:  s" RSQRTPS"  op$-xreg,xmem  ;
  $C6 run:  s" SHUFPS"   op$-xreg,xmem,imm8  ;
  $51 run:  s" SQRTPS"   op$-xreg,xmem  ;
  $AE run:  op-0FAE  ;
  $5C run:  s" SUBPS"    op$-xreg,xmem  ;
  $05 run:  s" SYSCALL"  op$>dis  ;
  $34 run:  s" SYSENTER" op$>dis  ;
  $2E run:  s" UCOMISS"  op$-xreg,xmem  ;
  $14 run:  s" UNPCKLPS" op$-xreg,xmem  ;
  $15 run:  s" UNPCKHPS" op$-xreg,xmem  ;
  $57 run:  s" XORPS"    op$-xreg,xmem  ;
switch]


\ $66 $0F $F1 $71 6 xreg,xmem/imm8 PSLLW
\ $66 $0F $E1 $71 4 xreg,xmem/imm8 PSRAW
: op-660F71
  case DisAddr c@c(t) 3 rshift $07 and	\ inspect reg field of mod/reg/mem
    4 of  s" PSRAW"      op$-xreg,imm8  endof
    6 of  s" PSLLW"      op$-xreg,imm8  endof
      illegal
  endcase
;

\ $66 $0F $F2 $72 6 xreg,xmem/imm8 PSLLD
\ $66 $0F $E2 $72 4 xreg,xmem/imm8 PSRAD
: op-660F72
  case DisAddr c@c(t) 3 rshift $07 and	\ inspect reg field mod/reg/mem
    4 of  s" PSRAD"      op$-xreg,imm8  endof
    6 of  s" PSLLD"      op$-xreg,imm8  endof
      illegal
  endcase
;

\ $66 $0F $73 7 xreg,imm8 PSLLDQ
\ $66 $0F $F3 $73 6 xreg,xmem/imm8 PSLLQ
\ $66 $0F $73 3 xreg,imm8 PSRLDQ
: op-660F73
  case DisAddr c@c(t) 3 rshift $07 and	\ inspect reg field mod/reg/mem
    2 of  s" PSRLQ"      op$-xreg,imm8  endof
    3 of  s" PSRLDQ"     op$-xreg,imm8  endof
    6 of  s" PSLLQ"      op$-xreg,imm8  endof
    7 of  s" PSLLDQ"     op$-xreg,imm8  endof
      illegal
  endcase
;

: ?MOVD/Q	\ -- caddr len
  D64bit? if  s" MOVQ"  else  s" MOVD"  then
;

: op-660F6E	\ --
  ?MOVD/Q  op$-xreg,r/m
;

: op-660F7E	\ --
  ?MOVD/Q op$-r/m,xreg
;

\ $66 $0F $54 xreg,xmem ANDPD
\ $66 $0F $55 xreg,xmem ANDNPD
\ $66 $0F $C2 0 cc-xreg,xmem CMPxxxPD
\ $66 $0F $2F xreg,xmem COMISD
\ $66 $0F $5A xreg,xmem CVTPD2PS
\ $66 $0F $5B xreg,xmem CVTPS2DQ
\ $66 $0F $E6 xreg,xmem CVTTPD2PQ
\ $66 $0F $5E xreg,xmem DIVPD
\ $66 $0F $58 xreg,xmem ADDPD
\ $66 $0F $F7 xreg,xreg MASKMOVDQU
\ $66 $0F $5F xreg,xmem MAXPD
\ $66 $0F $5D xreg,xmem MINPD
\ $66 $0F $28 $29 xmem,xmem MOVAPD
\ $66 $0F $6E $7E #32 xr/r/m,xr/r/m MOVD-X
\ $66 $0F $6E $7E #64 xr/r/m,xr/r/m MOVQ-X
\ $66 $0F $6F $7F xmem,xmem MOVDQA
\ $66 $0F $16 $17 xr/r/m,xr/r/m MOVHPD
\ $66 $0F $12 $13 xr/r/m,xr/r/m MOVLPD
\ $66 $0F $50 reg,xreg MOVMSKPD
\ $66 $0F $e7 mem,xreg MOVNTDQ
\ $66 $0F $2B mem,xreg MOVNTPD
\ $66 $0F $10 $11 xmem,xmem MOVUPD
\ $66 $0F $59 xreg,xmem MULPD
\ $66 $0F $56 xreg,xmem ORPD
\ $66 $0F $63 xreg,xmem PACKSSWB
\ $66 $0F $6B xreg,xmem PACKSSDW
\ $66 $0F $67 xreg,xmem PACKUSWB
\ $66 $0F $FC xreg,xmem PADDB
\ $66 $0F $FD xreg,xmem PADDW
\ $66 $0F $FE xreg,xmem PADDD
\ $66 $0F $D4 xreg,xmem PADDQ
\ $66 $0F $DC xreg,xmem PADDUSB
\ $66 $0F $DD xreg,xmem PADDUSW
\ $66 $0F $DB xreg,xmem PAND
\ $66 $0F $DF xreg,xmem PANDN
\ $66 $0F $E0 xreg,xmem PAVGB
\ $66 $0F $E3 xreg,xmem PAVGW
\ $66 $0F $74 xreg,xmem PCMPEQB
\ $66 $0F $75 xreg,xmem PCMPEQW
\ $66 $0F $76 xreg,xmem PCMPEQD
\ $66 $0F $64 xreg,xmem PCMPGTB
\ $66 $0F $65 xreg,xmem PCMPGTW
\ $66 $0F $66 xreg,xmem PCMPGTD
\ $66 $0F $C4 xreg,r32/m16,imm8 PINSRW
\ $66 $0F $F5 xreg,xmem PMADDWD
\ $66 $0F $EE xreg,xmem PMAXSW
\ $66 $0F $DE xreg,xmem PMAXUB
\ $66 $0F $EA xreg,xmem PMINSW
\ $66 $0F $DA xreg,xmem PMINUB
\ $66 $0F $E4 xreg,xmem PMULHUW
\ $66 $0F $E5 xreg,xmem PMULHW
\ $66 $0F $D5 xreg,xmem PMULLW
\ $66 $0F $F4 xreg,xmem PMULUDQ
\ $66 $0F $EB xreg,xmem POR
\ $66 $0F $F6 xreg,xmem PSADBWW
\ $66 $0F $70 xreg,xmem,imm8 PSHUFD
\ $66 $0F $F1 $71 6 xreg,xmem/imm8 PSLLW
\ $66 $0F $F3 $73 6 xreg,xmem/imm8 PSLLQ
\ $66 $0F $E1 $71 4 xreg,xmem/imm8 PSRAW
\ $66 $0F $E2 $72 4 xreg,xmem/imm8 PSRAD
\ $66 $0F $73 3 xreg,imm8 PSRLDQ
\ $66 $0F $D1 $71 2 xreg,xmem/imm8 PSRLW
\ $66 $0F $D2 $72 2 xreg,xmem/imm8 PSRLD
\ $66 $0F $D3 $73 2 xreg,xmem/imm8 PSRLQ
\ $66 $0F $F8 xreg,xmem PSUBB
\ $66 $0F $F9 xreg,xmem PSUBW
\ $66 $0F $FA xreg,xmem PSUBD
\ $66 $0F $FB xreg,xmem PSUBQ
\ $66 $0F $E8 xreg,xmem PSUBSB
\ $66 $0F $E9 xreg,xmem PSUBSW
\ $66 $0F $D8 xreg,xmem PSUBUSB
\ $66 $0F $D9 xreg,xmem PSUBUSW
\ $66 $0F $68 xreg,xmem PUNPCKHBW
\ $66 $0F $69 xreg,xmem PUNPCKHWD
\ $66 $0F $6A xreg,xmem PUNPCKHDQ
\ $66 $0F $6D xreg,xmem PUNPCKHQDQ
\ $66 $0F $60 xreg,xmem PUNPCKLBW
\ $66 $0F $61 xreg,xmem PUNPCKLWD
\ $66 $0F $62 xreg,xmem PUNPCKLDQ
\ $66 $0F $6C xreg,xmem PUNPCKLQDQ
\ $66 $0F $EF xreg,xmem PXOR
\ $66 $0F $C6 xreg,xmem,imm8 SHUFPD
\ $66 $0F $51 xreg,xmem SQRTPD
\ $66 $0F $5C xreg,xmem SUBPD
\ $66 $0F $57 xreg,xmem XORPD
\ $66 $0F $2E xreg,xmem UCOMISD
\ $66 $0F $14 xreg,xmem UNPCKLPD
\ $66 $0F $15 xreg,xmem UNPCKHPD
[+switch switch660Fxx	\ byte --
\ DisAddr points past xx byte, REX has been processed
  $10 run:  s" MOVUPD"     op$-xreg,xmem  ;
  $11 run:  s" MOVUPD"     op$-xmem,xreg  ;
  $12 run:  s" MOVLPD"     op$-xreg,xmem  ;
  $13 run:  s" MOVLPD"     op$-xmem,xreg  ;
  $14 run:  s" UNPCKLPD"   op$-xreg,xmem  ;
  $15 run:  s" UNPCKHPD"   op$-xreg,xmem  ;
  $16 run:  s" MOVHPD"     op$-xreg,xmem  ;
  $1F run:  op-660F1Fxx  ;
  $17 run:  s" MOVHPD"     op$-xmem,xreg  ;
  $54 run:  s" ANDPD"      op$-xreg,xmem  ;
  $55 run:  s" ANDNPD"     op$-xreg,xmem  ;
  $C2 run:  s" CMPPD"      op$-xreg,xmem,comp8  ;
  $58 run:  s" ADDPD"      op$-xreg,xmem  ;
  $2F run:  s" COMISD"     op$-xreg,xmem  ;
  $5A run:  s" CVTPD2PS"   op$-xreg,xmem  ;
  $5B run:  s" CVTPS2DQ"   op$-xreg,xmem  ;
  $E6 run:  s" CVTTPD2PQ"  op$-xreg,xmem  ;
  $5E run:  s" DIVPD"      op$-xreg,xmem  ;
  $F7 run:  s" MASKMOVDQU" op$-xreg,xmem  ;
  $5F run:  s" MAXPD"      op$-xreg,xmem  ;
  $5D run:  s" MINPD"      op$-xreg,xmem  ;
  $28 run:  s" MOVAPD"     op$-xreg,xmem  ;
  $29 run:  s" MOVAPD"     op$-xmem,xreg  ;
  $6E run:  op-660F6E  ;
  $7E run:  op-660F7E  ;
  $6F run:  s" MOVDQA"     op$-xreg,xmem  ;
  $7F run:  s" MOVDQA"     op$-xmem,xreg  ;
  $50 run:  s" MOVMSKPD"   op$-reg,xreg   ;
  $E7 run:  s" MOVNTDQ"    op$-xmem,xreg  ;
  $2B run:  s" MOVNTPD"    op$-xmem,xreg  ;
  $59 run:  s" MULPD"      op$-xreg,xmem  ;
  $56 run:  s" ORPD"       op$-xreg,xmem  ;
  $63 run:  s" PACKSSWB"   op$-xreg,xmem  ;
  $6B run:  s" PACKSSDW"   op$-xreg,xmem  ;
  $67 run:  s" PACKUSWB"   op$-xreg,xmem  ;
  $FC run:  s" PADDB"      op$-xreg,xmem  ;
  $FD run:  s" PADDW"      op$-xreg,xmem  ;
  $FE run:  s" PADDD"      op$-xreg,xmem  ;
  $D4 run:  s" PADDQ"      op$-xreg,xmem  ;
  $DC run:  s" PADDUSB"    op$-xreg,xmem  ;
  $DD run:  s" PADDUSW"    op$-xreg,xmem  ;
  $DB run:  s" PAND"       op$-xreg,xmem  ;
  $DF run:  s" PANDN"      op$-xreg,xmem  ;
  $E0 run:  s" PAVGB"      op$-xreg,xmem  ;
  $E3 run:  s" PAVGW"      op$-xreg,xmem  ;
  $74 run:  s" PCMPEQB"    op$-xreg,xmem  ;
  $75 run:  s" PCMPEQW"    op$-xreg,xmem  ;
  $76 run:  s" PCMPEQD"    op$-xreg,xmem  ;
  $64 run:  s" PCMPGTB"    op$-xreg,xmem  ;
  $65 run:  s" PCMPGTW"    op$-xreg,xmem  ;
  $66 run:  s" PCMPGTD"    op$-xreg,xmem  ;
  $C4 run:  s" PINSRW"     op$-xreg,r32/m16,imm8  ;
  $F5 run:  s" PMADDWD"    op$-xreg,xmem  ;
  $EE run:  s" PMAXSW"     op$-xreg,xmem  ;
  $DE run:  s" PMAXUB"     op$-xreg,xmem  ;
  $EA run:  s" PMINSW"     op$-xreg,xmem  ;
  $DA run:  s" PMINUB"     op$-xreg,xmem  ;
  $E4 run:  s" PMULHUW"    op$-xreg,xmem  ;
  $E5 run:  s" PMULHW"     op$-xreg,xmem  ;
  $D5 run:  s" PMULLW"     op$-xreg,xmem  ;
  $F4 run:  s" PMULUDQ"    op$-xreg,xmem  ;
  $EB run:  s" POR"        op$-xreg,xmem  ;
  $F6 run:  s" PSADBWW"    op$-xreg,xmem  ;
  $70 run:  s" PSHUFPD"    op$-xreg,xmem,imm8  ;
  $F1 run:  s" PSLLW"      op$-xreg,xmem  ;
  $F3 run:  s" PSLLQ"      op$-xreg,xmem  ;
  $E1 run:  s" PSRAW"      op$-xreg,xmem  ;
  $E2 run:  s" PSRAD"      op$-xreg,xmem  ;
  $73 run:  s" PSRLQ"      op$-xreg,imm8  ;
  $72 run:  s" PSRLD"      op$-xreg,imm8  ;
  $71 run:  s" PSRLW"      op$-xreg,imm8  ;
  $D3 run:  s" PSRLQ"      op$-xreg,xmem  ;
  $D2 run:  s" PSRLD"      op$-xreg,xmem  ;
  $D1 run:  s" PSRLW"      op$-xreg,xmem  ;
  $F8 run:  s" PSUBB"      op$-xreg,xmem  ;
  $F9 run:  s" PSUBW"      op$-xreg,xmem  ;
  $FA run:  s" PSUBD"      op$-xreg,xmem  ;
  $FB run:  s" PSUBQ"      op$-xreg,xmem  ;
  $E8 run:  s" PSUBSB"     op$-xreg,xmem  ;
  $E9 run:  s" PSUBSW"     op$-xreg,xmem  ;
  $D8 run:  s" PSUBUSB"    op$-xreg,xmem  ;
  $D9 run:  s" PSUBUSW"    op$-xreg,xmem  ;
  $68 run:  s" PUNPCKHBW"  op$-xreg,xmem  ;
  $69 run:  s" PUNPCKHWD"  op$-xreg,xmem  ;
  $6A run:  s" PUNPCKHDQ"  op$-xreg,xmem  ;
  $6D run:  s" PUNPCKHQDQ" op$-xreg,xmem  ;
  $60 run:  s" PUNPCKLBW"  op$-xreg,xmem  ;
  $61 run:  s" PUNPCKLWD"  op$-xreg,xmem  ;
  $62 run:  s" PUNPCKLDQ"  op$-xreg,xmem  ;
  $6C run:  s" PUNPCKLQDQ" op$-xreg,xmem  ;
  $EF run:  s" PXOR"       op$-xreg,xmem  ;
  $C6 run:  s" SHUFPD"     op$-xreg,xmem,imm8  ;
  $51 run:  s" SQRTPD"     op$-xreg,xmem  ;
  $5C run:  s" SUBPD"      op$-xreg,xmem  ;
  $2E run:  s" UCOMISD"    op$-xreg,xmem  ;
  $57 run:  s" XORPD"      op$-xreg,xmem  ;
switch]

\ $F2 $0F $58 xreg,xmem ADDSD
\ $F2 $0F $C2 0 cc-xreg,xmem CMPxxxSD
\ $F2 $0F $E6 xreg,xmem CVTPD2DQ
\ $F2 $0F $2D reg32/64,xmem CVTSD2SI
\ $F2 $0F $5A xreg,xmem CVTSD2SS
\ $F2 $0F $2A xreg,r/m32/64 CVTSI2SD
\ $F2 $0F $2C reg32/64,xmem CVTTSD2SI
\ $F2 $0F $5E xreg,xmem DIVSD
\ $F2 $0F $5F xreg,xmem MAXSD
\ $F2 $0F $5D xreg,xmem MINSD
\ $F2 $0F $10 $11 xmem,xmem MOVSD-XM
\ $F2 $0F $59 xreg,xmem MULSD
\ $F2 $0F $70 xreg,xmem,imm8 PSHUFLW
\ $F2 $0F $51 xreg,xmem SQRTSD
\ $F2 $0F $5C xreg,xmem SUBSD
[+switch switchF20Fxx	\ byte --
  $58 run:  s" ADDSD"    op$-xreg,xmem  ;
  $C2 run:  s" CMPSD"    op$-xreg,xmem,comp8  ;
  $E6 run:  s" CVTPD2DQ" op$-xreg,xmem  ;
  $2D run:  s" CVTSD2SI" op$-reg,xmem   ;
  $5A run:  s" CVTSD2SS" op$-xreg,xmem  ;
  $2A run:  s" CVTSI2SD" op$-xreg,r/m	;
  $5F run:  s" MAXSD"    op$-xreg,xmem  ;
  $5F run:  s" MAXSD"    op$-xreg,xmem  ;
  $2C run:  s" CVTTSD2SI" op$-reg,xmem  ;
  $5E run:  s" DIVSD"    op$-xreg,xmem  ;
  $5F run:  s" MAXSD"    op$-xreg,xmem  ;
  $5D run:  s" MINSD"    op$-xreg,xmem  ;
  $10 run:  s" MOVSD"    op$-xreg,xmem  ;
  $11 run:  s" MOVSD"    op$-xmem,xreg  ;
  $59 run:  s" MULSD"    op$-xreg,xmem  ;
  $70 run:  s" PSHUFLW"  op$-xreg,xmem,imm8  ;
  $51 run:  s" SQRTSD"   op$-xreg,xmem  ;
  $5C run:  s" SUBSD"    op$-xreg,xmem  ;
switch]

: op-F30F1Exx	\ --
  case  @byte
    $FA of  s" ENDBR64" op$>dis  endof
    $FB of  s" ENDBR32" op$>dis  endof
      s" UNKNOWN - may be 4 byte NOP" op$>dis
  endcase
;

\ $F3 $0F $58 xreg,xmem ADDSS
\ $F3 $0F $C2 7 cc-xreg,xmem CMPxxxSS
\ $F3 $0F $E6 xreg,xmem CVTDQ2PD
\ $F3 $0F $2A xreg,r/m32/64 CVTSI2SS
\ $F3 $0F $5A xreg,xmem CVTSS2SD
\ $F3 $0F $2D reg32/64,xmem CVTSS2SI
\ $F3 $0F $5B xreg,xmem CVTTPS2DQ
\ $F3 $0F $2C reg32/64,xmem CVTTSS2SI
\ $F3 $0F $5E xreg,xmem DIVSS
\ $F3 $0F $5F xreg,xmem MAXSS
\ $F3 $0F $5D xreg,xmem MINSS
\ $F3 $0F $6F $7F xmem,xmem MOVDQU
\ $F3 $0F $10 $11 xmem,xmem MOVSS-XM
\ $F3 $0F $59 xreg,xmem MULSS
\ $F3 $0F $70 xreg,xmem,imm8 PSHUFHW
\ $F3 $0F $53 xreg,xmem RCPSS
\ $F3 $0F $52 xreg,xmem RSQRTSS
\ $F3 $0F $51 xreg,xmem SQRTSS
\ $F3 $0F $5C xreg,xmem SUBSS
\ $F3 $0F $1E $FA       ENDBR64
\ $F3 $0F $1E $FB       ENDBR32
[+switch switchF30Fxx	\ byte --
\ DisAddr points past xx byte, REX has been processed
  $1E run:  op-F30F1Exx  ;
  $58 run:  s" ADDSS"    op$-xreg,xmem  ;
  $C2 run:  s" CMPSS"    op$-xreg,xmem,comp8  ;
  $E6 run:  s" CVTDQ2PD" op$-xreg,xmem  ;
  $2A run:  s" CVTSI2SS" op$-xreg,r/m   ;
  $5A run:  s" CVTSS2SD" op$-xreg,xmem  ;
  $2D run:  s" CVTSS2SI" op$-reg,xmem   ;
  $5F run:  s" MAXSS"    op$-xreg,xmem  ;
  $5B run:  s" CVTTPS2DQ" op$-xreg,xmem  ;
  $2C run:  s" CVTTSS2SI" op$-reg,xmem  ;
  $5E run:  s" DIVSS"    op$-xreg,xmem  ;
  $5D run:  s" MINSS"    op$-xreg,xmem  ;
  $6F run:  s" MOVDQU"   op$-xreg,xmem  ;
  $7F run:  s" MOVDQU"   op$-xmem,xreg  ;
  $7E run:  s" MOVQ"     op$-xreg,xmem  ;
  $10 run:  s" MOVSS"    op$-xreg,xmem  ;
  $11 run:  s" MOVSS"    op$-xmem,xreg  ;
  $59 run:  s" MULSS"    op$-xreg,xmem  ;
  $70 run:  s" PSHUFHW"  op$-xreg,xmem,imm8  ;
  $53 run:  s" RCPSS"    op$-xreg,xmem  ;
  $52 run:  s" RSQRTSS"  op$-xreg,xmem  ;
  $51 run:  s" SQRTSS"   op$-xreg,xmem  ;
  $5C run:  s" SUBSS"    op$-xreg,xmem  ;
  $b8 run:  s" POPCNT"   op$>dis  dis-reg,r/m  ;
  $bc run:  s" TZCNT"    op$>dis  dis-reg,r/m  ;
  $bd run:  s" LZCNT"    op$>dis  dis-reg,r/m  ;
switch]


\ ======
\ *> ###
\ ======

previous definitions
decimal
