\ hdasmSSE - disassembler for: prefix [REX] 0F instructions

((
Copyright (c) 2020
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone numbers are:
       011 44 23 8063 1441
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth  also disassembler definitions
decimal


\ ===========
\ *! hdasmsse
\ *T SSE/SSE2 disassembler
\ ===========

create regsXMM	\ -- addr ; table of XMM register names
  ", XMM0"  ", XMM1"  ", XMM2"  ", XMM3"  ", XMM4"  ", XMM5"  ", XMM6"  ", XMM7"
  ", XMM8"  ", XMM9"  ", XMM10" ", XMM11" ", XMM12" ", XMM13" ", XMM14" ", XMM15"

: xreg>dis	\ reg# -- ; output XMMn
  regsXMM table$ >disline
;

[-short-branches
: XR/M+DISP>DIS	\ -- ; process XR/M, maybe a sib, maybe a displacement, or xreg
  \ 32bit?= -1  32bit data
  \ mod= 0, 1 or 2 and r/m= 4 then sib byte present
  \ modr/m = 00 reg# 100
  r/m 4 =  modifier 3 < AND
  IF  MODRM32-2BYTE  EXIT  THEN		\ mod r/m + sib
  \ 00 101 mod= 0 & r/m= 5 data 32
  modifier 0= r/m 5 = NOT AND
  IF  MEM(32bits)  EXIT  THEN
  modifier 3 < IF
    modifier 0= r/m 5 = AND IF
      disp+rip>dis
    ELSE
      [>DIS
      r/m +rex>base areg>dis
      modifier 1 = IF			\ byte offset
        @BYTE ?DUP
        IF  +>DIS SEXT>DIS  THEN
      THEN
      modifier 2 = IF			\ 32 bit offset
        @LONG ?DUP
        IF  +>DIS LONG>DIS  THEN
      THEN
      ]>DIS
    THEN
  ELSE
    r/m +rex>base xreg>dis
  THEN
;
short-branches]

: dis-Imm8	\ --
  s" , # " >disline  @byte sext>dis
;

: dis-dreg	\ --
  reg# +rex>reg dreg>dis
;

: dis-xreg	\ --
  reg# +rex>reg xreg>dis
;

: dis-xreg,xmem	\ --
\ disassemble xreg, XR/M, maybe a sib, maybe a displacement, or xreg
  @MODRM  dis-xreg  ,>DIS  XR/M+DISP>DIS
;

: op$-xreg,xmem	\ caddr len --
  op$>dis    @MODRM  dis-xreg  ,>DIS  XR/M+DISP>DIS
;

: op$-xmem,xreg	\ caddr len --
  op$>dis  @modrm  XR/M+DISP>DIS  ,>dis  dis-xreg
;

: op$-xreg,xmem,imm8	\ caddr len --
  op$>dis  dis-xreg,xmem  dis-Imm8
;

: op$-memsz	\ caddr len --
  op$>dis  @modrm  XR/M+DISP>DIS
;

: op$-xreg,imm8	\ caddr len --
  op$>dis  @modrm  XR/M+DISP>DIS  dis-Imm8
;

: op$-xreg,r/m	\ caddr len --
  op$>dis  @modrm  dis-xreg  ,>dis  R/M+DISP>DIS
;

: op$-r/m,xreg	\ caddr len --
  op$>dis  @modrm  R/M+DISP>DIS  ,>dis  dis-xreg
;

: op$-xreg,r32/m16,imm8	\ caddr len
  op$>dis  @modrm  dis-xreg  ,>dis  R/M+DISP>DIS  dis-Imm8
;

: op$-reg,xreg	\ caddr len --
  op$>dis  @modrm  dis-dreg  ,>dis  R/M+DISP>DIS
;

: op$-reg,xmem	\ caddr len --
  op$>dis  @modrm  dis-dreg  ,>dis  XR/M+DISP>DIS
;

create xconds	\ --
  ", EQ"	", LT"		", LE"		", UNORD"
  ", NEQ"	", NLT"		", NLE"		", ORD"
\  --
  ", EQ_UQ"	", NGE"		", NGT"		", FALSE"
  ", NEQ_OQ"	", GE"		", GT"		", TRUE"
\  --
  ", EQ_OS"	", LT_OQ"	", LE_OQ"	", UNORD_S"
  ", NEQ_OS"	", NLT_UQ"	", NLE_UQ"	", ORD_S"
\  --
  ", EQ_US"	", NGE_UQ"	", NGT_UQ"	", FALSE_OS"
  ", NEQ_OS"	", GE_OQ"	", GT_OQ"	", TRUE_US"

: comp-SSE	\ --
  @byte $01F and xconds table$ >disline
;

: op$-xreg,xmem,comp8	\ caddr len --
  op$>dis  @modrm  dis-xreg  ,>dis  R/M+DISP>DIS  ,>dis  comp-SSE
;


: op-0F12	\ --
  @modrm  modifier 3 = if
    s" MOVHLPS" op$>dis  dis-xreg  ,>dis  XR/M+DISP>DIS
  else
    s" MOVHPS" op$>dis  dis-xreg  ,>dis  XR/M+DISP>DIS
  then
;

: op-0F16	\ --
  @modrm  modifier 3 = if
    s" MOVLHPS" op$>dis  dis-xreg  ,>dis  XR/M+DISP>DIS
  else
    s" MOVLPS" op$>dis  dis-xreg  ,>dis  XR/M+DISP>DIS
  then
;

: op-0FA0	\ --
  opcDsize @ 64 =
  if  s" FXSAVE64"  else  s" FXSAVE"  then
  op$-memsz
;

: op-0FAE	\ --
  @modrm
  case  modifier
    2 of  s" LDMXCSR" endof
    3 of  s" STMXCSR" endof
      s" unknown "
  endcase
  op$>dis  XR/M+DISP>DIS
;

\  -1 $0F $54 xreg,xmem ANDPS
\  -1 $0F $55 xreg,xmem ANDNPS
\  -1 $0F $C2 0 cc-xreg,xmem CMPxxxPS
\  -1 $0F $2F xreg,xmem COMISS
\  -1 $0F $5B xreg,xmem CVTDQ2PS
\  -1 $0F $5A xreg,xmem CVTPS2PD
\  -1 $0F $5E xreg,xmem DIVPS
\  -1 $0F $A0 0 #32 op3-memsz FXSAVE
\  -1 $0F $A0 0 #64 op3-memsz FXSAVE64
\  -1 $0F $5F xreg,xmem MAXPS
\  -1 $0F $5D xreg,xmem MINPS
\  -1 $0F $58 xreg,xmem ADDPS
\  -1 $0F $28 $29 xmem,xmem MOVAPS
\  -1 $0F $16 $17 xr/r/m,xr/r/m MOVHPS  $16 op2 <> reg, $17 op1 <> reg
\  -1 $0F $16 xreg,xreg MOVLHPS		op2 = reg only
\  -1 $0F $12 $13 xr/r/m,xr/r/m MOVLPS
\  -1 $0F $12 xreg,xreg MOVHLPS
\  -1 $0F $50 reg,xreg MOVMSKPS
\  -1 $0F $E7 mem,xreg MOVNTPS
\  -1 $0F $7E $D6 xmem,xmem MOVQ-XM
\  -1 $0F $10 $11 xmem,xmem MOVUPS
\  -1 $0F $59 xreg,xmem MULPS
\  -1 $0F $f6 xreg,xmem ORPS
\  -1 $0F $53 xreg,xmem RCPPS
\  -1 $0F $52 xreg,xmem RSQRTPS
\  -1 $0F $C6 xreg,xmem,imm8 SHUFPS
\  -1 $0F $51 xreg,xmem SQRTPS
\  -1 $0F $AE 2 #32 op3-memsz LDMXCSR
\  -1 $0F $AE 3 #32 op3-memsz STMXCSR
\  -1 $0F $5C xreg,xmem SUBPS
\  -1 $0F $57 xreg,xmem XORPS
\  -1 $0F $2E xreg,xmem UCOMISS
\  -1 $0F $14 xreg,xmem UNPCKLPS
\  -1 $0F $15 xreg,xmem UNPCKHPS
: (ops0Fxx)	\ --
\ DisAddr points at xx byte, REX has been processed
  case @byte
    $54 of  s" ANDPS"    op$-xreg,xmem  endof
    $55 of  s" ANDNPS"   op$-xreg,xmem  endof
    $C2 of  s" CMPPS"    op$-xreg,xmem,comp8  endof
    $58 of  s" ADDPS"    op$-xreg,xmem  endof
    $2F of  s" COMISS"   op$-xreg,xmem  endof
    $5B of  s" CVTDQ2PS" op$-xreg,xmem  endof
    $5A of  s" CVTPS2PD" op$-xreg,xmem  endof
    $5E of  s" DIVPS"    op$-xreg,xmem  endof
    $A0 of  op-0FA0  endof
    $5F of  s" MAXPS"    op$-xreg,xmem  endof
    $5D of  s" MINPS"    op$-xreg,xmem  endof
    $28 of  s" MOVAPS"   op$-xreg,xmem  endof
    $29 of  s" MOVAPS"   op$-xmem,xreg  endof
    $16 of  op-0F16  endof
    $17 of  s" MOVHPS"   op$-xmem,xreg  endof
    $12 of  op-0F12  endof
    $13 of  s" MOVLPS"   op$-xmem,xreg  endof
    $50 of  s" MOVMSKPS" op$-reg,xreg   endof
    $E7 of  s" MOVNTPS"  op$-xmem,xreg  endof
    $7E of  s" MOVQ-XM"  op$-xreg,xmem  endof
    $D6 of  s" MOVQ-XM"  op$-xmem,xreg  endof
    $10 of  s" MOVUPS"   op$-xreg,xmem  endof
    $11 of  s" MOVUPS"   op$-xmem,xreg  endof
    $59 of  s" MULPS"    op$-xreg,xmem  endof
    $F6 of  s" ORPS"     op$-xreg,xmem  endof
    $53 of  s" RCPPS"    op$-xreg,xmem  endof
    $52 of  s" RSQRTPS"  op$-xreg,xmem  endof
    $C6 of  s" SHUFPS"   op$-xreg,xmem,imm8  endof
    $51 of  s" SQRTPS"   op$-xreg,xmem  endof
    $AE of  op-0FAE  endof
    $5C of  s" SUBPS"    op$-xreg,xmem  endof
    $2E of  s" UCOMISS"  op$-xreg,xmem  endof
    $14 of  s" UNPCKLPS" op$-xreg,xmem  endof
    $15 of  s" UNPCKHPS" op$-xreg,xmem  endof
    $57 of  s" XORPS"    op$-xreg,xmem  endof
      illegal
  endcase
;  assign (ops0Fxx) to-do ops0Fxx

\ $66 $0F $F1 $71 6 xreg,xmem/imm8 PSLLW
\ $66 $0F $E1 $71 4 xreg,xmem/imm8 PSRAW
: op-660F71
  case DisAddr c@c(t) 3 rshift $07 and	\ inspect reg field of mod/reg/mem
    4 of  s" PSRAW"      op$-xreg,imm8  endof
    6 of  s" PSLLW"      op$-xreg,imm8  endof
      illegal
  endcase
;

\ $66 $0F $F2 $72 6 xreg,xmem/imm8 PSLLD
\ $66 $0F $E2 $72 4 xreg,xmem/imm8 PSRAD
: op-660F72
  case DisAddr c@c(t) 3 rshift $07 and	\ inspect reg field mod/reg/mem
    4 of  s" PSRAD"      op$-xreg,imm8  endof
    6 of  s" PSLLD"      op$-xreg,imm8  endof
      illegal
  endcase
;

\ $66 $0F $73 7 xreg,imm8 PSLLDQ
\ $66 $0F $F3 $73 6 xreg,xmem/imm8 PSLLQ
\ $66 $0F $73 3 xreg,imm8 PSRLDQ
: op-660F73
  case DisAddr c@c(t) 3 rshift $07 and	\ inspect reg field mod/reg/mem
    2 of  s" PSRLQ"      op$-xreg,imm8  endof
    3 of  s" PSRLDQ"     op$-xreg,imm8  endof
    6 of  s" PSLLQ"      op$-xreg,imm8  endof
    7 of  s" PSLLDQ"     op$-xreg,imm8  endof
      illegal
  endcase
;

: ?MOVD/Q	\ -- caddr len
  D64bit? if  s" MOVQ-XM"  else  s" MOVD-XM"  then
;

: op-660F6E	\ --
  ?MOVD/Q  op$-xreg,r/m
;

: op-660F7E	\ --
  ?MOVD/Q op$-r/m,xreg
;

\ $66 $0F $54 xreg,xmem ANDPD
\ $66 $0F $55 xreg,xmem ANDNPD
\ $66 $0F $C2 0 cc-xreg,xmem CMPxxxPD
\ $66 $0F $2F xreg,xmem COMISD
\ $66 $0F $5A xreg,xmem CVTPD2PS
\ $66 $0F $5B xreg,xmem CVTPS2DQ
\ $66 $0F $E6 xreg,xmem CVTTPD2PQ
\ $66 $0F $5E xreg,xmem DIVPD
\ $66 $0F $58 xreg,xmem ADDPD
\ $66 $0F $F7 xreg,xreg MASKMOVDQU
\ $66 $0F $5F xreg,xmem MAXPD
\ $66 $0F $5D xreg,xmem MINPD
\ $66 $0F $28 $29 xmem,xmem MOVAPD
\ $66 $0F $6E $7E #32 xr/r/m,xr/r/m MOVD-X
\ $66 $0F $6E $7E #64 xr/r/m,xr/r/m MOVQ-X
\ $66 $0F $6F $7F xmem,xmem MOVDQA
\ $66 $0F $16 $17 xr/r/m,xr/r/m MOVHPD
\ $66 $0F $12 $13 xr/r/m,xr/r/m MOVLPD
\ $66 $0F $50 reg,xreg MOVMSKPD
\ $66 $0F $e7 mem,xreg MOVNTDQ
\ $66 $0F $2B mem,xreg MOVNTPD
\ $66 $0F $10 $11 xmem,xmem MOVUPD
\ $66 $0F $59 xreg,xmem MULPD
\ $66 $0F $56 xreg,xmem ORPD
\ $66 $0F $63 xreg,xmem PACKSSWB
\ $66 $0F $6B xreg,xmem PACKSSDW
\ $66 $0F $67 xreg,xmem PACKUSWB
\ $66 $0F $FC xreg,xmem PADDB
\ $66 $0F $FD xreg,xmem PADDW
\ $66 $0F $FE xreg,xmem PADDD
\ $66 $0F $D4 xreg,xmem PADDQ
\ $66 $0F $DC xreg,xmem PADDUSB
\ $66 $0F $DD xreg,xmem PADDUSW
\ $66 $0F $DB xreg,xmem PAND
\ $66 $0F $DF xreg,xmem PANDN
\ $66 $0F $E0 xreg,xmem PAVGB
\ $66 $0F $E3 xreg,xmem PAVGW
\ $66 $0F $74 xreg,xmem PCMPEQB
\ $66 $0F $75 xreg,xmem PCMPEQW
\ $66 $0F $76 xreg,xmem PCMPEQD
\ $66 $0F $64 xreg,xmem PCMPGTB
\ $66 $0F $65 xreg,xmem PCMPGTW
\ $66 $0F $66 xreg,xmem PCMPGTD
\ $66 $0F $C4 xreg,r32/m16,imm8 PINSRW
\ $66 $0F $F5 xreg,xmem PMADDWD
\ $66 $0F $EE xreg,xmem PMAXSW
\ $66 $0F $DE xreg,xmem PMAXUB
\ $66 $0F $EA xreg,xmem PMINSW
\ $66 $0F $DA xreg,xmem PMINUB
\ $66 $0F $E4 xreg,xmem PMULHUW
\ $66 $0F $E5 xreg,xmem PMULHW
\ $66 $0F $D5 xreg,xmem PMULLW
\ $66 $0F $F4 xreg,xmem PMULUDQ
\ $66 $0F $EB xreg,xmem POR
\ $66 $0F $F6 xreg,xmem PSADBWW
\ $66 $0F $70 xreg,xmem,imm8 PSHUFD
\ $66 $0F $F1 $71 6 xreg,xmem/imm8 PSLLW
\ $66 $0F $F3 $73 6 xreg,xmem/imm8 PSLLQ
\ $66 $0F $E1 $71 4 xreg,xmem/imm8 PSRAW
\ $66 $0F $E2 $72 4 xreg,xmem/imm8 PSRAD
\ $66 $0F $73 3 xreg,imm8 PSRLDQ
\ $66 $0F $D1 $71 2 xreg,xmem/imm8 PSRLW
\ $66 $0F $D2 $72 2 xreg,xmem/imm8 PSRLD
\ $66 $0F $D3 $73 2 xreg,xmem/imm8 PSRLQ
\ $66 $0F $F8 xreg,xmem PSUBB
\ $66 $0F $F9 xreg,xmem PSUBW
\ $66 $0F $FA xreg,xmem PSUBD
\ $66 $0F $FB xreg,xmem PSUBQ
\ $66 $0F $E8 xreg,xmem PSUBSB
\ $66 $0F $E9 xreg,xmem PSUBSW
\ $66 $0F $D8 xreg,xmem PSUBUSB
\ $66 $0F $D9 xreg,xmem PSUBUSW
\ $66 $0F $68 xreg,xmem PUNPCKHBW
\ $66 $0F $69 xreg,xmem PUNPCKHWD
\ $66 $0F $6A xreg,xmem PUNPCKHDQ
\ $66 $0F $6D xreg,xmem PUNPCKHQDQ
\ $66 $0F $60 xreg,xmem PUNPCKLBW
\ $66 $0F $61 xreg,xmem PUNPCKLWD
\ $66 $0F $62 xreg,xmem PUNPCKLDQ
\ $66 $0F $6C xreg,xmem PUNPCKLQDQ
\ $66 $0F $EF xreg,xmem PXOR
\ $66 $0F $C6 xreg,xmem,imm8 SHUFPD
\ $66 $0F $51 xreg,xmem SQRTPD
\ $66 $0F $5C xreg,xmem SUBPD
\ $66 $0F $57 xreg,xmem XORPD
\ $66 $0F $2E xreg,xmem UCOMISD
\ $66 $0F $14 xreg,xmem UNPCKLPD
\ $66 $0F $15 xreg,xmem UNPCKHPD
: (ops660Fxx)	\ --
\ DisAddr points at xx byte, REX has been processed
  case  @byte
    $54 of  s" ANDPD"      op$-xreg,xmem  endof
    $55 of  s" ANDNPD"     op$-xreg,xmem  endof
    $C2 of  s" CMPPD"      op$-xreg,xmem,comp8  endof
    $58 of  s" ADDPD"      op$-xreg,xmem  endof
    $2F of  s" COMISD"     op$-xreg,xmem  endof
    $5A of  s" CVTPD2PS"   op$-xreg,xmem  endof
    $5B of  s" CVTPS2DQ"   op$-xreg,xmem  endof
    $E6 of  s" CVTTPD2PQ"  op$-xreg,xmem  endof
    $5E of  s" DIVPD"      op$-xreg,xmem  endof
    $F7 of  s" MASKMOVDQU" op$-xreg,xmem  endof
    $5F of  s" MAXPD"      op$-xreg,xmem  endof
    $5D of  s" MINPD"      op$-xreg,xmem  endof
    $28 of  s" MOVAPD"     op$-xreg,xmem  endof
    $29 of  s" MOVAPD"     op$-xmem,xreg  endof
    $6E of  op-660F6E  endof
    $7E of  op-660F7E  endof
    $6F of  s" MOVDQA"     op$-xreg,xmem  endof
    $7F of  s" MOVDQA"     op$-xmem,xreg  endof
    $16 of  s" MOVHPD"     op$-xreg,xmem  endof
    $17 of  s" MOVHPD"     op$-xmem,xreg  endof
    $12 of  s" MOVLPD"     op$-xreg,xmem  endof
    $13 of  s" MOVLPD"     op$-xmem,xreg  endof
    $50 of  s" MOVMSKPD"   op$-reg,xreg   endof
    $E7 of  s" MOVNTDQ"    op$-xmem,xreg  endof
    $2B of  s" MOVNTPD"    op$-xmem,xreg  endof
    $10 of  s" MOVUPD"     op$-xreg,xmem  endof
    $11 of  s" MOVUPD"     op$-xmem,xreg  endof
    $59 of  s" MULPD"      op$-xreg,xmem  endof
    $56 of  s" ORPD"       op$-xreg,xmem  endof
    $63 of  s" PACKSSWB"   op$-xreg,xmem  endof
    $6B of  s" PACKSSDW"   op$-xreg,xmem  endof
    $67 of  s" PACKUSWB"   op$-xreg,xmem  endof
    $FC of  s" PADDB"      op$-xreg,xmem  endof
    $FD of  s" PADDW"      op$-xreg,xmem  endof
    $FE of  s" PADDD"      op$-xreg,xmem  endof
    $D4 of  s" PADDQ"      op$-xreg,xmem  endof
    $DC of  s" PADDUSB"    op$-xreg,xmem  endof
    $DD of  s" PADDUSW"    op$-xreg,xmem  endof
    $DB of  s" PAND"       op$-xreg,xmem  endof
    $DF of  s" PANDN"      op$-xreg,xmem  endof
    $E0 of  s" PAVGB"      op$-xreg,xmem  endof
    $E3 of  s" PAVGW"      op$-xreg,xmem  endof
    $74 of  s" PCMPEQB"    op$-xreg,xmem  endof
    $75 of  s" PCMPEQW"    op$-xreg,xmem  endof
    $76 of  s" PCMPEQD"    op$-xreg,xmem  endof
    $64 of  s" PCMPGTB"    op$-xreg,xmem  endof
    $65 of  s" PCMPGTW"    op$-xreg,xmem  endof
    $66 of  s" PCMPGTD"    op$-xreg,xmem  endof
    $C4 of  s" PINSRW"     op$-xreg,r32/m16,imm8  endof
    $F5 of  s" PMADDWD"    op$-xreg,xmem  endof
    $EE of  s" PMAXSW"     op$-xreg,xmem  endof
    $DE of  s" PMAXUB"     op$-xreg,xmem  endof
    $EA of  s" PMINSW"     op$-xreg,xmem  endof
    $DA of  s" PMINUB"     op$-xreg,xmem  endof
    $E4 of  s" PMULHUW"    op$-xreg,xmem  endof
    $E5 of  s" PMULHW"     op$-xreg,xmem  endof
    $D5 of  s" PMULLW"     op$-xreg,xmem  endof
    $F4 of  s" PMULUDQ"    op$-xreg,xmem  endof
    $EB of  s" POR"        op$-xreg,xmem  endof
    $F6 of  s" PSADBWW"    op$-xreg,xmem  endof
    $70 of  s" PSHUFPD"    op$-xreg,xmem,imm8  endof
    $F1 of  s" PSLLW"      op$-xreg,xmem  endof
    $F3 of  s" PSLLQ"      op$-xreg,xmem  endof
    $E1 of  s" PSRAW"      op$-xreg,xmem  endof
    $E2 of  s" PSRAD"      op$-xreg,xmem  endof
    $73 of  s" PSRLQ"      op$-xreg,imm8  endof
    $72 of  s" PSRLD"      op$-xreg,imm8  endof
    $71 of  s" PSRLW"      op$-xreg,imm8  endof
    $D3 of  s" PSRLQ"      op$-xreg,xmem  endof
    $D2 of  s" PSRLD"      op$-xreg,xmem  endof
    $D1 of  s" PSRLW"      op$-xreg,xmem  endof
    $F8 of  s" PSUBB"      op$-xreg,xmem  endof
    $F9 of  s" PSUBW"      op$-xreg,xmem  endof
    $FA of  s" PSUBD"      op$-xreg,xmem  endof
    $FB of  s" PSUBQ"      op$-xreg,xmem  endof
    $E8 of  s" PSUBSB"     op$-xreg,xmem  endof
    $E9 of  s" PSUBSW"     op$-xreg,xmem  endof
    $D8 of  s" PSUBUSB"    op$-xreg,xmem  endof
    $D9 of  s" PSUBUSW"    op$-xreg,xmem  endof
    $68 of  s" PUNPCKHBW"  op$-xreg,xmem  endof
    $69 of  s" PUNPCKHWD"  op$-xreg,xmem  endof
    $6A of  s" PUNPCKHDQ"  op$-xreg,xmem  endof
    $6D of  s" PUNPCKHQDQ" op$-xreg,xmem  endof
    $60 of  s" PUNPCKLBW"  op$-xreg,xmem  endof
    $61 of  s" PUNPCKLWD"  op$-xreg,xmem  endof
    $62 of  s" PUNPCKLDQ"  op$-xreg,xmem  endof
    $6C of  s" PUNPCKLQDQ" op$-xreg,xmem  endof
    $EF of  s" PXOR"       op$-xreg,xmem  endof
    $C6 of  s" SHUFPD"     op$-xreg,xmem,imm8  endof
    $51 of  s" SQRTPD"     op$-xreg,xmem  endof
    $5C of  s" SUBPD"      op$-xreg,xmem  endof
    $2E of  s" UCOMISD"    op$-xreg,xmem  endof
    $14 of  s" UNPCKLPD"   op$-xreg,xmem  endof
    $15 of  s" UNPCKHPD"   op$-xreg,xmem  endof
    $57 of  s" XORPD"      op$-xreg,xmem  endof
      illegal
  endcase
;  assign (ops660Fxx) to-do ops660Fxx


\ $F2 $0F $58 xreg,xmem ADDSD
\ $F2 $0F $C2 0 cc-xreg,xmem CMPxxxSD
\ $F2 $0F $E6 xreg,xmem CVTPD2DQ
\ $F2 $0F $2D reg32/64,xmem CVTSD2SI
\ $F2 $0F $5A xreg,xmem CVTSD2SS
\ $F2 $0F $2A xreg,r/m32/64 CVTSI2SD
\ $F2 $0F $2C reg32/64,xmem CVTTSD2SI
\ $F2 $0F $5E xreg,xmem DIVSD
\ $F2 $0F $5F xreg,xmem MAXSD
\ $F2 $0F $5D xreg,xmem MINSD
\ $F2 $0F $10 $11 xmem,xmem MOVSD-XM
\ $F2 $0F $59 xreg,xmem MULSD
\ $F2 $0F $70 xreg,xmem,imm8 PSHUFLW
\ $F2 $0F $51 xreg,xmem SQRTSD
\ $F2 $0F $5C xreg,xmem SUBSD
: (opsF20Fxx)	\ --
\ DisAddr points at xx byte, REX has been processed
  case @byte
    $58 of  s" ADDSD"    op$-xreg,xmem  endof
    $C2 of  s" CMPSD"    op$-xreg,xmem,comp8  endof
    $E6 of  s" CVTPD2DQ" op$-xreg,xmem  endof
    $2D of  s" CVTSD2SI" op$-reg,xmem   endof
    $5A of  s" CVTSD2SS" op$-xreg,xmem  endof
    $2A of  s" CVTSI2SD" op$-xreg,r/m	endof
    $5F of  s" MAXSD"    op$-xreg,xmem  endof
    $5F of  s" MAXSD"    op$-xreg,xmem  endof
    $2C of  s" CVTTSD2SI" op$-reg,xmem  endof
    $5E of  s" DIVSD"    op$-xreg,xmem  endof
    $5F of  s" MAXSD"    op$-xreg,xmem  endof
    $5D of  s" MINSD"    op$-xreg,xmem  endof
    $10 of  s" MOVSD-XM" op$-xreg,xmem  endof
    $11 of  s" MOVSD_XM" op$-xmem,xreg  endof
    $59 of  s" MULSD"    op$-xreg,xmem  endof
    $70 of  s" PSHUFLW"  op$-xreg,xmem,imm8  endof
    $51 of  s" SQRTSD"   op$-xreg,xmem  endof
    $5C of  s" SUBSD"    op$-xreg,xmem  endof
      illegal
  endcase
;  assign (opsF20Fxx) to-do opsF20Fxx

\ $F3 $0F $58 xreg,xmem ADDSS
\ $F3 $0F $C2 7 cc-xreg,xmem CMPxxxSS
\ $F3 $0F $E6 xreg,xmem CVTDQ2PD
\ $F3 $0F $2A xreg,r/m32/64 CVTSI2SS
\ $F3 $0F $5A xreg,xmem CVTSS2SD
\ $F3 $0F $2D reg32/64,xmem CVTSS2SI
\ $F3 $0F $5B xreg,xmem CVTTPS2DQ
\ $F3 $0F $2C reg32/64,xmem CVTTSS2SI
\ $F3 $0F $5E xreg,xmem DIVSS
\ $F3 $0F $5F xreg,xmem MAXSS
\ $F3 $0F $5D xreg,xmem MINSS
\ $F3 $0F $6F $7F xmem,xmem MOVDQU
\ $F3 $0F $10 $11 xmem,xmem MOVSS-XM
\ $F3 $0F $59 xreg,xmem MULSS
\ $F3 $0F $70 xreg,xmem,imm8 PSHUFHW
\ $F3 $0F $53 xreg,xmem RCPSS
\ $F3 $0F $52 xreg,xmem RSQRTSS
\ $F3 $0F $51 xreg,xmem SQRTSS
\ $F3 $0F $5C xreg,xmem SUBSS
: (opsF30Fxx)	\ --
\ DisAddr points at xx byte, REX has been processed
  case  @byte
    $58 of  s" ADDSS"    op$-xreg,xmem  endof
    $C2 of  s" CMPSS"    op$-xreg,xmem,comp8  endof
    $E6 of  s" CVTDQ2PD" op$-xreg,xmem  endof
    $2A of  s" CVTSI2SS" op$-xreg,r/m   endof
    $5A of  s" CVTSS2SD" op$-xreg,xmem  endof
    $2D of  s" CVTSS2SI" op$-reg,xmem   endof
    $5F of  s" MAXSS"    op$-xreg,xmem  endof
    $5B of  s" CVTTPS2DQ" op$-xreg,xmem  endof
    $2C of  s" CVTTSS2SI" op$-reg,xmem  endof
    $5E of  s" DIVSS"    op$-xreg,xmem  endof
    $5D of  s" MINSS"    op$-xreg,xmem  endof
    $6F of  s" MOVDQU"   op$-xreg,xmem  endof
    $7F of  s" MOVDQU"   op$-xmem,xreg  endof
    $10 of  s" MOVSS-XM" op$-xreg,xmem  endof
    $11 of  s" MOVSS_XM" op$-xmem,xreg  endof
    $59 of  s" MULSS"    op$-xreg,xmem  endof
    $70 of  s" PSHUFHW"  op$-xreg,xmem,imm8  endof
    $53 of  s" RCPSS"    op$-xreg,xmem  endof
    $52 of  s" RSQRTSS"  op$-xreg,xmem  endof
    $51 of  s" SQRTSS"   op$-xreg,xmem  endof
    $5C of  s" SUBSS"    op$-xreg,xmem  endof
    $b8 of  s" POPCNT"   op$>dis  dis-reg,r/m  endof
    $bc of  s" TZCNT"    op$>dis  dis-reg,r/m  endof
    $bd of  s" LZCNT"    op$>dis  dis-reg,r/m  endof
      illegal
  endcase
;  assign (opsF30Fxx) to-do opsF30Fxx


\ ======
\ *> ###
\ ======

previous definitions
decimal
