\ REQDCODE.FTH - required code fragments

((
Copyright (c) 2001, 2002, 2006, 2008, 2009, 2013, 2014, 2019, 2020, 2022
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441


To do
=====
Recode SKIP SCAN to avoid double branch at JECX.


Change history
==============
20220804 SFP017 Added 64 bit CALL.
20200613 SFP016 Added and removed FLUSHCACHE.
20200603 SFP015 Added DLSHIFT and DRSHIFT.
20200310 SFP014 More 64 bit work.
20190706 SFP013 64 bit conversion
20140723 SFP012 Removed divide primitives, done by cross compiler.
20131212 SFP011 Added overlap checks to CMOVE and CMOVE>.
		On overlap, always use a byte by byte copy.
20090603 SFP010 Fixed RELLVS
20080930 SFP009 Added DU< and DU>.
20061002 SFP008 Recoded S= for longer strings.
20060807 SFP007 Rewrote FILL for P4.
20060606 SFP006 Added CMOVE primitives for P4 and above.
20020811 SFP005 Fixed bug in S= for len=0.
20020318 SFP004 Make SKIPCALL PATCHCALL GETCALLDEST !JUMP visible.
20010702 SFP003 Added S= SKIP and SCAN
19990924 SFP002 Made doLayAbsCall visible
19990905 SFP001 Added INLINE-ALWAYS where possible
		Added _/MOD _/ _MOD
))


\ On the x86_64 the following register usage is
\ assumed:
\ Forth regs.
\   RSI         UP (thread pointer)	volatile Linux, non-vol Windows
\   RDI         LP (locals pointer)	volatile Linux, non-vol Windows
\   RSP         RSP
\   RBP         PSP			non-volatile across ABIs
\   R13		FSP			non-volatile across ABIs
\ Special
\   R14		Index for DO/LOOP	non-volatile across ABIs
\   R15		Index~Limit		non-volatile across ABIs
\   XMM8	FTOS			volatile Linux, non-vol Windows
\   XMM9        FTEMP temp float
\ Stack
\   RBX         TOS
\   RDX
\   RCX
\   RAX
\   R8-R12


\ Undocumented fragments of carnal knowledge code for the Kernel
\ These are things you just can't do in portable high-level Forth

proc DODOES     \                                               MPE.0000
        pop     rax
        sub     rbp, # cell
        mov     0 [rbp], rbx
        pop     rbx
        jmp     rax

L: DOCREATE     \                                               MPE.0000
        sub     rbp, # cell
        mov     0 [rbp], rbx
        pop     rbx
        next,
end-code

code (do)	\ limit index -- ; exit address inline
	pop	rdx			\ rdx = return address
	mov	rcx, 0 [rdx]		\ exit address
	lea	rdx, cell [rdx]		\ step over it
	push	rcx			\ rcx = loop exit address
	push	r15			\ r15 = old index - limit - msb
	push	r14			\ r14 = old index

	mov	r15, rbx		\ rbx = index, r15 = index
	sub     r15, 0 [rbp]		\ r15 = index - limit
	btc	r15, # 63		\ r15 = index - limit - msb
	mov	r14, rbx		\ r14 = index

	mov	rbx, cell [rbp]		\ restore TOS
	lea	rbp, 2 cells [rbp]	\ step data stack

	jmp	rdx
end-code

code (?do)	\ limit index -- ; exit address inline
	cmp	rbx, 0 [rbp]		\ index=limit ?
	je	qdox
	pop	rdx			\ rdx = return address
	mov	rcx, 0 [rdx]		\ exit address
	lea	rdx, cell [rdx]		\ step over it
	push	rcx			\ rcx = exit address
	push	r15			\ r15 = old index - limit - msb
	push	r14			\ r14 = old index

	mov	r15, rbx		\ rbx = index, r15 = index
	sub     r15, 0 [rbp]		\ r15 = index - limit
	btc	r15, # 63		\ r15 = index - limit - msb
	mov	r14, rbx		\ r14 = index

	mov	rbx, cell [rbp]		\ restore TOS
	lea	rbp, 2 cells [rbp]	\ step data stack

	jmp	rdx

l: qdox
	mov	rbx, cell [rbp]		\ restore TOS
	lea	rbp, 2 cells [rbp]	\ step data stack
	pop	rdx
	mov	rdx, 0 [rdx]		\ take exit address
	jmp	rdx
end-code

code (+LOOP)    \ step --
\ RSP+18: ESP+C:  exit address
\ RSP+10: ESP+8:  old index-limit-$8000.0000  (incremented by LOOP)
\ RSP+08: ESP+4:  old index                   (incremented by LOOP)
\ RSP+00: return address
\ R15:    new index-limit-xorbit63
\ R14:    new index
        mov     rax, rbx
        mov     rbx, 0 [rbp]
        add     rbp, # cell		\ RAX=step, RBX restored
        add     r14, rax		\ step index
        add     r15, rax		\ step limit
        jo      leave_ploop
        pop     rax
        mov     rax, 0 [rax]
        jmp     rax
l: leave_ploop
        pop     rax			\ return address
        pop	r14			\ index
        pop	r15			\ index-limit-xorbit63
        add     rsp, # cell		\ exit address
        add     rax, # cell		\ step over branch target
        jmp     rax			\ exit
end-code

code (LOOP)    \ --
\ RSP+18: ESP+C:  exit address
\ RSP+10: ESP+8:  old index-limit-$8000.0000  (incremented by LOOP)
\ RSP+08: ESP+4:  old index                   (incremented by LOOP)
\ RSP+00: return address
\ R15:    new index-limit-xorbit63
\ R14:    new index
        inc     r14			\ step index
        inc     r15			\ step limit
\        add     r14, # 1		\ step index
\        add     r15, # 1		\ step limit
        jo      leave_loop
        pop     rax
        mov     rax, 0 [rax]
        jmp     rax
l: leave_loop
        pop     rax			\ return address
        pop	r14			\ index
        pop	r15			\ index-limit-xorbit63
        add     rsp, # cell		\ exit address
        add     rax, # cell		\ step over branch target
        jmp     rax			\ exit
end-code

code unloop	\ --
\ RSP+18: ESP+C:  exit address
\ RSP+10: ESP+8:  old index-limit-$8000.0000  (incremented by LOOP)
\ RSP+08: ESP+4:  old index                   (incremented by LOOP)
\ RSP+00: return address
\ R15:    new index-limit-xorbit63
\ R14:    new index
        pop     rax			\ return address and discard it
        pop	r14			\ index
        pop	r15			\ index-limit-xorbit63
        ret
end-code

\ **************************
\ machine code CALL handlers
\ **************************

[defined] <dp> [if]
<dp> constant DP
[else]
variable DP	\ -- addr
\ *G Holds the address of the next free location in the dictionary.
[then]

((
code t  mov rax, # $01234567:89abcdef  call rax  next,  end-code
dis t
T
( 000EE780    48B8EFCDAB8967452301 )  MOV     RAX, # 01234567:89ABCDEF
( 000EE78A    48FFD0 )                CALL    RAX
( 000EE78D    C3 )                    RET/NEXT
( 14 bytes, 3 instructions )
))
: doLayAbsCall  \ address --
  dup dp @ 5 + -		\ -- addr off64 ; 64 bit offset
  #32 arshift 1+ 		\ 0/1 for 32 bit
  2 u< if			\ -- addr ; check for 32 bit offset
    $E8 C,  dp @ 4+ - l,	\ 32 bit offset
  else				\ -- addr ; 64 bit abs
    $48 c,  $B8 c,  ,		\ mov rax, # addr ; 10 bytes
    $48 c,  $FF c,  $D0 c,	\ call rax        ; 3 bytes, 13 total
  then
;

: PatchCall     \ abs:32 address-of-call-instruction --
  dup 5 +			\ abs32 aoci after-ci
  swap -rot			\ aoci abs32 after-ci
  - swap 1+ l!
;

: SkipCall      \ a-addr -- 'a-addr
\ +G Modify address by size of a CALL instruction.
  5 +
;

: GetCallDest   \ a-addr -- abs:32
\ +G Given the address of a CALL instruction return the CALL
\ +* target address.
  dup 1 + l@s			\ get rel:32 part of instruction
  swap 5 + +			\ +end-of-instruction ptr = ABS:32
;

: !call         \ target ^ins --
\ +G Set a CALL instruction at *\fo{^ins} that calls address
\ +* *\fo{target}.
  $E8 over c!			\ make it a CALL instruction
  1+ tuck			\ -- ^off target ^off
  4+ - swap l!			\ --
;

: !jump         \ target where --
\ +G Set a JMP instruction at *\fo{^ins} that calls address
\ +* *\fo{target}.
  $E9 over c!
  1+ tuck
  4+ - swap l!
;

CODE LV!	\ n -- ; store local variable [offset in-line]
        pop     rax			\ return address
        mov     rdx, 0 [rax]		\ offset
        add     rax, # cell		\ step over offset
        add     rdx, rdi		\ offset+LP
        mov     0 [rdx], rbx		\ TOS -> [offset+LP]
        mov     rbx, 0 [rbp]		\ restore TOS
        add     rbp, # cell
        jmp     rax
END-CODE

CODE LV@	\ -- n ; read local variable [offset in-line]
        pop     rax			\ return address
        mov     rdx, 0 [rax]		\ offset
        add     rax, # cell		\ step over offset
        add     rdx, rdi		\ offset+LP
        sub     rbp, # cell		\ new TOS needed
        mov     0 [rbp], rbx		\ save old TOS
        mov     rbx, 0 [rdx]		\ [offset+LP] -> TOS
        jmp     rax
END-CODE

CODE LV		\ -- a-addr ; read local variable address [offset in-line]
        pop     rax
        mov     rdx, 0 [rax]
        add     rax, # cell
        add     rdx, rdi
        sub     rbp, #  cell
        mov     0 [rbp], rbx
        mov     rbx, rdx
        jmp     rax
END-CODE

CODE MAKELVS	\ -- ; make space for local variables
                \ #args (cells), #locals (cells) in-line.
                \ #args and #cells occupy 16 bits each.
                \ Frame structure is as OTA spec, vol1 3.2.5
                \ arguments +ve offset, locals -ve offset
\ allocate a frame for the arguments
        pop     rax			\ return address is in-line data pointer
        mov     rcx, rsp		\ RCX = old stack pointer

        sub     rdx, rdx
        mov     dx, 0 [rax]
        add     rax, 2			\ step over #args
        shl     rdx, 3			\ convert cells to bytes
        sub     rsp, rdx		\ make argument frame

        push    rcx			\ old stack pointer
        push    rdi			\ old locals pointer
        mov     rdi, rsp		\ new locals pointer

        cmp     rdx, # 0		\ any arguments?
        je      makelvs3		\ no
        mov     rcx, rsp
        add     rcx, # 2 cells		\ points to pushed SP and LP
        mov     0 [rcx], rbx		\ stash TOS
        add     rcx, # cell		\ stop over saved TOS
l: makelvs1				\ rest of arguments
        sub     rdx, # cell		\ count #args
        je      makelvs2		\ done
        mov     rbx, 0 [rbp]		\ copy arg
        add     rbp, # cell
        mov     0 [rcx], rbx
        add     rcx, # cell		\ step to next
        jmp     makelvs1
l: makelvs2
        mov     rbx, 0 [rbp]
        add     rbp, # cell
l: makelvs3
        sub     rdx, rdx
        mov     dx, 0 [rax]		\ get #locals
        add     rax, 2			\ step over #locals
        shl     rdx, 3			\ convert cells to bytes
        sub     rsp, rdx
        jmp     rax
END-CODE

CODE RELLVS  ( -- ; drop local variables )
	pop	rax
        mov     rsp, cell [rdi]
        mov     rdi, 0 [rdi]
	jmp	rax
END-CODE

\ Here you put things you want your cross-compiler code generator to simply
\ TARGET-REF because you can't write a code generator

code Nrev     \ XN..X1 count -- x1..XN
\ *G Reverse the order of the top N data stack items.
        cmp     rbx, # 1		\ ignore count <1
        g, if,
          dec     rbx			\ item n at offset n-1
          mov     rcx, rbp		\ data stack pointer
          shl     rbx, # 3		\ in cells
          add     rbx, rcx		\ RBX points to XN, RCX to X1
          begin,
            mov     rdx, 0 [rbx]	\ perform exchange
            mov     rax, 0 [rcx]
            mov     0 [rcx], rdx
            add     rcx, # cell
            mov     0 [rbx], rax
            sub     rbx, # cell
            cmp     rcx, rbx
          a, until,
        endif,
        mov     rbx, 0 [rbp]
        add     rbp, # cell
        next,
end-code

INTERNAL

CODE mpeWITHIN?    \ n1 n2 n3 -- flag
        mov     rax, 0 [rbp]		\ N2
        mov     rdx, cell [rbp]		\ N1
        mov     rcx, rbx		\ N3
        xor     rbx, rbx
        add     rbp, # 2 cells		\ drop two items
        cmp     rdx, rax
        ge, if,
          cmp	  rdx, rcx
          le, if,
            dec     rbx
          endif,
        endif,
        next,
END-CODE  inline-always	\ SFP001

((
: within	\ n1|u1 n2|u2 n3|u3 -- flag
  over - -rot			\ -- n3-n2 n1 n2
  -				\ -- n3-n2 n1-n2
  swap u<			\ (n1-n2)-(n3-n2), cy -> -1, ncy -> 0
;
))
CODE ansWITHIN	\ n1|u1 n2|u2 n3|u3 -- flag
        mov     rax, 0 [rbp]		\ RAX=n2|u2
        mov     rcx, cell [rbp]		\ RCX=n1|u1
        sub     rbx, rax		\ RBX=n3-n2, rebased to n2
        sub     rcx, rax		\ RCX=n1-n2, rebased to n2
        lea     rbp, 2 cells [rbp]	\ adjust data stack
        sub     rcx, rbx		\ RCX=(n1-n2)-(n3-n2)
	sbb	rbx, rbx		\ cy -> -1, ncy -> 0
        next,
END-CODE  inline-always	\ SFP001

code _D2*        \ d1 -- d1*2                                   8.6.1.1090
  mov rax, 0 [rbp]
  shl rax, # 1
  rcl rbx, # 1
  mov 0 [rbp], rax
  next,
end-code  inline-always	\ SFP001

code _D2/        \ d1 -- d1/2                                   8.6.1.1100
  mov rax, 0 [rbp]
  sar rbx, # 1
  rcr rax, # 1
  mov 0 [rbp], rax
  next,
end-code  inline-always	\ SFP001

EXTERNAL

code du<	\ d1l d1h d2l d2h -- flag
  mov  rax, 0 [rbp]			\ d2 = RBX:RAX
  mov  rdx, 2 cells [rbp]
  mov  rcx, cell [rbp]			\ d1 = RCX:RDX
  sub  rdx, rax				\ d1l - d2l
  sbb  rcx, rbx				\ d1h - d2h
  sbb  rbx, rbx				\ flag
  lea  rbp, 3 cells [rbp]
  next,
end-code

code du>	\ d1l d1h d2l d2h -- flag
  mov  rax, 0 [rbp]			\ d2 = RBX:RAX
  mov  rdx, 2 cells [rbp]
  mov  rcx, cell [rbp]			\ d1 = RCX:RDX
  sub  rax, rdx				\ d2l - d1l
  sbb  rbx, rcx				\ d2h - d1h
  sbb  rbx, rbx				\ flag
  lea  rbp, 3 cells [rbp]
  next,
end-code


\ ****************************************************************
\ If these words are not defined, high level versions will be used
\ ****************************************************************

code dlshift	\ d u -- d' ; double left shift, 0..127
  mov		rdx, 0 [rbp]		\ dhi
  mov		rax, cell [rbp]		\ dlo
  cmp		rbx, # #64		\ >64
  ge, if,	\ reduce range to 0..63
    sub		  rbx, # #64		\ reduce shift count by 64
    mov		  rdx, rax		\ low -> high
    xor		  rax, rax		\ 0 -> low
  then,
  mov		ecx, ebx		\ shift count to CL
  shld		rdx, rax, cl		\ shift high, fill from low
  shl		rax, cl			\ shift low
  add		rbp, # cell		\ put d' back
  mov		0 [rbp], rax		\ low
  mov		rbx, rdx		\ high
  next,
end-code

code drshift	\ d u -- d' ; double right shift, 0..127
  mov		rdx, 0 [rbp]		\ dhi
  mov		rax, cell [rbp]		\ dlo
  cmp		rbx, # #64		\ >64
  ge, if,	\ reduce range to 0..63
    sub		  rbx, # #64		\ reduce shift count by 64
    mov		  rax, rdx		\ high -> lo
    xor		  rdx, rdx		\ 0 -> hi
  then,
  mov		ecx, ebx		\ shift count to CL
  shrd		rax, rdx, cl		\ shift low, fill from high
  shr		rdx, cl			\ shift high
  add		rbp, # cell		\ put d' back
  mov		0 [rbp], rax		\ low
  mov		rbx, rdx		\ high
  next,
end-code

CODE */MOD	\ n1 n2 n3 -- rem quot                          6.1.0110
  mov	rax, 0 [rbp]			\ n1
  mov	rcx, cell [rbp]			\ n2
  imul	rcx				\ n1*n2
  idiv	rbx				\ n1*n2/n3
  add	rbp, # cell			\ adjust stack
  mov   0 [rbp], rdx			\ remainder
  mov	rbx, rax			\ quotient
  next,
end-code  inline

CODE */         \ n1 n2 n3 -- n4                                6.1.0100
  mov	rax, 0 [rbp]			\ n1
  mov	rcx, cell [rbp]			\ n2
  imul	rcx				\ n1*n2
  idiv	rbx				\ n1*n2/n3
  add	rbp, # 2 cells			\ adjust stack
  mov	rbx, rax			\ quotient
  next,
end-code  inline

CODE M/         \ d1 n2 -- quot                                 MPE.0000
  mov   rdx, 0 [rbp]			\ d1h
  mov   rax, cell [rbp]			\ d1l
  add	rbp, # 2 cells			\ adjust stack
  idiv  rbx
  mov   rbx, rax
  next,
END-CODE  inline

CODE SM/REM	\ d1 n2 -- rem quot                            6.1.2214
  mov   rdx, 0 [rbp]
  add   rbp, # cell
  mov   rax, 0 [rbp]
  idiv  rbx
  mov   0 [rbp], rdx
  mov   rbx, rax
  next,
END-CODE  inline-always	\ SFP001

\ replace ORs by TESTs
CODE FM/MOD	\ d1 n2 -- rem quot                            6.1.1561
  mov   rdx, 0 [rbp]			\ dividend high
  mov   rax, cell [rbp]			\ dividend low
  mov   rcx, rbx                	\ sign of quotient by XOR
  add	rbp, # cell			\ drop one item from stack
  xor   rcx, rdx
  idiv  rbx                     	\ do division
  or    rcx, rcx                	\ test sign of quotient
  jns   msx                     	\ skip if positive
    or    rdx, rdx                  	\ test remainder
    jz    msx                     	\ skip if 0
      dec   rax                     	\ decrement quotient
      add   rdx, rbx                	\ adjust remainder
l: msx
  mov   0 [rbp], rdx			\ set remainder
  mov   rbx, rax			\ set quotient
  next,
END-CODE


((
: compare       \ c-addr1 u1 c-addr2 u2 -- n                    17.6.1.0935
  rot swap                      \ c-addr1 c-addr2 u1 u2
  2dup - >r min                 \ c-addr1 c-addr2 minlen -- R: lendiff? --

  begin
    dup
  while
    -rot over c@ over c@ -      \ length c-addr1 c-addr2 (char1-char2)
\   ?dup if                     \ If chars are different
    dup if                      \ If chars are different
      r> drop >r                \  replace lendiff result with error code
      drop 0                    \  and put 0 on TOS (make len==0 at BEGIN)
    else                        \ otherwise
      drop			\ discard 0 lendiff
      1+ swap 1+ swap rot 1-    \ increment addresses and decrement length
    then
  repeat
  drop 2drop                    \ remove addresses and null count from stack
                                \ -- ; R: result --
  r> dup if 0< 1 or then        \ make nice flag, 0 becomes 0, -ve becomes -1
                                \            and  +ve becomes 1
;
))

code compare	\ c-addr1 u1 c-addr2 u2 -- n ; 17.6.1.0935
  mov	rcx, rbx			\ RCX=u2
  push	rsi				\ preserve Forth regs
  push	rdi
  xor	rbx, rbx			\ RBX=flag, assume success
  cmp	rcx, cell [rbp]			\ select min of u1/u2
  je	compare2			\ same
  jnb	compare1
  inc	rbx				\ result = 1
  jmp	near compare2
l: compare1
  dec	rbx				\ result = -1
  mov	rcx, cell [rbp]			\ u1 < u2
l: compare2
  jecxz	compare4			\ min(u1,u2)=0 check
  mov	rsi, 2 cells [rbp]		\ c-addr1
  mov	rdi, 0 [rbp]			\ c-addr1
  repe cmpsb				\ compare strings
  je	compare4			\ same
  jb	compare3
  mov	rbx, # 1
  jmp	short compare4
l: compare3
  mov	rbx, # -1
l: compare4
  pop	rdi
  pop	rsi
  lea	rbp, 3 cells [rbp]
  next,
end-code

((
: s=            \ addr1 addr2 len -- flag                          MPE.0000
  dup if
    bounds do
      dup c@ i c@ <> if
        drop false  unloop exit
      endif
      1 chars +
    loop
    drop true
  else
    3drop true
  endif
;
))

CODE s=		\ caddr1 caddr2 u -- flag ; true if strings same ; SFP008
  mov   rcx, 0 [rbp]			\ RCX=caddr2
  mov   r8, rbx				\ RBX=u, make copy
  shr   rbx, # 3			\ #cells **1
  mov   rdx, cell [rbp]			\ RDX=caddr1
  nz, if,				\ **1
    begin,				\ **2
      mov   rax, 0 [rdx]
      lea   rdx, cell [rdx]
      cmp   rax, 0 [rcx]
      lea   rcx, cell [rcx]
      jne   s=x
      dec   rbx
    z, until,
  endif,
  test	r8, # 4				\ r8 range 4..7?
  nz, if,
    mov	  eax, 0 [rdx]
    lea   rdx, 4 [rdx]
    cmp	  eax, 0 [rcx]
    lea   rcx, 4 [rcx]
    jne	  s=x
  endif,
  and   r8, # 3				\ #bytes **1
  nz, if,				\ **1
    begin,				\ **2
      mov   al, 0 [rdx]
      lea   rdx, 1 [rdx]
      cmp   al, 0 [rcx]
      lea   rcx, 1 [rcx]
      jne   s=x
      dec   r8
    z, until,
  endif,
  mov   rbx, # -1			\ same, return -1
  lea   rbp, 2 cells [rbp]			\ restore stack
  next,
l: s=x
  xor   rbx, rbx			\ different, return 0
  lea   rbp, 2 cells [rbp]			\ restore stack
  next,
end-code

((
: SKIP          \ c-addr u char -- 'c-addr 'u                    MPE.0000
  over 0= if drop exit then
  >r
  begin					\ -- caddr u
    over c@ dup ATAB =			\ if character is TAB
    if  drop BL  endif			\ convert to space
    r@ =
    over and				\ 0 length will stop
  while
    1- swap 1+ swap
  repeat
  r> drop
;
))
code SKIP	\ c-addr u char -- 'c-addr 'u                    MPE.0000
  mov	rdx, rbx			\ RDX=char
  mov	rcx, 0 [rbp]			\ RCX=u=len
  mov   rbx, cell [rbp]			\ RBX=caddr
l: skip1
  jecxz skipx				\ exit if len=0
  mov	al, 0 [rbx]			\ get character
  cmp	al, # 9				\ convert tab to space
  z, if,  mov  al, # $020  endif,
  cmp	al, dl				\ get out if not char to skip
  jne	skipx
  dec	rcx				\ step count and address
  inc	rbx
  jmp	skip1
l: skipx
  mov	cell [rbp], rbx			\ 'c-addr
  mov	rbx, rcx			\ 'u=len'
  lea	rbp, cell [rbp]			\ restore stack
  next,
end-code

((
: scan          \ caddr u char -- caddr2 u2                      MPE.0000
  over 0= if drop exit then
  >r
  begin
    over c@ dup ATAB =			\ if character is TAB
    if  drop BL  endif			\ convert to space
    r@ <>
    over and
  while
    1- swap 1+ swap
  repeat
  r> drop
;
))
code scan	\ caddr u char -- caddr2 u2                      MPE.0000
  mov	rdx, rbx			\ RDX=char
  mov	rcx, 0 [rbp]			\ RCX=u=len
  mov   rbx, cell [rbp]			\ RBX=caddr
l: scan1
  jecxz scanx				\ exit if len=0
  mov	al, 0 [rbx]			\ get character
  cmp	al, # 9				\ convert tab to space
  z, if,  mov  al, # $020  endif,
  cmp	al, dl				\ get out if char to scan for
  je	scanx
  dec	rcx				\ step count and address
  inc	rbx
  jmp	scan1
l: scanx
  mov	cell [rbp], rbx			\ 'c-addr
  mov	rbx, rcx			\ 'u=len'
  lea	rbp, cell [rbp]			\ restore stack
  next,
end-code

((
: fill          \ addr len char -- ; 6.1.1540
  -rot bounds ?do
    dup i c!
  loop
  drop
;
))

code fill	\ addr len char -- ; 6.1.1540
  mov	rcx, 0 [rbp]			\ len
  mov   rax, rbx			\ char
  mov	rdx, rdi			\ save RDI
  jrcxz fillx
  mov	rdi, cell [rbp]			\ addr
  rep stosb				\ copy - rely on AMD64 hardware
l: fillx
  mov	rbx, 2 cells [rbp]
  mov	rdi, rdx
  lea	rbp, 3 cells [rbp]
  next,
end-code


\ SFP006...
code cmove	\ src dest len --
\ load parameters
	mov	r8, rsi
	mov	r9, rdi
	mov	rsi, cell [rbp]		\ src
	mov	rdi, 0 [rbp]		\ dest
	mov	rcx, rbx		\ len
	jrcxz	cmovex
	rep movsb
l: cmovex
	mov	rdi, r9
	mov	rsi, r8
	mov	rbx, 2 cells [rbp]
	lea	rbp, 3 cells [rbp]
	next,
end-code

code cmove>	\ src dest len --
\ load parameters
  mov    rcx, cell [rbp]	\ ecx=src
  mov    rdx, 0 [rbp]		\ edx=dest
\ optimised move - eight bytes at a time
  mov    r8, rbx		\ ebx=len
  lea    rcx, -8 [rcx] [rbx]	\ point to last cells
  lea    rdx, -8 [rdx] [rbx]
  shr    rbx, # 3		\ number of cells to move
  nz, if,
    begin,
      mov    rax, 0 [rcx]
      sub    rbx, # 1
      lea    rcx, -8 [rcx]
      mov    0 [rdx], rax
      lea    rdx, -8 [rdx]
    z, until,			\ points to NEXT cells
  then,
\ single byte moves
  mov    rbx, r8		\ EBX=len
  and    rbx, # 7		\ number of bytes to move
  lea    rcx, 7 [rcx]		\ adjust cell to byte addresses
  lea    rdx, 7 [rdx]
  nz, if,			\ RBX <> 0
    begin,
      mov    al, 0 [rcx]
      sub    rbx, # 1
      lea    rcx, -1 [rcx]
      mov    0 [rdx], al
      lea    rdx, -1 [rdx]
    z, until,
  then,
\ remove parameters
  mov       rbx, 2 cells [rbp]
  lea       rbp, 3 cells [rbp]
  next,
end-code
((
code cmove>	\ src dest len --
\ load parameters
	mov	r8, rsi
	mov	r9, rdi
	mov	rsi, cell [rbp]		\ src
	mov	rdi, 0 [rbp]		\ dest
	mov	rcx, rbx		\ len
	jrcxz	cmove>x
	lea	rsi, -1 [rsi] [rcx]
	lea	rdi, -1 [rdi] [rcx]
	std
	rep movsb
	cld
l: cmove>x
	mov	rdi, r9
	mov	rsi, r8
	mov	rbx, 2 cells [rbp]
	lea	rbp, 3 cells [rbp]
	next,
end-code
))

code ucmove	\ src dest len --
\ As CMOVE but converts to upper case.
	mov	rdx, 0 [rbp]		\ rdx=dest
	mov	rcx, cell [rbp]		\ rcx=src
	cmp	rbx, # 0
	nz, if,
	  begin,
	    mov   al, 0 [rcx]
	    lea   rcx, 1 [rcx]
	    cmp   al, # char a
	    ae, if,
	      cmp   al, # char z
	      be, if,
	        and   al, # $0DF
	      endif,
	    endif,
	    sub   rbx, # 1
	    mov   0 [rdx], al
	    lea   rdx, 1 [rdx]
	  z, until,
	then,
	mov     rbx, 2 cells [rbp]
	lea     rbp, 3 cells [rbp]
	next,
end-code

code ucmove>	\ src dest len --
\ As CMOVE> but converts to upper case.
	mov	rcx, cell [rbp]		\ RCX=src
	mov     rdx, 0 [rbp]		\ RDX=dest
	cmp     rbx, # 0		\ RBX=len
	nz, if,
	  lea     rcx, -1 [rcx] [rbx]
	  lea     rdx, -1 [rdx] [rbx]
	  begin,
	    mov     al, 0 [rcx]
	    lea     rcx, -1 [rcx]
	    cmp     al, # char a
	    ae, if,
	      cmp     al, # char z
	      be, if,
	        and     al, # $0DF
	      endif,
	    endif,
	    sub    rbx, # 1
	    mov    0 [rdx], al
	    lea    rdx, -1 [rdx]
	  z, until,
	then,
	mov    rbx, 2 cells [rbp]
	lea    rbp, 3 cells [rbp]
	next,
end-code


\ ************
\ Name hashing
\ ************

\ If you change the hash function, you MUST also change the
\ hash functions in the cross compiler VfxHead.fth, the target
\ files KERNEL.FTH and REQDCODE.FTH to be the same.
((
compiler
: OldHash	\ c-addr len -- hash
  swap c@ +				\ count & first char
;

: NewHash1	\ c-addr len -- hash
  tuck  over c@ -rot			\ -- len first caddr len
  + 1- c@				\ -- len first last
  + +					\ -- hash
;

: NewHash2	\ c-addr len -- hash
  tuck  over c@ 1 lshift -rot		\ -- len first caddr len
  + 1 - c@ 2 lshift			\ -- len first last
  + +					\ -- hash
;

: NewHash3	\ caddr len -- hash ; previous best
  tuck					\ -- len caddr len
  over c@ 2 lshift -rot			\ -- len first caddr len
  + 1 - c@ 4 lshift			\ -- len first last
  + +					\ -- hash
;

: NewHash4	\ caddr len -- hash
  0 -rot  bounds
  do  1 lshift  i c@ +  loop
;

: NewHash5	\ caddr len -- hash
  dup 3 and 6 lshift -rot	\ -- len' caddr len
  over c@ 3 lshift -rot		\ -- len' first' caddr len
  + 1 - c@			\ -- len' first' last
  + +				\ -- hash
;
target
))

\ $8FDB equ CRCPoly
 $A001 equ CrcPoly
\ $013C equ CrcPoly

align
l: CRCTBL  #256 2* ALLOT

interpreter
: INIT-CRC16-TABLE	\ --
  #256 0 DO
    I  8 0 DO
      DUP 1 AND
      swap 1 rshift swap
\      >R  1 RSHIFT  R>
      IF  CrcPoly XOR  THEN
    LOOP
    I 2*  CRCTBL + W!
  LOOP
;
target
INIT-CRC16-TABLE

((
compiler
: CRC-16	\ crc byte -- crc'
  OVER  XOR  255 AND  2* CRCTBL + W@  SWAP  8 RSHIFT  XOR
;

: CRChash1	\ caddr len -- hash
  dup 3 lshift -rot		\ crc caddr len
  bounds ?do			\ -- crc
    i c@ crc-16
  loop
  $FFFF and
;

: GenHash	\ caddr len -- hash
  CRChash1
;
target
))

code GenHash	\ caddr len -- hash
  mov   rcx, rbx			\ RCX = len
  mov   rbx, 0 [rbp]			\ RBX = addr
  lea   rbp, cell [rbp]			\ update stack
  lea   rdx, 0 [rcx*8]			\ RDX = crc = len<<3
  jrcxz ghx
    movzx rax, byte 0 [rbx]		\ first byte	i c@	\ -- crc byte
    lea   rbx, 1 [rbx]
  begin,
    xor   rax, rdx			\		over xor
    and   rax, # $FF			\		255 and
    shr   rdx, # 8			\ avoid pipeline stutter
    movzx rax, word CRCTBL [rax*2]	\		2* CRCTBL + W@
    xor   rdx, rax			\		xor
    sub   rcx, # 1
    movzx rax, byte 0 [rbx]		\ next byte	i c@	\ -- crc byte
    lea   rbx, 1 [rbx]
  z, until,
l: ghx
  mov   rbx, rdx			\ return hash value
  and   rbx, # $FFFF
  ret
end-code

((
code flushCache	\ -- ; system dependent
  mfence
  ret
end-code
))
