\ SaveElf.fth - VFX Forth for Linux ELF image save

((
Copyright (c) 2004, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20120525 SFP002 Added protection for INCLUDE.
20071228 SFP001 Added new code size to new ELF header.
))

decimal

\ ==========
\ *> turnkey
\ ==========
\ *S Saving to an ELF file
\ *P The following sequence performs this operation:
\ *E ok SAVE myfile.elf
\ ** ok BYE
\ ** poop> ./myfile.elf

only forth definitions
((
: Mb		\ n -- nMb ; nMb = n * 1048576
\ *G Given *\i{n}, returns n megabytes. Useful before *\fo{SET-SIZE}
\ ** or *\fo{ALLOCATE}.
  1048576 *
;

: Kb		\ n -- nKb ; nKb = n * 1024
\ *G Given *\i{n}, returns n kilobytes. Useful before *\fo{SET-SIZE}
\ ** or *\fo{ALLOCATE}.
  1024 *
;
))
: get-size	\ -- size
\ *G Return the amount of memory used by the Forth dictionary and
\ ** system headers.
  codesize @ pe-coderva +
;

: set-size	\ size --
\ *G Set the amount of memory to be used by the Forth dictionary and
\ ** system headers. This will not take effect until the system has
\ ** been *\fo{SAVE}d to form a new ELF file. The new dictionary
\ ** size will be used when the new ELF file is run.
  4095 + -4096 and			\ force up to 4k page
  pe-codeRVA - newcodesize !
;

: get-stacks	\ -- size
\ *G Return the amount of memory used by the Forth stacks and user area.
\ ** OBSOLETE.
  StackSize @
;

: set-stacks	\ size --
\ *G Set the amount of memory to be used by the Forth stacks and
\ ** user area. This will not take effect until the system has
\ ** been *\fo{SAVE}d to form a new ELF file. The new size will
\ ** be used when the new ELF file is run.
\ ** OBSOLETE.
  StackSize !
;

: BeforeSave	\ --
\ +G Prepare the system for saving.
  ?markEndOfSource			\ SFP002
  CheckDict
  ['] FirstName here removeSinInRange	\ remove all tokeniser data
  freeze
;

: AppImage	\ -- addr len
\ +G Return the base address and length of the application.
  loadbase @ PE-CODERVA +  here over -
;

: RestoreGOT	\ addr --
\ +G Restore the GOT table in the ELF header, so that the dynamic linker can
\ +* work on the reloaded image
  ehdr -	\ -- (offset from running header to buffer)
  >r
  0 dlpriv1ref r@ + !
  0 dlpriv2ref r@ + !
  original_dlopen dlopenref r@ + !
  original_dlsym dlsymref r@ + !
  original_dlclose dlcloseref r@ + !
  original_dlerror dlerrorref r> + !
;

: CreatePatchedHeader	\ -- addr
\ +G Create a copy of the current ELF header, and patch it
\ +* back to how the executable started
  ehdr_size protalloc		\ Allocate a block to hold the ELF header copy
  dup ehdr swap ehdr_size move	\ Copy ELF header
  dup RestoreGOT		\ Patch copy of GOT table
  newcodesize @ over p_memszref1 ehdr - + !	\ patch requested code size ; SFP001
;

: ExpelWithGOTPatch	\ caddr len name namelen --
\ +G Write memory region defined by caddr/len to file defined by
\ +* name/namelen. The GOT region in the ELF header is patched
\ +* back to what it was in the original executable.
  r/w create-file #-412 ?throw >r		\ create
  CreatePatchedHeader
  dup ehdr_size r@ write-file #-415 ?throw	\ Write header
  protfree					\ Free buffer
  swap ehdr_size + swap ehdr_size -		\ Adjust addr & len
  r@ write-file #-415 ?throw			\ Write rest of image
  r> close-file #-416 ?throw			\ close
;

defer OS_GetUmask	\ -- mode
defer OS_Chmod		\ caddr u mode -- ior

also system definitions

assign (OS_GetUmask)		to-do OS_GetUmask
assign (OS_Chmod)		to-do OS_Chmod

previous definitions

$1ff equ MODE_777	\ Octal 777 - rwx for all

: Save		\ "<name>" -- ; SAVE <name>
\ *G Save the application to the file given by the following file
\ ** name. The .ELF extension must be supplied. The image is saved
\ ** as a complete ELF file.
  {: | buff[ 256 ] -- :}
  BeforeSave
  AppImage dup p_fileszref1 !		\ -- caddr len
  tuck GetPathSpec buff[ expandMacro
  2dup 2>r ExpelWithGOTPatch		\ -- len ; R: -- fname flen
  OS_GetUmask
  not MODE_777 and			\ Generate desired permission bits
  2r@ rot OS_Chmod drop			\ Ignore chmod failures
  cr  . ." bytes saved to " 2r> type
;

