//  VFX Forth tests of Linux

/* Original VFX test

For 32 bit use, compile with
  gcc -m32 osxtest.c  -o osxtest
For 64 bit use, compile with
  gcc -m64 osxtest.c  -o osxtest

*/

#include <stdio.h>
#include <signal.h>
#include <sys/ucontext.h>
// #include <asm/sigcontext.h>
#include <unistd.h>
#include <limits.h>
#include <sys/time.h>
#include <time.h>
#include <semaphore.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/termios.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <dlfcn.h>
#include <fcntl.h>


int main( int argc, char *argv[] )
{
   struct stat sbuf;
   struct sigaction act;
   siginfo_t info;
   ucontext_t context;
   long t1, t2, t3, t4, t5, sz1;
   sem_t sem;
   pthread_attr_t attr;
   pthread_mutex_t pmutex;
   struct ifreq myifreq;
   struct tm mytm;
   struct termios mytermios;
   struct timeval mytimeval;
   struct _fpstate myfpstate64;
   struct sigcontext mysigcontext64;
   gregset_t mygregset;
   fpregset_t myfpregset;

   uint32_t foobar;
   long long llfoo;

   printf("\nGeneral data types\n");
   printf(" long long sz  = %d\n",(int)sizeof(llfoo) );
   printf(" long size     = %d\n",(int)sizeof(long));
   printf(" int size      = %d\n",(int)sizeof(int));
   printf(" short size    = %d\n",(int)sizeof(short));
   printf(" clock_t size  = %d\n",(int)sizeof(clock_t));
   printf(" time_t size   = %d\n",(int)sizeof(time_t));
   printf(" timeval size  = %d\n",(int)sizeof(mytimeval) );
   printf(" pid_t size    = %d\n",(int)sizeof(pid_t));
   printf(" uid_t size    = %d\n",(int)sizeof(uid_t));
   printf(" mode_t size   = %d\n",(int)sizeof(mode_t));
   printf(" tm size       = %d\n",(int)sizeof(mytm));
   printf(" sizeof(off_t) = %d\n",(int)sizeof(off_t));
   printf(" sizeof addr   = %ld\n",(long)sizeof(&sbuf));
   printf(" sizeof size_t = %d\n",(int)sizeof(size_t));
   printf(" sizeof ssize_t= %d\n",(int)sizeof(ssize_t));

   printf("\ndlopen and friends\n");
   printf(" RTLD_LAZY = %d\n", (int)RTLD_LAZY );
   printf(" RTLD_NOW = %d\n", (int)RTLD_NOW );
   printf(" RTLD_GLOBAL = 0x%X\n", (int)RTLD_GLOBAL );
   printf(" RTLD_LOCAL = 0x%X\n", (int)RTLD_LOCAL );

   printf("\nLinux constants\n");
   printf(" PATH_MAX = %d\n", PATH_MAX);
//   printf(" SCK_TCK = %d\n", SCK_TCK );
   printf(" _SC_CLK_TCK = %d\n", _SC_CLK_TCK);		// get value with sysconf()
   printf(" CLOCKS_PER_SEC = %d\n", (int)CLOCKS_PER_SEC);
   printf(" FIONREAD = 0x%x\n", (int)FIONREAD);
   printf(" O_CREAT = 0x%x\n", (int)O_CREAT);
   printf(" O_TRUNC = 0x%x\n", (int)O_TRUNC);
   printf(" O_WRONLY = 0x%x\n", (int)O_WRONLY);
   printf(" O_RDWR = 0x%x\n", (int)O_RDWR);
   printf(" NCCS = %d\n", (int)NCCS);
   printf(" TCSANOW = %d\n", (int)TCSANOW);
   printf(" ICANON = 0x%x\n", (int)ICANON);
   printf(" ECHO = 0x%x\n", (int)ECHO);
   printf(" ISIG = 0x%x\n", (int)ISIG);
   printf(" sizeof termios = %d\n",(int)sizeof(mytermios));

   printf("\nFile structures\n");
   t1 = (long) (void *) &sbuf;
   t2 = (long) (void *) &(sbuf.st_size);
   sz1 = t2-t1;
   printf("offset st_size = %ld \n", sz1);

   printf("\nSignals\n");
   printf(" SA_SIGINFO     = %d\n", SA_SIGINFO );
   printf(" SIGILL         = %d\n", SIGILL );
   printf(" SIGFPE         = %d\n", SIGFPE );
   printf(" SIGSEGV        = %d\n", SIGSEGV );
   printf(" sigset_t size  = %d\n", (int)sizeof(sigset_t) );
   printf(" siginfo_t size = %d\n", (int)sizeof(info) );
   printf(" stack_t size   = %d\n", (int)sizeof(stack_t) );
   printf(" fpstate64 size = %d\n", (int)sizeof(myfpstate64) );
   printf(" sigcontext size= %d\n", (int)sizeof(mysigcontext64) );
   printf(" gregset_t size = %d\n", (int)sizeof(mygregset) );
   printf(" fpregset_t size= %d\n", (int)sizeof(myfpregset) );
   printf(" ucontext_t size= %d\n", (int)sizeof(context) );
   printf(" mcontext_t size= %d\n", (int)sizeof(mcontext_t) );
   printf(" sigaction size = %d\n", (int)sizeof(act) );
   t1 = (long) (void *) &act;
   t2 = (long) (void *) &(act.sa_handler);
   printf("   offset sa_handler = %ld\n", t2-t1 );
   t2 = (long) (void *) &(act.sa_mask);
   printf("   offset sa_mask    = %ld\n", t2-t1 );
   t2 = (long) (void *) &(act.sa_flags);
   printf("   offset sa_flags   = %ld\n", t2-t1 );
   t2 = (long) (void *) &(act.sa_restorer);
   printf("   offset sa_restorer= %ld\n\n", t2-t1 );

   t1 = (long) (void *) &info;
   t2 = (long) (void *) &(info.si_addr);
   sz1 = t2-t1;
   printf(" offset si_addr = %ld\n", sz1 );
//   printf(" offset si_addr = %d\n", (int)offsetof(siginfo_t, si_addr) );
   printf(" SIG_DFL = %ld\n", (long)SIG_DFL);
   printf(" SIG_IGN = %ld\n", (long)SIG_IGN);

   printf("\nSemaphores, Timers and Threads\n");
   printf(" sem_t size = %d\n", (int)sizeof(sem) );
   printf(" PTHREAD_STACK_MIN   = %d\n", PTHREAD_STACK_MIN );
   printf(" pthread_t size      = %d\n", (int)sizeof(pthread_t) );
   printf(" pthread_attr_t size = %d\n", (int)sizeof(attr) );
   printf(" ITIMER_REAL = %d\n", ITIMER_REAL );
   printf(" ITIMER_VIRTUAL = %d\n", ITIMER_VIRTUAL );
   printf(" ITIMER_PROF = %d\n", ITIMER_PROF );
   printf(" pthread_mutex_t size = %d\n", (int)sizeof(pmutex) );


   printf("\nsa_flags\n");
   printf(" SA_NOCLDSTOP = %d, $%X\n", SA_NOCLDSTOP, SA_NOCLDSTOP);
//   printf(" SA_ONESHOT = %d, $%X\n", SA_ONESHOT, SA_ONESHOT);
   printf(" SA_ONSTACK = %d, $%X\n", SA_ONSTACK, SA_ONSTACK);
   printf(" SA_RESTART = %d, $%X\n", SA_RESTART, SA_RESTART);
   printf(" SA_NODEFER = %d, $%X\n", SA_NODEFER, SA_NODEFER);
   printf(" SA_SIGINFO = %d, $%X\n", SA_SIGINFO, SA_SIGINFO);
   printf(" SIGFPE = %d\n", SIGFPE);

   t1 = (long)( void *) &act;
   t2 = (long)(void *) &(act.sa_flags);
   t3 = (long)(void *) &(act.sa_sigaction);
   t4 = (long)(void *) &(act.sa_mask);
   printf("offset of sa_flags = %ld\n", t2-t1);
   printf("offset of sa_sigaction = %ld\n", t3-t1);
   printf("offset of sa_mask = %ld\n", t4-t1);

   printf("\nSockets and Networking\n");
   printf(" IFNAMSIZ = %d, $%X\n", IFNAMSIZ, IFNAMSIZ);
//   printf(" IFHWADDRLEN = %d, $%X\n", IFHWADDRLEN, IFHWADDRLEN);
   printf(" ifreq size = %d\n", (int)sizeof(myifreq) );
   t1 = (long)( void *) &myifreq;
//   t2 = (int)(void *) &(myifreq.ifr_hwaddr.sa_data);
   t3 = (long)(void *) &(myifreq.ifr_name);
//   printf("offset of MAC address = %d\n", t2-t1);
   printf("offset of MAC name = %ld\n", t3-t1);

   return 0;
}
