\ startup64.fth - Start up code for 64 bit Linux ELF application

((
Copyright (c) 2005, 2007, 2008, 2009, 2011, 2012, 2017, 2020
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone number is:
  011 44 23 8063 1441


To do
=====

Change history
==============
20201006 SFP012 Fixed TOS in CALLBACK-ENTRY for n>6
20200824 SFP011 Fixed CALLBACK-ENTRY
20200608 SFP010 64 bit conversion
20170530 SFP009 Initialised SYSPAD in CALLBACK-ENTRY common code.
20120512 SFP008 Added EXECBASE and CB:
20110914 SFP007 Allowed dlopen() mode to be used.
20091125 SFP006 Recoded CALLBACK-ENTRY.
20090127 SFP005 Swapped position of ESP and EBP for new Extern:
20080326 SFP004 Refactored CALLBACK: and added CALLPROC:
20071217 SFP003 Changed CALLBACK: to have C behaviour for Linux signals.
20071203 MPE002 Added XCALL64 for functions that return doubles.
20050625 SFP001 Converted from Windows startup code
))

decimal

interpreter
: bin-align	\ n --
\ Force alignment to an N byte boundary where N is a
\ power of two. The space stepped over is set to 0.
  chere
  over 1- +  swap negate and		\ -- dp'
  here over over - erase		\ wipe gap
  corg
;
target

$0040:0000 equ _file0			\ memory address of first byte of file
$0080:0000 equ TextSize			\ ask for 8 megabytes for my interpreter

cdata
l: StartAddress				\ first location in cross-compiled code

here _file0 <> [if]
  cr primary section at wrong address" abort
[then]

StartAddress PE-LOADBASE - equ LOADBIAS	\ offset from code base to start of code
LOADBIAS PE-CODERVA <> [if]		\ sanity check
  cr ." LOADBIAS <> PE-CODERVA - whoops!" abort
[then]
StartAddress _file0 <> [if]		\ sanity check
  cr ." StartAddress <> _FILE0 - whoops!" abort
[then]

[undefined] -cell [if]  cell negate equ -cell  [then]
[undefined] -2cells [if]  2 cells negate equ -2cells  [then]

\ **********
\ ELF Header
\ *********
\ The ELF headers and file structure are built by hand.
\ This allows us to take advantage of some ELF facilities
\ not usually used by assembler and C toolchains.

l: ehdr				\ ehdr:   ; ELF file header
  $7f c,
  char E c, char L c, char F c,	\  db      0x7F, "ELF", 1, 1, 1	; e_ident
  2 c,				\ ei_class, 1=32 bit, 2=64 bit objects
  1 c,				\ ei_data, 1=little endian, 2=big endian
  1 c,				\ ei_version, 1=current
  0 c, 0 c, 0 c, 0 c, 0 c, 	\ times   9 db 0
  0 c, 0 c, 0 c, 0 c,
  2 w,				\  dw 2 ; e_type     2=executable, 3=shared lib
  #62 w,			\  dw 3 ; e_machine  3=i32/386, #40=ARM, #62=AMD64
  1 l,				\  dd 1 ; e_version  1=current
l: entryref  0 ,		\  dd entry ; e_entry    absolute entry point
l: PHTref  0 ,			\  dd PHT-$$ ; e_phoff    file offset to PHT, 0=none
  0 ,				\  dd 0 ; e_shoff    file offset to SHT, 0=none
  0 l,				\  dd 0 ; e_flags    0 for i386
l: ehdrsizeref  0 w,		\  dw ehdrsize ; e_ehsize   ELF header size
l: phdrsizeref  0 w,		\  dw phdrsize ; e_phentsize PHT entry size
 3 w,				\  dw 3 ; e_phnum    #entries in PHT
 0 w,				\  dw 0 ; e_shentsize SHT entry size
 0 w,				\  dw 0 ; e_shnum    #entries in SHT
 0 w,				\  dw 0 ; e_shstrndx file offset to string table, 0=none
l: ehdrend

ehdrend ehdr - ehdrsizeref w!

\ ***************
\ Program headers
\ ***************

#64 bin-align	\  align 64      ; for dumps
l: InterpName			\  ; name of the dynamic linker
z", /lib64/ld-linux-x86-64.so.2" \  db      "/lib64/ld-linux-x86-64.so.2",0
here InterpName - equ InterpNameSize

#64 bin-align	\  align 64      ; for dumps

l: PHT			\ Program Header Table (PHT) with three entries
			\ it is legal and useful to include the headers in
			\ memory that is visible to the program.

PHT ehdr - PHTref w!

\ Program interpreter for shared library access
l: PHinterp
  3 l,			\ dd 3 ; p_type     3=PT_INTERP
  4 l,			\ dd 4 ; p_flags  permissions R--, 4=read,2=write,1=exec
  InterpName ehdr - ,	\ dd InterpName-$$ ; p_offset offset into file
  InterpName ,		\ dd InterpName   ; p_vaddr (virtual) run-time address
  InterpName ,		\ dd InterpName   ; p_paddr (physical) run-time address
  InterpNameSize ,	\ dd InterpNameSize ; p_filesz  #bytes to load from file
  InterpNameSize ,	\ dd InterpNameSize ; p_memsz #bytes to occupy in memory
  1 ,			\ dd 0x1             ; p_align    page size
l: PHinterpEnd

PHinterpEnd PHinterp - phdrsizeref w!

\ Main code (.text) and fixed data
l: PHtext
  1 l,			\ dd 1     ; p_type     1=load into memory
  7 l,			\ dd 7 ; p_flags permissions RWX, 4=read,2=write,1=exec
  0 ,			\ dd 0     ; p_offset   file offset to start of segment
  ehdr ,		\ dd $$    ; p_vaddr    virt. address where loaded
  ehdr ,		\ dd $$    ; p_paddr    abs. address where loaded
l: p_fileszref1 0 ,	\ dd filesize   ; p_filesz   how much loaded from file
l: p_memszref1 0 ,	\ dd filesize   ; p_memsz    segment size in memory
  $1000 ,		\ dd 0x1000     ; p_align    alignment required

\ Dynamic info section for shared libraries, PT_DYNAMIC segment
l: PHdynamic
  2 l,			\ dd 2     ; p_type     1=load mem, 2=dynlink info
  6 l,			\ dd 6 ; p_flags permissions RW-, 4=read,2=write,1=exec
l: p_offsetref2 0 ,	\ dd DynOffset ; p_offset   offset into file
l: p_vaddrref2 0 ,	\ dd DynAddr   ; p_vaddr    (virtual) run-time address
l: p_paddrref2 0 ,	\ dd DynAddr   ; p_paddr    (physical) run-time address (irrelevant here)
l: p_fileszref2 0 ,	\ dd DynSize   ; p_filesz   #bytes to load from file
l: p_memszref2 0 ,	\ dd DynSize   ; p_memsz    #bytes to occupy in memory
  8 ,			\ dd 0x08      ; p_align    page size

\ ************
\ String Table
\ ************

#64 bin-align	\  align 64      ; for dumps

l: StringTable
  0 c,			\ db 0         ; first entry always 0
here StringTable - equ idxlibdl
  z", libdl.so.2"	\ db "libdl.so.2",0
here StringTable - equ idxdlopen
  z", dlopen"		\ db "dlopen",0
here StringTable - equ idxdlsym
  z", dlsym"		\ db "dlsym",0
here StringTable - equ idxdlclose
  z", dlclose"		\ db "dlclose",0
here StringTable - equ idxdlerror
  z", dlerror"		\ db "dlerror",0
here StringTable - equ idxlibc
  z", libc.so.6"	\ db "libc.so.6",0
here StringTable - equ StringTableSize

\ ************
\ Symbol Table
\ ************

#64 bin-align	\ align 64      ; for dumps

l: SymbolTable
\ entry 0 - dummy
  0 l,				\ dd 0    ; st_name   index into a string table
  0 c,				\ db 0    ; st_info   binding=1(global), type: 1=data 2=function
  0 c,				\ db 0    ; st_other
  0 w,				\ dw 0    ; st_shndx  0 indicates external symbol
  0 ,				\ dd 0    ; st_value  0 for external symbols
  0 ,				\ dd 0    ; st_size   0=unknown
\ entry 1 - dlopen
  idxdlopen l,			\ dd idxdlopen   ; st_name   index into a string table
  $12 c,			\ db 0x12 ; st_info   binding=1(global), type: 1=data 2=function
  0 c,				\ db 0    ; st_other
  0 w,				\ dw 0    ; st_shndx  0 indicates external symbol
  0 ,				\ dd 0    ; st_value  0 for external symbols
  0 ,				\ dd 0    ; st_size   0=unknown
\ entry 2 - dlsym
  idxdlsym l,			\ dd idxdlsym    ; st_name   index into a string table
  $12 c,			\ db 0x12 ; st_info   binding=1(global), type: 1=data 2=function
  0 c,				\ db 0    ; st_other
  0 w,				\ dw 0    ; st_shndx  0 indicates external symbol
  0 ,				\ dd 0    ; st_value  0 for external symbols
  0 ,				\ dd 0    ; st_size   0=unknown
\ entry 3 - dlclose
  idxdlclose l,			\ dd idxdlclose  ; st_name   index into a string table
  $12 c,			\ db 0x12 ; st_info   binding=1(global), type: 1=data 2=function
  0 c,				\ db 0    ; st_other
  0 w,				\ dw 0    ; st_shndx  0 indicates external symbol
  0 ,				\ dd 0    ; st_value  0 for external symbols
  0 ,				\ dd 0    ; st_size   0=unknown
\ entry 4 - dlerror
  idxdlerror l,			\ dd idxdlerror  ; st_name   index into a string table
  $12 c,			\ db 0x12 ; st_info   binding=1(global), type: 1=data 2=function
  0 c,				\ db 0    ; st_other
  0 w,				\ dw 0    ; st_shndx  0 indicates external symbol
  0 ,				\ dd 0    ; st_value  0 for external symbols
  0 ,				\ dd 0    ; st_size   0=unknown


\ **********
\ Hash Table
\ **********

l: HashTable
  1 l, 				\ dd 1    ; number of buckets, modulo 1
  5 l,				\ dd 5    ; number of chains = 0..4 (dummy plus four symbols)
l: buckets
  1 l,				\ dd 1    ; y=0, try SymIdx=1
l: chains
  0 l,				\ dd 0    ; 0=dummy
  2 l,				\ dd 2    ; next
  3 l,				\ dd 3    ; next
  4 l,				\ dd 4    ; next
  0 l,				\ dd 0    ; next=end of list


\ *******************
\ Global Offset Table
\ *******************

#64 bin-align	\  align 64      ; for dumps

l: GOT
l: dynamicref 0 ,		\ dd _DYNAMIC    ; points to start of _DYNAMIC structure
l: dlpriv1ref  0 ,		\ dd 0     	; fixed up by dynamic linker
l: dlpriv2ref  0 , 		\ dd 0     	; fixed up by dynamic linker
l: dlopenref 0 ,		\ dd dlopen+6   ; GOT+24  GOT[3]
l: dlsymref 0 ,			\ dd dlsym+6    ; GOT+32  GOT[4]
l: dlcloseref 0 ,		\ dd dlclose+6  ; GOT+40  GOT[5]
l: dlerrorref 0 ,		\ dd dlerror+6  ; GOT+48  GOT[6]


\ *********************
\ Program Linkage Table
\ *********************

#64 bin-align	\  align 64      ; for dumps

proc PLT0
  push    qword [] GOT 8 +       \ GOT[1] contents supplied by dynamic linker
  jmp     [] GOT 16 +            \ GOT[2] entry to dynamic linker
END-CODE
#16 bin-align	\  align 16
proc dlopen
  jmp     [] GOT #24 +           \ through GOT[3], initially points to next ins.
  push    qword # 0x00           \ index to relocation table below
  jmp     PLT0
END-CODE
#16 bin-align	\  align 16
proc dlsym
  jmp     [] GOT #32 +           \ through GOT[4], initially points to next ins.
  push    qword # 0x01           \ index to relocation table blow
  jmp     PLT0
END-CODE
#16 bin-align	\  align 16
proc dlclose
  jmp     [] GOT #40 +           \ through GOT[5], initially points to next ins.
  push    qword # 0x2            \ index to relocation table blow
  jmp     PLT0
END-CODE
#16 bin-align	\  align 16
proc dlerror
  jmp     [] GOT #48 +           \ through GOT[6], initially points to next ins.
  push    qword # 0x3            \ index to relocation table blow
  jmp     PLT0
END-CODE
#16 bin-align	\  align 16
l: PLTx

dlopen 6 +   dlopenref  !
dlsym 6 +    dlsymref   !
dlclose 6 +  dlcloseref !
dlerror 6 +  dlerrorref !

dlopen 6  + equ original_dlopen
dlsym 6   + equ original_dlsym
dlclose 6 + equ original_dlclose
dlerror 6 + equ original_dlerror

\ ****************
\ Relocation Table
\ ****************

#64 bin-align	\  align 64      ; for dumps

l: RelocTable
\ for dlopen
  GOT #24 + ,			\ dd GOT+24 ; first entry is GOT[3], GOT[0..2] reserved
  $0007 l,  1 l,		\ dd 0x01:0007 ; Symbol Index = 1<<32 + R_X86_64_JMP_SLOT
  0 ,
\ for dlsym
  GOT #32 + ,			\ dd GOT+32 ; second entry is GOT[4]
  $0007 l,  2 l,		\ dd 0x0207 ; Symbol Index = 2<<8 + R_X86_64_JMP_SLOT
  0 ,
\ for dlclose
  GOT #40 + ,			\ dd GOT+40 ; third entry is GOT[5]
  $0007 l,  3 l,		\ dd 0x0307 ; Symbol Index = 3<<8 + R_X86_64_JMP_SLOT
  0 ,
\ for dlerror
  GOT #48 + ,			\ dd GOT+48 ; fourth entry is GOT[6]
  $0007 l,  4 l,		\ dd 0x0407 ;  Symbol Index =4<<8 + R_X86_64_JMP_SLOT
  0 ,
here RelocTable - equ RelocTableSize


\ ***************
\ Dynamic segment
\ ***************

#256 bin-align	\  align 256

l: DynAddr                     \ base address of segment
l: _DYNAMIC
  1 , idxlibdl ,		\ dd 1, idxlibdl     ; DT_NEEDED, string table index ; library name
  1 , idxlibc ,			\ dd 1, idxlibc      ; DT_NEEDED, string table index ; library name
  4 , HashTable ,		\ dd 4, HashTable    ; DT_HASH, address of hash table
  #10 , StringTableSize ,	\ dd 10, StringTableSize  ; DT_STRSZ, string table size in bytes
  5 , StringTable ,		\ dd 5, StringTable  ; DT_STRTAB, address of string table
  6 , SymbolTable ,		\ dd 6, SymbolTable  ; DT_SYMTAB, address of symbol table
  #11 , #16 ,			\ dd 11, 16          ; DT_SYMENT, symbol table entry size in bytes
  3 , GOT ,			\ dd 3, GOT          ; DT_PLTGOT, base address of Global Offset Table
  2 , ReloctableSize ,		\ dd 2, RelocTableSize  ; DT_PLTRELSZ, size of relocation table in bytes
  #23 , RelocTable ,		\ dd 23, RelocTable  ; DT_JMPREL, base address of relocation table
  #20 , 7 ,			\ dd 20, 7          ; DT_PLTREL, DT_RELA, indicate type of PLT relocation
\ dd      21, 0                ; DT_DEBUG, none, no debug information
  0 , 0 ,			\ dd 0, 0      ; DT_NULL, ignored - end of list

#256 bin-align	\  align 256

_DYNAMIC dynamicref !		\ GOT[0] entry

DynAddr ehdr - equ DynOffset	\ offset of segment in file
here DynAddr - equ DynSize	\ length of segment

DynOffset p_offsetref2 !
DynAddr p_vaddrref2 !
DynAddr p_paddrref2 !

DynSize p_fileszref2 !
DynSize p_memszref2 !

\ Some buffer space for the command line
\ This is necessary because the Forth interpreter expects its
\ command line as one long string with all arguments in it;
\ This is necessary because some arguments only make sense
\ in conjunction with others, and it would complicate the rest of the code
\ dramatically to try to iterate over argv[] from within the interpreter
l: CmdLineBuffer

\ *********************************************
\ The ELF file header transfers control to here
\ *********************************************
\ ELF header is at 0x40:0000
$2000 equ ehdr_size		\ let header occupy 8kb
ehdr ehdr_size + org		\ This file then starts at 0x0804A000

((	\ Only required for testing
[undefined] tib-len [if]
  $0800 equ tib-len
[then]

[undefined] rp-size  [if]
  $10000 equ rp-size
[then]

[undefined] fs-size  [if]
  $1000 equ fs-size
[then]

[undefined] sp-size  [if]
  $10000 equ sp-size
[then]
))

prefix

\ This entry point code layout up to label GETRA cannot be changed
\ without first checking or modifying TOOLS\SRC\MAKE-EXE.C and
\ VFXSUPP.C to match.
asmcode
l: entry	\ must be first
	jmp 	entry2
end-code
entry entryref !			\ Set entry point in ELF header

StartAddress PE-LOADBASE - equ LOADBIAS	\ offset from code base to start of code

LOADBIAS PE-CODERVA <> [if]
  cr ." LOADBIAS <> PE-CODERVA - whoops!" abort
[then]

\ storage. N.B. Linux provides write permission in code segment
\ This first set of data is common to all VFX Forth hosted
\ implementations
	align
l: loadbase	\ holds base address of system		$0804:A008
	PE-LOADBASE ,
l: codesize	\ holds code size set by load time	$0804:A00C
	TextSize PE-CODERVA - dup , p_memszref1 !
l: newcodesize	\ holds requested code size by user	$0804:A010
	TextSize PE-CODERVA - ,
l: numrelocs	\ number of relocations, 0=none		$0804:A014
	0 ,
l: relocbase	\ OFFSET from ENTRY to reloc table	$0804:A018
	0 ,	\ 0 = no table
l: pDllExports	\ OFFSET from ENTRY to DLL chain start	$0804:A01C
	0 ,	\ 0 = no table

\ The second set is operating system specific to Linux implementations
l: StackSize	\ requested stack size			$0804:A020
	1 Mb ,
l: StackTop	\ top of allocated stack+1		$0804:A024
	0 ,
l: StackBase	\ base of Forth stack			$0804:A028
	0 ,
l: EntryRSP	\ RSP given by the loader		$0804:A02C
	0 ,	\ was used to find argc, argv, envc ....
l: posArgc	\ was in RDI, now in 0 [RSP]
	0 ,
l: posArgv	\ was in RSI, now in 8 [RSP]
	0 ,
l: posEnv	\ was in RDX, now needs calculating.
	0 ,

proc GetRA	\ return R12=return address
	mov	r12, 0 [rsp]
	ret
end-code

proc RelocSystem	\ returns RBX=runtime PE-LOADBASE, R14=len
\ perform the relocation, leaving R12=runtime PE-LOADBASE
\ N.B. this code must be position independent and not itself subject to relocation.
	call	GetRA				\ ESI=RelocSystem+5
	sub     r12, # 5			\ ESI=RelocSystem
	sub	r12, # RelocSystem StartAddress -	\ ESI=StartAddress
	sub	r12, # LOADBIAS			\ ESI=LoadBase

	mov	rcx, numrelocs PE-LOADBASE - [r12]	\ ECX := number of relocations
	mov	rbx, relocbase PE-LOADBASE - [r12]	\ EBX := offset from StartAddress to relocation table
	add	rbx, # LOADBIAS			\ rebias from loadbase
	add	rbx, r12			\ add loadbase, make absolute

	mov	rax, r12			\ new load address

	sub	rax, loadbase PE-LOADBASE - [r12]	\ - old = relocation
\ R12=runtime loadbase
\ RBX=start of relocation table
\ RCX=#relocations
\ RAX=relocation
\ N.B
\   RDI=argc
\   RSI=argv[]
\   RDX=env[]
l: relocloop
	jecxz	relocx
	mov	r13, 0 [rbx]			\ get offset from relocation table
	add	rbx, # 8			\ update table address
	add	LOADBIAS [r12] [r13], rax	\ relocate entry
	dec	rcx				\ count relocations down
	jmp	relocloop
l: relocx
	mov	rax, # 0
	mov	numrelocs PE-LOADBASE - [r12], rax	\ zap number of relocations
	mov	relocbase PE-LOADBASE - [r12], rax	\ zap offset to reloction table

\ set up the system after relocation, R12=runtime LOADBASE
\ *** the line below should reference p_memszref1
        mov     rax, [] p_memszref1		\ get CODE size CELL
	mov	[] codesize , rax		\ save dictionary size
	mov	[] newcodesize , rax		\ save dictionary size
	mov	rbx, r12			\ set return values
	mov	r14, rax
\ touch every 4 k from the base
	mov	rcx, rax			\ length
	shr	rcx, # #12			\ # 4k blocks
	mov	r15, rcx
	test	rax, # $FFF			\ round
	nz, if,  inc  rcx  endif,
l: touchloop
	mov	rax, 0 [r12]			\ read one cell per 4k block
	add	r12, # $01000
	sub	rcx, # 1
	jne	touchloop
	ret
end-code

\ hex
align
l: cld1		\ address of word to run, will be patched later
  0 ,
l: serial_no	\ serial number
  0 ,
l: copyright	\ DO NOT CHANGE THIS STRING - See STAMPEXE
  ", &(c) MicroProcessor Engineering Limited"

\ TIB - terminal input buffer
align l: init-tib
init-tib equ init-tib		\ make cross compiler happy
  tib-len allot


\ ****************************************
\ table of initial values of the user area
\ ****************************************

l: user-reset
  0 ,                   \ s0
  0 ,                   \ r0
  init-tib ,		\ 'TIB
here user-reset - equ user-reset-len
l: init-fence
  0 ,			\ FENCE, patched by cross compiler
l: init-dp
  0 ,			\ DP, patched by cross compiler
l: init-voc-link
  0 ,			\ VOC-LINK, patched by cross compiler
l: SavedFS0
  0 ,


\ ***********************************************************************
\ COLD start entry code. The stack value is saved for use by warm starts.
\ The stacks are allocated as frames on the Windows stack.
\ ***********************************************************************
\ STACKS GROW DOWN
\   .. USER-SIZE bytes ..
\ ESI user area
\ TIB terminal input buffer in callbacks
\   /FSP-guard
\ FS0/FSP
\   .. FP-SIZE bytes ..
\ EBP/S0 data stack
\   ... min SP-SIZE
\ ESP/R0 return stack
\   .. RP-SIZE bytes ..

$040 equ /RSPguard	\ 16 cell guard space at top of return stack
/RSPguard negate equ /-RSPguard
$040 equ /PSPguard	\ 16 cell guard space at top of data stack
/PSPguard negate equ /-PSPguard
$040 equ /FSPguard	\ 4*16 byte guard space at top of float stack
/FSPguard negate equ /-FSPguard

\ The required space used on the stack is:
RP-SIZE USER-SIZE + TIB-LEN + FS-SIZE + SP-SIZE +
#4095 + #12 >>n		\ number of 4k pages
  equ #T0pages

align
proc entry2
\ N.B
\   RDI=argc
\   RSI=argv[]
\   RDX=env[]
	call	RelocSystem			\ relocate system as required
\ RBX=runtime PE-LOADBASE, R14=len
	\ These need to occur after relocation
	mov	[] EntryRSP , rsp		\ preserve loader's stack
	mov	rax, 0 [rsp]			\ argc
	mov	[] posArgc , rax		\ save argc
	lea	rax, cell [rsp]			\ base address of argv[]
	mov	[] posArgv , rax		\ argv[]
	mov	[] posEnv , rdx			\ env[]

	\ Expand system for old stack definition - may be obsolete
	mov	rdi , # 0			\ NULL pointer -> curbrk
	mov	rax , # #12			\ Syscall 12 is brk
	syscall					\ Find end of allocated mem
	add	rax, [] StackSize		\ Add on the amount of stack we want
	mov	rdi, rax			\ required size
	mov	rax , # #12			\ Syscall 12 is brk
	syscall					\ extend segment
	\ Move our stack into newly-created area
\	mov	rsp, rax

l: ecld
((
\ Touch each 4k of stack space.
\ May be NECESSARY if not all stack space is committed.
  mov   esi, esp	\ ESI = top of stack
  mov   ecx, # #T0pages	\ ECX = number of pages
l: ecldTouchup
  mov   edi, #-4092 [esi]
\  mov   #-4092 [esi], eax
  sub   esi, # #4096
  dec   ecx
  jne   ecldTouchup
))
((
  mov   rbx, [] EntryRSP
  call  showhex

  mov	rbx, rsp
  call	showhex

  mov	rbx, [] cld1
  call	showhex

  mov	ax, 10
  call	primemit
  jmp	$
))

  mov	rbx, rsp			\ current stack pointer

\ User area
  sub   rbx, # USER-SIZE		\ point at base of USER area
  mov	rsi, rbx
  mov   cell [rsi], rsp			\ set R0
\ Float stack
  lea   rax, /-FSPguard [rbx]		\ float stack guard space
  mov   $14 cells [rsi], rax		\ FS0
  mov   $15 cells [rsi], rax		\ FSP (user var)
  mov   r13, rax			\ FSP (register for SSE/HFP)
  sub   rbx, # FS-SIZE
\ Data stack
  lea	rbp, /-PSPguard [rbx]		\ reserve guard space
  lea   rax, -cell [rbp]		\ set S0
  mov   0 [rsi], rax
  mov   rcx, # user-reset		\ point at initial user-var values
  mov   0 [rcx], rbp			\ INIT-S0
  sub	rbx, # SP-SIZE
\ Return stack
  mov	rax, rbx
  sub   rax, # /RSPguard		\ crash protection
  mov	rsp, rax
  mov   cell [rcx], rsp			\ INIT-R0

l: ewrm
  mov   rcx, # user-reset		\ point at initial user-var values
  mov   rbp, 0 [rcx]			\ INIT-S0
  add   rbp, # cell
  mov   rsp, cell [rcx]			\ INIT-R0
  xor	rbx, rbx			\ TOS=0

  mov	rax, [] cld1			\ initial word to run
  cld					\ force direction flag to increment
  call  rax				\ run it
  jmp   ewrm				\ whoops! - should never happen
end-code


\ ***********
\ Debug tools
\ ***********

#60 equ SYS_exit
1 equ SYS_write
\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
\ RAX=caddr, RBX=len, RCX=handle
proc primtype
  push	  rsi
  push	  rdi
  push	  rbp
  mov     r12, rsp		\ save and align return stack
  and	  rsp, $-010		\ on 16 byte boundary
  mov     rdi, rcx		\ STDOUT
  mov     rsi, rax              \ address
  mov     rdx, rbx		\ length
  mov     rax, SYS_write	\ syscall number
  syscall
  mov     rsp, r12		\ restore return stack
  pop     rbp
  pop	  rdi
  pop	  rsi
  ret
end-code

code sysType	\ addr len --
   mov    rax, 0 [rbp]		\ caddr
				\ RBX=len
   mov    rcx, 1		\ STDOUT
   call   primtype
   mov    rbx, cell [rbp]
   lea    rbp, 2 cells [rbp]
   next,
end-code

\ emit single character, RAX=char
proc primemit
   push    rax
   mov     rax, rsp		\ address of character
   mov     rbx, # 1		\ 1 char ln
   mov     rcx, # 1             \ STDOUT
   call    primtype
   pop     rax
   ret
 end-code

code sysEmit	\ char --
   mov     rax, rbx
   call    primemit
   mov    rbx, 0 [rbp]
   lea    rbp, cell [rbp]
   next,
end-code

: sysCR		\ --
  #10 sysEmit
;

\ Display RBX as 8 digit hex number in low half
proc showhex8
   push    rax
   push    rbx

   mov     rcx, # 8
l: shx1
   push    rcx
   push    rbx
   mov     rax, rbx
   shr     rax, # #28
   and     rax, # $0F
   cmp     rax, # 9
   jle     shx2
   add     rax, # 7
l: shx2
   add     rax, # $30
   call    primemit
   pop     rbx
   pop     rcx
   shl     rbx, # 4
   sub     rcx, # 1
   jne     shx1

\   mov     rax, # $20
\   call    primemit

   pop     rbx
   pop     rax
   ret
end-code

code sys.dword	( l -- )
   call showhex8
   mov     rbx, 0 [rbp]
   lea     rbp, cell [rbp]
   ret
end-code

\ Display RBX as 16 digit hex number
proc showhex
   push    rbx

   shr     rbx, 32
   call    showhex8
   mov     rax, ':'
   call    primemit

   pop     rbx
   call    showhex8
   mov     rax, $020
   call    primemit

   ret
end-code

code sys.xword	\ x --
   call    showHex
   mov     rbx, 0 [rbp]
   lea     rbp, cell [rbp]
   next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
\ Exit code in RAX
code primexit
   mov     r12, rsp		\ save and align return stack
   and     rsp, $-010		\ on 16 byte boundary

   mov     rdi, rax		\ exit code
   mov     rax, SYS_exit	\ call number
   syscall

   mov     rsp, r12		\ restore return stack
   ret
end-code

: .sysState	\ --
  syscr
  syscr  up@ sys.xword  sp@ sys.xword  rp@ sys.xword
  syscr
;


\ -----------------------------------------------------------------------------

decimal
external
#64 kb bin-align

here constant FirstName		\ -- addr ; marks boundary for ip>nfa etc
\ Must be the first named word in the system.
\ Must be above 64k for VFX code generator's code-len field to be good.

SECTION-START constant ExecBase	\ -- addr ; where file loads

((
\ from /usr/include/bits/dlfcn.h
/* The MODE argument to `dlopen' contains one of the following: */
#define RTLD_LAZY	0x00001	/* Lazy function call binding.  */
#define RTLD_NOW	0x00002	/* Immediate function call binding.  */
#define	RTLD_BINDING_MASK   0x3	/* Mask of binding time value.  */
#define RTLD_NOLOAD	0x00004	/* Do not load the object.  */
#define RTLD_DEEPBIND	0x00008	/* Use deep binding.  */

/* If the following bit is set in the MODE argument to `dlopen',
   the symbols of the loaded object and its dependencies are made
   visible as if the object were linked directly into the program.  */
#define RTLD_GLOBAL	0x00100

/* Unix98 demands the following flag which is the inverse to RTLD_GLOBAL.
   The implementation does this by default and so we can define the
   value to zero.  */
#define RTLD_LOCAL	0

/* Do not delete object when closed.  */
#define RTLD_NODELETE	0x01000
))

variable LoadLibMask	\ -- addr
\ Holds mask set for special cases when loading library
  RTLD_LAZY LoadLibMask !

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code LoadLibrary      \ z-addr -- handle|0
\ save Forth registers
  	push	rdi
	push	rsi
	mov     r12, rsp		\ save RSP
\ align return stack frame. Call has two parameters
	and	rsp, # -16		\ drop to 16 byte boundary
\ set parms, call, get value
	mov	rdi, rbx		\ z-addr
	mov	rsi, [] LoadLibMask	\ RTLD_LAZY
	call	dlopen			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rsi
	pop	rdi
	next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code FreeLibrary      \ addr -- status
\ save Forth registers
  	push	rdi
	push	rsi
	mov     r12, rsp		\ save RSP
\ align return stack frame. Call has one parameter
	and	rsp, # -16		\ drop to 16 byte boundary
\ set parms, call, get value
	mov	rdi, rbx		\ addr
	call	dlclose			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rsi
	pop	rdi
	next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code GetProcAddress   \ handle z-addr -- abs32|0
\ save Forth registers
  	push	rdi
	push	rsi
	mov     r12, rsp		\ save RSP
\ align return stack frame. Call has two parameters
	and	rsp, # -16		\ drop to 16 byte boundary
\ set parms, call, get value
        mov	rdi, 0 [rbp]		\ handle
	mov	rsi, rbx		\ z-addr
	call	dlsym			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
	lea     rbp, cell [rbp]		\ remove one item from data stack
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rsi
	pop	rdi
	next,
end-code

code RelocateSystem	\ --
\ Relocate the Forth system - can be called by shared library
\ load or initialisation routines.
	push	rsi
	push	rdi
	push	rbx
	push	r14
	call	RelocSystem
	pop	r14
	pop	rbx
	pop	rdi
	pop	rsi
	next,
end-code


\ *****************
\ NXCALL operations
\ *****************
\ Both Linux and MacOS use the SystemV ABI. Hence most of this code
\ is the same for both Forths. See also VFXBase/ExternSysVx64.fth

decimal

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code nxcall0      \ addr -- res
\ Call the external function at addr with 0 arguments.
\ preserve Forth state
        push    rsi                     \ ESI = UP
        push    rdi			\ EDI = LP
        mov	r12, rsp		\ save RSP
\ align return stack frame.
	and	rsp, # #-16		\ drop to 16 byte boundary
\ set parms, call, get value
	call	rbx			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rdi
	pop	rsi
	next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code nxcall1      \ p1 addr -- res
\ Call the external function at addr with 1 argument.
\ preserve Forth state
        push    rsi                     \ ESI = UP
        push    rdi			\ EDI = LP
        mov	r12, rsp		\ save RSP
\ align return stack frame.
	and	rsp, # -16		\ drop to 16 byte boundary
\ set parms, call, get value
        mov	rdi, 0 [rbp]		\ param1
	call	rbx			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
	lea     rbp, cell [rbp]		\ remove one item from data stack
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rdi
	pop	rsi
	next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code nxcall2      \ p1 p2 addr -- res
\ Call the external function at addr with 2 arguments.
\ preserve Forth state
        push    rsi                     \ ESI = UP
        push    rdi			\ EDI = LP
        mov	r12, rsp		\ save RSP
\ align return stack frame.
	and	rsp, # -16		\ drop to 16 byte boundary
\ set parms, call, get value
        mov	rdi, cell [rbp]		\ param1
        mov     rsi, 0 [rbp]		\ param2
	call	rbx			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
	lea     rbp, 2 cells [rbp]	\ two items from data stack
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rdi
	pop	rsi
	next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code nxcall3     \ p1 p2 p3 addr -- res
\ Call the external function at addr with 3 arguments.
\ preserve Forth state
        push    rsi                     \ ESI = UP
        push    rdi			\ EDI = LP
        mov	r12, rsp		\ save RSP
\ align return stack frame.
	and	rsp, # -16		\ drop to 16 byte boundary
\ set parms, call, get value
	mov	rdi, 2 cells [rbp]	\ p1
        mov	rsi, cell [rbp]		\ p2
        mov     rdx, 0 [rbp]		\ p3
	call	rbx			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
	lea     rbp, 3 cells [rbp]	\ three items from data stack
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rdi
	pop	rsi
	next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code nxcall4	\ p1 p2 p3 p4 addr -- res
\ Call the external function at addr with 4 arguments.
\ preserve Forth state
        push    rsi                     \ ESI = UP
        push    rdi			\ EDI = LP
        mov	r12, rsp		\ save RSP
\ align return stack frame.
	and	rsp, # -16		\ drop to 16 byte boundary
\ set parms, call, get value
	mov	rdi, 3 cells [rbp]	\ p1
	mov	rsi, 2 cells [rbp]	\ p2
        mov	rdx, cell [rbp]		\ p3
        mov     rcx, 0 [rbp]		\ p4
	call	rbx			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
	lea     rbp, 4 cells [rbp]	\ four items from data stack
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rdi
	pop	rsi
	next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code nxcall5      \ p1 p2 p3 p4 p5 addr -- res
\ Call the external function at addr with 5 arguments.
\ preserve Forth state
        push    rsi                     \ ESI = UP
        push    rdi			\ EDI = LP
        mov	r12, rsp		\ save RSP
\ align return stack frame.
	and	rsp, # -16		\ drop to 16 byte boundary
\ set parms, call, get value
        mov	rdi, 4 cells [rbp]	\ p1
        mov	rsi, 3 cells [rbp]	\ p2
        mov	rdx, 2 cells [rbp]	\ p3
        mov	rcx, cell [rbp]		\ p4
        mov     r8, 0 [rbp]		\ p5
	call	rbx			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
	lea     rbp, 5 cells [rbp]	\ five items from data stack
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rdi
	pop	rsi
	next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code nxcall6     \ p1 p2 p3 p4 p5 p6 addr -- res
\ Call the external function at addr with 6 arguments.
\ preserve Forth state
        push    rsi                     \ ESI = UP
        push    rdi			\ EDI = LP
        mov	r12, rsp		\ save RSP
\ align return stack frame.
	and	rsp, # -16		\ drop to 16 byte boundary
\ set parms, call, get value
        mov	rdi, 5 cells [rbp]	\ p1
        mov	rsi, 4 cells [rbp]	\ p2
        mov	rdx, 3 cells [rbp]	\ p3
        mov	rcx, 2 cells [rbp]	\ p4
        mov	r8, 1 cells [rbp]	\ p5
        mov     r9, 0 cells [rbp]	\ p6
	call	rbx			\ make call
	mov     rbx, rax		\ return value in EAX, move to TOS
	lea     rbp, 6 cells [rbp]	\ six items from data stack
\ restore regs
	mov	rsp, r12		\ restore return stack
	pop	rdi
	pop	rsi
	next,
end-code

\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
code varNcall      \ x1..xn n addr -- rc
\ Call the external function at addr with n arguments, where x1
\ will be the top of the calling stack, and n is greater than 6.
\ preserve Forth state
        push    rsi                     \ ESI = UP
        push    rdi			\ EDI = LP
        mov	r12, rsp		\ preserve RSP

\ make calling frame
        mov     rcx, 0 [rbp]		\ n=#args
	shl	rcx, # 3		\ cells to bytes
        mov     0 [rbp], rcx		\ n=#bytes
\ n cells [rbp] x1, first argument
\ cell [rbp]	xn, last argument
\ 0 [rbp]	#bytes of args
\ RBX		call address
\ extend RSP to hold parameters
	sub	rcx, 6 cells		\ RCX > 6, first six in registers
	sub	rsp, rcx		\ make call frame for params 7..n
	and	rsp, # -16		\ align stack
\ Copy first six arguments into registers
	mov	r10, 0 [rbp]		\ number of bytes of args
	mov	rdi, 0 [rbp] [r10]
	sub	r10, cell
	mov	rsi, 0 [rbp] [r10]
	sub	r10, cell
	mov	rdx, 0 [rbp] [r10]
	sub	r10, cell
	mov	rcx, 0 [rbp] [r10]
	sub	r10, cell
	mov	r8, 0 [rbp] [r10]
	sub	r10, cell
	mov	r9, 0 [rbp] [r10]
	sub	r10, cell
\ Copy args 7..n to deep stack
	mov	r11, 0
        begin,
          mov	  rax, 0 [rbp] [r10]
          mov	  0 [rsp] [r11] , rax
          add	  r11, cell
          sub	  r10, cell
        z, until,
\ call external function and restore stacks
        cld
        call	rbx   			\ call function
        cld
	mov	rbx, rax		\ return value
\ clean arguments from RBP data stack
	mov	rax, 0 [rbp]		\ number of bytes of arguments
	lea	rbp, cell [rbp] [rax]	\ remove arguments and n
\ restore Forth
	mov	rsp, r12		\ restore RSP
	pop	rdi
	pop	rsi
	next,
end-code


\ *****************************
\ Breakpoint handler primitives
\ *****************************

code breakpoint
        int     3
        next,
end-code

: breakpoint    \ --
  $48 c, $8B c, $C0 c,   $CC c,		\ lay MOV RAX, RAX ; INT 3
; immediate

cr ." This redefinition is OK!"

code int3	\ --
  mov  rax, rax				\ isolate from INT 3 padding bytes
  int  3
  next,
end-code

code reset-fpu
        finit
        next,
end-code


\ ***********************
\ Command line primitives
\ ***********************

EntryRSP constant EntryRSP	\ -- addr
\ The address holds the value of ESP on entry. It was used for
\ argc and argv extraction on 32 bit systems. It is also used
\ for this on 64 bit x64 Linux.

CmdLineBuffer constant CmdLine	\ -- addr
\ Returns the address of a buffer in which the command line is
\ recreated from the ARGV list.

: argc	\ -- u
\ The number of defined arguments.
  posArgc @
;

: argv	\ -- addr
\ The base address of the argv array.
  posArgv @
;

: argv[                 \ n -- pointer|0
\ Given an index of 0..argc-1 return a pointer to the command
\ line token's zero-terminated string. *\fo{0 argv[} returns the
\ executable's name. If the argument does not exist, the
\ pointer is zero.
  dup argc u>=
  if  drop 0 exit  endif
  cells posArgv @ + @
;

: CommandLine	\ -- c-addr len
\ Return the system command line. This is recreated from the
\ data passed to VFX Forth and may not be exact.
  CmdLine zcount
;


\ ===========
\ *! callback
\ *T Callback functions
\ ===========
\ *P The CALLBACK mechanism provides the facility to wrap Forth
\ ** definitions in code which is callable by Linux. The Forth
\ ** stacks and data areas are created as frames on the calling
\ ** stack.


\ ****************************
\ *S Simple CALLBACK functions
\ ****************************

external
variable ip-default	\ -- addr
\ *G Holds the default value of *\fo{IP-HANDLE} that is set for
\ ** each CALLBACK entry.

variable op-default	\ -- addr
\ *G Holds the default value of *\fo{OP-HANDLE} that is set for
\ ** each CALLBACK entry.


\ SFP004...
\ *******************************************
\ Callback stack construction and destruction
\ *******************************************
\ Callbacks are preceded by the number of outputs and inputs
\  8   |  8  |  5		   | ...
\ #out | #in | call callback-entry | ... | call callback-exit | ret # #in*8 | ret for disassembler
\
\ STACKS GROW DOWN
\   .. USER-SIZE bytes ..
\ ESI user area
\ TIB terminal input buffer in callbacks
\   /FSP-guard
\ FS0/FSP
\   .. FP-SIZE bytes ..
\ EBP/S0 data stack
\   ... min SP-SIZE
\ ESP/R0 return stack
\   .. RP-SIZE bytes assumed
\ ESP is lowest pointer.

\ The required space used on the stack after the entry frame and
\ the offsets are here

USER-SIZE TIB-LEN + FS-SIZE + SP-SIZE + equ #CBdropped

TIB-LEN FS-SIZE + SP-SIZE + equ #rsp_USER
FS-SIZE SP-SIZE + equ #rsp_TIB
FS-SIZE SP-SIZE + equ #rsp_FSPtop
SP-SIZE equ #rsp_PSPtop


#CBdropped #4095 + #12 >>n equ #CBpages	\ number of 4k pages
#CBdropped #1024 + equ /AverageCB	\ guess memory used by callback
2 Mb  /AverageCB /  equ #MaxNestings	\ number of nestings


8 cells equ #CBsaved		\ number of items (inc CBRA) below parameters

8 cells equ #CBF_pars
7 cells equ #CBF_SYSRET
6 cells equ #CBF_RA

\ ============= for #in <= 6 all params in registers
\ preserve callee save registers

5 cells equ #CBF_RBX
4 cells equ #CBF_RBP

3 cells equ #CBF_R12
2 cells equ #CBF_R13
1 cells equ #CBF_R14
      0 equ #CBF_R15

prefix
code callback-entry
\    mov rax, rax
\    int 3
\ Preserve registers and required information
	                        \ Parameters 1..n
\ --- Start of Forth callback frame ---
  \ CALLBACK RETURN ADDRESS
  \ CALLBACK-ENTRY RETURN ADDRESS - MUST be left for CALLBACK-EXIT
\ ============= for #in <= 6 all params in registers
\ preserve callee save registers
	push	rbx
	push	rbp
	push	r12
	push	r13
	push	r14
	push	r15
\ --- End of Forth callback frame ---

\ free up RDI and RSI for use by Forth, they are the first two args
	mov	r14, rdi		\ 1st param
	mov	r15, rsi		\ second param

((
\ Normally commented out
\ Touch each 4k of stack space.
\ NECESSARY if not all stack space is committed.
  mov   esi, esp		\ ESI = top of stack before drop
  mov   ecx, # #CBpages		\ ECX = number of pages
l: touchup
  mov   edi, #-4092 [esi]	\ read
\  mov   #-4092 [esi], eax	\ write
  sub   esi, # #4096
  dec   ecx
  jne   touchup
))

	lea	rsp, #CBdropped negate [rsp]	\ drop stack
\ User area
  	lea	rsi, #rsp_USER [rsp]
\ TIB
  	lea	rdi, #rsp_TIB [rsp]		\ EDI = TIB
	mov	$0B cells [rsi], rdi		\ set 'TIB
\ Float stack, prevent crash into TIB
	lea	rax, #rsp_FSPtop /FSPguard - [rsp]  \ float stack guard space
	mov	$14 cells [rsi], rax		\ FS0
	mov	$15 cells [rsi], rax		\ FSP (user var)
	mov	r13, rax			\ FSP (register)
\ Data stack, below Float Stack
	lea	rbp, #rsp_PSPtop /PSPguard - [rsp] \ data stack guard space
	mov	$00 cells [rsi], rbp		\ set S0
\ Return stack, no protection as below Parameter/Data Stack
	mov	$01 cells [rsi], rsp		\ set R0

\ Set USER variables
	mov	rax, # $0A
	mov	$06 cells [rsi], rax		\ BASE is decimal
	xor	rax, rax
	mov	$18 cells [rsi], rax		\ ThreadExit?
	mov	$19 cells [rsi], rax		\ ThreadTCB
	mov	$1A cells [rsi], rax		\ ThreadSync

	lea	rax, USER-SIZE /ExceptStack - /Syspad - [rsi]		\ Syspadstart ; SFP007
	mov	$1F cells [rsi], rax		\ ^SysPad

	lea	rax, USER-SIZE 4 cells - [rsi]	\ top of exception stack = UP+$0FF0
	mov	USER-SIZE 3 cells - [rsi], rax	\ EXC0
	mov	USER-SIZE 2 cells - [rsi], rax	\ EXCP

\ Set the default I/O pointers. Done this way so that it still
\ works with ATTACH/DETACH in a DLL before relocation.
	call	getRA				\ return address placed in R12
l: cbe1						\ R12 points here
	mov	rax, ip-default cbe1 - [r12]	\ contents of IP-DEFAULT
	mov	$0F cells [rsi], rax		\ IP-HANDLE
	mov	rax, op-default cbe1 - [r12]	\ contents of OP-DEFAULT
	mov	$0E cells [rsi], rax		\ OP-HANDLE

\ Get the return address and hence the callback address
	mov	rax, #CBdropped #CBF_RA + [rsp]	\ return address from CALLBACK-ENTRY
	lea	r11, #CBdropped #CBF_pars + [rsp] \ points to lowest parameter
	push	rax				\ return from CALLBACK-ENTRY to following routine
\ Set up data stack, parameters in C order, EBX=^params, EBX=return
\ first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
	mov	r10, #-13 [rax]			\ R10 = NUM INPUT PARAMS
	cmp	r10, # 0
	z, if,
l: cbparams0
	  ret					\ no params
	then,
	cmp	r10, # 1
	z, if,
l: cbparams1
	  sub	  rbp, # cell
	  mov	  rbx, r14
	  ret
	then,
	cmp	r10, # 2
	z, if,
l: cbparams2
	  sub	  rbp, # 2 cells
	  mov	  0 [rbp], r14		\ parm1, rdi
	  mov	  rbx, r15		\ parm2, rsi
	  ret
	then,
	cmp	r10, # 3
	z, if,
l: cbparams3
	  sub	  rbp, # 3 cells
	  mov	  cell [rbp], r14	\ parm1, rdi
	  mov	  0 [rbp], r15		\ parm2, rsi
	  mov	  rbx, rdx		\ parm3, rdx
	  ret
	then,
	cmp	r10, # 4
	z, if,
l: cbparams4
	  sub	  rbp, # 4 cells
	  mov	  2 cells [rbp], r14	\ parm1, rdi
	  mov	  cell [rbp], r15	\ parm2, rsi
	  mov	  0 [rbp], rdx		\ parm3, rdx
	  mov	  rbx, rcx		\ parm4, rcx
	  ret
	then,
	cmp	r10, # 5
	z, if,
l: cbparams5
	  sub	  rbp, # 5 cells
	  mov	  3 cells [rbp], r14	\ parm1, rdi/r14
	  mov	  2 cells [rbp], r15	\ parm2, rsi/r15
	  mov	  cell [rbp], rdx	\ parm3, rdx
	  mov	  0 [rbp], rcx		\ parm4, rcx
	  mov	  rbx, r8		\ parm5, r8
	  ret
	then,
	cmp	r10, # 6
	z, if,
l: cbparams6
	  sub	  rbp, # 6 cells
	  mov	  4 cells [rbp], r14	\ param1, rdi/r14
	  mov	  3 cells [rbp], r15	\ param2, rsi/r15
	  mov	  2 cells [rbp], rdx	\ param3, rdx
	  mov	  cell [rbp], rcx	\ param4, rcx
	  mov	  0 [rbp], r8		\ param5, r8
	  mov     rbx, r9		\ param6, r9
	  ret
	then,
l: cbparamsn>6
	shl	r10, # 3		\ 8 bytes per item
	mov	r12, rbp		\ points at bottom of Forth data stack
					\ R11 points at lowest C parameter
	sub	rbp, r10		\ drop data stack by n items
\ copy the first 6 C parameters to the Forth RBP stack
	mov	-1 cells [r12], r14	\ param1, rdi/r14
	mov	-2 cells [r12], r15	\ param2, rsi/r15
	mov	-3 cells [r12], rdx	\ param3, rdx
	mov	-4 cells [r12], rcx	\ param4, rcx
	mov	-5 cells [r12], r8	\ param5, r8
	mov	-6 cells [r12], r9	\ param6, r9
\ Copy the remaining C parameters to the Forth RBP stack
	sub	r12, # 6 cells		\ allow for what we've done, must predecrement
	sub	r10, # 6 cells		\ remaining number of items in bytes
l: copyparams
	sub	r12, # cell		\ predecrement dest
	mov	rax, 0 [r11]		\ get src
	add	r11, # cell		\ postincrement
	mov	0 [r12], rax		\ to dest
	sub	r10, # cell
	jne	copyparams
l: copyparamsx
\ Fix for Nick ...
	mov	rbx, 0 [rbp]		\ final parameter to RBX
	add	rbp, # cell		\ pop
\ ... fix for Nick
	ret
end-code

code callback-exit
	pop  	rax			\ save return address
	lea	rsp, #CBdropped [rsp]	\ restore ESP to before frame creation
	mov	rsi, #CBF_RA [rsp]	\ ESI = CALLBACK-ENTRY's return address
	mov	#CBF_RA [rsp], rax	\ becomes new return address
	mov	rsi, # -21 [rsi]	\ number of output parameters, 8+8+5 offset

	mov	rax, rbx		\ 1st return parameter (EAX is not preserved
        				\ so this is valid even for void functions)
	cmp	rsi, # 2		\ check for 128 bit return value in EDX:EAX
	jne	onlyone
	mov	rdx, rbx		\ EDX=most significant
	mov	rax, 0 [rbp]		\ EAX=least significant
l: onlyone
\ restore saved regs
	pop	r15
	pop	r14
	pop	r13
	pop	r12
	pop	rbp
	pop	rbx
  	ret				\ following RET # #in cells cleans stack
end-code
\ ...SFP004

\ constants needed by CALLDEF routines (v5.4 onwards)
#CBdropped  constant #CBdropped
#rsp_USER   constant #rsp_USER
#rsp_TIB    constant #rsp_TIB
#rsp_FSPtop constant #rsp_FSPtop
/FSPguard   constant /FSPguard
#rsp_PSPtop constant #rsp_PSPtop
/PSPguard   constant /PSPguard
/ExceptStack constant /ExceptStack
/Syspad      constant /Syspad
USER-SIZE    constant USER-SIZE

\ ============================================================================

: callback-crash
  cr ." Undefined callback" bye
;

: set-callback	\ xt callback --
\ *G Make the xt be the action of the callback.
  SkipCall PatchCall
;

: callback,	\ #in #out -- address
\ *G Lay down a callback data structure. The first cell contains
\ ** the address of the entry point. The address of the data structure
\ ** is returned.
  here  dup >r  3 CELLS + ,	\ pointer to callback code
  ,  ,				\ #out #in ; SFP003
  ['] callback-entry doCall,	\ call to entry code
  ['] callback-crash doCall,	\ callback action word
  ['] callback-exit  doCall,	\ call to exit code
  $C3 c,			\ SFP003
  r>
;

target-only
: CALLBACK:	\ #in #out "<name>" -- ; -- a-addr
\ *G Create a callback function. *\i{#IN} and *\i{#OUT} refer to
\ ** the number of input and output parameters required for the
\ ** callback. When the definition *\fo{<name>}is executed it will
\ ** return the address of the callback function. For example
\ *C   2 1 CallBack: Foo
\ *P creates a callback named *\fo{Foo} with two inputs, and one output.
\ ** Executing *\fo{Foo} returns the entry point used by Linux. To use
\ ** it, pass *\fo{Foo} as the entry point required by Linux, e.g as the
\ ** address of a task action. *\fo{Foo} is built to use the "C" calling
\ ** convention.
\ *C   ' FooAction to-callback foo
\ *P Having defined an action for the callback, you can now use the
\ ** callback as if it was a C or assembler function called by the
\ ** operating system.
  create
    callback, drop
  does>
    @
;

: CallProc:	\ #in #out "<name>" -- ; -- entry
\ *G Create a callback function and start compilation of its
\ ** action. *\i{#IN} and *\i{#OUT} refer to
\ ** the number of input and output parameters required for the
\ ** callback. When the definition *\fo{<name>}is executed it will
\ ** return the entry point address of the callback function.
\ *E 4 1 CallProc: <name>  \ #in #out -- ; -- entry
\ ** \ Callback action ; x1 x2 x3 x4 -- op
\ **   ...
\ ** ;
\ ** <name>   \ returns entry point address
  create
    callback, >r :noname r> @ set-callback
  does>
    @
;

: CB:		\ xt #in "<name>" -- ; -- entry
\ *G Create a callback function that executes the action of *\i{xt}.
\ ** action. *\i{#IN} refers to the number of input parameters.
\ ** The number of output parameters is 1. When *\fo{<name>}is
\ ** executed it will return the entry point address of the
\ ** callback function. This word is provided to ease porting
\ ** from other Forth systems.
\ *E :noname  ( a b c -- d )
\ ** ...
\ ** ; 3 CB: <name>
  create
    1 callback, @ set-callback
  does>
    @
;

host&target

: to-callback   \ xt <"name"> --
\ *G Assign an XT as the action of a defined callback. This word is
\ ** state smart.
  ' execute			\ executing a callback returns its entry point
  state @ if
    [compile] literal  ['] set-callback doCall,
  else
    set-callback
  then
; immediate


\ ========================================
\ *S An example. Creating a signal handler
\ ========================================
\ *P A Linux signal handler has the prototype
\ *C   void sa_siginfo( int signum, siginfo_t * siginfo, ucontext_t * uc );
\ *P As far as Forth is concerned we need to execute a Forth word that
\ ** receives three parameters and returns none.
\ *C  (SigTrap)   \ signum *siginfo *ucontext --
\ *P The code fragment below achives this.
\ *E 3 0 callback: SigTrap  \ -- addr
\ ** \ executing SigTrap in Forth returns the C entry point.
\ **
\ ** : (SigTrap)  \ signum *siginfo *ucontext --
\ ** \ Action of SigTrap.
\ **   nip                                   \ discard siginfo
\ **   cr
\ **   cr ." Signal number " swap .sigName
\ **   uc.*mcontext64 @                      \ point at CPU context
\ **   cr ." at address " dup sc.RIP @ dup .dword
\ **      ." , probably in " ip>nfa .name
\ **   cr
\ **   ['] SigThrow swap sc.RIP !            \ force return to SigThrow
\ ** ;
\ ** assign (SigTrap) to-callback SigTrap

\ *P The callback entry code provides you with a default I/O device
\ ** and sets *\fo{BASE} to decimal. It does *\b{not} set up a
\ ** default *\fo{THROW} handler. If your callbacks must cope
\ ** with exceptions, you must provide a top-level *\fo{CATCH}
\ ** yourself.


\ ***********************
\ *S Implementation notes
\ ***********************
\ *P Callbacks are (usually) C functions. In the case of VFX Forth
\ ** these functions create a Forth environment with two or more
\ ** stacks, a *\fo{USER} area and so on. In a GUI environment,
\ ** callbacks are very common, and so must be established and
\ ** discarded quickly. The easiest place to do this is to use
\ ** the calling C stack and build the Forth stacks and data
\ ** areas on the C stack. This has several consequences:
\ *(
\ *B VFX Forth uses stacks generously, so the C stack has to be
\ ** large. Between 1 and 8Mb is common. The need for this amount
\ ** of memory is caused by a callback triggering another callback.
\ ** We have observed nesting levels of 12 or more in applications.
\ *B To improve performance, VFX Forth does not "touch" the stack
\ ** it needs when the Forth environment is created. Therefore
\ ** all the stack space must be "committed" in advance.
\ *B Only a limited number of *\fo{USER} variables are initialised:
\ ** *\fo{S0}, *\fo{R0}, *\fo{BASE}, *\fo{IP-HANDLE}, *\fo{OP-HANDLE},
\ ** *\fo{ThreadExit?}, *\fo{ThreadTCB}, and *\fo{ThreadSync}.
\ *B You can make *\b{no} assumptions about the addresses used for
\ ** the Forth environment. Two separate invocations of the same
\ ** callback may use quite different addresses. You can assume
\ ** that the addresses will not change within a callback unless
\ ** the application has changed them.
\ *B Callbacks, tasks and signals all use the same entry mechanism.
\ *)


\ ======
\ *> ###
\ ======

variable ExitCode	\ -- addr ; holds exit code returned to Windows
  0 ExitCode !

internal

((
: foo
  sysCr s" Ret=" sysType r@ sys.dword sysCr
;
))

: PRE-COLD
  0 ExitCode !
  cold
;
' pre-cold cld1 !	\ set up entry point code

external

defer (bye)	\ --

\ -----------------------------------------------------------------------------

decimal

