; VFX Forth stub loader.
; This file produces the Mach-O headers and commands used by VFX
; Forth 64 for Linux x64. The file is extended with the Forth
; image by the file binder.fth after cross compilation of the kernel.
;
; compile with:
; nasm -O1 -f elf64 stublin64.asm
; gcc -m64 -Wl,-Ttext-segment,0x400000 -o stublin64 stublin64.elf
; strip -u stubosx64                           (removes unnecessary symbols)
; otool -h -l stubosx64 >stubosx64.txt
;

	bits 64
	default rel			; PC relative is the default
extern dlopen, dlsym, dlclose, dlerror

; data goes here
segment .data

szlibname:
   db      'libc.so.6', 0
szprintf:
   db      "printf",0
formatstring:
   db      "Hello from %s, my number is 0x%lx",0x0a,0
myname:
   db      "Stephen",0
   align 8, db 0
mynum:
   dq      0x1234567890ABCDEF
pprintf:  ; holds address of printf
   dq      -1


; ***************************
; storage later used by Forth
; ***************************
   align 8, db 0
pmagicb:		; data here identifies 32 or 64 bit system
   dq      0xfeedfacf			; magic number as identifier of 64 bit
   dq      _magiclen_			; length of this block from pmagic
plib:     ; holds library handle
   dq      -1				; handle for libSystem.B.dylib
pdlopen:
   dq     dlopen			; function address
pdlsym:
   dq     dlsym				; function address
pdlclose:
   dq     dlclose			; function address
pdlerror:
   dq     dlerror			; function address
pMainRSP:
   dq     -1				; RSP on entry to Main
pUnixRSP:
   dq     -1                            ; RSP that points to argc/argv block
osArgc:
   dq     -1				; argc
osArgv:
   dq     -1				; argv
pmagicbend:
_magiclen_ equ pmagicbend-pmagicb


; ***********************
; Magic addesses in Forth
; ***********************

StubBase equ 0x400000			; load address

ForthBase equ StubBase + 0x00003000	; Forth image start address
					; holds counted string starting "VFX Forth"
ForthData  equ StubBase + 0x00003100	; copy the data block at pmagic to Forth
					; good image contains 0xfeedfacf at ForthData
ForthEntry equ StubBase + 0x00004000	; Forth entry point


; ********************
; MPE code starts here
; ********************
; Much of the code here is redundant, but is left here in case
; the rules change again.

segment .text

global main

main:
; These first two are the old style pointer to argc/argv
; they do not seem to contain argc/argv any more
   mov     rax, rsp
   mov     [rel pMainRSP], rax		; save current RSP
   add     rax, 8
   mov     [rel pUnixRSP], rax		; where argc/argv block is (by inspection)
; these are the new versions
   mov     [rel osArgc], rdi		; should be here
   mov     [rel osArgv], rsi

   jmp     near IDgood

   mov     rbx, ForthData
   mov     eax, [rbx+0]			; get Forth ID
   cmp     eax, 0xfeedfacf		; check it ; fails if not bound
   mov     ebx, 43			; error code
   jmp     failed
IDgood:

; first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
; Open the system library
   mov     r12, rsp		; preserve RSP for return/exit
   and     rsp, -0x10		; 16 byte align down
   mov     rdi, szlibname	; "libSystem.B.dylib"
   mov     rsi, 1		; RTLD_LAZY
   mov     rdx, 0		; tradition/hidden third parameter
   call    [pdlopen]		; open library
   mov     [plib], rax		; EAX is library handle or error code
   mov     rbx, 1		; return code if failure
   cmp     rax, 0		; test return value
   je  near    failed
;   mov rbx, 0x12
;   call exitsub
   jmp skip1x

; Copy the pmagicb data block above (line 39) to the Forth image
   mov     rbx, pmagicb
   mov     rcx, ForthData
   mov     rdx, _magiclen_
copypm:
   mov     rax, [rbx]
   mov     [rcx], rax
   add     rbx, 8
   add     ecx, 8
   sub     edx, 8
   jne     copypm

; go run Forth
   mov     rax, ForthEntry
   jmp     rax

skip1x:
; Find printf
; first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
   mov     rdi, [plib]		; library handle
   mov     rsi, szprintf	; string "printf"
;   call    dlsym		; find function address
   call    [pdlsym]		; find function address
   mov     [pprintf], rax	; RAX = function address or error code
   mov     rbx, 2		; return code
   cmp     rax, 0		; test return value
   je      failed
;   mov rbx, 0x13
;   call exitsub

; call printf
; first/left: %rdi, %rsi, %rdx, %rcx, %r8, %r9 :last/right
   mov     al, 0		; no vectors
   mov     rdi, formatstring     ; format string itself
   mov     rsi, myname           ; first parameter to format string
   mov     rdx, [mynum]          ; second paramter to format string
   call    [pprintf]             ; print and expand format string
;   mov rbx, 0x14
;   call exitsub

; close library
   mov     rdi, [plib]           ; library handle
;   call    dlclose
   call    [pdlclose]
   mov     rbx, 3                ; error return code
   cmp     rax, 0                ; test result
   jne     failed

; back to Linux
   mov     rbx, 42               ; success return code
failed:                          ; exit code in RBX
exitsub:	; rbx=exit code
   mov     eax, 60
   mov     rdi, rbx
   syscall			; SYSCALL clobbers RCX (return addr) and R11 (flags)
   mov     rsp, r12		; restore RSP
   ret

   call    [pdlerror]
   ret
;
; the end
;
