\ SYSPATCH.FTH - Operating system hooks for startup

((
Copyright (c) 2001-2006, 2007, 2008, 2009, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441


To do
=====

Change history
==============
20230825 GFW021 Added temp-file
20200610 64 bit conversion
20110613 SFP019 Fixed OS_SetEndOfFile.
20110304 SFP018 Added NumCall selection.
20090917 SFP017 Decluttered.
20090601 SFP016 Added missing OS_RENAME-FILE.
20080304 SFP015 Added (OS_Type).
20080205 VRW014 Support shared library added.
20080122 SFP013 Moved ERRNO to VFXBase/MiscLinux.fth.
20080115 SFP012 Added SetInterp and SetLibc in SYSTEM voc.
20071231 SFP011 Added ERRNO.
20071227 SFP010 Added (OS_Flush-File)
20071218 VRW009 Linux release
20070624 SFP008 Refactored console startup.
20061115 SFP007 Moved support DLL initialisation from SYSPATCH.FTH.
20060419 SFP006 Restored setting of STDERR for Unix compatibility.
		Rewrote RESOLVE and friends.
20041201 SFP005 Updated OS_TYPE and OS_CR
20031127 SFP004 Separated system and local times
		and refactored the interface
20030908 SFP003 Fixed (OS_KEY?) at last!
20021018 SFP002 Added OS_KEY?
20011026 SFP001 Added SETENDOFFILE.
                Reduced size of (SYSPATCH).
))


also system definitions
decimal


\ **********************************
\ Resolving shared libraries and their functions
\ **********************************

\ SFP006...
create libc$	\ -- caddr ; counted string
  256 allot&erase
  s" libc.so.6" libc$ place
\  s" /lib/x86_64-linux-gnu/libc.so.6" libc$ place

: PleaseResolve	\ dll$ func$ -- xcall|0
\ *G Return address to call for a function in a DLL,
\ ** or 0 if not found.
  { | zname[ #256 ] -- }
  swap count zname[ zplace
  zname[ LoadLibrary          \ c-addr2 libptr --
  dup if
    swap count zname[ zplace
    zname[ GetProcAddress
  else
    nip
  then
;

: Resolve	\ dll$ func$ -- xcall
\ *G Return address to call for a function in a DLL.
\ ** A breakpoint occurs if the function fails.
  PleaseResolve dup 0=
  if  breakpoint  then
;

\ ...SFP006


\ ***************
\ VFX Support DLL or shared library
\ ***************

\ SFP007...

also Forth definitions
1 value InitSupport?	\ -- n
\ Returns true if the support DLL is to be loaded.
previous definitions

\ internal

variable suppcall	\ -- addr ; holds address of FindConstant code
variable simcall	\ -- addr ; holds address of FindSimilar code
variable NumCall	\ -- addr ; holds address of NumConstants code
((
variable savecall	\ -- addr ; holds address of Save code
variable SaveExtCall	\ -- addr ; holds address of SaveExt call
variable SuppVerCall	\ -- addr ; holds address of VersionNum call
))
create SuppDLL$	\ -- addr
  ", vfxsupp64.so.1"

: initVFXdll	\ --
  InitSupport? 0=
  if  suppcall off  exit  endif
  SuppDLL$ C" VFXSuppFindConstant"  PleaseResolve suppcall !
  SuppDLL$ C" VFXSuppFindSimilar"   PleaseResolve simcall !
  SuppDLL$ C" VFXSuppNumConstants"  PleaseResolve NumCall !
;

\ external
\ ...SFP007


\ ************
\ Standard I/O
\ ************

variable stdin	\ -- addr
\ *G Holds file handle used for UNIX-style input.
\ ** Leave as a *\fo{VARIABLE} for compatibility between
\ ** VFX Forth implementations.
variable stdout	\ -- addr
\ *G Holds file handle used for UNIX-style output.
\ ** Leave as a *\fo{VARIABLE} for compatibility between
\ ** VFX Forth implementations.
variable stderr	\ -- addr
\ *G Holds file handle used for UNIX-style error output.
\ ** Leave as a *\fo{VARIABLE} for compatibility between
\ ** VFX Forth implementations.
variable stddev	\ -- addr
\ *G Holds console type: 1=file, 2=char, 3=pipe, other=???

\ variable dll_SetConsoleMode
variable dll_WriteFile
variable dll_ReadFile
variable dll_GetSystemTime
variable dll_GetLocalTime
variable dll_time
variable dll_CreateFile
variable dll_CloseHandle
\ variable dll_GetFileSize
variable dll_SetFilePointer
variable dll_DeleteFile
variable dll_isatty
variable dll_poll
variable dll_ftruncate
variable dll_perror
variable dll_tcgetattr
variable dll_tcsetattr
\ variable dll_cfmakeraw
variable dll_ioctl
variable dll_exit
variable dll_umask
variable dll_chmod
variable dll_syscall
variable dll_select
variable dll_nanosleep
variable dll_fsync	\ SFP010
variable dll_rename	\ SFP016
variable dll_tmpfile	\ GFW021
variable dll_fileno 	\ GFW021

: (os_bin)      \ fam -- 'fam
;

: (os_r/o)      \ -- fam
  0
;

: (os_w/o)      \ -- fam
  1
;

: (os_r/w)	\ -- fam
  2
;

((
defer +instrument
  assign noop to-do +instrument
defer -instrument
  assign noop to-do -instrument
))

$40 equ O_CREAT
$200 equ O_TRUNC
$1b6 equ MODE_666	\ Octal 666
-1 equ INVALID_HANDLE_VALUE

: (OS_Create-File)      { c-addr u fam | zaddr[ #1024 ] -- fileid ior }
  c-addr u #1023 umin zaddr[ zplace	\ Convert name to 0 string
\  MODE_666 fam O_CREAT or O_TRUNC or zaddr[	\ SFP002
  zaddr[  fam O_CREAT or O_TRUNC or  MODE_666
  dll_CreateFile @ nxcall3		\ int return = 4 bytes
  lit#64 $FFFF,FFFF and
  dup lit#64 $FFFF,FFFF =
  if  #-63  else  0  then
;

: (OS_Open-File)      { c-addr u fam | zaddr[ #1024 ] -- fileid ior }
\ +instrument
  c-addr u #1023 umin zaddr[ zplace	\ Convert name to 0 string
  zaddr[ fam  dll_CreateFile @ nxcall2
  lit#64 $FFFF,FFFF and
  dup lit#64 $FFFF,FFFF =
  if  #-69  else  0  then
\ -instrument
;

: (OS_Close-File)       \ fileid -- ior
\ +instrument
  dll_CloseHandle @ nxcall1  lit#64 $FFFF,FFFF and
  if  #-62  else  0  then
\ -instrument
;

: (OS_Write-File)       { caddr u fileid -- ior }
  fileid caddr u  dll_WriteFile @ nxcall3
  -1 =
  if  #-75  else  0  then
;

: (OS_Read-File)	{ caddr u fileid -- u2 ior }
  fileid caddr u  dll_ReadFile @ nxcall3
  dup -1 =
  if  #-70  else  0  then
;

0 equ FILE_BEGIN	\ SEEK_SET in POSIX parlance
1 equ FILE_CURRENT	\ SEEK_CUR
2 equ FILE_END		\ SEEK_END

\ off_t lseek(int fd, off_t offset, int whence);
: (OS_file-position)    { fileid -- ud ior }
  fileid 0 FILE_CURRENT  dll_SetFilePointer @ nxcall3
  0 over -1 = if			\ -- res 0 ; fail
    2drop  0 0 #-73
  else					\ success
    0
  then
;

\ off_t lseek(int fd, off_t offset, int whence);
: (OS_reposition-file)  { low high fileid -- ior }
  fileid low FILE_BEGIN  dll_SetFilePointer @ nxcall3
  -1 =
  if  #-73  else  0  then
;

\ off_t lseek(int fd, off_t offset, int whence);
: (OS_file-end)	{ fileid -- ud ior }
  fileid 0 FILE_END  dll_SetFilePointer @ nxcall3
  0 over -1 = if			\ -- res 0 ; fail
    2drop  0 0 #-73
  else					\ success
    0
  then
;

: (OS_file-size)	{ fileid | high low -- ud ior }
  fileid (OS_file-position) drop to high to low
  fileid (OS_file-end)
  low high fileid (OS_reposition-file) drop
;

\ int ftruncate(int fd, off_t length);
: (OS_SetEndOfFile)	\ fileid -- ior ; SFP001 SFP002
  dup (OS_file-position) dup 0= if	\ -- fileid ud ior
    2drop  dll_ftruncate @ nxcall2
  else
    nip nip nip				\ Remove fileid and dummy position from stack
  then
;

\ int unlink(const char *pathname);
: (OS_delete-file)	{ c-addr u | zaddr[ #1024 ] -- ior }
  c-addr u zaddr[ zplace
  zaddr[  dll_DeleteFile @ nxcall1
  #-64 and
;

\ int rename(const char *old, const char *new);
: (OS_Rename-File)	\ ca-old lold ca-new lnew -- ior
  { | zold[ #1024 ] znew[ #1024 ] -- }
  #1024 2- min znew[ zplace
  #1024 2- min zold[ zplace
  zold[ znew[  dll_rename @ nxcall2
  if  #-72  else  0  then
;

\ mode_t umask(mode_t mask);
: (OS_GetUmask)	\ -- mode
  0  dll_umask @ nxcall1		\ Retrieve mask
  dup dll_umask @ nxcall1 drop		\ umask() is destructive; replace original mask
;

\ int chmod(const char *path, mode_t mode);
: (OS_Chmod)	{ c-addr u mode | zaddr[ #1024 ] -- ior }
  c-addr u zaddr[ zplace
  zaddr[ mode  dll_chmod @ nxcall2
  0=
;

\ int fdatasync(int fd);
: (OS_Flush-File)	\ fileid -- ior ; SFP010
  dll_fsync @ nxcall1 drop 0	\ always return ior=0
;

: (OS_Temp-File) \ -- fileid ; GFW021
  dll_tmpfile @ nxcall0 ?dup if
    dll_fileno @ nxcall1
  then
;


\ ************************
\ ALLOCATE FREE and RESIZE
\ ************************

variable dll_GlobalAlloc
variable dll_GlobalFree
variable dll_GlobalReAlloc

\ void *malloc(size_t size);
\ void *calloc(size_t nmemb, size_t size);
: (os_allocate)	\ size -- a-addr ior
\ Returns a pointer to a fixed memory block, with zeroed contents
  1 swap  dll_GlobalAlloc @ nxcall2	\ calls calloc
  ?dup
  if  0  else  -1 -59  then		\ see error codes in kernel64.fth
;

\ void *realloc(void *ptr, size_t size);
: (os_resize)	\ a-addr newsize -- 'a-addr ior
  over >r				\ preserve a-addr for error
  dll_GlobalReAlloc @ nxcall2
  ?dup if 0 else r@ -61 then		\ see error codes in kernel64.fth
  r> drop
;

\ void free(void *ptr);
: (os_free)	\ a-addr -- ior
  dup if
    dll_GlobalFree @ nxcall1 drop 0
  then
;

: InitVfxHeap	\ -- ; initialise heap operations
  libc$ C" calloc"             Resolve dll_GlobalAlloc !
  libc$ C" realloc"            Resolve dll_GlobalReAlloc !
  libc$ C" free"               Resolve dll_GlobalFree !
;


\ ***********************
\ Standard in and friends
\ ***********************

variable stdin	\ -- addr
\ *G Holds file handle used for UNIX-style input.
\ ** Leave as a *\fo{VARIABLE} for compatibility between
\ ** VFX Forth implementations.
variable stdout	\ -- addr
\ *G Holds file handle used for UNIX-style output.
\ ** Leave as a *\fo{VARIABLE} for compatibility between
\ ** VFX Forth implementations.
variable stderr	\ -- addr
\ *G Holds file handle used for UNIX-style error output.
\ ** Leave as a *\fo{VARIABLE} for compatibility between
\ ** VFX Forth implementations.
variable stddev	\ -- addr
\ *G Holds console type: 1=file, 2=char, 3=pipe, other=???

variable TermPrepped	\ -- addr
\ *G Set non-zero when the terminal functions have been set up.
variable StdinUnbuffered \ -- addr
\ *G Flags whether or not stdin has been explicitly unbuffered

(( in /usr/include/bits/termios.h
#define NCCS 32
struct termios
  {
    tcflag_t c_iflag;		/* input mode flags */
    tcflag_t c_oflag;		/* output mode flags */
    tcflag_t c_cflag;		/* control mode flags */
    tcflag_t c_lflag;		/* local mode flags */
    cc_t c_line;		/* line discipline */
    cc_t c_cc[NCCS];		/* control characters */
    speed_t c_ispeed;		/* input speed */
    speed_t c_ospeed;		/* output speed */
#define _HAVE_STRUCT_TERMIOS_C_ISPEED 1
#define _HAVE_STRUCT_TERMIOS_C_OSPEED 1
  };
))
#32 equ NCCS
struct /termios \ -- size
\ *S A structure to mimic the termios structure used by tcgetattr and tcsetattr
\ *[
  4 field termios.c_iflag	\ input mode flags
  4 field termios.c_oflag	\ output mode flags
  4 field termios.c_cflag	\ control mode flags
  4 field termios.c_lflag	\ local mode flags
  1 field termios.c_line	\ line discipline
  NCCS field termios.c_cc	\ control characters
  3 field termios.padding	\ C aligns everything to 32-bits
  4 field termios.c_ispeed	\ input speed
  4 field termios.c_ospeed	\ output speed
end-struct
\ *]
((
/* Copyright (C) 1992-2019 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */
#include <termios.h>
/* Set *T to indicate raw mode.  */
void
cfmakeraw (struct termios *t)
{
  t->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
  t->c_oflag &= ~OPOST;
  t->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
  t->c_cflag &= ~(CSIZE|PARENB);
  t->c_cflag |= CS8;
  t->c_cc[VMIN] = 1;                /* read returns when one char is available.  */
  t->c_cc[VTIME] = 0;
}
Generated on 2019-Mar-30 from project glibc revision glibc-2.29.9000-166-g656dd306d4
Powered by Woboq Code Browser 2.1
Generator usage only permitted with license.
))

/termios buffer: OrigTermios	\ buffer for original termios structure

0 equ TCSANOW
2 equ ICANON
8 equ IECHO

\ int tcgetattr(int fd, struct termios *termios_p);
: getTermios	\ addr -- flag ; true=success
  stdin @ swap  dll_tcgetattr @ nxcall2 0=
;

\ int tcsetattr(int fd, int optional_actions, const struct termios *termios_p);
: setTermios	\ addr --
  stdin @ TCSANOW rot  dll_tcsetattr @ nxcall3 drop
;

: lbic!		\ mask addr -- ; clear bits in 32 bit unit
  tuck l@  swap invert and  swap l!
;

: (SetUnbufferedStdin) { | termios_struct[ /termios ] -- }
  termios_struct[ getTermios if 		\ If we managed to get attributes...
    termios_struct[ OrigTermios /termios cmove	\ save structure for exit
    ICANON IECHO or termios_struct[ termios.c_lflag lbic!
    termios_struct[ setTermios
    1 StdinUnbuffered !
  then
;

: (RebufferStdin) { | termios_struct[ /termios ] -- }
  StdinUnbuffered @ if
    OrigTermios setTermios  0 StdinUnbuffered !
  then
;

\ int isatty(int fd);
: prepTerminal	\ --
\ Set up the terminal.
  stdin @  dll_isatty @ nxcall1 stddev !
  (SetUnbufferedStdin)
;

: ?PrepTerm	\ --
\ If not already done, set up the terminal.
  TermPrepped @ 0= if
    PrepTerminal  TermPrepped on
  then
;

: UnprepTerminal \ --
\ Restore the terminal.
  (RebufferStdin)
;

: ?UnprepTerm	\ --
\ Restore the terminal if set up.
  TermPrepped @ if
    UnprepTerminal  TermPrepped off
  then
;
' ?UnprepTerm AtExit		\ Cue up unbuffering action

: InitVfxConsole	\ -- ; SFP008
\ Initialise console I/O.
  libc$ C" write"            Resolve dll_WriteFile !
  libc$ C" read"             Resolve dll_ReadFile !
  libc$ C" isatty"	     Resolve dll_isatty !
  libc$ C" poll" 	     Resolve dll_poll !
  libc$ C" select" 	     Resolve dll_select !
  libc$ C" tcgetattr" 	     Resolve dll_tcgetattr !
  libc$ C" tcsetattr" 	     Resolve dll_tcsetattr !
\  libc$ C" cfmakeraw" 	     Resolve dll_cfmakeraw !
  libc$ C" ioctl" 	     Resolve dll_ioctl !
  libc$ C" umask" 	     Resolve dll_umask !
  libc$ C" chmod" 	     Resolve dll_chmod !
  \ The POSIX way of doing things...
  0 stdin !
  1 stdout !
  2 stderr !

  \ Default to "haven't touched it"
  0 TermPrepped !  0 StdinUnbuffered !  0 stddev !
\  stdin @ dll_isatty @ 1 nxcall stddev !
;

$541B equ FIONREAD

\ int ioctl(int d, int request, ...);
: (OS_key?)	{ | nread[ cell ] -- flag }
  ?PrepTerm  nread[ off
  stdin @ FIONREAD nread[  dll_ioctl @ nxcall3	\ returns an int (4 bytes)
  $7FFF:FFFF and  $7FFF:FFFF = if	\ -1 is error
    0					\ Error return from ioctl
  else
    nread[ @ 0<>
  then
;

: (OS_Key)	\ -- key ; SFP003
  { | iobuff[ cell ] -- char }
  ?PrepTerm
  stdin @ iobuff[ 1  dll_ReadFile @ nxcall3 drop
  iobuff[ c@
;

: (OS_Emit)     \ char -- ; SFP003
  { | iobuff[ cell ] -- }
  ?PrepTerm
  iobuff[ c!
  stdout @ iobuff[ 1 dll_WriteFile @ nxcall3 drop
;

: (OS_Type)     \ caddr len -- ; SFP015
  ?PrepTerm
  stdout @ -rot dll_WriteFile @ nxcall3 drop
;

\ --------------------------------------------------------------------------

#32 equ SPACE_CHAR

: (OS_CopyCommandLine)		\ --
\ *G Copy command line from argv[] into a single string buffer
  posArgc @  posArgv @		\ -- argc argv
  CmdLineBuffer			\ -- argc argv buff
  rot 0 do			\ -- argv buff
    over i cells + @ swap	\ -- argv argv[n] buff
    begin
      over c@			\ -- argv argv[n] buff char
      ?dup
    while
      case  dup
        $0A of  drop BL  endof
        $0D of  drop BL  endof
      endcase
      over c! 1+ swap 1+ swap
    repeat
    SPACE_CHAR over c! 1+ nip
  loop
  0 swap 1- c! drop
;

: (OS_GetLoadAddress)           \ -- a-addr
  StartAddress
;

: (OS_GetCommandLine)           \ -- z-addr
  CmdLineBuffer
;

((
struct tm
           int tm_sec;     /* 0, 4 seconds (0 - 60) */
           int tm_min;     /* 4, 4 minutes (0 - 59) */
           int tm_hour;    /* 8, 4 hours (0 - 23) */
           int tm_mday;    /* 12, 4 day of month (1 - 31) */
           int tm_mon;     /* 16, 4 month of year (0 - 11) */
           int tm_year;    /* 20, 4 year - 1900 */
           int tm_wday;    /* 24, 4 day of week (Sunday = 0) */
           int tm_yday;    /* 28, 4 day of year (0 - 365) */
           int tm_isdst;   /* 32, 4 is summer time in effect? */
           /* packing         36, 4 */
           char *tm_zone;  /* 40, 8 abbreviation of timezone name */
           long tm_gmtoff; /* 48, 8 offset from UTC in seconds */
))                         \  56

: >tdw		\ addr -- seconds mins hours day month year dow
\ Extract data from a POSIX struct tm structure.
  dup l@ swap			\ seconds
  dup 4 + l@ swap		\ minutes
  dup 8 + l@ swap		\ hours
  dup #12 + l@ swap		\ day
  dup #16 + l@ 1+ swap		\ month - 0-based for POSIX
  dup #20 + l@ #1900 + swap	\ year - 1900-based for POSIX
  #24 + l@			\ dow
;

\ time_t time(time_t *t);
\ struct tm *localtime_r(const time_t *timep, struct tm *result);
: (OS_GetLocalTime)	\ -- seconds mins hours day month year dow
  { | st[ 64 ] timeval[ cell ] -- }
  timeval[  dll_time @ nxcall1 drop
  timeval[ st[  dll_GetLocalTime @ nxcall2 drop
  st[ >tdw
;

\ struct tm *gmtime_r(const time_t *timep, struct tm *result);
: (OS_GetSystemTime)	\ -- seconds mins hours day month year dow
  { | st[ 64 ] timeval[ cell ] -- }
  timeval[  dll_time @ nxcall1 drop
  timeval[ st[  dll_GetSystemTime @ nxcall2 drop
  st[ >tdw
;

: (OS_Exit)		\ -- ; primary exit routine
  ExitCode @  dll_exit @ nxcall1 drop
;

struct timespec \ -- size
\ *S A structure to mimic the timespec structure for libc
\ *[
  8 field tv_sec
  8 field tv_nsec
end-struct
\ *]

\ int nanosleep(const struct timespec *req, struct timespec *rem);
: (OS_ms)	\ ms --
  { | tspec[ timespec ] -- }
  #1000 /mod				\ -- rem quotient
  tspec[ tv_sec !
  #1000000 * tspec[ tv_nsec !
  tspec[ 0  dll_nanosleep @ nxcall2 drop
;

\ --------------------------------------------------------------------------

: (syspatch)    \ --
\ *G Patch all xcall addresses for system hooks, configure console
\ ** ready for interpreter.
\ s" into (SysPatch)" systype .sysState
  initVFXdll				\ link to support DLL ; SFP007
  initVfxConsole			\ initialise base console ; SFP008
  InitVfxHeap
\ s" done initVFXheap" sysType SysCr
  libc$ C" gmtime_r"           Resolve dll_GetSystemTime !
  libc$ C" localtime_r"        Resolve dll_GetLocalTime !
  libc$ C" time"               Resolve dll_time !
  libc$ C" open"               Resolve dll_CreateFile !
  libc$ C" close"              Resolve dll_CloseHandle !
\  libc$ C" __fxstat"           Resolve dll_GetFileSize !
  libc$ C" lseek"              Resolve dll_SetFilePointer !
  libc$ C" ftruncate"          Resolve dll_ftruncate !
  libc$ C" unlink"             Resolve dll_DeleteFile !
  libc$ C" perror"             Resolve dll_perror !
  libc$ C" exit"               Resolve dll_exit !
  libc$ C" syscall"            Resolve dll_syscall !
  libc$ C" nanosleep"          Resolve dll_nanosleep !
  libc$ C" fsync"              Resolve dll_fsync !	\ SFP010
  libc$ C" rename"             Resolve dll_rename !	\ SFP016
  libc$ C" tmpfile"            Resolve dll_tmpfile !	\ GFW021
  libc$ C" fileno"             Resolve dll_fileno !	\ GFW021
\ s" done Resolves" sysType SysCr

  (os_CopyCommandLine)
\ .sysState  s" out of (SysPatch)" systype
;
' (syspatch) AtCold

assign (OS_Key)                 to-do OS_Key
assign (OS_Key?)		to-do OS_Key?
assign (OS_Emit)                to-do OS_Emit
assign (OS_Type)                to-do OS_Type

assign (os_allocate)            to-do allocate
assign (os_resize)              to-do resize
assign (os_free)                to-do free

assign (OS_GetLoadAddress)      to-do OS_GetLoadAddress
assign (OS_GetCommandLine)      to-do OS_GetCommandLine
assign (OS_GetSystemTime)       to-do OS_GetSystemTime
assign (OS_GetLocalTime)        to-do OS_GetLocalTime

assign (os_bin)                 to-do os_bin
assign (os_r/o)                 to-do os_r/o
assign (os_w/o)                 to-do os_w/o
assign (os_r/w)			to-do os_r/w
assign (OS_Create-File)         to-do OS_Create-File
assign (OS_Open-File)           to-do OS_Open-File
assign (OS_Close-File)          to-do OS_Close-File
assign (OS_Write-File)          to-do OS_Write-File
assign (OS_Read-File)           to-do OS_Read-File
assign (OS_File-Size)           to-do OS_File-Size
assign (OS_File-Position)       to-do OS_File-Position
assign (OS_Reposition-file)     to-do OS_Reposition-file
assign (OS_SetEndOfFile)        to-do OS_SetEndOfFile		\ SFP001
assign (OS_Delete-File)         to-do OS_Delete-File
assign (OS_Flush-File)		to-do OS_FLush-File		\ SFP010
assign (OS_Rename-File)		to-do OS_Rename-File		\ SFP016
assign (OS_Temp-File)		to-do OS_Temp-File		\ GFW021

assign (OS_ms)			to-do ms
assign (OS_Exit)		to-do (bye)

((
\ These words were provided for testing various Unices
: SetInterp	\ caddr len --
  InterpName zplace
;

: SetLibc	\ caddr len --
  libc$ place
;
))

previous definitions

