

### Memory Buffer Device

This Generic IO Device uses blocks of memory for input and/or output\. The source code is in _LIB\GENIO\Buffer.fth_\. This is **not** a circular buffer system\. After a buffer has been used, the read and write pointers are **not** reset\. You must reset the buffer pointers using the IOCTL functions\.

As of VFX Forth build 2380, this code has been overhauled\. If this causes you problems, the file _BUFFER.old.FTH_ contains the previous \(but now unsupported\) version\. The major changes are:


1. More error checking\.
1. The **`KEY`** operation \(and hence **`ACCEPT`** blocks if no
data is available\.
1. The **`KEY?`** operation returns the number of unread bytes\.
1. The **`EMIT?`** operation returns the number of unwritten bytes
in the buffer\.\.
1. The close operation is protected if the device is already
closed\.
1. The GENIO **`ReadEx`** function is implemented\.
1. The **`Textbuff-sid`** GENIO structure has been documented\.
1. IOCTL functions have been added\. See later for details\.

Note that the **`ACCEPT`** operation does **not** echo\. It is designed for extracting lines from saved input\.

In order to create a device use the **`TEXTBUFF:`** definition given later\. **`TEXTBUFF:`** is compatible with ProForth 2\.

When opening a memory device the parameters to **`OPEN-GEN`** have the following meaning:

| Term | Description |
| --- | --- |
| ADDR |        Address of memory to use as buffer or ignored if | 
|   |            dynamic allocation is required\. | 



| Term | Description |
| --- | --- |
| LEN |         The maximum length of the memory image\. | 



| Term | Description |
| --- | --- |
| ATTRIBS |     Bit 0: When zero the ADDR parameter is ignored and LEN | 
|   |            bytes of memory are allocated from the heap; When 1, ADDR | 
|   |            is used as the buffer address\. | 



| Term | Description |
| --- | --- |
| ATTRIBS |     Bit 1: When 0, the buffer is FREEd on close; when 1, | 
|   |            buffer is not freed on close\. | 



--------------------------------------------------------------------------------
```
struct textbuff-sid     \ -- len
```
Defines the length of a SID for a text buffer device\.
```
  gen-sid +             \ handle=buffer, reuse field names of GEN-SID
  int tb-len            \ length of buffer
  int tb-attribs        \ attributes
  int tb-wr             \ address to write to, set by SETPOS, WRITE
  int tb-rd             \ address to read from set by SETPOS, READ
end-struct
```

--------------------------------------------------------------------------------
```
: ff-tb         \ sid -- ; page/cls on display devices
```
This word is run by **`PAGE`**, **`CLS`** and **`FF-GEN`**\. It resets
\(empties\) the buffer\.

--------------------------------------------------------------------------------
```
: setpos-tb     \ x y mode sid -- ior
```
This word is run by **`SETPOS-GEN`**\. Mode controls the x and y
input values as follows\.

| Term | Description |
| --- | --- |
| 0 |    x = \#bytes written, y is ignored | 



| Term | Description |
| --- | --- |
| \-1 |   x=col, y=line for next character to be written | 



| Term | Description |
| --- | --- |
| \-2 |   x = \#bytes read, y is ignored | 



| Term | Description |
| --- | --- |
| \-3 |   x=col, y=line for next character to be read | 



--------------------------------------------------------------------------------
```
: getpos-tb     \ mode sid -- x y ior
```
This word is run by **`GETPOS-GEN`**\. Mode controls the x and y
return values as follows\.

| Term | Description |
| --- | --- |
| 0 |    x = \#bytes written, y = 0 | 



| Term | Description |
| --- | --- |
| \-1 |   x,y for next character to be written | 



| Term | Description |
| --- | --- |
| \-2 |   x = \#bytes read, y = 0 | 



| Term | Description |
| --- | --- |
| \-3 |   x,y for next character to be read | 



| Term | Description |
| --- | --- |
| \-4 |   x = addr, y = len of unread data | 



| Term | Description |
| --- | --- |
| \-5 |   x = base address, y = size of data area | 



--------------------------------------------------------------------------------
```
: ioctl-tb      \ addr len fn sid -- ior
```
This word is run by **`IOCTL-GEN`** and **`IOCTL-GIO`**\.
_Fn_ controls the meaning of _addr_, _len_ and the
_ior_ return value as follows:

| Term | Description |
| --- | --- |
| 0 |    Get buffer address: _addr_=0, _len_=0, _ior_=addr\. | 



| Term | Description |
| --- | --- |
| \-1 |   Set write pointer: _addr_=0, _len_=offset, _ior_=0\. | 



| Term | Description |
| --- | --- |
| \-2 |   Get write pointer: _addr_=0, _len_=0, _ior_=offset\. | 



| Term | Description |
| --- | --- |
| \-3 |   Set read pointer: _addr_=0, _len_=offset, _ior_=0\. | 



| Term | Description |
| --- | --- |
| \-4 |   Get read pointer: _addr_=0, _len_=0, _ior_=offset\. | 




#### Device Creation

--------------------------------------------------------------------------------
```
: initTextBuffSid       \ addr --
```
Initialise a previously allocated SID for
a text buffer to the default values\.

--------------------------------------------------------------------------------
```
: textbuff:     \ "name" -- ; Exec: -- sid
```
Create a memory buffer called _name_\.

--------------------------------------------------------------------------------
```
: SizedTextBuff \ size -- sid|0
```
Allocates and opens a SID with a buffer of size _size_
bytes, and returns the SID on success or 0 on failure\.

--------------------------------------------------------------------------------
```
: AllocTextBuff \ -- sid|0
```
Allocates and opens a SID with a default 16kb text buffer
and returns the SID on success or 0 on failure\.

--------------------------------------------------------------------------------
```
: FreeTextBuff  \ sid --
```
Closes the SID and frees memory allocated by **`AllocTextBuff`**
or **`SizedTextBuff`**\.


### Memory Buffer Device

This Generic IO Device uses blocks of memory for input and/or output\. The source code is in _LIB\GENIO\Buffer.fth_\. This is **not** a circular buffer system\. After a buffer has been used, the read and write pointers are **not** reset\. You must reset the buffer pointers using the IOCTL functions\.

As of VFX Forth build 2380, this code has been overhauled\. If this causes you problems, the file _BUFFER.old.FTH_ contains the previous \(but now unsupported\) version\. The major changes are:


1. More error checking\.
1. The **`KEY`** operation \(and hence **`ACCEPT`** blocks if no
data is available\.
1. The **`KEY?`** operation returns the number of unread bytes\.
1. The **`EMIT?`** operation returns the number of unwritten bytes
in the buffer\.\.
1. The close operation is protected if the device is already
closed\.
1. The GENIO **`ReadEx`** function is implemented\.
1. The **`Textbuff-sid`** GENIO structure has been documented\.
1. IOCTL functions have been added\. See later for details\.

Note that the **`ACCEPT`** operation does **not** echo\. It is designed for extracting lines from saved input\.

In order to create a device use the **`TEXTBUFF:`** definition given later\. **`TEXTBUFF:`** is compatible with ProForth 2\.

When opening a memory device the parameters to **`OPEN-GEN`** have the following meaning:

| Term | Description |
| --- | --- |
| ADDR |        Address of memory to use as buffer or ignored if | 
|   |            dynamic allocation is required\. | 



| Term | Description |
| --- | --- |
| LEN |         The maximum length of the memory image\. | 



| Term | Description |
| --- | --- |
| ATTRIBS |     Bit 0: When zero the ADDR parameter is ignored and LEN | 
|   |            bytes of memory are allocated from the heap; When 1, ADDR | 
|   |            is used as the buffer address\. | 



| Term | Description |
| --- | --- |
| ATTRIBS |     Bit 1: When 0, the buffer is FREEd on close; when 1, | 
|   |            buffer is not freed on close\. | 



--------------------------------------------------------------------------------
```
struct textbuff-sid     \ -- len
```
Defines the length of a SID for a text buffer device\.
```
  gen-sid +             \ handle=buffer, reuse field names of GEN-SID
  int tb-len            \ length of buffer
  int tb-attribs        \ attributes
  int tb-wr             \ address to write to, set by SETPOS, WRITE
  int tb-rd             \ address to read from set by SETPOS, READ
end-struct
```

--------------------------------------------------------------------------------
```
: ff-tb         \ sid -- ; page/cls on display devices
```
This word is run by **`PAGE`**, **`CLS`** and **`FF-GEN`**\. It resets
\(empties\) the buffer\.

--------------------------------------------------------------------------------
```
: setpos-tb     \ x y mode sid -- ior
```
This word is run by **`SETPOS-GEN`**\. Mode controls the x and y
input values as follows\.

| Term | Description |
| --- | --- |
| 0 |    x = \#bytes written, y is ignored | 



| Term | Description |
| --- | --- |
| \-1 |   x=col, y=line for next character to be written | 



| Term | Description |
| --- | --- |
| \-2 |   x = \#bytes read, y is ignored | 



| Term | Description |
| --- | --- |
| \-3 |   x=col, y=line for next character to be read | 



--------------------------------------------------------------------------------
```
: getpos-tb     \ mode sid -- x y ior
```
This word is run by **`GETPOS-GEN`**\. Mode controls the x and y
return values as follows\.

| Term | Description |
| --- | --- |
| 0 |    x = \#bytes written, y = 0 | 



| Term | Description |
| --- | --- |
| \-1 |   x,y for next character to be written | 



| Term | Description |
| --- | --- |
| \-2 |   x = \#bytes read, y = 0 | 



| Term | Description |
| --- | --- |
| \-3 |   x,y for next character to be read | 



| Term | Description |
| --- | --- |
| \-4 |   x = addr, y = len of unread data | 



| Term | Description |
| --- | --- |
| \-5 |   x = base address, y = size of data area | 



--------------------------------------------------------------------------------
```
: ioctl-tb      \ addr len fn sid -- ior
```
This word is run by **`IOCTL-GEN`** and **`IOCTL-GIO`**\.
_Fn_ controls the meaning of _addr_, _len_ and the
_ior_ return value as follows:

| Term | Description |
| --- | --- |
| 0 |    Get buffer address: _addr_=0, _len_=0, _ior_=addr\. | 



| Term | Description |
| --- | --- |
| \-1 |   Set write pointer: _addr_=0, _len_=offset, _ior_=0\. | 



| Term | Description |
| --- | --- |
| \-2 |   Get write pointer: _addr_=0, _len_=0, _ior_=offset\. | 



| Term | Description |
| --- | --- |
| \-3 |   Set read pointer: _addr_=0, _len_=offset, _ior_=0\. | 



| Term | Description |
| --- | --- |
| \-4 |   Get read pointer: _addr_=0, _len_=0, _ior_=offset\. | 




#### Device Creation

--------------------------------------------------------------------------------
```
: initTextBuffSid       \ addr --
```
Initialise a previously allocated SID for
a text buffer to the default values\.

--------------------------------------------------------------------------------
```
: textbuff:     \ "name" -- ; Exec: -- sid
```
Create a memory buffer called _name_\.

--------------------------------------------------------------------------------
```
: SizedTextBuff \ size -- sid|0
```
Allocates and opens a SID with a buffer of size _size_
bytes, and returns the SID on success or 0 on failure\.

--------------------------------------------------------------------------------
```
: AllocTextBuff \ -- sid|0
```
Allocates and opens a SID with a default 16kb text buffer
and returns the SID on success or 0 on failure\.

--------------------------------------------------------------------------------
```
: FreeTextBuff  \ sid --
```
Closes the SID and frees memory allocated by **`AllocTextBuff`**
or **`SizedTextBuff`**\.
