: charinstring ( char c-addr n -- f )
\ true if char has been found in string
\ implement using scan?
    bounds ?do
        dup i c@ = if
            drop -1 unloop exit
        then
    loop drop 0 ;
T{ CHAR a s" hallo" charinstring -> -1 }T
T{ CHAR x s" hallo" charinstring ->  0 }T

: docgen-line? ( c-addr n -- flag )
\ *G check if the given string is a docgen-line
    -leading                \ skip leading whitespace
    4 < if drop 0 exit then \ not long enough
    s" \ *" s=              \ check for docgen prefix
;
T{ s" \ *G"         docgen-line? -> -1 }T
T{ s"       \ *G"   docgen-line? -> -1 }T
T{ s" \ some"       docgen-line? ->  0 }T
T{ s" 1"            docgen-line? ->  0 }T

: whereis-doc ( xt -- c-addr u offset length true | false )
\ *G Given the XT of a word this will return the filename string,
\ ** the offset in characters for the start of the line,
\ ** the length of the docgen comment in chars and TRUE
\ ** If the XT cannot be found, 0 is returned.
    (whereis) if
        >r 2dup r> \ save filename
        -rot r/o open-file throw >r
            r@ swap 1- skip-file-lines

            r@ file-position throw drop \ start of file-block

            \ ignore first line
            line-buffer /line-buffer r@ read-line throw 2drop

            0 \ dummy line-length
            begin
                drop \ drop last line-length
                line-buffer /line-buffer r@ read-line throw
                >r \ save success
                dup \ save line-length
                dup if \ EOF not reached
                    line-buffer swap docgen-line?
                then
            r> and 0=
            until
            \ get end of line
            r@ file-position throw drop
            swap - \ remove last line-length
            over - \ remove start, now we got the proper length

        r> close-file throw
        -1
    else
        0
    then ;

: read-file-chunk ( c-addr n n-start c-addr-chunk n-length -- c-addr-chunk n-length' )
\ *G read chunk of file into chunk-buffer
\ ** n-length' returns the number of bytes actually read
    { n-start c-addr-chunk n-length | fd -- }
    r/o open-file throw to fd
    n-start 0 fd reposition-file throw
    c-addr-chunk dup n-length fd read-file throw
    fd close-file throw
;

: read-file-chunk-allocate ( c-addr n n-start n-length -- c-addr-chunk n-chunk )
\ *G read chunk of file into newly allocated string
    >r r@ allocate throw
    r> read-file-chunk
;

: doc-chunk ( xt -- c-addr n -1 | 0 )
\ *G reads docgen comments from sourcefile into newly allocated buffer
    whereis-doc if
        ['] read-file-chunk-allocate silent-catch if
            0
        else
            -1
        then
    else
        0
    then
;

: type-file ( fileid -- )
\ *G Reads the file chunkwise, outputting it
    { | buf[ 256 ] -- }
    >r
    begin
        buf[ 254 r@ read-file throw
    ?dup while
        buf[ swap type
    repeat
;

defer doc-personality
assign docgen_vt100 to-do doc-personality
\ assign docgen_html to-do doc-personality

: docgen-wrap-chunk ( c-addr-source n-source -- fileid )
\ Wrap  chunk into docgen markup
\ return fd
\ Hint: does not run -docgen, which would close the temp files,
\       thereby deleteing them
    +docgen doc-personality
    tempfile >r
    s" \ *TF"   r@ write-line throw
    r@ write-line throw
    0 0 r@ reposition-file throw
    r> parse-file

\ read output fileid
    DG_FileId
;

: docgen-wrap-chunk-buffer ( c-addr-source n-source -- c-addr-output n-output )
\ returns newly allocate buffer containing the docgen-generated output
    docgen-wrap-chunk
    dup file-position throw drop >r \ drop upper size, save lower size
    dup 0 0 rot reposition-file throw

    r@ ProtAlloc dup rot
    r> swap read-file throw
;

: docgen-type-chunk ( c-addr-source n-source -- )
\ types chunked output
    docgen-wrap-chunk >r
    0 0 r@ reposition-file throw
    r> type-file

    -docgen \ closes input and output file
;

: doc-raw ( <parse-name> -- )
\ *G parse name, and display docgen comments in terminal
    ' doc-chunk if
        over >r
        type
        r> protfree
    else
        ." No documentation found"
    then
;

: xt-doc-string ( xt -- c-addr n -1 | 0 )
\ *G return xt's docgen-parsed comment as newly allocated string
    doc-chunk if
        docgen-wrap-chunk-buffer -1
    else
        0
    then
;    
: doc-string ( <parse-name> -- c-addr n -1 | 0 )
\ *G parsing version of *\fo{xt-doc-string}
    ' xt-doc-string
;

: doc ( <parse-name> -- )
\ *G parse name, and display parsed docgen in terminal
    ' doc-chunk if
        docgen-type-chunk
    else
        ." Not found"
    then
;

: doctest ( -- )
\ *G just a word to test some formatting
\ *T This is a section title.
\ *P some text in this section
\ *S This is a section sub-title.
\ *P some text in this section
\ *N This is a section sub-sub-title.
\ *P some text in this section
\ *H This is a simple heading.
\ *P some text in this section
\ *P Some more *\b{bold}, or *\i{italic}, or *\f{fixed}, or *\fo{forth} text.
\ ** Visit us at *\href{"https://vfxforth.com" "vfxforth.com"}
\ ** See our image: *\image{"Ultimate 12 Months" "https://vfxforth.com/images-shop/product/products/subscriptions/ultimate/12/vfx-ultimate-12" "jpg" 100 100}
\ ** thanks!
\ *P other image: *\image{"logo text 1" "technics" "jpg" 50 100}
\ *L
\ *(
\ *B Apples
\ *B Bananas
\ *B Pineapples
\ *)
\ *DF Fruits edible suggar delivery vehicles
\ ** can be very complicated thingies
\ ** or simple?
\ *DF Vegetables edible *\i{sadness} delivery vehicles
\ *E This is an example
\ ** 2nd line of example
\ ** 3rd line of example
\ *E Next example (single line)
\ *E Last example
\ ** 2nd line of last example
\ *C some code
\ ** more code
;
