$400 constant /PromptBuff
/PromptBuff SizedTextBuff constant PromptBuff

: silent-catch ( xt -- )
\ *G if a signal is received during a catch,
\ ** catch shows the registers and pauses.
\ ** silent-catch does neither.
    SigShow? SigPause? 2>r  \ preserve current signal-handler setup
    0 to SigShow?
    0 to SigPause?
        catch
    2r> to SigPause? to SigShow?
;

: addr? ( x -- flag )
\ *G check if x is an address
\ *E 0 addr? \ -- 0
\ *E s" hi" drop addr? \ -- -1
    ['] c@ silent-catch nip 0=
;

: string? ( c-addr n -- flag )
\ *G check if c-addr n is a string.
\ ** this is done by using *\fo{addr?} at the string's limits,
\ ** followed by checking for ascii-characters in the string.
\ ** Also the size is expected to be between 1 and 80 chars.
    dup 1 81 within 0= if 2drop 0 exit then \ ensure size
    bounds
    2dup addr? swap addr? and 0= if \ ensure address space
        2drop 0 exit
    then
    -1 -rot do  \ printable-chars-flag
        \ TODO: allow for UTF-8
        i c@ $20 $7F within and
    loop
;

: addr. ( x -- )
\ *G Display an address prefixed by $ in hex.
\ ** groups of 4 digits are separated by *\f{:}.
    [CHAR] $ emit
    base @ >r
    hex
    0 <#
    begin
        # # # #
        2dup or 0= dup 0= if
            [char] : hold
        then
    until
    #> type
    r> base !
;

: string. ( c-addr n -- )
\ *G Displays a string in double-quotes
    [CHAR] " emit type [CHAR] " emit space
;

: .s-item ( x -- )
\ *G Smart-display a cell on the stack:
\ ** Words are displayed by `<name>
\ ** Addresses are prefixed by $
\ ** Other values are using .
    dup InForth? if
        dup ip>nfa? if [char] ` emit .name drop exit then
    then
    dup addr? if
        addr. exit
    then
    .
;

: .s-item ( n1 -- n2 )
\ *G smart print item at stack-depth n1.
\ ** pick *\fo{n1}, perform smart print.
\ ** Return new depth *\fo{n2}.

    dup pick addr? 0= if    \ no address -> show number
        dup pick . 1- exit
    then  

    dup 1 > if  \ check for string if enough depth
        dup pick over pick string? if
            dup pick over pick string.
            2 - exit
        then
    then

    dup 1- swap pick \ new-depth value

    dup InForth? if
        dup ip>nfa? if [char] ` emit .name drop exit then
    then

    addr. space
;

: .s-items ( -- )
\ Smart-display the data stack
    depth 0 max begin
        dup 0>
    while
        .s-item
    repeat drop
;

: <depth> ( -- )
\ display current depth in pointy-brackets
    ." <" depth 0 .r ." > " ;

: ..s ( -- )
\ *G display number of stack items and the items
    <depth>
    .s-items
;

: .f-item ( r -- )
\ *G Display float *\i{f} followed by a space. Floating-point
\ ** is used if the exponent is in the range -4 to 5 otherwise
\ ** use scientific notation. Non-essential zeros and signs are
\ ** removed.
    fdup BadFloat? if
        fdrop type space exit
    then
    >exp -4 6 within if
        0 (f.) type ." E0 "
    else
        0 (fs.) type
    then
;

: .f-items ( -- )
\ fisplay floating point stack
    fdepth 0 max dup 0 ?do
        dup i - 1- fpick .f-item
    loop drop
;

: cls-.s-items
    cls
    .s-items
    -1 italic-text
    .f-items
;

: .s-items>string
    ['] cls-.s-items PromptBuff catch>gio
;

: term-prompt 
    update-terminal-size
    cyan text-color -1 bold-text    \ nice juicy bold green
    ."  ok"  depth if
      ." -" depth 0 .r              \ data stack
    then
    fdepth if
        ." -f" fdepth .             \ floating point stack
    else
        space
    then
    0 bold-text
\ 0 italic-text
    reset-colors -1 faint-text              \ faint for stack content

    .s-items>string throw >r

\ jump forward
\ TODO: use spaces or at-xy?
    terminal-w @ cursor-position? drop -
    4 + \ remove itelic escape sequence from .f-items
    r@ - 1+
    dup 0< if   \ stack picture longer than screen
        ." …"
        r> swap negate 1 + /string type
    else        \ right-align by spaces
        spaces
        r> type
    then

    reset-text                              \ restore
;

: term-pre-input ( -- )
    cr
    -1 bold-text    \ bold input
;

: term-post-input ( -- )
    reset-text      \ restore
;

: (~~)
    -1 faint-text
    cr ." ~~ "
    type            \ CurrSourceName
    [char] : emit . \ line#

    dup InForth? if
        dup ip>nfa? if [char] ` emit .name drop then
    then

    cyan text-color 0 faint-text -1 bold-text
    <depth>
    reset-text
    .s-items
;

: ~~ ( -- )
\ *G Display file name and line number,
\ ** if applicable show current definition's name
\ ** followed by *\fo{..s}
    last-colon-xt @ postpone literal
    line# @ postpone literal
    CurrSourceName postpone 2literal
    POSTPONE (~~)
; immediate

' term-prompt is .prompt
' term-pre-input is .PreInput
' term-post-input is .PostInput

