\ FPorg.fth - organisation of VFX floating point

((
Copyright (c) 2022, 2023
Wodni & Pelc GmbH
Ödenburgerstraße 21/6/9,
1210 Wien
Austria

Copyright (c) 2000-2009, 2010, 2011, 2015, 2016, 2018, 2019, 2020
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441


To do
=====

Change History
==============
20230125 SFP001 First release
))

\ ========
\ *! fporg
\ ========
\ *T VFX Floating Point organisation

\ **********
\ *S Introduction
\ **********
\ *P 64 bit VFX Forth systems for AMD64/Intel64 processors
\ ** released from January 2023 onwards can use one of several
\ ** floating point packages. The *\fo{Extern:} interface is
\ ** adjusted so that FP doubles and FP floats are converted
\ ** to the form required by the native ABI. Locals are also
\ ** according to the selection. You, the user must make this
\ ** selection. There are are three choices.
\ *(
\ *B Lib/x64/FPSSE64S.fth - Uses the SSE2 instruction set with 64 bit
\ ** floats and an external FP stack (R13=FSP). This is the form
\ ** used by operating systems.
\ ** However, without an SSE optimiser (in development, no
\ ** timescale defined) floats are slow and of limited
\ ** precision.
\ *B Lib/x64/Ndpx64.fth - Uses the NDP (80386+) instruction set
\ ** in 80 bit mode and only the eight-level internal stack.
\ ** For most uses, this is the fastest and most accurate package.
\ ** However, you must test thoroughly for floating point stack
\ ** overflow.
\ *B Lib/x64/Hfpx64 - Uses the NDP (80386+) instruction set
\ ** in 80 bit mode and an external FP stack (R13=FSP). This
\ ** often provides the best speed/accuracy tradeoff.
\ *)

((
0 value FpSystem	\ -- n
\ *G The value *\fo{FPSYSTEM} defines which floating point pack is installed
\ ** and active. See the Floating Point chapters for
\ ** further details.
\ ** Each floating point pack defines its own type as follows:
\ *(
\ *B 0 constant NoFPSystem
\ *B 1 constant HFP387System (also 64 bit)
\ *B 2 constant NDP387System (also 64 bit)
\ *B 3 constant OpenGL32System (obsolete)
\ *B 4 constant SSE64System (default)
\ *)
\ *P When *\i{FPSystem} changes, the following files that use *\i{FPSystem}
\ ** are affected:
\ *C   Extern*.fth  kernel64.fth  Tokeniser.fth
\ *C   Lib/x64/Ndpx64.fth  Lib/Hfpx64  Lib/x64/FPSSE64.fth
\ *P At present, only 0, 1, 2 and 4 are valid values of *\i{FPSystem}
\ ** in 64 systems.
))


\ ********************
\ *S Extern: and CallDef: interfaces
\ ********************
\ *P The *\fo{EXTERN:} interfaces use *\fo{FPSYSTEM} to show how floats
\ ** and doubles are generated. The float is taken from its stack,
\ ** converted as required and placed in the register of memory
\ ** location required by the call. On return, the return result
\ ** is converted back as required by *\fo{FPSYSTEM}.


\ ************
\ *S FP locals
\ ************
\ *P FP locals are held in memory in the form defined by *\fo{FPSYSTEM}.


\ **********************
\ *S Only one FP package
\ **********************
\ *P Only one float pack can be installed. This is checked at compile time.
\ ** To replace the floating point pack use:
\ *E integers
\ ** remove-FP-pack
\ ** include <sourcefile>

\ *P By default, x64 systems come with Lib/x64/Ndpx64.fth compiled.
\ ** At present this offers the best options for precision and speed. 


\ ======
\ *> ###
\ ======



