\ defmac.fth - default macros

((
Copyright (c) 2004, 2007, 2011, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone is:
  011 44 23 8063 1441


To do
=====

Change history
==============
20120225 MPE003 Moved more into kernel.fth.
20110517 MPE002 Fixed edit on error action.
20071211 SFP001 Made editor configurable.
))

decimal

\ ======
\ *> tmacro
\ *S Linux specifics
\ ======
\ *P The code described here is specific to VFX Forth for Linux.
\ ** Do not rely on any of the words documented here being
\ ** present in any other VFX Forth implementation.

also substitutions definitions

(( now in KERNEL.FTH
#256 buffer: LOAD_PATH  0 LOAD_PATH c!
[undefined] LIBRARYDIR [if]
#256 buffer: LIBRARYDIR  0 LIBRARYDIR c!
[then]
#256 buffer: BasePath  0 BasePath c!

: lib LIBRARYDIR ;
: bin LOAD_PATH ;
#256 buffer: LOCATE_PATH  0 LOCATE_PATH c!
#256 buffer: LOCATE_LINE  0 LOCATE_LINE c!
: f locate_path ;
: l locate_line ;
))

previous definitions

also substitutions

: GetExeName	\ caddr --
\ *G Get the fully qualified name of the executable. It is saved
\ ** as a counted string at *\i{caddr}.
  { | pname[ 64 ] exename[ 512 ] -- }
  base @ >r decimal
  s" /proc/" pname[ zplace		\ make "/proc/123456/exe"
  getpid (u.) pname[ zAppend
  s" /exe" pname[ zAppend
  pname[ exename[ realpath drop
  exename[ zcount #255 min rot place
  r> base !
;

: InitMacros	\ --
\ *G Initialises the directory macros. Run at start up.
  { | buf[ MAX_PATH ] -- }
  buf[ GetExeName  buf[ stripFilename
  buf[ LOAD_PATH $move
  S" /../Lib" buf[ append
  buf[ LIBRARYDIR $move
  cDot to IncDir
;
' InitMacros AtCold

previous


\ ****************************
\ *S Editor and LOCATE actions
\ ****************************

#256 buffer: editor$	\ -- addr
\ *G A buffer holding the path and name of the preferred editor
\ ** as a counted string, e.g.
\ *C   /bin/vi
  0 editor$ c!

0 value EditInBackground? \ -- flag
\ *G Set true to run editor in backgdound (detached).
\ ** Hint: running vim in the terminal requires this to be false,
\ ** where as GUI editors like GVim or gedit will work fine in background.

0 value EditOnError?	\ -- flag
\ *G Set true to call the editor on an error.

: editor-is	\ "<editor-name>" --
\ *G Set your preferred editor, e.g.
\ *C   editor-is /bin/vi
  GetPathSpec editor$ place
;

: .ed		\ --
\ *G Display the name of your preferred editor
  cr  editor$ c@ if
    ." Editor is: " editor$ count type
  else
    ." No editor set"
  endif
;

#256 buffer: locate$	\ -- addr
\ *G A buffer holding the macro expansion required to edit a
\ ** specific line of a file. This information is used by
\ ** *\fo{LOCATE}. In the example below the macros
\ ** *\i{%f%} and *\i{%l%} will be replaced by the file name and
\ ** line number.
\ *C   %f% -# %l%
  s" %f% -# %l%" locate$ place

max_path buffer: EditCommand$	\ -- addr

: edit$		\ -- cstring
\ *G If the preferred editor has been set, return the program name,
\ ** otherwise return the default editor string for *\i{vi}.
  editor$ c@				\ select editor
  if  editor$  else  c" /bin/vi"  endif
  count EditCommand$ ExpandMacro 2drop
  EditCommand$
;

: $edit		\ cstring --
\ *G Edit the file provided as a counted string. The editor is run
\ ** detached.
  edit$ swap $ExpandMacros
  EditInBackground? if
     >r  s"  &" r@ append  r>   \ run detached
  then
  $linux
;

: edit		\ "<filename>" --
\ *G Edit the file whose name follows in the input stream, e.g.
\ *C   EDIT release.txt
  { | temp[ #256 ] -- }
  GetPathSpec #255 min temp[ place  temp[ $edit
;

: .edlocate	\ --
  { | temp[ MAX_PATH ] }
  locate$ count temp[ ExpandMacro 2drop
  edit$ temp[ $linux
;
\ assign .edlocate to-do .locate

: (EditOnError)	\ -- ; run editor on error
\ *G Edit the file at an error, using the contents of the variables
\ ** *\fo{'SOURCEFILE} and *\fo{LINE#}.
  'SourceFile @ -1 =
  line# @ -1 = or
  EditOnError? 0= or
  if  exit  endif
  CurrSourceName line# @ LocateInfo	\ set locate macros
  .edlocate				\ go do it
;

: SetLocate	\ --
\ *G Tells VFX Forth how your editor can be called to go a
\ ** particular file and line. Use in the form
\ *C   SetLocate  <rest of line>
\ *P where the text after *\fo{SetLocate} is used to define
\ ** how parameters are passed to the editor, e.g. for Emacs,
\ ** use
\ *C   SetLocate +%l% "%f%"
\ *P The rest of line following *\fo{SetLocate} is used as the
\ ** editor configuration string. Within the editor configuration
\ ** string '%f%' will be replaced by the file name and '%l%' will be
\ ** replaced by the line number. If you use file names with spaces,
\ ** you should put quotation marks around the *\f{%f%} text macro.
\ ** You do not need to finish the line with " &" to run the editor
\ ** detached from VFX Forth - *\fo{LOCATE} adds this for you.
  0 word count locate$ place		\ Collect string
  assign .edlocate to-do .locate	\ locate through editor
  1 to EditOnError?
  assign (EditOnError) to-do EditOnError \ locate on error
;


\ ======
\ *> ###
\ ======

decimal

