\ SIGNAL.FTH - Linux signal handling

((
Copyright (c) 2007, 2009, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140804 SFP002 ARM conversion.
20090416 SFP001 Added Exception frame dump.
))

only forth definitions
decimal


\ ===========
\ *> lintools
\ *S Linux signal handling
\ ===========

\ *************
\ *N Structures
\ *************
\ *P Signal numbers
\ ** See linux/include/uapi/asm-generic/sigcontext.h
\ *[
#define _NSIG           32
#define _NSIG_WORDS	 1	( number of cells to hold _NSIG bits )

#define SIGHUP		 1
#define SIGINT		 2
#define SIGQUIT		 3
#define SIGILL		 4
#define SIGTRAP		 5
#define SIGABRT		 6
#define SIGIOT		 6
#define SIGBUS		 7
#define SIGFPE		 8
#define SIGKILL		 9
#define SIGUSR1		10
#define SIGSEGV		11
#define SIGUSR2		12
#define SIGPIPE		13
#define SIGALRM		14
#define SIGTERM		15
#define SIGSTKFLT	16
#define SIGCHLD		17
#define SIGCONT		18
#define SIGSTOP		19
#define SIGTSTP		20
#define SIGTTIN		21
#define SIGTTOU		22
#define SIGURG		23
#define SIGXCPU		24
#define SIGXFSZ		25
#define SIGVTALRM	26
#define SIGPROF		27
#define SIGWINCH	28
#define SIGIO		29
#define SIGPOLL		SIGIO
#define SIGPWR		30
#define SIGSYS		31
#define	SIGUNUSED	31
\ *]

struct /sigcontext	\ -- len
\ *G CPU *\b{sigcontext} structure.
\ ** See *\i{linux/arch/arm/include/asm/sigcontext.h}.
\ ** Note that this is *\b{not} the same as a *\b{ucontext} structure.
  int sc.trapno		\ $30
  int sc.err
  int sc.oldmask	\ $50
  int sc.r0
  int sc.r1
  int sc.r2
  int sc.r3
  int sc.r4
  int sc.r5
  int sc.r6
  int sc.r7
  int sc.r8
  int sc.r9
  int sc.r10
  int sc.r11
  int sc.r12
  int sc.r13
  int sc.r14
  int sc.pc
  int sc.cpsr
  int sc.fault_address
end-struct

struct /stack_t		\ -- len
\ *S Stack description structure.
  int st.sp		\ pointer to ...
  int st.flags		\ ...
  int st.size		\ ..
end-struct

((
struct /gregset_t	\ -- len
\ *G CPU *\b{gregset_t} structure.
\ ** See *\i{/usr/src/linux-2.6.8-24/include/asm-i386/ucontext.h}.
  int gr.gs		\ really a 16 bit item plus a 16 bit dummy, use w@ and w!
  int gr.fs		\ really a 16 bit item plus a 16 bit dummy, use w@ and w!
  int gr.es		\ really a 16 bit item plus a 16 bit dummy, use w@ and w!
  int gr.ds		\ really a 16 bit item plus a 16 bit dummy, use w@ and w!
  int gr.edi
  int gr.esi
  int gr.ebp
  int gr.esp
  int gr.ebx
  int gr.edx
  int gr.ecx
  int gr.eax
  int gr.trapno
  int gr.err
  int gr.eip
  int gr.cs		\ really a 16 bit item plus a 16 bit dummy, use w@ and w!
  int gr.eflags
  int gr.esp_at_signal
  int gr.ss		\ really a 16 bit item plus a 16 bit dummy, use w@ and w!
end-struct

struct /mcontext	\ -- len
\ *G System *\b{uncontext} structure returned by signal handlers.
\ ** for Linux i32 this is the same as the *\b{/sigcontext} structure.
  /gregset_t field mc.gregs
  int mc.*fpregs
  int mc.oldmask
  int mc.cr2
end-struct
))

struct /ucontext	\ -- len
\ *G System *\b{ucontext} structure returned by signal handlers.
\ ** See *\i{linux/arch/arm/include/asm/ucontext.h}.
  int uc.flags
  int uc.link
  /stack_t field uc.stack
  /sigcontext field uc.mcontext
  #128 field uc.sigmask		\ 1024 bits
  7 + -8 and			\ 8 byte aligned
  128 cells field uc.regspace	\ coprocessors inc FPUs
end-struct


struct /sigaction	\ -- len
\ *G System *\b{sigaction} structure.
\ ** See *\i{linux/arch/arm/include/asm/signal.h}.
  int sa.action			\ action of signal, entrypoint
  128 field sa.mask		\ 1024 bits
  int sa.flags			\ special flags
  int sa.restorer		\ restore handler
end-struct

4 constant SA_SIGINFO	\ for sa.mask

Extern: int sigaction(int signum, void * act, void * oldact);
\ act can specify either
\   void sa_handler( int signum );
\ or
\   void sa_siginfo( int signum, siginfo_t * siginfo, ucontext_t * uc );
\ return 0 on success, -1 on error
Extern: int sigemptyset(void * set);
\ generate an empty signal set.
Extern: int sigaddset(void *set, int signum);
Extern: int raise(int sig);


\ ******************
\ *N Signal handling
\ ******************
\ *P The following is the stack structure seen by the *\b{siginfo}
\ ** signal handler.
\ *E
\ **    |             |
\ **    +-------------+
\ **    | Return code |		8 bytes of code to return to kernel context at
\ **    |             |		end of the handler run via a special system call
\ **    |             |		(now a dummy left as a signature for debuggers
\ **    |             |		as many systems don't allow execution from the
\ **    |             |		stack)
\ **    +-------------+
\ **    | Floating    |		FPU state if FPU has been used by this process
\ **    | point state |
\ **    +-------------+
\ **    | User        |		POSIX extension user context for the signal
\ **    | Context     |         handler.  Includes register contents etc, many
\ ** +->|             |		of which are modifiable by the signal handler
\ ** |  +-------------+
\ ** |  | Siginfo     |		Traditional signal information structure.
\ ** |  | structure   |<-+	Duplicates/simplifies some of User Context
\ ** |  +-------------+  |
\ ** +--| void *      |	 |	Pointer to POSIX user context
\ **    +-------------+  |
\ **    | siginfo_t * |--+	Pointer to Siginfo structure
\ **    +-------------+
\ **    | Signum      |		Actual signal number generated
\ **    +-------------+
\ **    | Return      |		Originally this pointed at the stack based
\ **    | address     |		return code above.  Now points directly at the
\ **    |             |		sigreturn syscall gate in the vDSO
\ **    +-------------+


create sigNames	\ -- addr
\ *G Holds the signal numbers and names as counted strings.
   0 , ", ???"	      1 , ", SIGHUP"    2 , ", SIGINT"     3 , ", SIGQUIT"
   4 , ", SIGILL"     5 , ", SIGTRAP"   6 , ", SIGIOT"     7 , ", SIGBUS"
   8 , ", SIGFPE"     9 , ", SIGKILL"  10 , ", SIGUSR1"   11 , ", SIGSEGV"
  12 , ", SIGUSR2"   13 , ", SIGPIPE"  14 , ", SIGALRM"   15 , ", SIGTERM"
  16 , ", SIGSTK"    17 , ", SIGCHLD"  18 , ", SIGCONT"   19 , ", SIGSTOP"
  20 , ", SIGTSTP"   21 , ", SIGTTIN"  22 , ", SIGTTOU"   23 , ", SIGURG"
  24 , ", SIGXCPU"   25 , ", SIGFSXZ"  26 , ", SIGVTALRM" 27 , ", SIGPROF"
  28 , ", SIGWINCH"  29 , ", SIGIO"    30 , ", SIGPWR"    31 , ", SIGSYS"

: .sigName	\ n --
\ *G Given a signal number, display its name.
  dup #31 u>
  if  drop 0  endif
  sigNames swap 0 ?do
    cell +  count + aligned
  loop
  cell + count type
;

: .RSitem	\ x --
\ *G Display an item retrieved from the faulting return stack.
  cr ."  $" dup .dword			\ display stack item as number
  dup ['] firstname u>			\ above start of dictionary
  over here u< and if			\ below end of dictionary
    space  ip>nfa .name			\ show word name
  else
    drop
  endif
; doNotSin

: .SigContext	\ sc --
\ *G Display data from the sigcontext structure.
  cr
  ."  R0 =" dup sc.r0 @ .dword  ."  R1 =" dup sc.r1 @ .dword
  ."  R2 =" dup sc.r2 @ .dword  ."  R3 =" dup sc.r3 @ .dword
  cr
  ."  R4 =" dup sc.r4 @ .dword  ."  R5 =" dup sc.r5 @ .dword
  ."  R6 =" dup sc.r6 @ .dword  ."  R7 =" dup sc.r7 @ .dword
  cr
  ."  R8 =" dup sc.r8 @ .dword  ."  R9 =" dup sc.r9 @ .dword
  ."  R10=" dup sc.r10 @ .dword ."  R11=" dup sc.r11 @ .dword
  cr
  ."  PSP=" dup sc.r12 @ .dword ."  RSP=" dup sc.r13 @ .dword
  ."  LR =" dup sc.r14 @ .dword ."  PC =" dup sc.pc @ .dword
  cr
  ."  Flt=" dup sc.fault_address @ .dword
  ."  CPSR=" dup sc.cpsr @ .dword

  cr ." ----- LR -----"
  dup sc.r14 @ .RSitem
  cr ." --- RS top ---"
  dup sc.r13 @ 8 cells bounds
  do  i @ .RSitem  cell +loop
  drop
; doNotSin

: SigThrow	\ --
\ *G Runs the O/S THROW action.
  #-57005 throw
;

variable #SigNest	\ --
\ Counts Signal nesting level.

: .sigTrap	\ signum *siginfo *ucontext --
\ ." signum=" 2 pick .dword  ."  *siginfo=" over .dword  ."  *ucontext=" dup .dword
\ key drop
  nip					\ discard siginfo
  uc.mcontext				\ point at CPU context
  dup .SigContext			\ display context
  cr ." Signal number " swap .sigName	\ display signal number
  cr ." at address " dup sc.pc @ dup .dword
     ." , probably in " ip>nfa .name
  cr
;

3 0 callback: SigGenTrap	\ signum *siginfo *ucontext --
\ *G Generic trap handler that causes a *\fo{-57005 THROW} on return.
\ ** Callbacks are documented in a separate section of the manual.

1 value -NestedSigs?	\ -- x
\ *G Set non-zero to cause an exit if a nested signal exception
\ ** occurs in *\fo{(SigGenTrap)} below.
1 value SigPause?	\ -- x
\ *G Set non-zero to cause a pause when a signal is processed
\ ** in *\fo{(SigGenTrap)} below.

create restartData
  up@ ,  r0 @ ,  s0 @ ,

: setRestart	\ --
\ Set up the restart data area.
  restartData  up@ over !  r0 @ over cell + !  s0 @ swap 2 cells + !
;
' setRestart AtCold

: doRestart	\ --
  restartData  dup @ up!  dup cell + @ rp!  2 cells + @ sp!
  ip-default @ ip-handle !  op-default @ op-handle !
  quit
;

: ?SigBye	\ --
  begin key? while key drop repeat
  cr ." Press E to exit, R to restart, other to continue: "
  case  key upc
    [char] E of  (bye)  endof
    [char] R of  doRestart  endof
  endcase
;

: (SigGenTrap)	\ signum *siginfo *ucontext --
\ *G Action of *\fo{SigGenTrap}. Displays an error message. On
\ ** return to Linux, a Forth *\fo{THROW} will occur.
  #SigNest @ if				\ skip display
    nip nip  #SigNest off		\ clear nesting
    -NestedSigs? if			\ check for permission
      cr ." Exiting on nested signal" cr
      ['] (bye) swap sc.pc !  exit
    endif
  else
    #SigNest incr  .sigTrap  #SigNest decr
    SigPause?
    if  ?SigBye  endif
  endif
  ['] SigThrow swap sc.pc !		\ force return to SigThrow
;
assign (SigGenTrap) to-callback SigGenTrap

: setSignal	\ callback signum --
\ *G Set a signal handler to execute the given callback. The
\ ** callback must have the stack effect *\fo{( signum *siginfo *ucontext -- )}
  {: | act[ /sigaction ] -- :}
  act[ sa.mask sigemptyset drop		\ clean masks
\  act[ sa.mask over sigaddset drop	\ add this signum
  SA_SIGINFO act[ sa.flags !		\ which type of handler
  swap act[ sa.action !			\ what to run
  act[ NULL sigaction drop		\ set it
;

: setSigTraps	\ --
\ *G Install the *\fo{SigGenTrap} signal handler for signals SIGILL,
\ ** SIGFPE and SIGSEGV. Performed at startup.
  #SigNest off
  SigGenTrap SIGILL setSignal
  SigGenTrap SIGFPE setSignal
  SigGenTrap SIGSEGV setSignal
\  SigGenTrap SIGUSR1 setSignal
;
' setSigTraps AtCold

\ Check for default I/O device
ip-default @ 0= [if]
  default-console  dup ip-default !  op-default !
[then]


\ ======
\ *> ###
\ ======

decimal

