/* Test file to make a shared library */

typedef struct _Entry
{
	struct _Entry * link;
	char * name;
	long value;
//	int value;
}Entry;

/* The table full of preprocessor symbols & their values... */

extern Entry entry[];

/* _GNU_SOURCE gets us goodies like strcasestr() */
#define _GNU_SOURCE
#include <string.h>
#include <stdio.h>
#include <stdint.h>

enum {FALSE, TRUE};

// Hash variables

// Create a hash table of the names for quick lookup.

#define HASHSIZE 8197 // 2003

Entry *hashTable[HASHSIZE];

int numconsts = 0;
static int hashBuilt = 0;

unsigned int hash(char *name)
{
	unsigned int h = 0;
	char c;

	while ( (c = *name++) )
		h += c;
	return h % HASHSIZE;

}

// In some editions of glibc, strcasestr() is broken.
// To avoid problems, we use our own.
// The original version of this code is public domain:
// http://code.google.com/p/wmii/source/browse/lib/libstuff/util/strcasestr.c
char * repl_strcasestr(const char *dst, const char *src)
{
        int len, dc, sc;

        if(src[0] == '\0')
                return (char*)(uintptr_t)dst;

        len = strlen(src) - 1;
        sc  = tolower(src[0]);
        for(; (dc = *dst); dst++) {
                dc = tolower(dc);
                if(sc == dc && (len == 0 || !strncasecmp(dst+1, src+1, len)))
                        return (char*)(uintptr_t)dst;
        }
        return NULL;
}

void BuildHash()
{
	int i;

	for (i = 0; entry[i].name; i++)
	{
		Entry **bucket = &hashTable[hash(entry[i].name)];
		entry[i].link = *bucket;
		*bucket = &entry[i];
		numconsts++;
	}

	hashBuilt = 1;	/* Mark it as built so we don't do it again */
}



void CountHash()	// Display results of hashing
{
	int sizes[21];
	int i, n, maxi=0;
	Entry *ep;

	for (i = 0; i < 21; i++)
		sizes[i] = 0;

	for (i = 0; i < HASHSIZE; i++)
	{
		n = 0;
		for (ep = hashTable[i]; ep; ep = ep->link)
			n += 1;
		if (n > 20)
		{	if (n > maxi) maxi = n;
			n = 20;
		}

		sizes[n] += 1;
	}

	for (i = 0; i < 21; i++)
		printf("%2d: %4d\n", i, sizes[i]);

	printf("Max = %d\n", maxi);
}



// Find an entry in the hash table. Return NULL if not found.

Entry *FindHash(char *name)
{
	Entry *ep = hashTable[hash(name)];

	while (ep && strcmp(name, ep->name))
		ep = ep->link;
	return ep;
}




long VFXSuppFindConstant(char *addr, int *value)
{
	char name[256];
	Entry *ep;

        int len = strlen(addr);

	if (len <= 0 || len > 255)
		return FALSE;

	memcpy(name, addr, len);
	name[len] = 0;

	ep = FindHash(name);
	if (!ep) return FALSE;
	*value = ep->value;
	return TRUE;
}

long VFXSuppFindSimilar(char *inpaddr, int index, char *outaddr)
{
    int len = strlen(inpaddr);

    if (len <= 0 || len > 255)
        return -1;
    if (index < 0 || index >= numconsts)
        return -1;

    for (  ; index < numconsts; index++)
    {
    	  // In some editions of glibc, strcasestr() is broken.
    	  // To avoid problems, we use our own.
        if (repl_strcasestr(entry[index].name, inpaddr))
            break;
    }

    if (index == numconsts)
        return -1;

    len = strlen(entry[index].name);
    memcpy(outaddr, entry[index].name, len);
    outaddr[len] = 0;
    return index;
}

long VFXSuppNumConstants( void )
{
  return numconsts;
}


/* Library setip / teardown calls */

void __attribute__ ((constructor)) my_init(void)
{
    BuildHash();
}

void __attribute__ ((destructor)) my_fini(void)
{
    /* Nothing to do yet! */
}

