\ hoptx64.fth - Optimiser for x86_64 desktop
\ VFX Forth has NO free scratch registers.

((
Copyright (c) 1998-2007, 2008, 2009, 2017, 2018, 2020, 2021, 2023
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com


To do
=====
for DUP DP ! and friends, leave final TOS in REG not MEM.
Change PICK to use reference count instead of move.

Add FLAGSETID check to GENCMP1

Convert "<lit> +" to use "LEA dest, <lit> [src]" when src reg is not dreg.
preserve src in optimiser data block.
When src is dreg, set src in optimiser data block.
In both cases, use src reg in <lit>+@/!.
This will avoid many redundant register moves.
Move this to 68K compiler.


Change History
==============
20231030 SFP519 Added code intrinsics for R14/R15 fetch/store.
20230923 SFP518 Updated for selectable FPSYSTEM float packs.
20230106 SFP517 Corrected size in (INC/DEC).
20220223 SFP516 Increased items held on stack.
20211112 SFP515 Rewrote L@S and L@ to follow Intel recommendation
		not to use MOVSXD with a 32 bit destination.
20210527 SFP514 Corrected (COMPILE-REF) for NDCS words although
		this is out of spec.
20210114 SFP513 Changed way NOPs are laid.
20201222 SFP512 Converted 64 bit lits to reg before manipulation.
20200907 SFP511 Restored C_WITHIN.
20200818 SFP510 Fixed PUSH in INTARGS,
20200807 SFP509 Removed C_WITHIN code generation - fails somewhere.
20200626 SFP508 Prevented binary inline of RIP relative.
20200623 SFP507 Forced more sizes to default QWORD.
20200621 SFP506 Updated FP locals code for SSE.
20200529 SFP505 Added FSP@ and FSP!
20200330 SFP504 Added NSLWIDEN and NULWIDEN.
20181023 SFP502 Added C_GUARDEDSHUFFLE.
20181016 SFP501 More 64 bit conversion.
20170705 SFP500 AMD64 conversion.

20180510 SFP426 Removed obsolete words
20170530 SFP425 Optimised more user variables.
20161128 SFP424 Added C_3DUP.
20151110 SFP423 Fixed DEFER! typo.
20140414 SFP422 First fix of volatile bug.
20130515 SFP421 Corrected moves into EDX, EAX for division.
		Some moves incorrectly eliminated code.
20130209 SFP420 Improved LOADDEST to reduce shuffles.
20120222 SFP419 Added better generation for U< and U>.
		Added code generation for WITHIN.
20110720 SFP418 Protected system registers in indirect moves to
		prevent MOV edi, n [edi] and so on.
20110420 SFP417 Another tweak to DUP.
20101203 SFP416 Forced FlushCode to prevent eliminations. This is
		needed because FlushCode is used in unoptimised code
		at basic block boundaries.
20100914 SFP415	Added 2ROT generator.
		Optimised DUP further.
20090910 SFP414 Corrected token generation.
20090909 SFP413 Removed nxxWiden words. Put back because of CCS.
20090908 SFP412 Removed FASTDO switches, always using +FASTDO.
20090724 SFP411 Added code generators for U<= and U>=.
20090406 SFP410 Tokeniser changes.
20090303 SFP409	Added floating point locals.
20090301 SFP408 Corrected C_XM* for literals.
20090119 SFP407 Added WorkReg?
		Fixed bug in ReduceIndMem?
20081219 SFP406	Fixed OF code generation bug.
		Improved handling of INDMEMs.
20080815 SFP405 Enhanced ABS, added ?NEGATE.
20071018 SFP404 Added MIN and MAX optimisers.
		Factored 2LITS?
20070716 SFP403 Fixed LIT bug in C_XM*.
		Fixed HISETEDX:EAX.
20060911 SFP402 Fixed genCMP2.
20060724 SFP401 v4 with NO free registers
20060719 SFP098 Cleaned up ?REM-LITPLUS and its usage.
20060606 SFP097 Added P4 switch and applied it to CMOVE to use
		the target CMOVE and friends.
		Moved shuffle in C_LOOP
20041115 SFP096 Corrected for INDMEMs in division operations.
20041109 SFP095 Added loop head alignment.
20041105 SFP094 Added >R R@ optimisation.
		Improved LOOP +LOOP
20041102 SFP093 Added optimisers for UM* DEFER@ and DEFER!.
		Major overhaul to permit INDMEM resources to
		be placed on the working stack.
		Added bulletproofing to >R and 2>R.
20041102 SFP092 Brought into line with VFXW OPT386.FTH
... change numbers now match OPT386.FTH
20041022 SFP063 Added UM/MOD generator.
		Improved compares for CMP1.
20040605 SFP062 Treated BUFFER: in RAM systems as a VARIABLE.
20030917 SFP061 Removed literal shifts from / generator as these
		used floored division for negative dividends.
		Exposed ARSHIFT.
20030803 SFP060 Added D0= code generator.
20030512 SFP059 Fixed potential conflict when reusing resources
		that are part of an eliminatable move.
20020516 SFP058 Updated DO..LOOP to favour I/J.
                Major overhaul of EXECUTE optimiser.
20020320 SFP057	Prevented binary inlining of DO ... LOOP and friends
SFP056  29 Dec 2001	Ignore literal shift by 0.
SFP055	2 November 2001	Added xxx-BIT operators.
SFP054  27 Oct 2001	Updated PICK generator
SFP053	15 July 2001	Added OR! AND! XOR! BIC!
SFP052	11 July 2001	Restored 2 PICK optimisation.
SFP051	26 June 2001	Incorporated changes from VFX Forth version with
			much major hacking!
			- Removed FAST-USER switch.
			- Added FLAGSETID optimisation.
			- introduced redundant move elimination
			- Removed #CHANGED(WS) nightmare.
SFP050	15 Dec 2000	Added ROL and ROR
SFP049  26 July 2000	Improved COMDYAD for reg, reg
SFP048	15 July 2000	Added short forward branches
SFP047	14 July 2000	Made LOADDEST more intelligent. It now replaces
			the memory reference on the working stack by a
			register. Note that LOADNOS and LOADTOS do not
			guarantee that the resulting TOS is a destination
			register (dreg).
			Added C_WVARIABLE and C_VAL for v6.1
SFP046	6 June 2000	Enhanced C_- for literals. Refactored D+ D- for clarity.
			Added /STRING.
			Inhibited inlining in RP!.
SFP045	24 May 2000	Updated to v6.1 with C_C/V
SFP044	4 Feb 2000	Added EXECUTE
SFP043  1 Nov 1999	Changed TO-DO to parse
SFP042	20 Sept 1999	Changed COMPILE to lay LIT <name> COMPILE, as this
			reduces complexity in the target kernel.
			Replace C_COMPILE by C_COMPILE_NEW
SFP041  11 Sept 1999	Improved multiplies, added divide generators
			Added generators for M* / M/ SM/REM
			Recoded generator for *
SJC040  10 Sept 1999    Added code to allow the use of EDI as an optimiser
                        register when locals are not in use.
                        Adds some control definitions and effects:
                          c_:
                          c_makelvs
                        The use of this code is handled by the variable
                                <recycle-regs>
                        which is OFF by default (cos it's not fully tested)
...			Change numbers stepped to 040

SFP037  9 Sept 1999	Enhanced SP@ SP! UP@ UP! added LP@ LP!
SFP036	8 Sept 1999	Added MakeBestTOS and MakeBestNOS optimisations from OPT386.FTH
			Added FAST-DO and SLOW-DO switches
SFP035  20 August 1999  Added R> DROP and DOES> >R optimisations
                        Minor code tidy up.
SFP034  19 August 1999  Modified DISAMBIGUATE to take the minimum number
                        of items of the stack and restore it on exit. This
                        avoids errors when the stack is shuffled by
                        DISAMBIGUATE.
                        Added REG/LIT?
                        Replaced C_2! by a macro

SJC030  99/05/21        Added naive generators for UP@ UP! SP@ SP! RP@ RP!
SJC029  99/05/21        Added naive generators for CMOVE and CMOVE>
SJC028  99/05/20        Added naive generator for RPICK

SFP030  19 July 1999    Added C_+C C_-C and D+ D- code generators
                        Added C_M+
SFP029  14 July 1999    Fixed bug in comparing a literal against 0,
                        e.g. const IF ...
SFP028  23 May 1999     Added C_CVARIABLE
SFP027  8 Apr 1999      Added C_FIELD-CHILD
SFP026  1 Apr 1999      Added DISAMBIGUATE to avoid problem of:
                          VARIABLE V
                          : FOO V @ 1 V !  ;
                         in which V @ is not flushed before the V !
                         Disambiguation is what the compiler weenies call this
                         problem. Added to (STOROP) C_2! (INC/DEC)
                        Added code generator for S>D.
                        Added LIT + LIT + optimisation
                        Added LIT + @ and LIT + ! optimisations
SFP025  12 Mar 1999     Rewrote (corrected) LIT + optimiser in C_+
                        Changed name of a defining word called STOREOP to
                        STOROP since it collided with the storage operator
                        in the methods table. This broke C_LVARG when the local
                        operator was anything other than default.
                        Added code generators for >BODY BL NULL MAX_PATH TRUE
                        FALSE CELL+ CELL-
                        Added signed-extended W@ and C@ as W@S and C@S
                        Added NSWWIDEN NSBWIDEN NUWWIDEN NUBWIDEN to extend
                        and sign-extend 8 and 16 values on the stack
SFP024  10 Mar 1999     Corrected defaults for inline local generation
SFP023  9 Mar 1999      Added LIT + @ and LIT + ! optimisers
SFP022  18 Jan 1999     Added CHAR+ optimiser, LSHIFT, RSHIFT optimisers
SJC021  17 Jan 1999     Swtched USER base pointer from ECX to ESI
SFP020  15 Dec 1998     Added COMPILE,(T) C_COMPILE, and COMPILE,s
SFP019  20 Nov 1998     Added XC: words for 2>R 2R@ 2R>
SFP018  28 Oct 1998     Corrected shuffle routines. All completed moves,
                        exchanges, or inplaces, now set the working stack
                        resource to rDUMMY.
                        Local variables now default to inline.
SFP017  23 Oct 1998     Corrected compile time handling of literals
SFP016  7 Oct 1998      Corrected compile time action of C_4+
SFP015  27 Sep 1998     Added inlining
SFP014  11 Sep 1998     Rewrote s_xchg to avoid 15 cycle instruction
SFP013  18 Aug 1998     Added C_BUFFER:
SFP012  17 Aug 1998     Added code in UNMARK to substitute a dummy resource
                        in the working stack. This prevents a register being
                        falsely copied to the output stack when it is reused as
                        a scratch register.
SFP011  15 Aug 1998     Added PAGE-COMPILE-REF for bank support
SFP010  10 Aug 1998     Added 3DROP, 4DROP, UNLOOP, ON and OFF code generators.
SFP009  22 July 1998    Restored memory pool after ensuring that a stack
                        destination held in a memory pool resource is first
                        copied to a register. This forces the operation to
                        be performed in the register, and NOT at the memory
                        location itself. See LOADDEST, LOADTOS and LOADNOS.
                        Made >NUMBER(T) base sensitive.
SFP008  21 July 1998    Provided code generators for children of VALUE and
                        USER.
                        Prevented GETSHUFFLEREG from finding TOS.
                        Removed memory pool handling as they may only be used
                        as sources of operations. Use as destinations causes
                        the location to be updated, rather than a copy on the
                        working stack! Wrong!
SFP007  20 July 1998    Rationalised literal handling. Added memory pool to
                        add INSTR reg, mem handling such as VAR @ AND and to
                        reduce the shuffle overhead as this avoids using an
                        additional register.
SFP006  16 July 1998    Fixed fowward reference problems in DOLITERALxxx
SFP005  15 July 1998    Fixed ForceNewTOS+NOS
SFP004  2 July 1998     Moved XC: words and analogues from XDTC41-4
                        Fixed conditional branches taking literal values
SFP003  29 June 1998    Converted to cross compiler version
SFP002  4 June 1998     Converted from using strings to access assembler directly
SFP001  3 May 1998      First cut

The following deferred words in the kernel are modified by the optimiser:
DOCOLON,        --      lays the entry code for : definitions
DOSEMICOLON,    --      lays the exit code for ; definitions
DOEXIT,         --      lays the code EXIT in : definitions
CLIT,           n --    lays the code for a lteral in : definitions
))


\ ===========
\ *! opcontrl
\ *T X64 VFX Code Generator
\ ===========
\ *P The VFX code generator is a black box that simply does its job.
\ ** Some implementations may have switches for special cases.

\ *****************************
\ *S Enabling the VFX optimiser
\ *****************************
\ *P The optimiser can be enabled and disabled by the
\ ** words *\fo{OPTIMISED} and *\fo{UNOPTIMISED}. The state of the
\ ** optimiser can be detected by inspecting the variable
\ ** *\fo{OPTIMISING}.

\ ******************
\ *S Binary inlining
\ ******************
\ *P Binary inlining consists of copying the binary code for a word
\ ** inline without the final return instruction. This avoids the
\ ** overhead of the call and return instructions. It is useful for
\ ** very short coded instruction sequences. For high level definitions
\ ** the source inliner usually gives better results.

\ *P The VFX code generator gives some control over the use of
\ ** binary inlining, controlled by the word *\fo{INLINING ( n -- )}.
\ ** When the code generator has completed a word, the length of
\ ** the word is stored. When the word is to be compiled, its
\ ** length is compared against the value passed to *\fo{INLINING},
\ ** and if the length is less than the system value, the word is
\ ** compiled inline, with the procedure entry and exit code
\ ** removed. This avoids pipeline stalls, and is very useful
\ ** for short definitions.

\ *P By default four constants are available for inlining control,
\ ** although any number will be accepted by *\fo{INLINING}.
\ *E  NO INLINING               \ 0, binary inlining turned off
\ **  NORMAL INLINING           \ 12-16, ~10% increase in size
\ **  AGGRESSIVE INLINING       \ 255, useful when time critical
\ **  ABSURD INLINING           \ 4096, unlikely to be useful

\ *P You can use *\fo{INLINING} anywhere in the code outside a
\ ** definition.

\ ====================
\ *N Colon definitions
\ ====================
\ *P Any word that uses words that affect the return stack such as
\ ** *\fo{EXIT}, or takes items off the return stack that you
\ ** didn't put there in the same word, will automatically be
\ ** marked as not being able to be inlined.

\ *P Implementations that use absolute calls will disable inlining
\ ** of any word that makes an absolute call.

\ *P Note that when words are inlined, the effects may not be as
\ ** expected.
\ *E : A ... ;                  \ inlined
\ ** : B ... A ... ;            \ A inlined, B can be inlined
\ ** : C ... B ... B ... ;      \ A, B inlined, C can be inlined

\ ===================
\ *N Code definitions
\ ===================
\ *P By default *\fo{CODE} definitions are not marked for inlining
\ ** because the assembler cannot detect all cases which may upset
\ ** the return stack. If you want to make a code definition
\ ** available for binary inlining, follow it with the word
\ ** *\fo{INLINE}.
( *E    CODE <name>                                                     )
( **      ...                                                           )
( **    END-CODE InLine                                                 )

\ *************************
\ *S VFX Optimiser Switches
\ *************************
\ *P Some instructions are only available on later CPUs.
\ ** Note that CPU selection affects the assembler and the VFX code
\ ** code generator and compile time, *\b{not} the run time
\ ** instruction usage of your application. If you select a higher
\ ** CPU level than the application runs on, incorrect operation will
\ ** occur. The default selection is for the x64 instruction set.
\ *C  CPU=x64     \ -- ; select base instruction set with SSE and SSE2

\ *P Aspects of the VFX code generator are controllable by
\ ** switches. In particular the inlining of the *\fo{DO ... LOOP} entry code
\ ** and local variable entry code may be turned on and off to suit your
\ ** particular coding style.

\ *P Note also that for large computationally intensive definitions,
\ ** the *\fo{SMALLER} and *\fo{FASTER} pair of switches may
\ ** actually give better performance using *\fo{SMALLER}. The
\ ** impact of these switches varies considerably between CPU types
\ ** and cache/memory architecture.

(( for DOCGEN
#16 value /code-alignment	\ -- n
\ *G The default code alignment used by *\fo{FASTER} below.
\ ** Must be a power of two.

: smaller	\ --
\ *G Selects smaller code using the minimum of alignment.
  4 to code-alignment
;

: faster	\ --
\ *G Selects faster code using 16 byte alignment, which will
\ ** increase the size of the dictionary headers.
  /code-alignment to code-alignment
;
))

only forth  also h-c  definitions
decimal
external


\ ***********************
\ Error codes and strings
\ ***********************

-300	#ErrDef opt_oflow	"OPT: Working stack overflow in code optimiser. Working stack reset"
-301	#ErrDef opt_literals	"OPT: No free literals for code optimiser. Definition too complex."
-302	#ErrDef opt_mempool	"OPT: Full memory pool in code optimiser. Definition too complex."
-303	#ErrDef opt_regs	"OPT: No free registers in code optimiser. Definition too complex."
-304	#ErrDef opt_scratch	"OPT: Couldn't free scratch resource in code optimiser."
-305	#ErrDef opt_shuffle	"OPT: Couldn't shuffle stack in code optimiser."
-306	#ErrDef opt_pick	"OPT: Attempt to optimise PICK with a -ve input."
-307	#ErrDef opt_short	"OPT: Short branch out of range in optimiser."
-308	#ErrDef opt_argneg	"OPT: -Ve offset to argument promotor (nsbwiden etc...)"
-309	#ErrDef opt_xXT		"OPT: Invalid xXT operation - internal software error"
-310	#ErrDef opt_sizeop	"OPT: Invalid size operator - internal software error"
-311	#ErrDef	opt_SETx	"OPT: SETx should never execute - internal software error"
-312	#ErrDef opt_set/get-comp "OPT: SET-COMPILER or GET-COMPILER called with no valid LATEST"
-313	#ErrDef	opt_internal	"OPT: Optimiser internal error - contact MPE"
-314	#ErrDef opt_inline	"OPT: Attempt to INLINE a high level definition"
-315	#ErrDef opt_uflow	"OPT: Working stack underflow in code optimiser. Working stack reset"
-316	#ErrDef opt_div0	"OPT: Division by constant of zero"

-317    #ErrDef opt_toIndMem    "OPT: ISE - XferRes to Indmem ; contact MPE"
-318	#ErrDef opt_sdmove	"OPT: ISE - Invalid one instruction move ; contact MPE"
-319    #ErrDef opt_cmove	"OPT: ISE - Error in CMOVE ; contact MPE"
external

defer optThrow	\ n --
\ Optimiser's version of THROW.

: ?optThrow	\ flag n --
\ Optimiser's version of ?THROW.
  swap if  optThrow  else  drop  endif
;


\ ******************
\ Target description
\ ******************

1 constant tos-cached?  \ -- n ; #stack items cached in registers at a boundary

compiler
: cells(t)      \ n -- n'
  cell(t) *  ;
target


\ ********************
\ Assembler management
\ ********************

also asm-core
: start-asm     \ --
  asm-init asm_start
;

: flush-asm     \ --
  asm_end asm_start
;
previous

\ Define words to switch to and from the main assember vocabulary
\ during code generation. The definition order is important.
\ Too much cross-compiler voodoo here.
(( from PostAsm.fth
: a[            \ -- ; start assembler sequence to be compiled later, A[ ... ]A
  also asm-access
  postpone asm-init
;  immediate

: ]a            \ -- ; end of assembler sequence
  postpone asm_end
  previous  ;  immediate
))

: a[            \ -- ; start of assembler sequence
  also asm-access
;  immediate
interpreter
: a[            \ -- ; start of assembler sequence
  also asm-access
;
target
compiler
: a[            \ -- ; start of assembler sequence
  [interp]  also asm-access  [comp]
;
target

: ]a            \ -- ; end of assembler sequence
  previous  ;  immediate
interpreter
: ]a            \ -- ; end of assembler sequence
  previous
;
target
compiler
: ]a            \ -- ; end of assembler sequence
  [interp]  previous  [comp]
;
target



\ *******************************************
\ High level branches - marking and resolving
\ These branch targets are 32 bit absolute
\ *******************************************

: mrk>          \ -- addr ; mark start of forward branch
  chere(t)                              \ address to patch up later
  0 x,c(t)  ;                           \ lay dummy value

: >res          \ addr -- ; resolve forward branch
  flush-asm                             \ flush assembler
  chere(t) swap x!c(t)  ;               \ fix up destination

: mrk<          \ -- addr ; mark end of backward branch
  flush-asm                             \ flush assembler
  chere(t)  ;                           \ address to branch to

: <res          \ addr -- ; resolve backward branch
  x,c(t)  ;                             \ lay target address


\ **********************
\ Parsing quoted strings
\ **********************

: !xt++         \ addr xt -- addr' ; set pointer to given XT and step to next slot
  over !  cell +  ;

interpreter

: !xt++         \ addr xt -- addr' ; set pointer to given XT and step to next slot
  over !  cell+  ;

: lay-asm:      \ addr -- addr'
\ compile a string as an an assembler macro, putting xt at addr
  :noname !xt++  !csp                   \ generate : portion, put xt at addr, step on
  also asm-access                       \ enable assmbler vocabulary
  "xxx" evaluate                        \ use the string
  previous
  postpone ;
;

: lay-drop-asm: \ addr -- addr'
\ compile a string as an an assembler macro, putting xt at addr
  :noname !xt++  !csp			\ generate : portion, put xt at addr, step on
  postpone drop                         \ compile a DROP to discard the resource
  also asm-access                       \ enable assembler vocabulary
  "xxx" evaluate
  previous
  postpone ;
;

target


\ ********************
\ Optimisation control
\ ********************

((
The code generator is NOT intended to be a full optimiser. It is intended to
be used with two additional modules, a token optimiser and an instruction
scheduler. The token optimiser is a pass that converts the code into a token
sequence, and then modifies it to use more efficient sequences. For example
the sequence "lit + @" may be translated to "indexed@" which will then produce
shorter code. Similarly, phrases such as "var lit1 + lit2 +" can be folded to
a single literal "var+lit1+lit2" generated at compile time.

Despite this, there are some sequences which are simple to recognise at
compile time, and to optimise simply. The main one of these is the "compare
condbranch" pair. During a normal comparison, a code tail is produced which
only has to be processed by the conditional branch code. The code generator
detects this pair, and removes the comparison epilogue and the branch
prologue.

These simple optimisations are controlled by a data structure
SEQBEG          holds HERE before the generated code sequence, so
                that the optimiser can be extended to multiple look backs
SEQEND          holds HERE after code is generated
SEQBKUP         holds the start of the code to backup (can be removed with ALLOT)
SEQID           holds a sequence type indicator
SEQFLAGS        holds information used later
SEQRES		Holds more info, usually a resource.
		Holds the index register for scaled index addresses.
SEQPREV		holds information from a previous optimisation in a compound
		or the register in an nREGID optimisation
SEQPTR          Holds a pointer to the comparison data structure for compares.
		Holds the base register for scaled indix addressing
))

struct /sequence
  int .seqbeg
  int .seqend
  int .seqbkup
  int .seqid
  int .seqflags
  int .seqres			\ SFP093
  int .seqprev                  \ SFP026
  int .seqptr
end-struct

/sequence buffer: seqbeg        \ -- addr ; returns address of SEQBEG
/sequence buffer: seqsave	\ -- addr ; for back up copy ; SFP093

: seqend        \ -- addr
  seqbeg .seqend  ;

: seqbkup       \ -- addr
  seqbeg .seqbkup  ;

: seqid         \ -- addr
  seqbeg .seqid  ;

: seqflags      \ -- addr
  seqbeg .seqflags  ;

: seqres	\ -- addr		\ SFP093
  seqbeg .seqres  ;

: seqprev       \ -- addr               \ SFP026
  seqbeg .seqprev  ;

: seqptr        \ -- addr
  seqbeg .seqptr  ;

\ These must be constants for the F.P. and external optimiser(s)
0 constant nooptID                      \ no optimisation
1 constant 1CMPID                       \ Comparison of 1 item, e.g. 0=
2 constant 2CMPID                       \ comparison of 2 items, e.g. <
3 constant QDUPID                       \ ?DUP
4 constant QDUPZEID                     \ ?DUP 0= etc
5 constant LITPLUSID                    \ LIT <x> + followed by @ or !          ; SFP023
6 constant LITPLUSLITID                 \ LIT <x> + then LIT <y> +      \ SFP026
7 constant DOESID                       \ DOES> >R                      \ SFP035
8 constant RFROMID                      \ R> DROP, R> IF etc            \ SFP035
9 constant nREGID			\ USER variables		\ SFP051
#10 constant RFROMnREGID		\ R> USERx !			\ SFP051
#11 constant FLAGSETID			\ for when required CPU flag is set
#12 constant IID			\ for I I
#13 constant >RID			\ for >R R@
#14 constant CELLSID			\ for LIT <x> CELLS + ( addr n -- addr+n*cells )
#15 constant CELLSLITPLUSID		\ for CELLS LIT <x> + ( n addr -- addr+n*cells )
#16 constant CELLSREGPLUSID		\ for CELLS reg +   ( n addr -- addr+n*cells )
#17 constant nextOptID

create OptXts	\ -- addr
  ' nooptID ,	' 1CMPID ,	' 2CMPID ,	' QDUPID ,
  ' QDUPZEID ,	' LitPlusID ,	' LitPlusLitID ,  ' DoesID ,
  ' RFromID ,	' nRegID ,	' RFromnRegID ,  ' FlagSetID ,
  ' IID ,	' >RID ,	' CellsID ,	' CellsLitPlusID ,
  ' CellsRegPlusID ,

: .OptName	\ opt# --
\ Display optimisation name if opt# non-zero.
  ?dup if
    ."  Opt="  dup nextOptID >=
    if  . exit  endif
    cells OptXTs + @ >name .name
  endif
;

: start-seq     \ seqID -- ; mark the start of a sequence
  seqid !  chere(t) seqbeg !  ;

: start-bkup    \ -- ; mark the start of the code which will be removed
  chere(t) seqbkup !  ;

: end-seq       \ -- ; mark the end of the sequence and code to be removed
  chere(t) seqend !  ;

\ SFP051...
Defer NoElim	\ -- ; Prevents spurious move eliminations

: seq-backup    \ -- ; remove the optimisable sequence - safely
  seqbkup @ seqend @ - callot(t)  NoElim  ;

: seq-backup2   \ -- ; remove the optimisable sequence - unsafe
  seqbkup @ seqend @ - callot(t)  ;
\ ...SFP051

: seq-prevbackup    \ -- ; remove back to the previous optimisable sequence - safely ; SFP401
  seqprev @ seqend @ - callot(t)  NoElim  ;

: reset-opt     \ -- ; resets optimiser
  -1 seqend !  nooptID seqid !  ;       \ will never be matched

: optimisable?  \ -- ; true if optimisation ends here
  seqend @ chere(t) =  ;

: optid?        \ seqid -- flag ; true if sequence ID matches
  seqid @ =  ;

: opt?          \ seqid -- flag ; true if this sequence can be optimised        \ SFP035
  optid?  optimisable? and  ;

: [opt		\ seqid -- ; start a removable sequence ; SFP051
  start-seq  start-bkup
;

: opt]		\ -- ; end a removable sequence ; SFP051
  end-seq
;

: [save-opt	\ -- ; backup optimisation data ; SFP093
  seqbeg seqsave /sequence cmove
;

: save-opt]	\ -- ; restore optimisation data ; SFP093
  seqsave seqbeg /sequence cmove
;

-1 seqbeg .seqend !  nooptID seqbeg .seqid !


\ *********************
\ Structure definitions
\ *********************

\ ********************************************************
\ A resource is somewhere to keep data about a stack item,
\ it may be a register, or it may be on the stack itself.
\ ********************************************************

struct /resource        \ -- ; -- len ;
        int res-flags           \ zero if unused
        int res-refcount        \ number of stack items represented
\        int res-type            \ data type
        int res-value           \ value of literal/address/associated data
	int res-assoc		\ associated resource		SFP051
        int res-reslink         \ link to previous resource
\    0 field res-name            \ was resource name, now end of modifiable data
        int xxt                 \ xt of word to execute when XLONG
        int lxt                 \ xt of word to execute when LONG
        int wxt                 \ xt of word to execute when WORD
        int bxt                 \ xt of word to execute when BYTE
        int ixt                 \ xt of word to execute when ADDRESS
        int i*2xt               \ xt of word to execute when ADDRESS*2
        int i*4xt               \ xt of word to execute when ADDRESS*4
        int i*8xt               \ xt of word to execute when ADDRESS*8
end-struct

\ The following bit masks are used by flags field in a resource structure
%00000001 constant bit0
%00000010 constant bit1
%00000100 constant bit2
%00001000 constant bit3
%00010000 constant bit4
%00100000 constant bit5
%01000000 constant bit6
%10000000 constant bit7
$0100     constant bit8
$0200     constant bit9

$8000:0000 constant bit31

bit0 constant rmask     \ -- n ; register bit mask, if clear memory assumed
bit1 constant br        \ -- n ;  has 8 bit form
bit2 constant wr        \ -- n ;  has 16 bit form
bit3 constant lr        \ -- n ;  has 32 bit form
bit4 constant xr        \ -- n ;  has 64 bit form
bit5 constant litmask   \ -- n ; is a literal, value field contains literal
bit6 constant deepmask  \ -- n ; is in deep stack, value field contains index into stack model
bit7 constant srcmask   \ -- n ; is the source of a stack shuffle
bit8 constant memmask   \ -- n ; is at an absolute address

bit31 constant hiregmask \ -- u ; resource is a high register

rmask  xr or  lr or  wr or  br or
  constant rxlwb         \ -- n ; register with 64, 32, 16, 8 bit forms

rmask  xr or  lr or  wr or  br or
  constant fmask        \ -- n ; initial flag mask

: init-resource \ addr -- ; initialise a resource
  dup  res-flags dup @ fmask and swap !
  dup  res-refcount off
\  dup  res-type off
  dup  res-value off
  drop
;

defer bad-report

variable res-link       \ -- addr ; holds address of last resource declared
  0 res-link !

: say-bad       \ -- ; error if executed
  cr ." SAY-BAD probably in " r> ip>nfa .name
  bad-report
\  1 abort" Invalid xXT operation - internal software error"  ;
  opt_xXT optThrow
;

: bad           \ addr -- addr' ; set pointer to be invalid
  ['] say-bad !xt++  ;
interpreter
: bad           \ addr -- addr' ; set pointer to be invalid
  ['] say-bad !xt++  ;
target

defer xlit>a    \ res -- ; put 64 bit literal in output
defer llit>a    \ res -- ; put 32 bit literal in output
defer wlit>a    \ res -- ; put 16 bit literal in output
defer blit>a    \ res -- ; put 8 bit literal in output

defer xmem>a    \ res -- ; put 64 bit memory in output
defer lmem>a    \ res -- ; put 32 bit memory in output
defer wmem>a    \ res -- ; put 16 bit memory in output
defer bmem>a    \ res -- ; put 8 bit memory in output

: res-allot     \ flags -- ; lay data section of resource and initialise the flags field
  here                                  \ address of new resource
  /resource allot&erase                 \ reserve data space
  res-link @ over res-reslink !         \ old value to link field
  dup res-link !                        \ set new value
  res-flags !                           \ and initialise flags field
;

interpreter
: allot&erase	\ n --
  here swap  dup allot  erase
;

: res-allot     \ flags -- ; lay data section of resource and initialise the flags field
  here                                  \ address of new resource
  /resource allot&erase                 \ reserve data space
  res-link @ over res-reslink !         \ old value to link field
  dup res-link !                        \ set new value
  res-flags !                           \ and initialise flags field
;

: resource:     \ index -- ; -- addr
\ Resource with no indirect forms.
  create
    here                                        \ address of data area
    deepmask res-allot                          \ reserve and link data space
    tuck res-value !                            \ stash index
    xxt                                         \ start of word table
      lay-drop-asm:  lay-drop-asm:		\ xlong, long strings
      lay-drop-asm:  lay-drop-asm:		\ word, byte strings
      bad bad bad bad				\ no indirect forms
    drop
;

: reg-xlwb:	\ reg# -- ; -- addr
\ Register with Xlong, Long, Word, Byte forms and indirect forms.
\ The high portion of the flags contains the
  create
    >r				\ preserve register number
    here
    r@ 7 >
    if   bit31  else  0  then	\ high register mask?
    r> #32 lshift or		\ merge reg num
    rxlwb or res-allot		\ reserve data space
    xxt                         	\ start of word table
      lay-drop-asm:  lay-drop-asm:	\ xlong, long strings
      lay-drop-asm:  lay-drop-asm:	\ word, byte strings
      lay-drop-asm:  lay-drop-asm:      \ indirect forms
      lay-drop-asm:  lay-drop-asm:
    drop
;

((
: reg-xlw:	\ -- ; -- addr ; register with Long, Word forms
  create
    here
    rlw res-allot                       \ reserve and link data space
    xxt                                 \ start of word table
      lay-drop-asm:  lay-drop-asm	\ xlong, long forms
      lay-drop-asm:  bad		\ long, word, no byte strings
      lay-drop-asm:  lay-drop-asm:      \ indirect forms
      lay-drop-asm:  lay-drop-asm:
    drop
;
))

: res-lit:      \ -- ; -- addr ; literal
  create
    here
    litmask res-allot                           \ reserve and link data space
    xxt                                         \ start of word table
      ['] xlit>a !xt++  ['] llit>a !xt++	\ x/l/w/b forms
      ['] wlit>a !xt++  ['] blit>a !xt++
      bad bad bad bad                           \ no indirect forms
    drop
;

: res-mem:      \ -- ; -- addr ; memory
  create
    here
    memmask res-allot                           \ reserve and link data space
    xxt                                         \ start of word table
      ['] xmem>a !xt++  ['] lmem>a !xt++	\ x/l/w/b forms
      ['] wmem>a !xt++  ['] bmem>a !xt++
      bad bad bad bad                           \ no indirect forms
    drop
;
target


\ ********************
\ Resource definitions
\ ********************
\ On the x64 the following register usage is
\ assumed:
\ Forth regs.
\   RSI         UP (thread pointer)	volatile Linux, non-vol Windows
\   RDI         LP (locals pointer)	volatile Linux, non-vol Windows
\   RSP         RSP
\   RBP         PSP			non-volatile across ABIs
\   R13		FSP			non-volatile across ABIs
\ Special
\   R14		Index for DO/LOOP	non-volatile across ABIs
\   R15		Index~Limit		non-volatile across ABIs
\   XMM8	FTOS			volatile Linux, non-vol Windows
\   XMM9        FTEMP temp float
\ Stack
\   RBX         TOS
\   RDX
\   RCX
\   RAX
\   R8-R12

\ Working Registers - N.B. the leading spaces MUST be provided
 0 reg-xlwb: rRAX  " rax" " eax" " ax" " al"  " [rax]" " [rax*2]" " [rax*4]" " [rax*8]"
 3 reg-xlwb: rRBX  " rbx" " ebx" " bx" " blreg"  " [rbx]" " [rbx*2]" " [rbx*4]" " [rbx*8]"
 2 reg-xlwb: rRDX  " rdx" " edx" " dx" " dl"  " [rdx]" " [rdx*2]" " [rdx*4]" " [rdx*8]"
 1 reg-xlwb: rRCX  " rcx" " ecx" " cx" " cl"  " [rcx]" " [rcx*2]" " [rcx*4]" " [rcx*8]"

\ These four registers are Forth registers - use with extreme caution!	SFP034
 6 reg-xlwb: rRSI  " rsi" " esi" " si" " sil" " [rsi]" " [rsi*2]" " [rsi*4]" " [rsi*8]"
 7 reg-xlwb: rRDI  " rdi" " edi" " di" " dil" " [rdi]" " [rdi*2]" " [rdi*4]" " [rdi*8]"
 5 reg-xlwb: rRBP  " rbp" " ebp" " bp" " bpl" " [rbp]" " [rbp*2]" " [rbp*4]" " [rbp*8]"
 4 reg-xlwb: rRSP  " rsp" " esp" " sp" " spl" " [rsp]" " say-bad" " say-bad" " say-bad"

\ These are the extra registers provided by the 64 bit extension.
\ R12 is used by the EXTERN interface
\ R13 is the Float stack pointer.
\ R14, R15 are used by DO ... LOOP and friends
\ R12..R15 are preserved by x64 ABIs.
 8 reg-xlwb: rR8   " r8" " r8d" " r8w" " r8l" " [r8]"  " [r8*2]"  " [r8*4]"  " [r8*8]"
 9 reg-xlwb: rR9   " r9" " r9d" " r9w" " r9l" " [r9]"  " [r9*2]"  " [r9*4]"  " [r9*8]"
10 reg-xlwb: rR10  " r10" " r10d" " r10w" " r10l" " [r10]" " [r10*2]" " [r10*4]" " [r10*8]"
11 reg-xlwb: rR11  " r11" " r11d" " r11w" " r11l" " [r11]" " [r11*2]" " [r11*4]" " [r11*8]"
12 reg-xlwb: rR12  " r12" " r12d" " r12w" " r12l" " [r12]" " [r12*2]" " [r12*4]" " [r12*8]"
13 reg-xlwb: rR13  " r13" " r13d" " r13w" " r13l" " [r13]" " [r13*2]" " [r13*4]" " [r13*8]"
14 reg-xlwb: rR14  " r14" " r14d" " r14w" " r14l" " [r14]" " [r14*2]" " [r14*4]" " [r14*8]"
15 reg-xlwb: rR15  " r15" " r15d" " r15w" " r15l" " [r15]" " [r15*2]" " [r15*4]" " [r15*8]"

\ Deep stack  N.B. the index into the stack model MUST be provided
tos-cached?
     dup resource: DS1   " qword $000 [rbp]" " dword $000 [rbp]" " worda $000 [rbp]" " byte $000 [rbp]"
  1+ dup resource: DS2   " qword $008 [rbp]" " dword $008 [rbp]" " worda $008 [rbp]" " byte $008 [rbp]"
  1+ dup resource: DS3   " qword $010 [rbp]" " dword $010 [rbp]" " worda $010 [rbp]" " byte $010 [rbp]"
  1+ dup resource: DS4   " qword $018 [rbp]" " dword $018 [rbp]" " worda $018 [rbp]" " byte $018 [rbp]"
  1+ dup resource: DS5   " qword $020 [rbp]" " dword $020 [rbp]" " worda $020 [rbp]" " byte $020 [rbp]"
  1+ dup resource: DS6   " qword $028 [rbp]" " dword $028 [rbp]" " worda $028 [rbp]" " byte $028 [rbp]"
  1+ dup resource: DS7   " qword $030 [rbp]" " dword $030 [rbp]" " worda $030 [rbp]" " byte $030 [rbp]"
  1+ dup resource: DS8   " qword $038 [rbp]" " dword $038 [rbp]" " worda $038 [rbp]" " byte $038 [rbp]"
  1+ dup resource: DS9   " qword $040 [rbp]" " dword $040 [rbp]" " worda $040 [rbp]" " byte $040 [rbp]"
  1+ dup resource: DS10  " qword $048 [rbp]" " dword $048 [rbp]" " worda $048 [rbp]" " byte $048 [rbp]"
  1+ dup resource: DS11  " qword $050 [rbp]" " dword $050 [rbp]" " worda $050 [rbp]" " byte $050 [rbp]"
  1+ dup resource: DS12  " qword $058 [rbp]" " dword $058 [rbp]" " worda $058 [rbp]" " byte $058 [rbp]"
  1+ dup resource: DS13  " qword $060 [rbp]" " dword $060 [rbp]" " worda $060 [rbp]" " byte $060 [rbp]"
  1+ dup resource: DS14  " qword $068 [rbp]" " dword $068 [rbp]" " worda $068 [rbp]" " byte $068 [rbp]"
  1+ dup resource: DS15  " qword $070 [rbp]" " dword $070 [rbp]" " worda $070 [rbp]" " byte $070 [rbp]"
  1+ dup resource: DS16  " qword $078 [rbp]" " dword $078 [rbp]" " worda $078 [rbp]" " byte $078 [rbp]"
  1+ dup resource: DS17  " qword $080 [rbp]" " dword $080 [rbp]" " worda $080 [rbp]" " byte $080 [rbp]"
  1+ dup resource: DS18  " qword $088 [rbp]" " dword $088 [rbp]" " worda $088 [rbp]" " byte $088 [rbp]"
  1+ dup resource: DS19  " qword $090 [rbp]" " dword $090 [rbp]" " worda $090 [rbp]" " byte $090 [rbp]"
  1+ dup resource: DS20  " qword $098 [rbp]" " dword $098 [rbp]" " worda $098 [rbp]" " byte $098 [rbp]"
  1+ dup resource: DS21  " qword $0A0 [rbp]" " dword $0A0 [rbp]" " worda $0A0 [rbp]" " byte $0A0 [rbp]"
  1+ dup resource: DS22  " qword $0A8 [rbp]" " dword $0A8 [rbp]" " worda $0A8 [rbp]" " byte $0A8 [rbp]"
  1+ dup resource: DS23  " qword $0B0 [rbp]" " dword $0B0 [rbp]" " worda $0B0 [rbp]" " byte $0B0 [rbp]"
  1+ dup resource: DS24  " qword $0B8 [rbp]" " dword $0B8 [rbp]" " worda $0B8 [rbp]" " byte $0B8 [rbp]"
  1+ dup resource: DS25  " qword $0C0 [rbp]" " dword $0C0 [rbp]" " worda $0C0 [rbp]" " byte $0C0 [rbp]"
  1+ dup resource: DS26  " qword $0C8 [rbp]" " dword $0C8 [rbp]" " worda $0C8 [rbp]" " byte $0C8 [rbp]"
  1+ dup resource: DS27  " qword $0D0 [rbp]" " dword $0D0 [rbp]" " worda $0D0 [rbp]" " byte $0D0 [rbp]"
  1+ dup resource: DS28  " qword $0D8 [rbp]" " dword $0D8 [rbp]" " worda $0D8 [rbp]" " byte $0D8 [rbp]"
  1+ dup resource: DS29  " qword $0E0 [rbp]" " dword $0E0 [rbp]" " worda $0E0 [rbp]" " byte $0E0 [rbp]"
  1+ dup resource: DS30  " qword $0E8 [rbp]" " dword $0E8 [rbp]" " worda $0E8 [rbp]" " byte $0E8 [rbp]"
  1+ dup resource: DS31  " qword $0F0 [rbp]" " dword $0F0 [rbp]" " worda $0F0 [rbp]" " byte $0F0 [rbp]"
  1+ dup resource: DS32  " qword $0F8 [rbp]" " dword $0F8 [rbp]" " worda $0F8 [rbp]" " byte $0F8 [rbp]"
  1+ dup resource: DS33  " qword $100 [rbp]" " dword $100 [rbp]" " worda $100 [rbp]" " byte $100 [rbp]"
  1+ dup resource: DS34  " qword $108 [rbp]" " dword $108 [rbp]" " worda $108 [rbp]" " byte $108 [rbp]"
  1+ dup resource: DS35  " qword $110 [rbp]" " dword $110 [rbp]" " worda $110 [rbp]" " byte $110 [rbp]"
  1+ dup resource: DS36  " qword $118 [rbp]" " dword $118 [rbp]" " worda $118 [rbp]" " byte $118 [rbp]"
  1+ dup resource: DS37  " qword $120 [rbp]" " dword $120 [rbp]" " worda $120 [rbp]" " byte $120 [rbp]"
  1+ dup resource: DS38  " qword $128 [rbp]" " dword $128 [rbp]" " worda $128 [rbp]" " byte $128 [rbp]"
  1+ dup resource: DS39  " qword $130 [rbp]" " dword $130 [rbp]" " worda $130 [rbp]" " byte $130 [rbp]"
  1+ dup resource: DS40  " qword $138 [rbp]" " dword $138 [rbp]" " worda $138 [rbp]" " byte $138 [rbp]"
  1+ dup resource: DS41  " qword $140 [rbp]" " dword $140 [rbp]" " worda $140 [rbp]" " byte $140 [rbp]"
  1+ dup resource: DS42  " qword $148 [rbp]" " dword $148 [rbp]" " worda $148 [rbp]" " byte $148 [rbp]"
  1+ dup resource: DS43  " qword $150 [rbp]" " dword $150 [rbp]" " worda $150 [rbp]" " byte $150 [rbp]"
  1+ dup resource: DS44  " qword $158 [rbp]" " dword $158 [rbp]" " worda $158 [rbp]" " byte $158 [rbp]"
  1+ dup resource: DS45  " qword $160 [rbp]" " dword $160 [rbp]" " worda $160 [rbp]" " byte $160 [rbp]"
  1+ dup resource: DS46  " qword $168 [rbp]" " dword $168 [rbp]" " worda $168 [rbp]" " byte $168 [rbp]"
  1+ dup resource: DS47  " qword $170 [rbp]" " dword $170 [rbp]" " worda $170 [rbp]" " byte $170 [rbp]"
  1+ dup resource: DS48  " qword $178 [rbp]" " dword $178 [rbp]" " worda $178 [rbp]" " byte $178 [rbp]"
drop

\ Literals
res-lit:  LITP1
res-lit:  LITP2
res-lit:  LITP3
res-lit:  LITP4
res-lit:  LITP5
res-lit:  LITP6
res-lit:  LITP7
res-lit:  LITP8
res-lit:  LITP9
res-lit:  LITP10
res-lit:  LITP11
res-lit:  LITP12
res-lit:  LITP13
res-lit:  LITP14
res-lit:  LITP15
res-lit:  LITP16
res-lit:  LITP17
res-lit:  LITP18
res-lit:  LITP19
res-lit:  LITP20
res-lit:  LITP21
res-lit:  LITP22
res-lit:  LITP23
res-lit:  LITP24
res-lit:  LITP25
res-lit:  LITP26
res-lit:  LITP27
res-lit:  LITP28
res-lit:  LITP29
res-lit:  LITP30
res-lit:  LITP31
res-lit:  LITP32

\ Memory
res-mem:  MEMP1
res-mem:  MEMP2
res-mem:  MEMP3
res-mem:  MEMP4
res-mem:  MEMP5
res-mem:  MEMP6
res-mem:  MEMP7
res-mem:  MEMP8
res-mem:  MEMP9
res-mem:  MEMP10
res-mem:  MEMP11
res-mem:  MEMP12
res-mem:  MEMP13
res-mem:  MEMP14
res-mem:  MEMP15
res-mem:  MEMP16
res-mem:  MEMP17
res-mem:  MEMP18
res-mem:  MEMP19
res-mem:  MEMP20
res-mem:  MEMP21
res-mem:  MEMP22
res-mem:  MEMP23
res-mem:  MEMP24
res-mem:  MEMP25
res-mem:  MEMP26
res-mem:  MEMP27
res-mem:  MEMP28
res-mem:  MEMP29
res-mem:  MEMP30
res-mem:  MEMP31
res-mem:  MEMP32

res-mem:  IndDestMem	\ for temporary destination memory ; SFP051
res-mem:  AbsDestMem	\ for temporary destination memory ; SFP051

res-mem:  rDUMMY        \ never select this in a table, this resource is
                        \ reserved for use as an unused resource to prevent
                        \ the stack shuffle from mis-using a source register
                        \ which is moved and then re-used as a scratch reg.

\ return stack
-1 resource: DR1   " qword $000 [rsp]" " dword $000 [rsp]" " word $000 [rsp]" " byte $000 [rsp]"
-2 resource: DR2   " qword $008 [rsp]" " dword $008 [rsp]" " word $008 [rsp]" " byte $008 [rsp]"
-3 resource: DR3   " qword $010 [rsp]" " dword $010 [rsp]" " word $010 [rsp]" " byte $010 [rsp]"
-4 resource: DR4   " qword $018 [rsp]" " dword $018 [rsp]" " word $018 [rsp]" " byte $018 [rsp]"
-5 resource: DR5   " qword $020 [rsp]" " dword $020 [rsp]" " word $020 [rsp]" " byte $020 [rsp]"
-6 resource: DR6   " qword $028 [rsp]" " dword $028 [rsp]" " word $028 [rsp]" " byte $028 [rsp]"

\ Dummies for SDMOVE, and friends
-99 resource: >DR1 " bad" " bad" " bad" " bad"		\ SFP051
-98 resource: DR1> " bad" " bad" " bad" " bad"		\ SFP051


rRBX constant rTOS	\ define the register used to cache TOS

\ resource tables - lists of resources under various conditions
create resource-table   \ -- addr
\ holds available stack resources in preference order
\ deep stack resources are provided automagically
  rRBX ,          rRDX ,          rRCX ,        \ TOS *MUST* be first           \ SJC021
  rRAX ,          rR8 ,	          rR9 ,
  rR10 ,          rR11 ,          rR12 ,
  0 , 0 ,                                       \ ends with a 0

#40 constant /stack-picture     \ -- n
\ number of cells on the working stack and the initial model.

create stack-model      \ -- addr ; holds stack model for input and output
  rRBX ,        DS1 ,           DS2 ,           DS3 ,
  DS4 ,         DS5 ,           DS6 ,           DS7 ,
  DS8 ,         DS9 ,           DS10 ,          DS11 ,
  DS12 ,        DS13 ,          DS14 ,          DS15 ,
  DS16 ,        DS17 ,          DS18 ,          DS19 ,
  DS20 ,        DS21 ,          DS22 ,          DS23 ,
  DS24 ,        DS25 ,          DS26 ,          DS27 ,
  DS28 ,        DS29 ,          DS30 ,          DS31 ,
  DS32 ,	DS33 ,		DS34 ,		DS35 ,
  DS36 ,	DS37 ,		DS38 ,		DS39 ,
  DS40 ,	DS41 ,		DS42 ,		DS43 ,
  DS44 ,	DS45 ,		DS46 ,		DS47 ,
  0 ,

/stack-picture 1+ cells buffer: stack-picture	\ -- addr
\ the working stack, initialised from STACK-MODEL

/stack-picture 1+ cells buffer: dest-picture	\ -- addr
\ output stack picture after stack shuffling

create litpool  \ -- addr ; holds table of literal resources
  litp1 ,       litp2 ,         litp3 ,         litp4 ,
  litp5 ,       litp6 ,         litp7 ,         litp8 ,
  litp9 ,       litp10 ,        litp11 ,        litp12 ,
  litp13 ,      litp14 ,        litp15 ,        litp16 ,
  litp17 ,      litp18 ,        litp19 ,        litp20 ,
  litp21 ,      litp22 ,        litp23 ,        litp24 ,
  litp25 ,      litp26 ,        litp27 ,        litp28 ,
  litp29 ,      litp30 ,        litp31 ,        litp32 ,
  0 ,

create mempool  \ -- addr ; holds table of memory resources
  memp1 ,       memp2 ,         memp3 ,         memp4 ,
  memp5 ,       memp6 ,         memp7 ,         memp8 ,
  memp9 ,       memp10 ,        memp11 ,        memp12 ,
  memp13 ,      memp14 ,        memp15 ,        memp16 ,
  memp17 ,      memp18 ,        memp19 ,        memp20 ,
  memp21 ,      memp22 ,        memp23 ,        memp24 ,
  memp25 ,	memp26 ,	memp27 ,	memp28 ,
  memp29 ,	memp30 ,	memp31 ,	memp32 ,
  0 ,

\ external

: clean-resources	\ -- ; clean all resources
  res-link
  begin
    @ ?dup
   while
    dup res-refcount off
    res-reslink
  repeat
;


\ **************************
\ VFX stack access operators
\ **************************

: in(ws)        \ n -- addr ; get address of nth slot in working stack
  cells stack-picture +  ;

: in(os)        \ n -- addr ; get address of nth slot in output stack
  cells dest-picture +  ;

: in(ms)        \ n -- addr ; get address of nth slot in model stack
  cells stack-model +  ;

interpreter
: nget(ws)      \ n -- ; -- res ; get nth resource from working stack
  create  ,  does> @ in(ws) @  ;

: nput(ws)      \ n -- ; res -- ; set nth resource in working stack
  create  ,  does> @ in(ws) !  ;
target

: nget(ws)      \ n -- ; -- res ; get nth resource from working stack
  create  ,  does> @ in(ws) @  ;

: nput(ws)      \ n -- ; res -- ; set nth resource in working stack
  create  ,  does> @ in(ws) !  ;

0 nget(ws) tos-res      0 nput(ws) set-tos
1 nget(ws) nos-res      1 nput(ws) set-nos
2 nget(ws) 3os-res      2 nput(ws) set-3os
3 nget(ws) 4os-res      3 nput(ws) set-4os

variable #changed(ws)   \ -- addr ; stack depth change since start of word
variable #on-stack(ws)  \ -- addr ; number of items on stack
variable #max-stack(ws) \ -- addr ; max input number of items


\ *************************
\ Resource access operators
\ *************************

: reg?          \ res -- flag ; true if register
  res-flags @ rmask and 0<>  ;

: byte?         \ res -- flag ; true if resource has byte form
  res-flags @ br and 0<>  ;

: word?         \ res -- flag ; true if resource has word form
  res-flags @ wr and 0<>  ;

: lit?          \ res -- flag ; true if resource is a literal
  res-flags @ litmask and 0<>  ;

: lit=0?	\ res -- flag
  dup res-flags @ litmask and 0=
  if  drop  0 exit  endif
  res-value @ 0=
;

\ 64 BIT VERSION
: lit-s32?	\ res -- flag ; true if resource is signed 32 bit literal
  dup lit? if
    res-value @ $-8000:0000 $7FFF:FFFF within?
  else
    drop false
  then
;

((
\ 32 BIT VERSION
: lit-s32?	\ res -- flag ; true if resource is (sort of) signed 32 bit literal
  dup lit? if
    res-value @ $-1000:0000 $0FFF:FFFF within?
  else
    drop false
  then
;
))

: reg/lit-s32?	\ res -- flag
\ true if resource is register or 32 bit signed literal
  dup reg? if
    drop true
  else
    lit-s32?
  endif
;

: reg/lit?      \ res -- flag ; true if resource is register or literal SFP034
  res-flags @ [ rmask litmask or ] literal and 0<>  ;

: mem?          \ res -- flag ; true if resource is memory
  res-flags @ memmask and 0<>  ;

: deep?         \ res -- flag ; true if resource is in deep stack
  res-flags @ deepmask and 0<>  ;

: source?       \ res -- flag ; true if resource is a source for a shuffle
  res-flags @ srcmask and 0<>  ;

: count=1?      \ res -- flag ; true if ref count=1
  res-refcount @ 1 =  ;

: free?         \ res -- flag ; true if count=0
  res-refcount @ 0=  ;

: dreg?         \ res -- flag ; true if reg and count=1, can be used as dest
  dup reg? swap count=1? and  ;

: unmarkable?   \ res -- flag ; true if this resource is source with count=1
  dup source? swap count=1? and  ;

: mark          \ res -- ; set source bit
  srcmask swap res-flags or!  ;

: set-dummy(ws) \ n -- ; set nth item of the working stack to rDUMMY    SFP018
  rDUMMY swap in(ws) !
;

: unmark        \ res -- ; reset source bit, remove resource from working stack
  srcmask over res-flags bic!		\ clear source flag in resource
  #on-stack(ws) @ 0                     \ walk the stack
  ?do
    i in(ws) @ over =                   \ if that resource is being cleared
    if  i set-dummy(ws)  endif          \   put rDUMMY in its place
  loop
  drop                                  \ discard res
;

defer ResNoElim	\ res -- ; checks for elimination conflict ; SFP059
defer ResNoElim? \ res -- flag ; true if elimination safe ; SFP059

: getRes        \ table -- res|0 ; res1, res2, ... resn, 0
  begin
    dup @ dup                           \ get resource
   while                                \ 0 marks end of chain
    free?                               \ resource free?
    if  @  dup ResNoElim  exit  endif   \ yes, return it ; SFP059
    cell+                               \ no, step to next
  repeat
  nip
;

: GetByteRes    \ table -- res|0 ; res1, res2, ... resn, 0
  begin
    dup @ dup                           \ get resource
   while                                \ 0 marks end of chain
    dup free?  swap byte? and           \ resource free and has byte version?
    if  @  dup ResNoElim  exit  endif   \ yes, return it ; SFP059
    cell+                               \ no, step to next
  repeat
  nip  ;

: GetWordRes    \ table -- res|0 ; res1, res2, ... resn, 0
  begin
    dup @ dup                           \ get resource
   while                                \ 0 marks end of chain
    dup free?  swap word? and           \ resource free and has word version?
    if  @  dup ResNoElim  exit  endif	\ yes, return it ; SFP059
    cell+                               \ no, step to next
  repeat
  nip  ;

: getPoolRes	\ table -- res|0 ; res1, res2, ... resn, 0 ; SFP059
  begin
    dup @ dup                           \ get resource
   while                                \ 0 marks end of chain
    dup free? swap ResNoElim? and	\ resource free and safe?
    if  @ exit  endif			\ yes, return it
    cell+                               \ no, step to next
  repeat
  nip
;

: GetLitRes     \ -- res ; get a free literal resource
  LitPool GetPoolRes			\ SFP059
  dup 0= opt_literals ?optThrow		\  abort" No free literals"
;

: GetMemRes     \ -- res ; get a free memory resource
  MemPool GetPoolRes			\ SFP059
  dup 0= opt_mempool ?optThrow		\  abort" No free memory resources"
;

: GetRegRes     \ -- res ; get a free register resource
  Resource-table GetRes dup 0= opt_regs ?optThrow  ;	\  abort" No free registers"  ;

: GetShuffleReg \ -- res | 0 ; used in the shuffle routine
  Resource-Table cell+ GetRes  ;        \ skips over TOS, which *MUST* be first

: GetNewReg     \ -- res|0 ; get a new register
  Resource-Table GetRes  ;

: -GetNewReg	\ reg -- res|0 ; SFP401
\ Get a new register that is not the specified one.
  dup res-refcount incr
  GetNewReg
  swap res-refcount decr
;

: --GetNewReg	\ reg1 reg2 -- res|0 ; SFP401
\ Get a new register that is not either of the specified ones.
  over res-refcount incr  dup res-refcount incr
  GetNewReg
  swap res-refcount decr  swap res-refcount decr
;

: GetByteReg    \ table -- res | 0
  Resource-Table GetByteRes  ;

: GetWordReg    \ table -- res | 0
  Resource-Table GetWordRes  ;

: WorkReg?	\ res -- flag ; SFP407
\ Return true if res is a working register in RESOURCE-TABLE.
  resource-table begin			\ res tab --
    dup @
   while
    2dup @ =
    if  2drop -1 exit  endif
    cell +
  repeat
  2drop 0
;


\ ************************************
\ Putting code in the assembler output
\ ************************************

: x>a           \ res -- ; add XLONG string to ASM
  dup xxt @ execute  ;

: l>a           \ res -- ; add LONG string to ASM
  dup lxt @ execute  ;

: w>a           \ res -- ; add WORD string to ASM
  dup wxt @ execute  ;

: b>a           \ res -- ; add BYTE string to ASM
  dup bxt @ execute  ;

: i>a           \ res -- ; use as address
  dup ixt @ execute  ;

: i*2>a		\ res -- ; use as scaled*2 address
  dup i*2xt @ execute  ;

: i*4>a		\ res -- ; use as scaled*4 address
  dup i*4xt @ execute  ;

: i*8>a		\ res -- ; use as scaled*8 address
  dup i*8xt @ execute  ;

: sz>a          \ size res -- ; add SIZEd string to ASM$
  tuck xxt swap cells + @ execute  ;

: n>a           \ n -- n ; add number to assmbler output
;               \ consumed by assembler opcode


0 constant XSIZE	\ largest must be 0 to match table order
1 constant LSIZE
2 constant WSIZE
3 constant BSIZE

: size-error    \ -- ; size error detected
  opt_sizeop optThrow ;		\ 1 abort" invalid size operator"  ;

: use-a         \ -- ; display assembler string
  flush-asm  ;


\ ******************
\ Memory referencing
\ ******************

\ SFP051...
: SetAbsMem	\ addr res -- ; set resource to be Absolute Memory ref
  0 over res-assoc !		\ clear associated resource
  tuck res-value !		\ set memory address
  xxt				\ start of word table
    ['] xmem>a !xt++	        \ x/l/w/b forms
    ['] lmem>a !xt++
    ['] wmem>a !xt++
    ['] bmem>a !xt++
    bad bad bad bad             \ no indirect forms
  drop
;

: xind>a	\ res -- ; put out "n [reg]"
  dup res-value @		\ get offset
  swap res-assoc @ i>a		\ get indirect register
;

: SetIndMem	\ n reg res -- ; set resource to be Indirect through register
  tuck res-assoc !		\ set associated register
  tuck res-value !		\ set offset
  xxt				\ start of word table
    ['] xind>a !xt++	        \ x/l/w/b forms
    ['] xind>a !xt++
    ['] xind>a !xt++
    ['] xind>a !xt++
    bad bad bad bad             \ no indirect forms
  drop
;

: AbsMem?	\ res -- flag ; true if resource is absolute ; SFP093
  wxt @ ['] wmem>a =
;

: MakeIndMem	\ n reg -- res ; only for Forth registers!
  GetMemRes >r
  r@ SetIndMem
  r>
;

: MakeAbsMem	\ addr -- res
  GetMemRes >r
  r@ SetAbsMem
  r>
;

: MakeIndDest	\ n reg -- res ; only for Forth registers! ; SFP034
  IndDestMem SetIndMem  IndDestMem
;

: MakeAbsDest	\ addr -- res
  AbsDestMem SetAbsMem  AbsDestMem
;
\ ...SFP051

\ SFP401...

: xind*8>a	\ res -- ; put out "n [reg*8]"
  dup res-value @		\ get offset
  swap res-assoc @ i*8>a	\ get indirect register
;

: SetInd*8Mem	\ n reg res -- ; set resource to be Indirect through register*4
  tuck res-assoc !		\ set associated register
  tuck res-value !		\ set offset
  xxt				\ start of word table
    ['] xind*8>a !xt++	        \ x/l/w/b forms
    ['] xind*8>a !xt++
    ['] xind*8>a !xt++
    ['] xind*8>a !xt++
    bad bad bad bad             \ no indirect forms
  drop
;

: MakeInd*8Mem	\ n reg -- res
  GetMemRes >r
  r@ SetInd*8Mem
  r>
;

: IndMem?	\ res -- flag ; true if resource is indirect ; SFP093
  xxt @  dup ['] xind>a =  swap ['] xind*8>a = or
;

: WorkIndMem?	\ res -- flag ; SFP418
\ True if resource is working (not system) indirect.
  dup IndMem? if
    res-assoc @ WorkReg?
  else
    drop 0
  endif
;

\ ...SFP401


\ ******************************
\ Working stack access operators
\ ******************************

: init-stack(ws)        \ -- ; initialise the stack picture
  stack-model stack-picture /stack-picture cells cmove
  0 #changed(ws) !
[ tos-cached? ] [if]
\ The next section is only required if TOS is cached
  tos-res init-resource
  1 tos-res res-refcount !
  1 #on-stack(ws) !  1 #max-stack(ws) !
[else]
\ this section is required if TOS is not cached
  0 #on-stack(ws) !  0 #max-stack(ws) !
[then]
;

\ init-stack(ws)

: set-max-stack(ws)     \ -- ; set maximum input stack depth
  #on-stack(ws) @ #max-stack(ws) @ max #max-stack(ws) !
;

: refcount++	\ res -- ; increment reference count ; SFP093
  dup res-refcount incr			\ update reference count
  dup IndMem? if			\ if indirect reg access
    res-assoc @ res-refcount incr	\ update register as well
  else
    drop
  endif
;

: refcount--	\ res -- ; decrement reference count ; SFP093
  dup res-refcount decr			\ update reference count
  dup IndMem? if			\ if indirect reg access
    res-assoc @ res-refcount decr	\ update register as well
  else
    drop
  endif
;

: push(ws)      \ res -- ; add a resource to the top of the working stack
  #on-stack(ws) @ /stack-picture >=
  opt_oflow ?optThrow
  stack-picture dup cell+ /stack-picture 1- cells cmove>
  dup stack-picture !                   \ add as TOS
  refcount++				\ update reference count ; SFP093
  #on-stack(ws) incr
  #changed(ws) incr			\ SFP051
;

: drop(ws)      \ -- ; drop the top item of the working stack
  stack-picture @ refcount--		\ decrement reference count ; SFP093
  stack-picture cell+ stack-picture /stack-picture cells cmove
  #on-stack(ws) decr
  #changed(ws) decr			\ SFP051
;

: pop(ws)       \ -- res ; remove a resource from the top of the working stack
  stack-picture @  drop(ws)
;

: swap(ws)	\ -- ; Exchange on working stack ; SFP051
  pop(ws) pop(ws) swap push(ws) push(ws)
;

: nip(ws)       \ -- ; remove NOS
  pop(ws)  drop(ws)  push(ws)  ;

: min-depth(ws) \ n -- ; set minimum depth of stack
  #on-stack(ws) @ max                   \ set minimum input required
  dup #on-stack(ws) !
\ all resources USED on stack have minimum reference count of 1
  0 ?do
    i in(ws) @ res-refcount
    dup @ 1 max swap !
  loop
  set-max-stack(ws)
;

: in-range(ws)  \ n -- res | 0 ; 0 if the nth item is not on the working stack
  dup /stack-picture u<
  if  in(ws) @ 0<>  else  drop 0  endif
;

: ordered(ws)?	\ -- flag ; true if working stack = model stack ; SFP051
\ Don't check the last eight entries, may have been affected by push/pops.
  stack-model stack-picture /stack-picture 8 - cells s=
  #on-stack(ws) @ 1 = and
  #changed(ws) @ 0= and
;


\ **************************
\ Stack display and checking
\ **************************

variable <impolite>	\ -- addr ; true to display messages
  <impolite> off

also forth definitions
: +polite	\ -- ; suppresses some warnings
\ *G Suppresses some warning messages which some users may feel
\ ** are commenting on their code. In particular, if you define
\ ** constants to enable and disable code without using conditional
\ ** compilation, you can use *\fo{+POLITE} to disable the warnings
\ ** about conditional branches against a constant. See also
\ ** *\fo{-POLITE}.
  <impolite> off
;

: -polite	\ -- ; enables some warnings
\ *G Enables some warning messages which warn you if have used a
\ ** phrase such as "*\fo{<literal> IF}". See *\fo{+POLITE}.
  <impolite> on
;

previous definitions

: .res-name     \ res -- ; display resource name
  body> >name .name
;

: .resource     \ res -- ; list resouce ; SFP093
  cr
  dup .res-name
  #16 >pos  ." RC = " dup res-refcount @ .
  dup lit?
  if  #24 >pos  ." LIT " dup res-value @ .  endif
  dup mem?
  if  #24 >pos  ." ADDR " dup res-value @ .  endif
  dup IndMem? if
    #32 >pos  dup res-assoc @  ." AR = " dup .res-name
    #44 >pos  ." RC = " res-refcount @ .
  endif
  dup source?
  if  #52 >pos  ." source"  endif
  drop
;

: .resources    \ -- ; list current resources
  cr ." Resource list"
  res-link
  begin
    @ ?dup
   while
    dup .resource
    res-reslink
  repeat
;

: check(ws)     \ -- ; Check working stack
  #on-stack(ws) @ 0< if
    cr ." Depth = " #on-stack(ws) @ .
    opt_uflow optThrow
  endif
  #on-stack(ws) @ /stack-picture > if
    cr ." Depth = " #on-stack(ws) @ .
    opt_oflow optThrow
  endif
;

: .ws           \ -- ; display working stack
  check(ws)
  #on-stack(ws) @
  cr ." Depth = " dup .
     ."  max depth was " #max-stack(ws) @ .
     ."  change so far " #changed(ws) @ .
     seqid @ .OptName
  0 max  0 ?do
    i in(ws) @ .resource
  loop
;
assign .ws to-do bad-report

: .os           \ -- ; display output stack
  #on-stack(ws) @
  cr ." Depth = " dup .
  0 max  0 ?do
    i in(os) @ .resource
  loop
;

: check-stack(ws)       \ -- ; check current resources after shuffle
  res-link
  begin
    @ ?dup
   while
    dup rTOS <> if
      dup res-refcount @ if
        <impolite> @
        if  .ws .resources .rs  endif
	opt_internal optThrow		\ cr ." Optimiser internal error" #299 optThrow
      endif
    endif
    res-reslink
  repeat
;

: .wu		\ -- ; show what resources are in use ; SFP093
  cr ." Resources in use:"
  res-link
  begin
    @ ?dup
   while
    dup res-refcount @
    if  dup .resource  endif
    res-reslink
  repeat
;

: .xws		.ws  ;
: .xwu		.wu  ;


\ **************************************
\ CPU assembler specific string handling
\ **************************************

: #x>a          \ res -- n ; add lteral from resource to output stream
  a[ # ]a  res-value @  ;
assign #x>a to-do xlit>a
assign #x>a to-do llit>a
assign #x>a to-do wlit>a
assign #x>a to-do blit>a

: ,>a           \ -- ; add ", " to output string
  a[ ,a ]a  ;

: ,x>a          \ res -- ; second operand form
  ,>a  x>a  ;

: ,l>a          \ res -- ; second operand form
  ,>a  l>a  ;

: ,w>a          \ res -- ; second operand form
  ,>a  w>a  ;

: ,b>a          \ res -- ; second operand form
  ,>a  b>a  ;

: ,sz>a         \ size res -- ; second operand form
  ,>a  sz>a  ;

: ,#x>a         \ res -- n ; literal, must be second/third operand
  ,>a #x>a  ;

: ,gsz>a       \ size res -- [n] ; copes with #, reg, DS
  ,sz>a  ;

: +0[reg]       \ res -- ; add offset address to output string e.g. 0 [ESI]
  0 swap i>a  ;

: +cell[reg]    \ res -- ; add offset address to output string e.g. 4 [ESI]
  cell(t) swap i>a  ;

: +n[reg]       \ n res -- ; add offset address to output string e.g. n [ESI]
  i>a  ;

: +n[reg*2]     \ n res -- ; add offset address to output string e.g. n [ESI*2]
  i*2>a  ;

: +n[reg*4]     \ n res -- ; add offset address to output string e.g. n [ESI*4]
  i*4>a  ;

: +n[reg*8]     \ n res -- ; add offset address to output string e.g. n [ESI*8]
  i*8>a  ;

: []lit         \ res -- ; put out "[] n"
  a[ [] ]a  res-value @  not-inlined	\ relative addresses cannot be binary inlined.
;
assign []lit to-do xmem>a
assign []lit to-do lmem>a
assign []lit to-do wmem>a
assign []lit to-do bmem>a

\ SFP051...

: dgx>a		\ res -- ; general - handles destinations
  dup lit? if
    []lit  exit
  endif
  x>a
;

: sgx>a		\ res -- ; general - handles sources - e.g. push
  dup lit? if
    #x>a  exit
  endif
  x>a
;

: ,sgx>a	\ res -- ; general - handles sources
  dup lit? if
    ,#x>a  exit
  endif
  ,x>a
;
\ ...SFP051

: +QWORD        \ --
  a[ qword ]a  ;

: +DWORD        \ --
  a[ dword ]a  ;

: +WORD         \ --
  a[ worda ]a  ;

: +BYTE         \ --
  a[ byte ]a  ;

: +SIZE         \ size --
  case
    XSIZE of  +QWORD  endof
    LSIZE of  +DWORD  endof
    WSIZE of  +WORD   endof
    BSIZE of  +BYTE   endof
              size-error
  endcase  ;

: +,SIZE        \ size --
  ,>a  +SIZE  ;


\ ******************************************************
\ Single instruction move and redundant move elimination
\ ******************************************************

0 value GuardedShuffle?	\ -- flag ; true for GuardedShuffle

\ SFP051...
((
A particular edge optimisation is to merge two moves in which the destination
of the first is the source of the second. The primitive SDMOVE, defined later
generates all the single instruction sequences. This is used by SDCOPY and SDXFER
to mark instructions which can be eliminated.
))
variable MoveEnd	\ -- addr ; address at end of last move
  -1 MoveEnd !
variable MoveLen	\ -- addr ; length of last move instruction
  0 MoveLen !
variable MoveSrc	\ -- addr ; source resource for move
variable MoveDest	\ -- addr ; destination resource for move

: (NoElim)	\ -- ; mark that last instruction cannot be eliminated
  -1 MoveEnd !  0 MoveLen !  ;
assign (NoElim) to-do NoElim

: (ResNoElim)	\ res -- ; SFP059
\ If a resource is reused, but was part of a previous move,
\ elimation is inhibited if a previous move is outstanding.
\ This mainly applies to registers which are valuable.
  chere(t) MoveEnd @ = if		\ if previous opcode was a move
    dup MoveSrc @ =  swap MoveDest @ = or
    if  NoElim  endif
  else
    drop
  endif
;  assign (ResNoElim) to-do ResNoElim

: (ResNoElim?)	\ res -- flag ; SFP059
\ If a resource is reused, but was part of a previous move,
\ return false so that another resource can be used.
\ This mainly applies to literal and memory pool items.
  chere(t) MoveEnd @ = if		\ if previous opcode was a move
    dup MoveSrc @ =  swap MoveDest @ =  or 0=
  else
    drop true
  endif
;  assign (ResNoElim?) to-do ResNoElim?

: Elim?		{ src dest -- len }
\ length of last ins. if an eliminatable move
  chere(t) MoveEnd @ = if		\ if previous opcode was a move
    MoveDest @ src =			\ can remove if Dest of last = src of new
    MoveDest @ >DR1 =  src DR1> = and  or
    MoveLen @ and			\ merge length and flag
  else
    0
  endif
;

: .Elim		\ -- ; display elimination state
  cr ." CHERE    = "  chere(t) .dword
  cr ." MoveEnd  = "  MoveEnd @ .dword
  cr ." MoveLen  = "  MoveLen @ .dword
  cr ." MoveSrc  = "  MoveSrc @ .res-name
  cr ." MoveDest = "  MoveDest @ .res-name
;

variable <merged>	\ -- addr
\ Set when two move instructions have been merged.

: 1InsMove?	{ sres dres -- flag }
\ Returns true if move can be done in one instruction.
  sres reg/lit-s32?  dres reg? or
  if  true  exit  endif
  sres lit?  dres reg? and
  if  true  exit  endif
  sres DR1> = if			\ POP ...
    dres dup DR1 <>  swap DR2 <> and	\ but not to return stack location
    exit
  endif
  dres >DR1 = if			\ PUSH ...
    sres reg/lit-s32?
    if  true exit  endif
    sres dup DR1 <>  swap DR2 <> and	\ but not from return stack location
    exit
  endif
  false
;

: lit>reg	{: slit dreg -- :}
\ You can't use the XOR reg, reg trick for 0 because it upsets
\ operations like DNEGATE = 0 0 2SWAP D- that reorder literals
\ which are assumed not to modify flags.
\  GuardedShuffle? 0= if			\ use XOR reg,reg
\    slit res-value @ 0= if
\      a[ XORa ]a  dreg dgx>a  dreg ,sgx>a  use-a  exit
\    endif
\  endif
  slit res-value @ $0 $7FFF:FFFF within? if
    a[ MOV ]a  dreg l>a  slit ,sgx>a  use-a  exit
  endif
  a[ MOV ]a  dreg dgx>a  slit ,sgx>a  use-a
;

[-short-branches
: (sdmove,)	{ src-res dest-res -- ; copy source to dest }
  <merged> off				\ assume we won't merge anything

  src-res dest-res Elim? if		\ Can we remove the previous move?
    MoveSrc @ dest-res 1InsMove? if	\ and the new one is a single instruction
((
cr ." Merging:
MoveSrc @ .res-name space  MoveDest @ .res-name space
." with: "
src-res .res-name space  dest-res .res-name space
." in: " chere(t) ip>nfa .name
cr source type
))
      <merged> on			\ mark that we have merged instructions
      MoveLen @ negate callot(t)	\ remove previous instruction
      MoveSrc @ -> src-res		\ use old source for this move
      src-res dest-res =		\ avoid redundant moves
      src-res >DR1 =  dest-res DR1> = and  or
      src-res DR1> =  dest-res >DR1 = and  or
      if
\ cr ." Eliminating " src-res .res-name space  dest-res .res-name space
        reset-opt  exit
      endif
    endif
  endif

  dest-res >DR1 = if
    a[ PUSH qword ]a  src-res sgx>a  use-a
    exit
  endif
  src-res DR1> = if
    a[ POP qword ]a  dest-res dgx>a  use-a
    exit
  endif
\ SFP097, SFP096...
  dest-res reg? src-res lit? and if
    src-res dest-res lit>reg  exit
  endif
\ ...SFP097, SFP098
  a[ MOV ]a  dest-res dgx>a  src-res ,sgx>a  use-a
;
short-branches]


\ SDMOVE, SDCOPY and SDXFER
\ The prefix Px uses x to indicate the minimum reference count
\ required for removal or a previous instruction. In other words,
\ if the source has reference count greater than x, a previous
\ move will not be eliminated. Thus P0xxxx indicates
\ that the reference count must be 0 for the previous instruction to
\ be removed. This form is usually used after the source resource has
\ been popped from the working stack, or for a temporary resource
\ which gets as far as the working stack.

\ SDMOVE, : words of the form xxSDMOVE may eliminate a previous instruction
\ that has performed a move. If a merge is performed the merged instruction
\ will not be eliminated.

\ SDCOPY : words of the form xxSDCOPY may eliminate a previous instruction,
\ but no further elimination can take place. This ensures that the
\ destination is always valid.

\ SDXFER : words of the form xxSDXFER may eliminate a previous instruction,
\ in which case no further elimation can take place. If no elimination
\ has taken place the instrucion laid down can itself be eliminated.

: sdmove,	\ src-res dest-res -- ; copy source to dest
\ +G This is the one instruction move, which is used as a primitive.
  (sdMove,)  <merged> @
  if  NoElim  endif
;

: sdCopy	\ src dest -- ; copy a resource, cannot be eliminated
  (sdMove,)  NoElim
;

: sdXfer	\ src dest -- ; copy a resource, instruction can be eliminated
  2dup >r >r  chere(t) >r
  (sdmove,)
  <merged> @ if
    r> r> 2drop  r> drop  NoElim
  else
    chere(t) r> - MoveLen !  chere(t) MoveEnd !
    r> MoveSrc !  r> MoveDest !
  endif
;

: P1sdmove,	\ src dest -- ; Merge inhibited if src has count>1
  over res-refcount @ 1 >
  if  NoElim  endif
  sdmove,
;

: P1sdXfer	\ src dest -- ; Merge inhibited if src has count>1
  over res-refcount @ 1 >
  if  NoElim  endif
  sdXfer
;

: P1sdCopy	\ src dest -- ; Merge inhibited if src has count>1
  over res-refcount @ 1 >
  if  NoElim  endif
  sdCopy
;

: P0sdmove,	\ src dest -- ; Merge inhibited if src in use
  over res-refcount @ 0>
  if  NoElim  endif
  sdmove,
;

: P0sdXfer	\ src dest -- ; Merge inhibited if src in use
  over res-refcount @ 0>
  if  NoElim  endif
  sdXfer
;

: P0sdCopy	\ src dest -- ; Merge inhibited if src in use
  over res-refcount @ 0>
  if  NoElim  endif
  sdCopy
;

: CopyTOS       \ res -- ; copy TOS to new register resource which becomes TOS
  pop(ws) over P0sdXfer  push(ws)
;

: CopyNOS       \ res -- ; copy NOS to new register resource which becomes TOS
  swap(ws)  pop(ws) over P0sdXfer  push(ws)  ;

: ChangeNOS	\ res -- ; SFP404
\ Copy NOS to new register resource which becomes NOS
  pop(ws) >r
  pop(ws) over P0sdXfer  push(ws)
  r> push(ws)
;



\ *********************************************
\ Shuffling the stack at a basic block boundary
\ *********************************************

((
N.B. ***WARNING*** Ugly code follows!

At the end of a basic block, the array STACK-PICTURE holds the current state
of the data stack. The variable #CHANGED(WS) tells us by how many cells the
data stack has grown (+ve) or shrunk (-ve) since the stack was last in model
form as defined by STACK-MODEL. If the stack has grown, additional space must
allocated on the stack *before* all the stack items are shuffled onto the stack
in model order. If the stack has shrunk, the stack items must be reordered into
model form, and the stack reduced *after* the shuffle.

This means that in the case of a changed stack, translation is necessary from
before shuffle locations to after shuffle locations.

The position of the data on the working stack is held in the array
STACK-PICTURE, whose contents are adjusted for a growing stack. The required
output picture is held in DEST-PICTURE, which is initialised from STACK-MODEL,
and then adjusted if the stack is to shrink.

The algorithm for shuffling the stack is as follows:
  mark all resources in STACK-PICTURE as sources
  find all dest=source, already in place
    unmark if refcount=1
  begin
    find exchanges (source=dest(dest) with refcounts=1)
      lay exchange, unmark source and dest
    lay moves to free resources
      decrement source refcount
      unmark if source refcount=0
    if sources left, and no changes made
      get a scratch resource
      copy most used source in output to scratch
      lay moves as above
      free scratch
      if still no changes made
        panic!
  until all sources unmarked

Note that the strategy in this optimiser is to delay code generation for as
long as possible, and not to move the stack pointer unless absolutely
necessary. All the problems are passed to the stack shuffling code, which
is ugly and contains heuristics designed to cope with the peculiarities
of the target CPU. However, if it can cope with an 386/486/Pentium, it
can probably cope with most CPUs.
))

struct /temp-res        \ -- len ; holds information during renumbering
  int temp-resource
  int temp-refcount
end-struct

/stack-picture /temp-res * buffer: xlat-buffer

\ Renumber the working stack to allow for the stack pointer having been
\ moved BEFORE the shuffle takes place. For entries in deep stack only,
\ the reference count of the entry is preserved, along with the resource
\ it must be transferred to. When the whole stack has been scanned and the
\ list built, it is scanned again, and the deep stack entries are replaced
\ by their new entries, and the reference count is transferred.
: prepare-stack(ws)     \ -- ; updates the current stack items to allow for #CHANGED(WS)
  check(ws)

\ Save the reference counts of the working deep stack entries
  xlat-buffer  #on-stack(ws) @ 0 ?do	\ -- temp
     i in(ws) @ tuck deep?              \ copy if in deep stack
     if                                 \ -- res temp
       \ save the new resource in TEMP-RESOURCE
       over res-value @                 \ current deep stack index
       #changed(ws) @ abs +             \ update index by stack change
       in(ms) @                         \ get new resource
       over temp-resource !             \ this will be the resource afterwards
       \ save the reference count in TEMP-REFCOUNT
       over res-refcount @  over temp-refcount !
       \ update the temporary buffer pointer
       /temp-res +                      \ step to next item
     endif
     nip                                \ discard resource
  loop
  drop                                  \ discard temp

\ Zero the reference counts of the deep stack entries. These will
\ be restored later, but we need to ensure that any previous values
\ that are now unused are set to zero.
  #on-stack(ws) @ 0 ?do			\ -- temp
     i in(ws) @ dup deep?               \ zero it if in deep stack
     if  res-refcount off  else  drop  endif
  loop

\ Update the contents of the working stack, ensuring that the
\ reference counts of the new deep stack positions are also
\ updated.
  xlat-buffer  #on-stack(ws) @ 0 ?do	\ for each item
    i in(ws) @ deep? if			\ deep stack item?
      dup temp-resource @               \ get the new resource
      dup i in(ws) !                    \ make it current
      over temp-refcount @              \ get the required reference count
      swap res-refcount !               \ and set it in the resource
      /temp-res +                       \ step on
    endif
  loop
  drop
;

\ If the stack pointer is moved BEFORE the shuffle, the STACK-MODEL already
\ represents the output picture. However, if the stack pointer is to be
\ moved afterwards, the output stack picture must be modified.
[-short-branches
: prepare-dest(ws)      \ -- ; prepare the output stack picture
  stack-model dest-picture /stack-picture cells cmove
  #changed(ws) @ 0< if
    #on-stack(ws) @ 0 ?do
      i in(ms) @ deep? if               \ if in deep stack
        i #changed(ws) @ abs + in(ms) @ \ take new item from model
        i in(os) !                      \ and store in destination
      endif
    loop
  endif
;
short-branches]

: mark-sources          \ -- ; mark all items on the working stack
  #on-stack(ws) @ 0 ?do
    i in(ws) @ mark
  loop
;

: adjust-stack          \ -- ; update stack pointer
                        \ uses LEA because no flags are affected
  a[ LEA    RBP, ]a  #changed(ws) @ negate cells(t) n>a  a[ [RBP] ]a  use-a
;

: pre-balance(ws)       \ -- ; update stack pointer BEFORE shuffling the stack
  #changed(ws) @ 0> if                  \ if more cells are needed on the stack
    adjust-stack
    prepare-stack(ws)
  endif
  prepare-dest(ws)                      \ set up the output stack picture
  mark-sources                          \ marking working stack items
;

: post-balance(ws)      \ -- ; update stack pointer AFTER shuffling the stack
  #changed(ws) @ 0<                     \ if fewer cells are needed on the stack
  if  adjust-stack  endif
;

: 0shuffle      \ -- ; clean up empty stack
  a[ MOV    RBX, ]a  #changed(ws) @ abs 1- cells(t) n>a  a[ [RBP] ]a  use-a
;

: 1shuffle      \ -- ; clean up stack at a boundary
  tos-res rRBX <> if
    tos-res rRBX sdXfer			\ SFP051
    tos-res dup unmark  refcount--	\ SFP093
    rRBX set-tos  1 rRBX res-refcount !
  endif
;

: dec-source    \ res -- ; decrement the ref count of a source
  dup count=1?                          \ only one use?
  if  dup unmark  endif                 \ y: all done with source
  refcount--				\ count down ; SFP093
;

variable shuffled       \ -- addr ; set by any shuffle routine that does something
variable relaxed	\ -- addr ; set to allow more time-consuming operations

: find-inplace  \ -- ; find all sources already in right place
  #on-stack(ws) @ 0 ?do
    i in(ws) @ i in(os) @ = if          \ source = dest?
      i in(ws) @ dup source?            \ must still be a source
      swap count=1? and if              \ and only one reference
\ cr i in(ws) @ .res-name ."  is in place"
        i in(ws) @ dec-source           \ decrement ref count and unmark
        shuffled incr
      endif
    endif
  loop
;

: on-os?        \ res -- pos -1 | 0 ; determine where resource is
  0 swap                                \ assume not there
  #on-stack(ws) @ 0 ?do
    i in(os) @ over =
    if  2drop  i true  unloop  exit  endif
  loop
  drop  ;

: on-ws?        \ res -- pos -1 | 0
  0 swap                                \ assume not there
  #on-stack(ws) @ 0 ?do
    i in(ws) @ over =
    if  2drop  i true  unloop exit  endif
  loop
  drop  ;

\ SFP051...
: xchg-r/r,	{ res1 res2 -- ; lay XCHG res1, res2 ; res1 is always a register }
  a[ XCHG ]a  res1 dgx>a  res2 ,sgx>a  use-a
;

: xchg-r/m,	{ reg mem -- }	 \ SFP401
\ does not use EAX
  GetShuffleReg dup if
    >r
    mem r@ P1sdCopy
    reg mem sdmove,
    r> reg sdxfer
  else
    drop
    mem >DR1 P1sdCopy
    reg mem sdmove,
    DR1> reg sdxfer
  endif
;

[-short-branches
: xchg-m/m,	{ mem1 mem2 | rtmp1 rtmp2 -- }	\ SFP401
\ does not use EAX
  GetShuffleReg dup -> rtmp1 if
    rtmp1 res-refcount incr
    GetShuffleReg dup -> rtmp2
    rtmp1 res-refcount decr
    if
      mem1 rtmp1 P1sdCopy
      mem2 rtmp2 sdmove,
      rtmp1 mem2 sdmove,
      rtmp2 mem1 sdXfer
    else
      mem1 rtmp1 P1sdCopy
      mem2 >DR1 sdmove,
      rtmp1 mem2 sdmove,
      DR1> mem1 sdXfer
    endif
    exit
  endif

  mem1 >DR1 P1sdCopy
  mem2 >DR1 sdmove,
  DR1> mem1 sdmove,
  DR1> mem2 sdXfer
;

: s_xchg        { src-res dest-res -- ; lay code to exchange two items ; SFP014 }
                \ XCHG reg, mem on a Pentium costs 15 clocks!
  src-res reg?  dest-res reg?  and
  if  src-res dest-res xchg-r/r,  exit  endif	\ XCHG sreg, dreg

  src-res reg? if
    src-res dest-res xchg-r/m,
    exit
  endif

  dest-res reg? if
    src-res dest-res xchg-r/m,
    exit
  endif

  src-res dest-res xchg-m/m,
;
\ ...SFP051

: find-exchange { | src dest -- ; find all sources whose dest is the source }
  #on-stack(ws) @ 0 ?do
    i in(ws) @ -> src
    src unmarkable? if
      i in(os) @ -> dest
      dest unmarkable? if               \ both sources, both count=1
        src on-os? if                   \ source on dest stack?
          dest on-ws? if                \ dest on source stack?
            = if                        \ at same place?
\ cr ." Exchanging " src .res-name ."  and " dest .res-name
              src dest s_xchg           \ lay code
              src  dup unmark refcount--  \ unmark both ; SFP093
              dest dup unmark refcount--  \ unmark both ; SFP093
              i set-dummy(ws)           \ remove source from working stack      SFP018
              shuffled incr
            endif
          endif
        endif
      endif
    endif
  loop
;

: move,         {: sres dres -- :}	\ SFP401
\ Move source to dest, does not use EAX unless available.
\ cr ." Moving from " sres .res-name ."  to " dres .res-name
  sres lit? if                          \ if source is a literal
    dres reg? if
      sres dres lit>reg  exit
    then
    sres lit-s32? if
      a[ MOV qword ]a  dres dgx>a  sres ,#x>a  use-a \ MOV dest, # lit
      exit
    then
    \ 64 bit literal, dres NOT a register
    GetShuffleReg dup 0= abort" MOVE, cannot get register for 64 bit literal"
    sres over lit>reg			\ 64 bit lit to register
    >r  a[ MOV qword ]a  dres dgx>a  r> ,sgx>a  use-a \ MOV dest, src
    exit
  endif
  sres DR1 = if                         \ if move from TOR (rare)
    sres count=1? if                    \ and it's the last one?
      a[ POP qword ]a  dres dgx>a  use-a	\ y:, POP rather than MOV
      exit
    endif
  endif
  sres reg?  dres reg? or if            \ if either is a register
    a[ MOV ]a  dres dgx>a  sres ,sgx>a  use-a  \ MOV dest, source
    exit
  endif
  GetShuffleReg dup if
    >r
    a[ MOV ]a  r@ dgx>a  sres ,sgx>a  use-a
    a[ MOV ]a  dres dgx>a  r> ,sgx>a  use-a	\ SFP401
    exit
  endif
  drop
  a[ PUSH qword ]a sres sgx>a  use-a	\ PUSH SOURCE
  a[ POP qword ]a  dres dgx>a  use-a	\ POP DEST
;
short-branches]

: can-move?	\ sres dres -- flag ; can we move sres to dres? ; SFP093
\ DRES is on the output stack, and is a register or deep stack.
  over source? 0=			\ item must be marked as a source
  if  2drop  false  exit  endif

  dup source? 0= if			\ dest must not be a source
\    dup reg? 0= if
\      2drop true  exit
\    endif
    dup res-refcount @ 0= if		\ and must not be an index reg in use
      2drop true  exit
    endif
  endif

\ 1) this test assumes that a move from: n [reg] to: reg is valid
\ 2) this assumption is true for MOST processors.
  over WorkIndMem? if			\ if the source is an INDMEM
    over res-assoc @ count=1? if	\ whose reg has a count of 1
      over res-assoc @  over = if	\ and dest is same register
        2drop true  exit		\ then we can move it
      endif
    endif
  endif

  2drop false
;

\ We can improve this.
\ 1) Only permit 1 instruction moves to begin with.
\ 2) check for free register, and permit these
\ 3) only permit intermediate register usage inside second level shuffle
\    after some resource has been saved/moved. Done in shuffle.
[-short-branches
: find-move     \ -- ; lay available moves ; SFP093
  #on-stack(ws) @ 0 ?do
    i in(ws) @ i in(os) @		\ -- source dest
    2dup can-move? if			\ if we can make the move
      2dup <> if			\ check for null move
        2dup 1insMove?			\ can always do 1 instruction moves
        GetShuffleReg 0<> or		\ can always do it if there's a spare register
        relaxed @ or if			\ or PUSH/POP pairs are permitted
\ cr ." Moving from " over .res-name ."  to " dup .res-name
          2dup move,  over dec-source	\ copy and count down
          i set-dummy(ws)		\ remove source from working stack
          shuffled incr			\ count number of moves
        endif
      endif
    endif
    2drop
  loop
;
short-branches]

: #sources      \ -- n ; return number of marked sources
  0  #on-stack(ws) @ 0 ?do
    i in(ws) @ source?
    if  1+  endif
  loop
;

: replace(ws)   \ oldres newres -- ; replace old by new on working stack
  swap
  #on-stack(ws) @ 0 ?do			\ -- newres oldres
    i in(ws) @ over = if
      over i in(ws) !
    endif
  loop
  2drop
;

: XferRefcounts	\ src dest -- ; SFP093
\ Transfer the reference counts from one resource to another,
\ and zero the source refcount. The source may be an INDMEM,
\ but the destination may not.
  dup IndMem? opt_toIndMem ?optThrow
  over IndMem? if			\ if indirect memory
    over dup res-refcount @ negate	\ subtract src indirect ref count
    swap res-assoc @ res-refcount +!	\ from that of associated register
  endif					\ -- src dest
  over res-refcount @			\ copy reference count
  swap res-refcount !
  res-refcount off			\ src is now unused
;

: XferRes	\ src dest --
\ copy resource from src to dest. At least one must be a register
  2dup sdXfer				\ copy it
  2dup XferRefcounts			\ move the reference count ; SFP093
  replace(ws)				\ replace all on working stack
;

\ SFP406...
: ReduceIndMem?	\ res cnt -- 0/1
\ Res is an INDMEM. If its refcount is greater than or equal to
\ cnt, try to copy it to its register. Return 1 if done, 0 if not.
  over res-assoc @ WorkReg? 0=		\ SFP407
  if  2drop 0 exit  endif
  >r  0 swap				\ -- 0 res ; R: -- cnt
  dup res-refcount @ r> >= if		\ multiply used
    dup res-refcount @			\ main ref count
    over res-assoc @ res-refcount @	\ assoc ref count
    = if				\ safe to move
      dup dup res-assoc @ XferRes	\ transfer
      swap 1+ swap
    endif
  endif
  drop
;

: ReduceIndMems	\ -- n ; SFP406
\ Copy (where possible) any multiply used INDMEMs on the working
\ stack to their registers. Return the number of changes made.
  0  #on-stack(ws) @ 0 ?do		\ -- n
    i in(ws) @ IndMem? if		\ its an INDMEM
      i in(ws) @ 2 ReduceIndMem? +	\ reduce if count is 2 or more
    endif
  loop
;
\ ...SFP406

: GetMaxOS      \ -- res|0
\ Get the output stack item with the highest ref-count.
\ We do not have to deal with INDMEMs because all output stack
\ items are in registers or DEEP stack. GETMAXOS will fail if
\ deep stack items are free, but no registers are available
\ because they are being used as index registers on the working
\ stack.
  0 0                                   \ -- dummy-res dummy-refcount
  #on-stack(ws) @ 0 ?do
    i in(os) @ source? if
      i in(os) @ res-refcount @ over > if
        2drop  i in(os) @ dup res-refcount @
      endif
    endif
  loop
  drop
;

: GetScratch    \ -- newres ; Find a scratch resource and copy to it
  GetMaxOS                              \ find dest that is most used source
  GetShuffleReg ?dup if                 \ can we use a spare register?
    a[ MOV ]a  dup dgx>a  over ,sgx>a  use-a
  else                                  \ n:
    DR1                                 \ top of return stack
    over >r  a[ PUSH qword ]a  r> sgx>a  use-a
  endif
  over res-refcount @                   \ copy refcount into new resource
  over res-refcount !
  2dup replace(ws)                      \ replace all source occurrences by new
  swap  dup unmark  res-refcount off    \ clear source and refcount on old
  dup mark                              \ mark as source
;

: FreeScratch   \ res -- ; free up the scratch resource
  res-refcount @ opt_scratch ?optThrow	\  abort" Couldn't free scratch resource"
;

: FreeIndReg?	{ | res -- res|0 }
\ Find an IndMem on the working stack whose working register has the
\ same reference count. This can be copied to its register, so making it
\ available for an easy move. This is only applicable for CPUs which use
\ INDMEMs and do not have mem->mem single instruction moves.
\ We use the INDMEM with the highest reference count as it generates
\ the highest number of single instruction stores.
  0 0                                   \ -- dummy-res dummy-refcount
  #on-stack(ws) @ 0 ?do			\ -- res cnt
    i in(ws) @ dup -> res WorkIndMem? if	\ IndMem?
      res res-refcount @		\ counts match?
      res res-assoc @ res-refcount @ = if
        res res-refcount @ over > if	\ more than previous?
          2drop  res dup res-refcount @
        endif
      endif
    endif
  loop
  drop
;

: IndMem>IndReg	\ res --
\ Move an INDMEM on the working stack to its associated register.
  dup res-assoc @ 2dup XferRes		\ -- src dest
  swap unmark  mark
;

variable show-shuffle

: ss            \ --
  show-shuffle on
;

variable <show-shuffle>

: shuffle-details       \ -- ; display shuffle state
  <show-shuffle> on  ;

: -shuffle-details      \ -- ; don't display shuffle state
  <show-shuffle> off  ;

: ?show-shuffle \ --
  <show-shuffle> @
  if .ws  endif
;

: keep-moving   \ -- ; keep moving until there's nothing left to do
  begin
?show-shuffle
    shuffled @
    find-move
    shuffled @ =
  until
;

: FreeIndMoves	\ -- ; free index reg, try move
  FreeIndReg? ?dup if
    IndMem>IndReg
    keep-moving
  endif
;

: ShufflePart1	\ --
\ The primary shuffle routine with the current constraints on
\ instruction sequence selection
  find-inplace  find-exchange  find-move
;

: ShufflePart2	\ --
\ Secondary shuffle, progressively releasing constraints on
\ instruction sequence selection
   FreeIndMoves  shuffled @ 0= if	\ try for index reg elimination
     relaxed on				\ permit PUSH/POP pairs
     ShufflePart1  shuffled @ 0= if	\ repeat part 1 with reduced constraint
       GetScratch			\ get a scratch resource
       keep-moving			\ let data be moved
       FreeScratch			\ free up the scratch resource
     endif
   endif
   relaxed off
;

: nshuffle      \ -- ; shuffle the general stack
  begin
?show-shuffle
    shuffled off			\ no shuffles yet this time
    relaxed off				\ tight constraints
    ShufflePart1
    #sources dup if			\ if shuffles left to do
      shuffled @ 0= if			\ did not manage a change?
        drop
        ShufflePart2
        shuffled @ 0= opt_shuffle ?optThrow \ check for disaster \ abort" Can't shuffle!"
        #sources                        \ see if there's more to do
      endif
    endif
    0=
  until
?show-shuffle
;

: (c_shuffle)	\ -- ; clean up stack at a boundary
  ReduceIndMems drop			\ SFP406
  pre-balance(ws)
  show-shuffle @off
  if  .ws .os  endif
  case  #on-stack(ws) @
    0 of  0shuffle  endof
    1 of  1shuffle  endof
          nshuffle
  endcase
  post-balance(ws)
  init-stack(ws)
;

: c_shuffle	\ -- ; clean up stack at a boundary
  (c_shuffle) check-stack(ws)
;

: c_GuardedShuffle	\ -- ; clean up stack at a boundary ; SFP411
  1 -> GuardedShuffle?  (c_shuffle)  0 -> GuardedShuffle?  check-stack(ws)
;

also forth definitions
: flushcode   \ -- ; force stack and optimiser to known state
  c_shuffle reset-opt NoElim		\ SFP412
;
previous definitions


\ ***************************************************************************
\ Copying literal when modified at run time. If a compile time operation
\ is performed on a multiply referenced literal, ALL copies are changed.                SFP0017
\ To prevent this, only literals with a reference count of 1 may be modified.
\ ***************************************************************************

: ?NewLitNeeded \ n -- ; replaces item n on stack if refcount<>1 ; SFP017
  in(ws) dup @                          \ preserve address, get resource
  dup lit?  swap count=1? 0= and        \ if not a destination (count=1)
  if                                    \ -- ^stack
    GetLitRes                           \ -- ^stack resource
    over @ res-value @ over res-value ! \ copy literal value to new
    over @ res-refcount decr            \ decrement old reference count
    1 over res-refcount !               \ set new reference count to 1
    swap !                              \ replace resource on stack
  else
    drop
  endif
;

: ?NewTOSlit    \ -- ; ensure literal TOS can be updated ; SFP017
  0 ?NewLitNeeded
;

: ?NewNOSlit    \ -- ; ensure literal NOS can be updated ; SFP017
  1 ?NewLitNeeded
;

: ?New3OSlit    \ -- ; ensure literal 3OS can be updated ; SFP096
  2 ?NewLitNeeded
;


\ ******************************************
\ Register requests that may cause a shuffle
\ ******************************************

\ If the nth item (0..n) is in memory, load it into a new register.
\ This is done when the destination of a stack operation is in
\ memory. The memory resource for that item is removed and replaced
\ by the register. If the operation fails, a zero flag is returned

: LoadDest      \ n -- flag ; SFP051
\ If the nth stack item is in memory, copy it to a register and
\ replace all its stack refences with the register reference.
\ Return true if the operation succeeds.
\ Note that this will NOT replace deep stack entries.
  in(ws) @ dup mem? if			\ if it is a memory reference
\ SFP420...
    dup IndMem? if			\ -- res
     dup 1 reduceIndMem?
     if  drop 1  exit  endif
    endif				\ -- res
\ ...SFP420
    GetNewReg ?dup if			\ -- oldres newres ; get new register
      XferRes -1
    else
      drop 0                            \ couldn't get register
    endif
  else
    drop  -1
  endif
;

((
: LoadDest      \ n -- flag ; SFP056
\ If the nth stack item is in memory, copy it to a register and
\ replace all its stack refences with the register reference.
\ Return true if the operation succeeds.
\ Note that this will NOT replace deep stack entries.
  in(ws) @ dup mem? if			\ if it is a memory reference
    GetNewReg ?dup if			\ -- oldres newres ; get new register
      XferRes -1
    else
      drop 0                            \ couldn't get register
    endif
  else
    drop  -1
  endif
;
))

: LoadTOS       \ -- flag ; if TOS is in memory, copy to register
\ Note that this will NOT replace deep stack entries.
  0 LoadDest
;

: LoadNOS       \ -- flag ; if NOS is in memory, copy to register
\ Note that this will NOT replace deep stack entries.
  1 LoadDest
;

: DisAmbiguate  \ min-depth -- ; Flush memory pool to registers   \ SFP026, SFP034
                \ Very simple, could be more complex, as TOS is always
                \ the address to store to. If a lit, see if it matches a
                \ MEMPOOL item, but even this is not always adequate
  dup min-depth(ws)                     \ SFP031
  #on-stack(ws) @ 0 ?do
    i LoadDest 0=
    if  c_shuffle  leave  endif
  loop
  min-depth(ws)                         \ SFP031
;

: -DisAmbiguate \ min-depth -- ; SFP401
\ Flush memory pool to registers, except the stack items within
\ min-depth. This is used by store operations to prevent a load of the
\ value destroying optimisation opportunities.
  dup >r min-depth(ws)
  #on-stack(ws) @ dup r@ min ?do
    i LoadDest 0=
    if  c_shuffle  leave  endif
  loop
  r> min-depth(ws)
;

: ForceNewTOS           \ -- ; get new register as TOS
  GetNewReg ?dup 0=
  if  c_shuffle  rRCX  endif
  push(ws)
;

: ForceNewTOS+NOS       \ -- ; get two new registers for TOS and NOS
  GetNewReg ?dup if
    dup res-refcount incr               \ so that GetNewReg won't find it
    GetNewReg
    over res-refcount decr              \ remove previous protection
    ?dup 0=
    if  drop  c_shuffle  rRCX  rRDX  endif
  else
    c_shuffle  rRCX  rRDX
  endif
  push(ws)  push(ws)
;


\ ***************
\ Compiling words
\ ***************

\ ----------------------
\ Start and end of words
\ ----------------------

: init-opt      \ -- ; set optimiser to unused/known state ; SFP035
  init-stack(ws)  reset-opt  NoElim
;
' init-opt AtCold

: c_:           \ -- ; initialise optimiser and compile code for :
  init-stack(ws)  reset-opt  start-asm
  start-inline  NoElim
;

: c_exit        \ -- ; compile code for EXIT
  not-inlined-exit			\ SFP410
  c_shuffle  reset-opt
  a[ ret ]a  use-a
;

: c_;		\ -- ; compile code for EXIT after ";"
  c_shuffle  reset-opt
  a[ ret ]a  use-a
  end-inline
;

also forth definitions
: clean-opt	\ --
\ Clean up the optimiser after an error that has NOT been
\ trapped by optThrow below.
  clean-resources init-opt
;
previous definitions

: (optThrow)	\ n --
  ?dup
  if  clean-opt  throw  endif
; assign (optThrow) to-do optThrow


\ --------------------------
\ Words involved with CREATE
\ --------------------------

: DODOES,       \ -- ; lay the entry code needed at the start of the
                \ run-time code defined by DOES>
  init-opt  start-asm
  DOESID start-seq  start-bkup          \ SFP035
    a[  lea  rbp, -8 [rbp]  ]a use-a    \ save old TOS
    a[  mov  0 [rbp], rbx   ]a use-a
    a[  pop  rbx            ]a use-a    \ new TOS is PFA is return address
  end-seq                               \ SFP035
;

: c_does>_>r    \ -- ; back up and lay new code         SFP032
;
((
: c_does>_>r    \ -- ; back up and lay new code         SFP035
  single-section? if
    \ it's a NOOP!
  else
    \ CAN use RAX because it's after a shuffle
    a[  mov  rax, 0 [rsp]   ]a use-a    \ get address after CALL runtime
    a[  mov  rax, 0 [rax]   ]a use-a    \ get contents (data address)
    a[  mov  0 [rsp], rax   ]a use-a    \ put CREATE address on return stack
  endif
;
))
+short-branches


\ ***********************************
\ Referencing a target word or symbol
\ ***********************************

\ Set up data for inlining control
0 #:entry !             \ size of COLON defn. entry code
1 #:exit !              \ size of COLON defn. exit code
0 #code-entry !         \ size of CODE defn. entry code
1 #code-exit !          \ size of CODE defn. exit code

: has-compiler?         \ xt -- flag ; true if word has compiler version
  >code-len @ $FFFF u>                  \ must have bit set in top 16 bits
;

: (compile-ref)	\ xt -- ; compile reference to xt
  dup has-compiler? if
    dup ndcs? if			\ NDCS word - no parameter ; SFP514
      >code-len @ execute
    else				\ normal code generator,
      dup >code-len @ execute		\   pass XT to compiling word
    endif
    exit
  endif
  c_shuffle  reset-opt                  \ flush stack
  dup inline? if
    inline,(t)
  else
    not-inlined doLayAbsCall		\ SFP015, SFP020 SFP043
  endif
;

0 value GenTokens?	\ -- flag ; true to generate a token stream
0 Value pSourceSin	\ -- addr ; pointer to token stream for word ; SFP414

: Compile-Ref           \ xt -- ; SFP015 SFP020 SFP414
  dup original-xt = if			\ prevent redefinition problem ; SFP070
    discard-sinline  warnings? if
      cr  space space dup >NAME dup .name ." contains a reference to " .name
    endif
  then
  GenTokens? if
    dup ,tok  0 -> GenTokens?
    (compile-ref)  pSourceSin
    if  -1 -> GenTokens?  endif
  else
    (compile-ref)
  endif
;

((
: NDCS-Ref           \ xt --
  dup original-xt = if			\ prevent redefinition problem ; SFP070
    discard-sinline  warnings? if
      cr  space space dup >NAME dup .name ." contains a reference to " .name
    endif
  then
  \ NDCS words may parse literals at compile time.
  \ Without source code you cannot parse. The NDCS portion has to be
  \ able to generate tokens, e.g. for parsed lits such as xts.
  (ndcs,)
;
))

: NDCS-Ref           \ xt --
  dup original-xt = if			\ prevent redefinition problem ; SFP070
    discard-sinline  warnings? if
      cr  space space dup >NAME dup .name ." contains a reference to " .name
    endif
  then
  \ This version is wrong because NDCS words may parse at compile time.
  \ Without source code you cannot parse. The NDCS portion has to be
  \ able to generate tokens, e.g. for parsed lits such as xts.
  GenTokens? if
    dup ,tok  0 -> GenTokens?
    (ndcs,)  pSourceSin
    if  -1 -> GenTokens?  endif
  else
    (ndcs,)
  endif
;

: o_docall,     \ xt -- ; needs a shuffle ; SFP020
  c_shuffle  reset-opt
  GenTokens? if
    callToken ,tok  dup ,tok
  then
  doLayAbsCall  not-inlined	\ SFP033 SFP043
;


((
: Compile-Run-Time-Ref  \ symbol --
  c_shuffle  reset-opt  not-inlined     \ flush stack
  $E8 c,c(t)
  2 cells+ 0 run-time-ref [ also asm-core ] REL32 [ previous ]
;
))


\ **********************************
\ value literals and symbol literals
\ **********************************

: c_lit         \ n -- ; compile code for resolved literal      \ SFP026
  dup genLitToken			\ SFP410
  LITPLUSID opt? if			\ if it follows a LIT + sequence
\ cr ." LITPLUSLITID"
    LITPLUSLITID seqid !                \ transform it into a LIT + LIT sequence
    seqflags @ seqprev !                \ preserve previous literal value
  else
    reset-opt
  endif
  GetLitRes tuck res-value !            \ -- reslit ; put value in literal resource
  dup lit-s32? 0= if			\ if 64 bit lit
    getRegRes tuck lit>reg		\ put in register
  then
  push(ws)				\ make TOS ; SFP051
;

: DoLiteral,    \ val -- ; lay code for literal handling
  c_lit  ;

: compile-literal       \ x -- ; compile the smallest literal form
  c_lit
;

: doliteralsym, \ xt -- ; lay symbol as a literal ; SFP019
  c_lit
;

: doliteralcfa, \ symbol -- ; lay symbol XT as a literal
  c_lit
;

\ ***************
\ Code Generators
\ ***************

\ ---------------
\ Stack operators
\ ---------------

\ 460 -> 460 no difference
: nth-mem>reg	\ u -- ; SFP422
\ If the nth stack item is in memory, copy it to a register.
  in(ws) @
  dup AbsMem?  over IndMem? or
  over Mem? or if			\ res --
    GetNewReg dup 			\ res reg --
    if  XferRes  else  2drop  ( c_shuffle )  endif
  else
    drop
  endif
;

(( \ 460 -> 490 slower
: nth-mem>reg	\ u -- ; SFP422
\ If the nth stack item not a reg or lit, copy it to a register.
  {: u | res -- :}
  u in(ws) @ dup -> res
  reg/lit-s32? 0= if
    GetNewReg ?dup if
      res swap XferRes
    then
  then
;
))
((
: uth-lit>reg	\ u -- ; SFP516
\ If uth item is a lit, copy it to a register
  {: u | res -- :}
  u in(ws) @ dup -> res lit-s32? if
    GetNewReg ?dup if
      res swap XferRes
    then
  then
;
))
: uth-lit/mem>reg	\ u -- ; SFP516
\ If uth item is a lit or in memory, copy it to a register
  {: u | res -- :}
  u in(ws) @ dup -> res lit-s32? if
    GetNewReg ?dup if
      res swap XferRes
    then
    exit
  then
  u in(ws) @ dup -> res
  dup AbsMem?  over IndMem? or  swap Mem? or if	\ --
    GetNewReg ?dup if			\ reg --
      res swap XferRes
    then
  then
;

: c_dup         \ -- ; compile code for DUP
  1 min-depth(ws)  reset-opt		\ SFP051
  0 uth-lit/mem>reg			\ SFP516
\  0 uth-lit>reg
\  0 nth-mem>reg			\ SFP422
  tos-res push(ws)
;

\ C_?DUP moved to make optimisation easier

: c_2dup        \ -- ; compile code for 2DUP
  2 min-depth(ws)  reset-opt		\ SFP051
  tos-res nos-res push(ws) push(ws)	\ this order is IMPORTANT!
;

: c_3dup        \ -- ; compile code for 3DUP
  3 min-depth(ws)  reset-opt
  tos-res nos-res 3os-res push(ws) push(ws) push(ws)  ;

: c_drop        \ -- ; compile code for DROP    SFP035
  1 min-depth(ws)			\ SFP051
  RFROMID opt? if                       \ SFP035
    seq-backup
    a[ lea  rsp, 8 [rsp]  ]a  use-a
  endif
  reset-opt  drop(ws)  ;

: c_2drop       \ -- ; compile code for 2DROP
  2 min-depth(ws)  reset-opt		\ SFP051
  drop(ws) drop(ws)  ;

: c_over        \ -- ; compile code for OVER
  2 min-depth(ws)  reset-opt		\ SFP051
  1 uth-lit/mem>reg			\ SFP516
\  1 uth-lit>reg
\  1 nth-mem>reg				\ SFP422
  nos-res push(ws)  ;

: c_2over       \ -- compile code for 2OVER
  4 min-depth(ws)  reset-opt		\ SFP051
  4os-res push(ws)  4os-res push(ws)  ;

: (c_swap)	\ --
  nos-res tos-res set-nos set-tos  ;

: c_swap        \ -- ; compile code for SWAP
  2 min-depth(ws)  reset-opt  (c_swap)  ;

: c_2swap       \ compile code for 2SWAP
  4 min-depth(ws)  reset-opt
  pop(ws) pop(ws) pop(ws) pop(ws)
  2swap
  push(ws) push(ws) push(ws) push(ws)  ;

: c_nip         \ -- ; compile code for NIP
  2 min-depth(ws)  reset-opt		\ SFP051
  pop(ws) drop(ws) push(ws)  ;

: c_tuck        \ -- ; compile code for TUCK
  2 min-depth(ws)  reset-opt		\ SFP051
  0 uth-lit/mem>reg			\ SFP516
\  0 uth-lit>reg				\ SFP516
\  0 nth-mem>reg				\ SFP422
  pop(ws) pop(ws)                       \ tos nos --
  over push(ws) push(ws) push(ws)  ;    \ put back

: c_rot         \ -- ; compile code for ROT
  3 min-depth(ws)  reset-opt
  pop(ws) pop(ws) pop(ws) -rot push(ws) push(ws) push(ws)  ;

: c_-rot        \ -- ; compile code for -ROT
  3 min-depth(ws)  reset-opt
  pop(ws) pop(ws) pop(ws) rot push(ws) push(ws) push(ws)  ;

: c_2rot	\ -- ; compile code for 2ROT ; SFP415
\ x1 x2 x3 x4 x5 x6 -- x3 x4 x5 x6 x1 x2
  6 min-depth(ws)  reset-opt
  pop(ws) pop(ws) pop(ws) pop(ws)
  pop(ws) pop(ws)			\ -- x6 x5 x4 x3 x2 x1
  >r >r
  push(ws) push(ws) push(ws) push(ws)
  r> r>					\ -- x2 x1
  push(ws) push(ws)
;

: c_2pick       \ -- ; compile code for 2 PICK
  3 min-depth(ws)  reset-opt		\ SFP051
  2 uth-lit/mem>reg			\ SFP516
\  2 uth-lit>reg
\  2 nth-mem>reg				\ SFP422
  3os-res push(ws)  ;

-short-branches
: c_pick        \ -- ; generate code for PICK
  1 min-depth(ws)  reset-opt
  tos-res lit? if
    pop(ws) res-value @				\ remove literal from working stack
    dup 0< opt_pick ?optThrow			\ Negative input to PICK?
    dup in-range(ws)                            \ Is it in the stack model?
    if                                          \ y: required source on the working stack
      case                                      \ check for equivalents
        0 of  c_dup    endof
        1 of  c_over   endof
        2 of  c_2pick  endof                    \ SFP052
        \ -- lit
\ XC6 version - fastest so far
              in(ws) @ push(ws)			\ SFP054
      end-case
      exit
    endif
    c_lit                                       \ can't optimise, put literal back
  endif
  c_shuffle
  a[ MOV    RBX, 0 [RBP] [RBX*8] ]a use-a
;
+short-branches


\ The following 4 optimisers are to handle the 0 and sign extension of
\ items already on the stack. They have the stack picture:
\   nn..n0 n -- nn'..n0
\ where nn is sign or zero extended as required. Thus functions are provided
\ to permit promotion of items less than one cell in size when fetched from
\ items already on the stack.
\
\ These are crude code generators. During their normal use TOS will be a lit
\ and the item will always be on the stack. Also the stack item affected will
\ almost certainly be a literal (constant or address).
\
\ A proper generator should be able to at least work without the shuffle if
\ the dest. is in-range (which it probably will be).
\
\ Also, if the target stack item is a literal, why not do the sign/zero
\ extension at compile time and put it back in the resource (no target
\ code would be generated then)??? Hmm.. Have to think about that, perhaps
\ it would only be safe to do the 0 extension in this way.
\
\ Actually, the Unsigned generators could check. If the target is a literal
\ and already has the top bits clear (quite often the case) then it could
\ just EXIT and do nothing.

: nsGetValid	\ -- n flag
\ Get the TOS literal and return true if the code can be optimised.
  pop(ws) res-value @			\ remove literal from working stack ; SFP063
  dup 0< opt_argneg ?optThrow		\ Negative offset
  dup in-range(ws)			\ Is it in the stack model?
  over in(ws) @ count=1?  and		\   and only referenced once
;

: msbwiden	{ rtmp rmem -- }
\ Widen a signed 8 bit word in memory rmem, using rtmp as a
\ temporary register.
  a[ MOVSX ]a  rtmp dgx>a  rmem ,b>a  use-a
  a[ MOV ]a  rmem dgx>a  rtmp ,x>a  use-a
;

-short-branches
: c_nsbwiden    \ -- ; Signed byte widen Nth Item on stack                      \ SJC025
  1 DisAmbiguate  reset-opt
  tos-res lit? if
    nsGetValid if                       \ -- n ; y: required source on the working stack
      in(ws) @ >r                       \ get resource
      r@ reg? if
        a[ MOVSX ]a  r@ dgx>a  r> ,b>a  use-a
        exit
      endif
      r@ lit? if
        r@ res-value @
        dup $080 and
        if  $FFFFFF00 or  endif
        r> res-value !
        exit
      endif
      GetNewReg dup if
        r> msbwiden
      else
        drop
        a[ PUSH RAX ]a use-a
        rRAX r> msbwiden
        a[ POP RAX ]a use-a
      endif
      exit
    endif
    c_lit
  endif
  c_shuffle
\ **** TOS MUST BE IN RBX, RAX is free after shuffle
  a[ MOVSX      RAX, byte ptr 0 [RBP] [RBX*8] ]a  use-a
  a[ MOV        0 [RBP] [RBX*8] ,a RAX ]a  use-a
  c_drop
;

: mswwiden	{ rtmp rmem -- }
\ Widen a signed 16 bit word in memory rmem, using rtmp as a
\ temporary register.
  a[ MOVSX ]a  rtmp dgx>a  rmem ,w>a  use-a
  a[ MOV ]a  rmem dgx>a  rtmp ,sgx>a  use-a
;

: c_nswwiden    \ -- ; Signed word widen Nth Item on stack                      \ SJC025
  1 DisAmbiguate  reset-opt
  tos-res lit? if
    nsGetValid if                       \ -- n ; y: required source on the working stack
      in(ws) @ >r                       \ get resource
      r@ reg? if
        a[ MOVSX ]a  r@ dgx>a  r> ,w>a  use-a
        exit
      endif
      r@ lit? if
        r@ res-value @
        dup $08000 and
        if  $FFFF0000 or  endif
        r> res-value !
        exit
      endif
      GetNewReg dup if
        r> mswwiden
      else
        drop
        a[ PUSH RAX ]a use-a
        rRAX r> mswwiden
        a[ POP RAX ]a use-a
      endif
      exit
    endif
    c_lit
  endif
  c_shuffle
\ **** TOS MUST BE IN RBX, RAX is free after shuffle
  a[ MOVSX      RAX, word ptr 0 [RBP] [RBX*8] ]a use-a
  a[ MOV        0 [RBP] [RBX*8] ,a RAX ]a use-a
  c_drop
;

: mslwiden	{ rtmp rmem -- }
\ Widen a signed 32 bit word in memory rmem, using rtmp as a
\ temporary register.
  a[ MOVSX ]a  rtmp dgx>a  rmem ,l>a  use-a
  a[ MOV ]a  rmem dgx>a  rtmp ,sgx>a  use-a
;

: c_nslwiden    \ -- ; Signed 32 bit dword widen Nth Item on stack                      \ SJC025
  1 DisAmbiguate  reset-opt
  tos-res lit? if
    nsGetValid if                       \ -- n ; y: required source on the working stack
      in(ws) @ >r                       \ get resource
      r@ reg? if
        a[ MOVSX ]a  r@ dgx>a  r> ,l>a  use-a
        exit
      endif
      r@ lit? if
        r@ res-value @
        dup $0000:0000:8000:0000 and	\ bit 31 set?
        if  $FFFF:FFFF:8000:0000 or  endif
        r> res-value !
        exit
      endif
      GetNewReg dup if
        r> mslwiden
      else
        drop
        a[ PUSH RAX ]a use-a
        rRAX r> mslwiden
        a[ POP RAX ]a use-a
      endif
      exit
    endif
    c_lit
  endif
  c_shuffle
\ **** TOS MUST BE IN RBX, RAX is free after shuffle
  a[ MOVSX      RAX, dword ptr 0 [RBP] [RBX*8] ]a use-a
  a[ MOV        0 [RBP] [RBX*8] ,a RAX ]a use-a
  c_drop
;

: c_nubwiden    \ -- ; Unsigned byte widen Nth Item on stack                      \ SJC025
  1 DisAmbiguate  reset-opt
  tos-res lit? if
    nsGetValid if                       \ -- n ; y: required source on the working stack
      in(ws) @ >r                       \ get resource
      r@ lit? if
        r@ res-value @
        $0FF and
        r> res-value !
        exit
      endif
      a[ ANDa qword ]a  r> dgx>a  a[ ,a # $000000FF ]a use-a
      exit
    endif
    c_lit
  endif
  c_shuffle
\ **** TOS MUST BE IN EBX
  a[ ANDa qword  0 [RBP] [RBX*8] ,a # $000000FF ]a use-a
  c_drop
;

: c_nuwwiden    \ -- ; Unsigned 16 bit Word widen Nth Item on stack                      \ SJC025
  1 Disambiguate  reset-opt
  tos-res lit? if
    nsGetValid if                       \ -- n ; y: required source on the working stack
      in(ws) @ >r                       \ get resource
      r@ lit? if
        r@ res-value @
        $0FFFF and
        r> res-value !
        exit
      endif
      a[ ANDa qword ]a  r> dgx>a  a[ ,a # $0000FFFF ]a use-a
      exit
    endif
    c_lit
  endif
  c_shuffle
\ **** TOS MUST BE IN EBX
  a[ ANDa qword  0 [RBP] [RBX*8] ,a # $0000FFFF ]a use-a
  c_drop
;

: c_nulwiden    \ -- ; Unsigned 32 bit Word widen Nth Item on stack                      \ SJC025
  1 Disambiguate  reset-opt
  tos-res lit? if
    nsGetValid if                       \ -- n ; y: required source on the working stack
      in(ws) @ >r                       \ get resource
      r@ lit? if
        r@ res-value @
        $0FFFF:FFFF and
        r> res-value !
        exit
      endif
      getRegRes >r			\ R: -- dest temp
      a[ MOV ]a  r@ dgx>a  a[ ,a # $0000:FFFF:FFFF ]a use-a
      r> r> swap >r >r			\ R: -- temp dest
      a[ ANDa qword ]a  r> dgx>a  r> ,sgx>a  use-a
      exit
    endif
    c_lit
  endif
  c_shuffle
\ **** TOS MUST BE IN EBX, RAX is free after shuffle
  a[ MOV   RAX ,a # $0000:0000:FFFF:FFFF ]a use-a
  a[ ANDa  0 [RBP] [RBX*8] ,a RAX ]a use-a
  c_drop
;


\ --------------------
\ LITPLUSxx optimisers
\ --------------------

: ?rem-litplus  \ -- n ; remove LIT + code if possible, and return offset ; SFP026
  LITPLUSID opt? if                     \ SFP035
\ cr ." Got LITPLUSID x"
    seq-backup    seqflags @            \ remove LIT + code, get value
  else
    0
  endif
;

: ?TOSrem-litplus	\ -- n ; as above but check tos=dreg ; SFP098
  tos-res dreg? if
    ?rem-litplus
  else
    0
  endif
;

: ?rem-lit+lit  \ -- n ; remove LIT + code if possible, and return offset       \ SFP026
  LITPLUSLITID opt? if                  \ SFP035
\ cr ." Got LITPLUSLITID x"
    seq-backup  seqprev @               \ remove LIT + code, get value
  else
    0
  endif
;

: litplus-seq   \ res -- ; start a LITPLUSID sequence           \ SFP026
  ?rem-lit+lit over res-value +!        \ deal with previous LIT + sequence
\ cr ." LITPLUSID"
  LITPLUSID start-seq                   \ start sequence
  res-value @ seqflags !                \ save literal value
  start-bkup                            \ mark start of code
;

: ?beg-litplus  \ res xt -- ; start sequence if xt is of +      \ SFP026
  ['] + =
  if  litplus-seq  else  drop  reset-opt  endif
;

: ?end-litplus  \ xt -- ; end sequence if xt is of +            \ SFP026
  ['] + =
  if  end-seq  endif
;

: ?beg-reglitplus       \ lit dest xt -- ; start sequence if dest is reg and xt is of +      \ SFP026
  ['] + =  swap reg? and
  if  litplus-seq  else  drop  reset-opt  endif
;

: ?end-reglitplus       \ dest xt -- ; end sequence if xt is of +            \ SFP026
  ['] + =  swap reg? and
  if  end-seq  endif
;

\ --------------------
\ Arithmetic operators
\ --------------------

: IndexReg?	\ res -- flag ; true if res is an index register ; SFP093
  0 swap  dup reg? if			\ -- 0 res
    #on-stack(ws) @ 0 ?do		\ -- 0 res
      i in(ws) @ IndMem? if		\ if item is an INDMEM
        dup i in(ws) @ res-assoc @ = if	\ and the index register is res
          nip true swap leave		\ then check is true ; true res
        endif
      endif
    loop
  endif
  drop
;

: MakeBestTOS	\ res -- ; if rEBX is TOS, leave it there, else TOS:=res ; SFP034, SFP093
\ MAKEBESTTOS is used when TOS is required for a destination on the
\ working stack, but the current tos cannot be used. The parameter
\ RES is a free register.
\ If the current TOS is rTOS (refcount > 1) then rTOS is made the new
\ TOS by copying all but one references to RES.
\ If the current TOS is not rTOS, then RES is made the new NOS
\ by copying one reference to RES
  NoElim
  tos-res rTOS = if
    tos-res IndexReg? 0= if		\ but not if its an index register
      pop(ws)				\ remove TOS (rTOS) ; -- new tos
      dup rot XferRes			\ transfer remaining to new
      push(ws)				\ put TOS back
      exit
    endif
  endif
  CopyTOS
;

: MakeBestNOS	\ res -- ; if rEBX is NOS, leave it there, else NOS:=res ; SFP034, SFP093
\ MAKEBESTNOS is used when NOS is required for a destination on the
\ working stack, but the current nos cannot be used. The parameter
\ RES is a free register.
\ If the current NOS is rTOS (refcount > 1) then rTOS is made the new
\ NOS by copying all but one references to RES.
\ If the current NOS is not rTOS, then RES is made the new NOS
\ by copying one reference to RES
  NoElim  nos-res rTOS = if
    nos-res IndexReg? 0= if		\ but not if its an index register ; SFP005
      nos-res swap			\ -- nos new
      2dup XferRes			\ move NOS to new
      refcount--			\ all but one to dest
      dup refcount++			\ one left in nos
      set-nos				\ restore NOS as NOS
      exit
    endif
  endif
  nos-res over sdXfer			\ move NOS to new register
  pop(ws) drop(ws)		        \ discard previous NOS
  swap push(ws) push(ws)	        \ install new NOS
;

: Pshuffle	\ --
\ Shuffle, but preserve the working stack depth.
  #on-stack(ws) @  c_shuffle  min-depth(ws)
;

\ SFP405...
: TOS>dreg	\ --
\ Get TOS into a dreg for manipulation; can shuffle. The working
\ stack depth is preserved.
  tos-res dreg? if  exit  endif
  GetNewReg ?dup 0=
  if  Pshuffle exit  endif
  pop(ws) over P0sdmove,  push(ws)
;

: TOS>dRes	\ --
\ Get TOS into a resource for manipulation; can shuffle. The working
\ stack depth is preserved.
  tos-res count=1? if
    tos-res deep? tos-res reg? or
    if  exit  endif
  endif
  TOS>dreg
;
\ ...SFP405

: FlagsSetOpt	\ -- ; mark flags as set ; SFP051
  FLAGSETID [opt opt]
;

: ?FlagsSetOpt	\ -- ; mark flags as set if no LITPLUSID ; SFP051
  seqID @ LITPLUSID <>
  if  FlagsSetOpt  endif
;

: 2lits?	\ -- flag ; SFP404
\ Return true if TOS and NOS are both literals.
  tos-res lit?  nos-res lit? and
;

: 2lit1op	\ xt --
\ Manipulates two literals at compile time to produce one.
  ?NewNOSLit				\ ensure NOS is free for modification ; SFP017
  nos-res res-value @  tos-res res-value @
  rot execute				\ perform compile time function
  nos-res res-value !
  drop(ws)				\ SFP027
;

: ?tosLit64>reg	\ -- ; SFP512
\ If TOS lit is not a 32 bit lit, put TOS in a new register. For
\ now, barf if we run out of registers.
  tos-res lit? if
    tos-res lit-s32? 0= if
      tos-res getRegRes xferRes
    then
  then
;

: ?nosLit64>reg	\ -- ; SFP512
\ If NOS is lit and not a 32 bit lit, put NOS in a new register. For
\ now, barf if we run out of registers.
  nos-res lit? if
    nos-res lit-s32? 0= if
      nos-res getRegRes xferRes
    then
  then
;

: ?3osLit64>reg	\ -- ; SFP512
\ If 3OS is lit and not a 32 bit lit, put 3OS in a new register. For
\ now, barf if we run out of registers.
  3os-res lit? if
    3os-res lit-s32? 0= if
      3os-res getRegRes xferRes
    then
  then
;

: (comdyad)     { opc xt -- }
\ Code generates for commutative dyads (a OP b = b OP a) other than xt=+.
\ Use of this factor permits recursion when we have to shuffle.
  2 min-depth(ws)			\ SFP026 SFP051

\ Deal with 2 literals first, more optimisation available
  2lits? if				\ can do everything at compile time ; SFP404
    xt 2lit1op  reset-opt  exit
  endif

  nos-res lit?				\ commutative, so can do this ; SFP401
  if  (c_swap)  endif
  ?tosLit64>reg				\ if tos=64 bit lit, make reg ; SFP512

\ Deal with single literal next
  tos-res lit? if
    nos-res count=1? if                 \ TOS is LIT, is NOS a destination?
      nos-res deep? if			\ Faster? - yes ; SFP401
        GetNewReg ?dup
        if  nos-res swap XferRes  endif
      endif
      LoadNOS 0=
      if  c_shuffle  opc xt recurse  exit  endif	\ SFP051
      tos-res nos-res xt ?beg-reglitplus                \ SFP026
      opc execute  nos-res dgx>a  pop(ws) ,sgx>a  use-a    \ ADD NOS, TOSLIT
      tos-res xt ?end-reglitplus                        \ SFP026
    else                                \ n: need a clean resource
      GetNewReg ?dup 0=
      if  c_shuffle  opc xt recurse  exit  endif	\ SFP051
      MakeBestNOS			\ NOS is now reg with count=1	  		SFP036
      tos-res xt ?beg-litplus                           \ SFP027, 		SFP036
      opc execute  nos-res dgx>a  tos-res ,sgx>a  use-a
      xt ?end-litplus                                   \ SFP027, 		SFP034
      drop(ws)						\ 			SFP034
    endif
    ?FlagsSetOpt			\ SFP051
    exit
  endif

\ Check whether TOS or NOS can be used as a destination (ref count=1)
  nos-res rRBX =			\ if NOS=RBX, make it TOS
  if  nos-res tos-res set-nos set-tos  endif	\ SFP053

\ Check whether TOS or NOS can be used as a destination (ref count=1)
  tos-res dreg? if
    opc execute  tos-res dgx>a  nos-res ,sgx>a  use-a       \ ADD TOS, NOS
    nip(ws)  FlagsSetOpt  exit		\ SFP026, SFP051
  endif
  nos-res dreg? if
    opc execute  nos-res dgx>a  tos-res ,sgx>a  use-a       \ ADD NOS, TOS
    drop(ws)  FlagsSetOpt  exit		\ SFP026, SFP051
  endif

\ Check for special case (publicity, really) in which tos=nos, count=2
  tos-res reg? if			\ SFP085
    nos-res tos-res = if
      tos-res res-refcount @ 2 = if
        opc execute  nos-res dgx>a  tos-res ,sgx>a  use-a	\ ADD TOS, NOS
        drop(ws)  FlagsSetOpt  exit
      endif
    endif
  endif

\ TOS and NOS not available, need a free resource, use sequence MOV NEW, TOS  ADD NEW, NOS
  GetNewReg ?dup 0=
  if  c_shuffle  opc xt recurse  exit  endif	\ SFP051
  MakeBestTOS					\ select and copy to best TOS reg	SFP036
  opc execute  tos-res dgx>a  nos-res ,sgx>a  use-a \ ADD NEW, NOS
  nip(ws)					\					SFP036
  FlagsSetOpt				\ SFP051
;

interpreter
: comdyad       \ -- ; -- ; COMDYAD C_+ + "ADD"
  create
    ' ,                                 \ get XT of compile time action
    here  0 ,  lay-asm: drop            \ for xt of opcode action
  does>
    dup cell+ @  swap @                 \ -- opc xt
    (comdyad)                           \ use common code
;
target

: +n	+ ;	\ n1 n2 -- n2 ; allows system to distinguish + and +n ; SFP401
: +c    1 abort" Cannot execute +C"  ;  \ SFP030

comdyad (c_+) +    "ADD   "		\ SFP056
comdyad c_+n  +n   "ADD   "		\ SFP056
comdyad c_+c  +c   "ADC   "
comdyad c_and and  "ANDa  "
comdyad c_or  or   "ORa   "
comdyad c_xor xor  "XORa  "

: c_+		\ -- ; SFP056
  2 min-depth(ws)
  CELLSID opt? nos-res lit-s32? and if	\ transform CELLSID to CELLSLITPLUSID
    seqbkup @  CELLSLITPLUSID [opt  seqprev !
    nos-res res-value @ seqflags !
    a[ ADD qword ]a  tos-res dgx>a  nos-res ,#x>a  use-a  nip(ws)
    opt]
    exit
  endif
  CELLSID opt? nos-res reg? and if	\ transform CELLSID to CELLSREGPLUSID
    seqbkup @  CELLSREGPLUSID [opt  seqprev !
    nos-res seqptr !
    a[ ADD ]a  tos-res dgx>a  nos-res ,sgx>a  use-a  nip(ws)
    opt]
    exit
  endif
  tos-res lit=0?
  if  drop(ws) reset-opt  exit  endif
  nos-res lit=0?
  if  nip(ws) reset-opt  exit  endif
  (c_+)
;

: (noncomdyad)  { opc xt -- ; code generates for a noncommutative dyad (a OP b != b OP a) }
                \ use of this factor permits recursion when GetNewReg fails
  2 min-depth(ws)  reset-opt		\ SFP051

  \ Deal with 2 literals first, more optimisation available
  2lits? if				\ can do everything at compile time ; SFP404
    xt 2lit1op  exit			\ SFP404
  endif

  ?tosLit64>reg  ?nosLit64>reg		\ SFP512

\ Deal with single literal next
  tos-res lit? if
    nos-res count=1? if                 \ NOS a destination?
      nos-res deep? if			\ Faster? - yes ; SFP401
        GetNewReg ?dup
        if  nos-res swap XferRes  endif
      endif
      LoadNOS 0=
      if  c_shuffle  opc xt recurse  exit  endif	\ SFP051
      opc execute  nos-res dgx>a  pop(ws) ,#x>a  use-a	\ OPx NOS, # TOSLIT
    else                                \ n: need a clean resource
      GetNewReg ?dup 0=
      if  c_shuffle  opc xt recurse  exit  endif	\ SFP051
      a[ MOV ]a  dup dgx>a  pop(ws) pop(ws) ,sgx>a  use-a  \ MOV NEW, NOS
      opc execute  over dgx>a  ,#x>a  use-a             \ OPx NEW, # TOSLIT
      push(ws)                                          \ new resource becomes TOS
    endif
    FlagsSetOpt				\ SFP051
    exit
  endif

  nos-res lit? if
    GetNewReg ?dup 0=
    if  c_shuffle  opc xt recurse  exit  endif		\ SFP051
    \ -- newreg
    pop(ws) >r						\ -- newreg ; R: -- tos ; SFP064
    pop(ws) over P0sdCopy				\ MOV NEW, # NOSLIT ; SFP064
    opc execute  dup dgx>a  r> ,sgx>a  use-a            \ OPx NEW, TOS ; SFP064
    push(ws)                                            \ new resource becomes TOS
    FlagsSetOpt				\ SFP051
    exit
  endif

\ Check whether NOS can be used as a destination (ref count=1)
  nos-res dreg? if
    opc execute  nos-res dgx>a  tos-res ,sgx>a  use-a	\ ADD NOS, TOS
    drop(ws)
    FlagsSetOpt				\ SFP051
    exit
  endif

\ TOS and NOS not available, need a free resource,
\ use sequence MOV NEW, NOS  OPx NEW, TOS
  GetNewReg ?dup 0=
  if  c_shuffle  opc xt recurse  exit  endif		\ SFP051
  \ -- newreg
  >r					\ SFP064
  pop(ws) pop(ws) r@ P0sdCopy		\ MOV NEW, NOS ; SFP064
  opc execute  r@ dgx>a  ,sgx>a  use-a	\ SUB NEW, TOS ; SFP064
  r> push(ws)				\ SFP064
  FlagsSetOpt				\ SFP065
;

interpreter
: noncomdyad    \ -- ; -- ; COMDYAD C_- - "SUB"
  create
    ' ,					\ get XT of compile time action
    here  0 ,  lay-asm: drop            \ for xt of opcode action
  does>
    dup cell+ @  swap @                 \ -- opc xt
    (noncomdyad)                        \ use common code
;
target

: -c    1 abort" Cannot execute -C"  ;  \ SFP030

noncomdyad (c_-)  -   "SUB   "
noncomdyad c_-n   -   "SUB   "
noncomdyad c_-c   -c  "SBB   "

: c_-		\ -- ; optimises x lit - ; SFP047
  2 min-depth(ws)
  tos-res lit? if
    ?NewTosLit				\ SFP055
    tos-res res-value @			\ negate literal
    negate
    tos-res res-value !
    c_+					\ perform x -lit +
  else
    (c_-)
  endif
;

variable FragBeg	\ -- addr ; holds start of code fragment
variable FragEnd	\ -- addr ; holds end of code fragment

: (monad)       { xt opc arg -- ; generate code for a monadic instruction OPC tos, arg }
  1 min-depth(ws)  reset-opt
  FragBeg off FragEnd off
  tos-res lit? if
    ?NewTOSLit                          \ ensure TOS is free for modification ; SFP017
    tos-res res-value  dup @  xt execute  swap !
    exit
  endif
  tos-res count=1? if
    LoadTOS 0=				\ only loads MEMs
    if  c_shuffle  xt opc arg recurse  exit  endif
    tos-res reg? 0= if			\ do not perform op in memory
      GetNewReg ?dup
      if  MakeBestTOS  endif
    endif
    chere(t) FragBeg !
    opc execute  tos-res dgx>a  arg execute  use-a      \ OP tos, arg
    chere(t) FragEnd !
    exit
  endif
  GetNewReg ?dup if
    MakeBestTOS						\ decide what to use as TOS	SFP034 SFP036
    chere(t) FragBeg !
    opc execute  tos-res dgx>a  arg execute  use-a      \ OP newreg, arg
    chere(t) FragEnd !
  else
    c_shuffle  xt opc arg recurse
  endif
;

interpreter
: monad         \ -- ; -- ; generate code for a monadic instruction OPC tos, arg
  create
    ' ,                                 \ xt for compile time action
    here
    0 ,                                 \ instruction
    0 ,                                 \ argument
    lay-asm: lay-asm: drop              \ get required actions
  does>
    dup @  swap cell+ dup @  swap cell+ @       \ -- xt opc arg
    (monad)
;
target

monad c_1+      1+     "INC   "  " "
monad c_2+      2+     "ADD   "  ",a # 2"
monad c_4+      4+     "ADD   "  ",a # 4"
monad c_8+      8+     "ADD   "  ",a # 8"
monad c_1-      1-     "DEC   "  " "
monad c_2-      2-     "SUB   "  ",a # 2"
monad c_4-      4-     "SUB   "  ",a # 4"
monad c_8-      8-     "SUB   "  ",a # 8"
monad c_2*      2*     "SHL   "  ",a # 1"
monad c_4*      4*     "SHL   "  ",a # 2"
monad (c_8*)    8*     "SHL   "  ",a # 3"
monad c_2/      2/     "SAR   "  ",a # 1"
monad c_4/      4/     "SAR   "  ",a # 2"
monad c_8/      8/     "SAR   "  ",a # 3"
monad c_u2/     u2/    "SHR   "  ",a # 1"
monad c_u4/     u4/    "SHR   "  ",a # 2"
monad c_u8/     u8/    "SHR   "  ",a # 3"
monad c_negate  negate "NEG   "  " "
monad (c_invert) invert "NOTa   "  " "	\ SFP093
monad (c_not)   not    "NOTa   "  " "	\ SFP093


: ?SetFragOpt	\ seqid -- ; start a removable sequence ; SFP401
  FragBeg @ if
    seqid !
    FragBeg @ dup seqbeg ! seqbkup !
    FragEnd @ seqend !
  else
    drop
  endif
;

: c_8*		\ --
\ version with optimiser sequence
  (c_8*)  tos-res dreg? if
    CELLSID ?SetFragOpt			\ set sequence
    tos-res seqres !			\ save register
  endif
;


\ ---------------------------
\ Logical and shift operators   SFP022
\ ---------------------------

: (shifter)     { xt opc -- ; generate code for given shift instruction }
  2 min-depth(ws)  reset-opt		\ SFP051

\ Deal with 2 literals first, more optimisation available
\  tos-res lit?  nos-res lit? and if     \ can do everything at compile time
  2lits? if				\ can do everything at compile time ; SFP404
    xt 2lit1op  exit			\ SFP404
  endif

\ Deal with TOS (shift count) = literal next
  tos-res lit? if
\ SFP056...
    tos-res res-value @ 0= if
      <impolite> @ if
        cr ." Urk! literal shift by zero, ignored"
        .source-line
      endif
      drop(ws)  exit
    endif
\ ...SFP056
    nos-res dreg?                       \ TOS is LIT, is NOS a destination register
    if                                  \ y: can add in place
      opc execute  nos-res dgx>a  pop(ws) ,#x>a  use-a    \ OPx NOS, # TOSLIT
    else                                \ n: need a clean resource
      GetNewReg ?dup 0=
      if  c_shuffle  xt opc recurse  exit  endif	\ SFP051
      \ -- newreg
      >r  pop(ws) pop(ws) r@ P0sdCopy	\ MOV NEW, NOS ; -- tos ; R: -- newreg ; SFP064
      opc execute  r@ dgx>a  ,#x>a  use-a	\ OPx NEW, #TOSLIT ; SFP064
      r> push(ws)                       \ new resource becomes TOS ; SFP064
    endif
\    FlagsSetOpt				\ SFP051
    exit
  endif

\ shift count not a literal, MUST be in RCX
  rRCX free? if
    pop(ws) rRCX P0sdCopy               \ force TOS into rRCX ; SFP051
    rRCX push(ws)
  else
    tos-res rRCX <> if                  \ if not already there
      c_shuffle  xt opc recurse  exit	\ SFP051
    endif
  endif

\ shift count in TOS=RCX, force NOS into register
  nos-res dreg? 0= if
    GetNewReg ?dup 0=
    if  c_shuffle  xt opc recurse  exit  endif	\ SFP051
    pop(ws) >r                          \ preserve TOS
    pop(ws) over P0sdCopy               \ copy NOS to new register ; SFP051
    push(ws)  r> push(ws)               \ restore NOS and TOS
  endif

\ We now have TOS in RCX, and NOS in a register
  opc execute  nos-res dgx>a  a[ ,a CL ]a  use-a   \ generate  SHx nos-reg, CL
  drop(ws)
\  FlagsSetOpt				\ SFP051
;

interpreter
: shifter       \ -- ; -- ; generate code for a shift instruction OPC nos, tos
  create
    ' ,                                 \ xt for compile time action
    here
    0 ,                                 \ instruction
    lay-asm: drop                       \ get required actions
  does>
    dup @  swap cell+ @                 \ -- xt opc
    (shifter)
;
target

\ : >>an		\ n cnt -- n>>cnt ; arithemetic
\  0 ?do  2/  loop  ;

shifter c_lshift  lshift   "SHL   "
shifter c_rshift  rshift   "SHR   "
shifter c_arshift arshift  "SAR   "
shifter c_rol	  rol      "ROLa  "	\ SFP050
shifter c_ror	  ror      "RORa  "	\ SFP050


\ --------------
\ Multiplication
\ --------------

: IMUL3         { destreg srcres litres -- }
  a[ IMUL qword ]a  destreg dgx>a  srcres ,sgx>a  litres ,#x>a  use-a
;

: IMUL2         \ dreg srcres --
  a[ IMUL qword ]a  swap dgx>a  ,sgx>a  use-a
;

: IMUL1		\ src -- ; SFP041
  a[ IMUL qword ]a  sgx>a  use-a
;

: MUL1		\ src -- ; SFP093
  >r  a[ MUL qword ]a  r> sgx>a  use-a
;

: c_*           \ -- ; generate code for *
  2 min-depth(ws)  reset-opt

\ Deal with 2 literals first, more optimisation available
\  tos-res lit?  nos-res lit? and if
  2lits? if				\ can do everything at compile time ; SFP404
    ['] * 2lit1op  exit			\ SFP404
  endif

\ Deal with single literal next
  nos-res lit?                          \ single literal in TOS
  if  (c_swap)  endif			\ multiplication is commutative ; SFP401
  ?tosLit64>reg				\ SFP512

  tos-res lit? if
    case  tos-res res-value @
      0 of  nip(ws)  exit  endof	\ result is literal 0
      1 of  drop(ws)  exit  endof	\ result is nos*1=nos
      2 of  drop(ws) c_2*  exit  endof
      4 of  drop(ws) c_4*  exit  endof
      8 of  drop(ws) 3 c_lit  c_lshift  exit  endof
    #16 of  drop(ws) 4 c_lit  c_lshift  exit  endof
    #32 of  drop(ws) 5 c_lit  c_lshift  exit  endof
    #64 of  drop(ws) 6 c_lit  c_lshift  exit  endof
   #128 of  drop(ws) 7 c_lit  c_lshift  exit  endof
   #256 of  drop(ws) 8 c_lit  c_lshift  exit  endof
 $10000 of  drop(ws) #16 c_lit  c_lshift  exit  endof
$1000000 of  drop(ws) #24 c_lit  c_lshift  exit  endof
    endcase
    nos-res dreg? if                    \ IMUL nos, nos, # tos
      nos-res dup pop(ws) IMUL3
    else                                \ IMUL new, nos, # tos
      GetNewReg ?dup 0=
      if  c_shuffle  recurse  exit  endif	\ SFP051
      dup pop(ws) pop(ws) swap IMUL3
      push(ws)                          \ new resource becomes TOS
    endif
    exit
  endif

\ Deal with case where TOS or NOS is in a destination register
  tos-res dreg? if
    tos-res nos-res IMUL2               \ IMUL tosreg, nos
    nip(ws)  exit
  endif

  nos-res dreg? if
    nos-res tos-res IMUL2               \ IMUL nosreg, tos
    drop(ws)  exit
  endif

\ Deal with the general case where a new resource must be used
  GetNewReg ?dup 0=
  if  c_shuffle  recurse  exit  endif	\ SFP051
  MakeBestTOS				\ MOV  newreg, tos	SFP034
  tos-res nos-res IMUL2  nip(ws)        \ IMUL newreg, nos
;

: regFrom2?	\ reg -- flag
\ Return true if the register can used as a destination when two
\ operands will be consumed.
  dup free?
  if  drop -1 exit  endif
  dup count=1? if
    dup tos-res =
    if  drop -1 exit  endif
    dup nos-res =
    if  drop -1 exit  endif
  endif
  dup res-refcount @ 2 = if		\ tos=nos, count=2
    dup tos-res =  over nos-res = and
    if  drop -1 exit  endif
  endif
  drop 0
;

: c_xm*		{ xt opc -- }	\ SFP093
\ generate code for M* and UM*. Uses MUL1 or IMUL1
  2 min-depth(ws)  reset-opt

\ Deal with 2 literals first, more optimisation available
\  tos-res lit?  nos-res lit? and if
  2lits? if				\ can do everything at compile time ; SFP404
    ?NewNOSLit ?NewTOSlit               \ ensure TOS and NOS are free for modification ; SFP017
    tos-res res-value @  nos-res res-value @
    xt execute				\ SFP093
    tos-res res-value !  nos-res res-value !
    exit
  endif

\ Deal with single literal next
  nos-res lit?                          \ single literal in TOS
  if  (c_swap)  endif			\ multiplication is commutative
  ?tosLit64>reg				\ SFP512

  tos-res lit=0? if
    drop(ws) drop(ws)			\ SFP063
    0 c_lit  0 c_lit   exit
  endif
\ At this stage, NOS is not a literal.

\ remove index register problems
  rRDX IndexReg?  rRAX IndexReg? or
  if  2 DisAmbiguate  endif

\ SFP403...
\ If using MUL1 and TOS=lit, get TOS into a register.
\  opc ['] MUL1 =  tos-res lit? and if
  tos-res lit? if			\ SFP408
    GetNewReg ?dup 0= if
      c_shuffle  2 min-depth(ws)	\ shuffle and retry
      xt opc recurse  exit
    endif
    CopyTOS
  endif
\ ...SFP403

\ Make sure that RAX and RDX are consumable or free.
  rRAX regFrom2? 0= if			\ if RAX cannot be used
    rRDX rRAX --GetNewReg ?dup 0= if	\ and we can't find a spare
\ insert check for reg/reg exchange
      c_shuffle  2 min-depth(ws)	\ shuffle and retry
      xt opc recurse  exit
    endif
    rRAX swap XferRes			\ copy RAX, frees it up
  endif
  rRDX regFrom2? 0= if			\ if RDX cannot be used
    rRDX rRAX --GetNewReg ?dup 0= if	\ and we can't find a spare
\ insert check for reg/reg exchange
      c_shuffle  2 min-depth(ws)	\ shuffle and retry
      xt opc recurse  exit
    endif
    rRDX swap XferRes			\ copy RDX, frees it up
  endif

\ if RAX contains one of TOS/NOS, we're fine.
\ Otherwise get TOS (NOS is not a literal) into RAX.
  tos-res rRAX <>  nos-res rRAX <> and if
    tos-res rRAX P1sdCopy
    tos-res refcount--
    1 rRAX res-refcount !  rRAX set-tos
  endif

  tos-res rRAX = if
    nos-res opc execute
  else
    nos-res rRAX <> #307 ?OptThrow	\ sanity check
    tos-res opc execute
  endif
  drop(ws) drop(ws)
  rRAX push(ws)  rRDX push(ws)
;

: c_m*		['] m*  ['] IMUL1 c_xm*  ;
: c_um*		['] um*  ['] MUL1 c_xm*  ;


\ --------
\ Division
\ --------

: IDIV1		\ res -- ; generate IDIV r/m32 ; SFP041
  a[ IDIV qword ]a  sgx>a  use-a
;

: DIV1		\ res -- ; generate DIV r/m32 ; SFP063
  a[ DIV qword ]a  sgx>a  use-a
;

((
: arshifts	\ n -- ; shift arithmetic right instead
  c_lit  c_arshift
;
))

: GetNotRDX:RAX	\ -- res|0
\ Get a new register that is not RDX or RAX
  rRAX rRDX --GetNewReg
;

: TOSnotRDX:RAX	\ --
\ Move TOS to a new register that is not RDX or RAX.
  GetNotRDX:RAX dup if		\ divisor must not be in RDX:RAX
    CopyTOS			\ lit in reg
  else
    drop  c_shuffle
  endif
;

: 2opTOSnotRDX:RAX	\ --
\ Move TOS to a new register that is not RDX or RAX. Force two operands.
  TOSnotRDX:RAX  2 min-depth(ws)
;

: 3opTOSnotRDX:RAX	\ --
\ Move TOS to a new register that is not RDX or RAX. Force three operands.
  TOSnotRDX:RAX  3 min-depth(ws)
;

: FreeRDX	\ -- ior ; 0=success
\ Leave RDX empty and unused without touching RAX.
\ Return zero on success.
  rRDX free? 0= if
    GetNotRDX:RAX ?dup 0=
    if  -1 exit  endif
    rRDX swap XferRes		\ RDX copied somewhere
  endif
  0
;

: FreeRAX	\ -- ior ; 0=success
\ Leave RAX empty and unused without touching RDX.
\ Return zero on success.
  rRAX free? 0= if
    GetNotRDX:RAX ?dup 0=
    if  -1 exit  endif
    rRAX swap XferRes		\ RAX copied somewhere
  endif
  0
;

: loSetRDX:RAX	{ rsrc -- ior ; 0=success }
\ Get rsrc into RAX, without using RDX unless it is the source.
\ If there are not enough registers, no action is taken and ior
\ is returned true. Reference counts are updated.

  rsrc rRAX =  rsrc count=1? and if
    \ rsrc already in RAX and modifiable
    0 exit
  endif

  rsrc rRDX =  rsrc count=1? and if
    \ rsrc in RDX and modifiable, free up RAX and copy RDX to RAX
    FreeRAX if  -1 exit  endif
    a[ MOV  RAX, RDX ]a use-a
    rRDX res-refcount decr
    1 rRAX res-refcount !
    0 exit
  endif

  \ rsrc is not a dreg in RAX or RDX, so get it into RAX
  rsrc rRAX = if
    \ rsrc in RAX but is not dreg
    GetNotRDX:RAX ?dup 0=
    if  -1 exit  endif
    rRAX over XferRes  refcount--
    1 rRAX res-refcount !
  else
    \ rsrc not in RAX
    rRAX free? 0= if
      GetNotRDX:RAX ?dup 0=
      if  -1 exit  endif
      rRAX swap XferRes
    endif
    rsrc rRAX sdCopy  rsrc refcount--
    1 rRAX res-refcount !
  endif
  0
;

: c_/mod	\ --
\ generate code for /mod
  2 min-depth(ws)  reset-opt

\ Deal with 2 literals first, more optimisation available
\  tos-res lit?  nos-res lit? and if
  2lits? if				\ can do everything at compile time ; SFP404
    ?NewNOSLit ?NewTOSLit				\ ensure NOS is free for modification
    nos-res res-value @  tos-res res-value @
    /mod
    tos-res res-value !  nos-res res-value !
    exit
  endif
  ?tosLit64>reg  ?nosLit64>reg		\ big lits to regs ; SFP512

\ If TOS = NOS the quotient is 1, remainder 0
  tos-res nos-res = if
    c_drop  c_drop  0 c_lit  1 c_lit  exit
  endif

\ remove index register problems
  rRDX IndexReg?  rRAX IndexReg? or
  if  2 DisAmbiguate  endif

\ do the obvious literal checks first
  tos-res lit? if
    case  tos-res res-value @
      0 of  opt_div0 optThrow  endof
      1 of  c_drop  0 c_lit  c_swap  exit  endof
        2opTOSnotRDX:RAX
    endcase
  endif

\ Make sure the divisor (TOS) is not in RDX or RAX.
\ It is already not a lit.
  tos-res rRDX =  tos-res rRAX = or
  if  2opTOSnotRDX:RAX  endif

\ Now TOS is somewhere we can use it, because we have forced TOS
\ out of RDX:RAX. What we next do is make sure that RDX:RAX has a
\ sign-extended NOS.
  nos-res loSetRDX:RAX
  if  c_shuffle 2 min-depth(ws)  recurse  exit  endif
  rRAX set-nos  FreeRDX
  if  c_shuffle 2 min-depth(ws)  recurse  exit  endif
  a[ CQO ]a  use-a			\ extend to double, RDX = sign ext RAX
  tos-res IDIV1				\ divide by TOS
  drop(ws) drop(ws)
  rRDX push(ws)  rRAX push(ws)		\ return remainder and quotient
;

: c_/		\ --
  c_/mod  c_nip
;

: c_mod		\ --
  c_/mod  c_drop
;

: hiSetRDX:RAX	{ rsrc -- ior ; 0=success }
\ Get src into RDX without touching RAX. If there are not enough
\ registers, no action is taken and ior is returned true.
\ Reference counts are updated.

  rsrc rRDX =  rsrc count=1? and if
    \ src in RDX and modifiable
    0 exit
  endif

  \ rsrc is not a dreg in RDX, so get it into RDX
  rsrc rRDX = if
    \ rsrc in RDX but is not dreg
    GetNotRDX:RAX ?dup 0=
    if  -1 exit  endif
    rRDX over XferRes  refcount--
    1 rRDX res-refcount !
    0 exit
  endif

  \ rsrc not in RDX
  FreeRDX
  if  -1 exit  endif			\ SFP403
  rsrc rRDX sdCopy  rsrc refcount--
  1 rRDX res-refcount !
  0
;

: c_div128:64	{ opc xt -- }
\ generate code for SM/REM and UM/MOD
  3 min-depth(ws)  reset-opt

\ Deal with 3 literals first, more optimisation available
  3os-res lit?  nos-res lit? and  tos-res lit? and if
    ?NewNOSLit ?New3osLit		\ ensure 3OS and NOS are free for modification ; SFP096
    3os-res res-value @  nos-res res-value @
    tos-res res-value @ xt execute		\ SFP063
    nos-res res-value !  3os-res res-value !
    drop(ws)  exit		\ SFP051
  endif
  ?tosLit64>reg  ?nosLit64>reg  ?3osLit64>reg	\ SFP512

  rRDX IndexReg?			\ EAX also?
  if  3 DisAmbiguate  endif

\ do the obvious literal checks first
  tos-res lit? if
    case  tos-res res-value @
      0 of  opt_div0 optThrow  endof
      1 of  c_drop  exit  endof
        3opTOSnotRDX:RAX
    endcase
  endif

\ Make sure the divisor (TOS) is not in EDX or EAX.
\ It is already not a lit.
  tos-res rRDX =  tos-res rRAX = or
  if  3opTOSnotRDX:RAX  endif

\ Now TOS is somewhere we can use it, because we have already
\ copied literals, EDX or EAX into a new register. What we will do
\ now is get NOS to EDX and 3OS to EAX.
  3os-res loSetRDX:RAX
  if  c_shuffle  opc xt recurse  exit  endif
  rRAX set-3os
  nos-res hiSetRDX:RAX
  if  c_shuffle  opc xt recurse  exit  endif
  rRDX set-nos

  tos-res opc execute			\ divide by TOS ; SFP063
  drop(ws) drop(ws) drop(ws)
  rRDX push(ws)				\ remainder
  rRAX push(ws)				\ quotient
;

: c_sm/rem	\ -- ; SFP063
  ['] IDIV1 ['] sm/rem c_div128:64
;

: c_um/mod	\ -- ; SFP063
  ['] DIV1 ['] um/mod c_div128:64
;

: c_m/		\ -- ; SFP401
  c_sm/rem nip(ws)
;


\ ----------------
\ Memory operators
\ ----------------

also Forth definitions

0 value MustLoad?	\ -- n
\ *G Returns true if indirect accesses are loaded rather than delayed.

: +MustLoad	\ --
\ *G Forces indirect memory loads to be fetched into a register rather
\ ** than delayed. For some applications (mostly calculations with
\ ** array indexing) this can lead to a performance gain.
  -1 -> MustLoad?
;

: -MustLoad	\ --
\ *G Permits indirect memory loads to be delayed. This is the default
\ ** condition.
  0 -> MustLoad?
;

previous definitions

: (fetchop)     { size opc -- }
\ compile code for fetch operations
  1 min-depth(ws)

\ SFP051... ; Deal with address = n[reg]
  nREGID opt?  RFROMnREGID opt? or if
    pop(ws)  GetNewReg ?dup if		\ try for free register, including assicated
      push(ws)  drop			\ discard previous TOS
      seq-backup			\ discard LEA
      opc execute  tos-res dgx>a
      size +,SIZE
      seqflags @ seqprev @ +n[reg]  use-a
      reset-opt  exit
    else
      push(ws)
    endif
  endif
\ ...SFP051

\ Deal with TOS=lit
  tos-res lit? if			\ MOVxx reg, size [] lit
    GetNewReg ?dup 0=
    if  c_shuffle  size opc recurse  exit  endif
    opc execute  dup dgx>a  size +,SIZE  pop(ws) []lit  use-a
    push(ws)  reset-opt  exit
  endif

\ Deal with TOS=dreg
  tos-res dreg? if			\ MOVxx reg, size 0 [reg]
\ SFP093...
    MustLoad? 0= if			\ improves sandmark not to delay
      size XSIZE = if			\ address registers are XSIZE
        NoElim				\ in case previous move used same mem resource
        ?rem-litplus  			\ remove LIT + code
        pop(ws) MakeIndMem push(ws)
        reset-opt  exit
      endif
    endif
\ ...SFP093
    size BSIZE <>  tos-res byte? or if
      ?rem-litplus >r                           \ remove LIT + code     \ SFP026
      opc execute  tos-res dgx>a  size +,SIZE  r> tos-res +n[reg]  use-a  \ SFP026
      reset-opt  exit                           \ ... SFP023
    endif
  endif
  reset-opt

\ Deal with the general case where a new resource must be used
  GetNewReg ?dup 0=
  if  c_shuffle  size opc recurse  exit  endif
  tos-res reg? if			\ MOVxx newreg, size 0 [tosreg]
    ?TOSrem-litplus >r			\ remove LIT + code ; SFP026 SFP098
    opc execute  dup dgx>a  size +,SIZE  r> pop(ws) +n[reg]  use-a      \ SFP026
    push(ws)  exit                      \ ... SFP023
  endif
  MakeBestTOS                           \ MOV newreg, tos   MOVxx newreg, size 0 [newreg]	SFP036
  opc execute  tos-res dgx>a  size +,SIZE  tos-res +0[reg]  use-a
  reset-opt                                                             \ SFP026
;

interpreter
: fetchop       \ size -- ; -- ; xsize FETCHOP <name> "instruction"
  create
    ,                                   \ size
    here  0 ,  lay-asm: drop            \ opc
  does>
    dup @  swap cell+ @ (fetchop)  ;
target

XSIZE fetchop (c_@) "MOV   "
\ LSIZE fetchop c_l@  "MOVZX "		\ no 64 to 32 operation in MOVZX
WSIZE fetchop c_w@  "MOVZX "
BSIZE fetchop c_c@  "MOVZX "

\ LSIZE fetchop c_l@s "MOVSX "		\ no 64 to 32 operation in MOVSX
WSIZE fetchop c_w@s "MOVSX "                                                    \ SJC024
BSIZE fetchop c_c@s "MOVSX "                                                    \ SJC024

: c_@           \ -- ; rewritten SFP051
  1 min-depth(ws)

\ Deal with literal address
  tos-res lit? if			\ literal address?
    pop(ws) res-value @ MakeAbsMem
    push(ws)  exit
  endif

\ SFP051... ; Deal with offset/reg address
  MustLoad? 0= if
    nREGID opt?  RFROMnREGID opt? or if	\ USER address?
      seq-backup  drop(ws)		\ remove: LEA reg1, n [reg2]
      seqflags @ seqprev @ MakeIndMem	\ convert to MEM ref
      push(ws)  reset-opt  exit
    endif
  endif
\ ...SFP051

\ SFP401...
  CELLSLITPLUSID opt? if
\    seq-backup
    seq-prevbackup			\ remove: SHL reg, ADD reg, # lit
    a[ MOV ]a  tos-res dgx>a  ,>a  seqflags @ tos-res +n[reg*8]  use-a
    reset-opt exit
  endif

  CELLSREGPLUSID opt? if
    seq-prevbackup			\ remove: SHL reg, ADD reg1, reg2
    a[ MOV ]a  tos-res dgx>a  ,>a  0 n>a  seqptr @ i>a  seqres @ i*8>a  use-a
    reset-opt exit
  endif
\ ...SFP401

\ default
  (c_@)
;

: c_l@s		\ -- ; SFP515
\ Compile code for 32 bit fetch to 64 bit sign extended L@S
\ operations. The regular MOV will not do 32->64
\ sign extend. Both are handled by
\   MOVSXD reg32/64 r/m
\ where r/m is always 32 bit. If reg is 64 bit, the result is sign extended.
\ If reg is 32 bit, the result is zero extended.
  1 min-depth(ws)

\ Deal with address = n[reg]
  nREGID opt?  RFROMnREGID opt? or if
    drop(ws)  GetRegRes push(ws)	\ get free register, including associated
    seq-backup				\ discard LEA
    a[ MOVSXD ]a  tos-res x>a
    ,>a  seqflags @ seqprev @ +n[reg]  use-a
    reset-opt  exit
  endif

\ Deal with TOS=lit
  tos-res lit? if			\ MOVxx size reg, [] lit
    GetRegRes >r
    a[ MOVSXD ]a  r@ x>a  ,>a  pop(ws) []lit  use-a
    r> push(ws)  reset-opt  exit
  endif

\ Deal with TOS=dreg
  tos-res dreg? if			\ MOVxx size reg, 0 [reg]
    ?rem-litplus >r                           \ remove LIT + code
    a[ MOVSXD ]a  tos-res x>a  ,>a  r> tos-res +n[reg]  use-a  \ SFP026
    reset-opt  exit
  endif
  reset-opt

\ Deal with the case where a new resource must be used for the destination
  GetRegRes
  tos-res reg? if			\ -- res ; MOVxx newreg, size 0 [tosreg]
    ?TOSrem-litplus >r			\ remove LIT + code
    dup >r  a[ MOVSXD ]a  r> x>a  ,>a  r> pop(ws) +n[reg]  use-a
    push(ws)  exit                      \ ... SFP023
  endif

\ Deal with the general case where a new resource must be used for
\ both destination and address.
  pop(ws) over P0sdmove,  push(ws)
  a[ MOVSXD ]a  tos-res x>a  ,>a  tos-res +0[reg]  use-a
  reset-opt
;

: c_l@		\ -- ; SFP515
\ Compile code for 32 bit fetch to 64 bit zero extended
\ operation L@. The regular MOV does 32->64 bit zero extension
\ when the destination is a GPR.
  1 min-depth(ws)

\ Deal with address = n[reg]
  nREGID opt?  RFROMnREGID opt? or if
    drop(ws)  GetRegRes push(ws)	\ get free register, including associated
    seq-backup				\ discard LEA
    a[ MOV ]a  tos-res l>a
    ,>a  seqflags @ seqprev @ +n[reg]  use-a
    reset-opt  exit
  endif

\ Deal with TOS=lit
  tos-res lit? if			\ MOVxx size reg, [] lit
    GetRegRes >r
    a[ MOV ]a  r@ l>a  ,>a  pop(ws) []lit  use-a
    r> push(ws)  reset-opt  exit
  endif

\ Deal with TOS=dreg
  tos-res dreg? if			\ MOVxx size reg, 0 [reg]
    ?rem-litplus >r                           \ remove LIT + code
    a[ MOV ]a  tos-res l>a  ,>a  r> tos-res +n[reg]  use-a  \ SFP026
    reset-opt  exit
  endif
  reset-opt

\ Deal with the case where a new resource must be used for the destination
  GetRegRes
  tos-res reg? if			\ -- res ; MOVxx newreg, size 0 [tosreg]
    ?TOSrem-litplus >r			\ remove LIT + code
    dup >r  a[ MOV ]a  r> l>a  ,>a  r> pop(ws) +n[reg]  use-a
    push(ws)  exit                      \ ... SFP023
  endif

\ Deal with the general case where a new resource must be used for
\ both destination and address.
  pop(ws) over P0sdmove,  push(ws)
  a[ MOV ]a  tos-res l>a  ,>a  tos-res +0[reg]  use-a
  reset-opt
;
((
: reg32/64	\ res size --
\ lay register of required size
  XSIZE =
  if  x>a  else  l>a then
;

: fetch-sz32	{: size -- :}
\ Compile code for 32 bit fetch to 64 bit sign/zero extend
\ operations L@S and L@. The regular MOV will not do 32->64
\ zero extend. Both are handled by
\   MOVSXD reg32/64 r/m
\ where r/m is always 32 bit. If reg is 64 bit, the result is sign extended.
\ If reg is 32 bit, the result is zero extended.
  1 min-depth(ws)

\ Deal with address = n[reg]
  nREGID opt?  RFROMnREGID opt? or if
    drop(ws)  GetRegRes push(ws)	\ get free register, including associated
    seq-backup				\ discard LEA
    a[ MOVSXD ]a  tos-res size reg32/64
    ,>a  seqflags @ seqprev @ +n[reg]  use-a
    reset-opt  exit
  endif

\ Deal with TOS=lit
  tos-res lit? if			\ MOVxx size reg, [] lit
    GetRegRes >r
    a[ MOVSXD ]a  r@ size reg32/64  ,>a  pop(ws) []lit  use-a
    r> push(ws)  reset-opt  exit
  endif

\ Deal with TOS=dreg
  tos-res dreg? if			\ MOVxx size reg, 0 [reg]
    ?rem-litplus >r                     \ remove LIT + code
    a[ MOVSXD ]a  tos-res size reg32/64  ,>a  r> tos-res +n[reg]  use-a  \ SFP026
    reset-opt  exit
  endif
  reset-opt

\ Deal with the case where a new resource must be used for the destination
  GetRegRes
  tos-res reg? if			\ -- res ; MOVxx newreg, size 0 [tosreg]
    ?TOSrem-litplus >r			\ remove LIT + code
    dup >r  a[ MOVSXD ]a  r> size reg32/64  ,>a  r> pop(ws) +n[reg]  use-a
    push(ws)  exit                      \ ... SFP023
  endif
\ Deal with the general case where a new resource must be used for
\ both destination and address.
  pop(ws) over P0sdmove,  push(ws)
  a[ MOVSXD ]a  tos-res size reg32/64  ,>a  tos-res +0[reg]  use-a
  reset-opt
;

: c_l@s		\ --
  XSIZE fetch-sz32
;

: c_l@		\ --
  LSIZE fetch-sz32
;
))

: (storeop)     { size opc | rtmp1 rtmp2 -- }
\ compile code for store operations
  2 min-depth(ws)

((
\ ...SFP051 ; Deal with  addr=n[reg], n=reg/lit
  nREGID opt?  RFROMnREGID opt? or if
    seq-backup				\ remove LEA reg1, n [reg2]
    drop(ws)				\ flush stack ; SFP063
    1 -Disambiguate			\ SFP401
    tos-res reg? if			\ reg1 already discarded, n in TOS
      opc execute
      seqflags @ seqprev @ +n[reg]  size pop(ws) ,sz>a  use-a
    else
      tos-res lit? if
        opc execute  size +SIZE
        seqflags @ seqprev @ +n[reg]  pop(ws) ,#x>a  use-a
      else
        GetNewReg dup if
          >r
          pop(ws) r@ P0sdCopy		\ MOV  EAX, nos ; SFP064
          opc execute  seqflags @ seqprev @ +n[reg]  size r> ,sz>a  use-a
        else
\ BAD BAD BAD
          drop
          a[ PUSH  EAX ]a use-a
          pop(ws) rEAX P0sdCopy		\ MOV  EAX, nos ; SFP064
          opc execute  seqflags @ seqprev @ +n[reg]  size rEAX ,sz>a  use-a
          a[ POP  EAX ]a use-a
        endif
      endif
    endif
    reset-opt  exit			\ SFP063
  endif
\ ...SFP051
))
\ ...SFP051 SFP402 ; Deal with  addr=n[reg], n=reg/lit
  nREGID opt?  RFROMnREGID opt? or if
    seq-backup				\ remove LEA reg1, n [reg2]
    drop(ws)				\ flush stack ; SFP063
    1 -Disambiguate			\ SFP401
    tos-res reg? if			\ reg1 already discarded, n in TOS
      opc execute
      seqflags @ seqprev @ +n[reg]  size pop(ws) ,sz>a  use-a
      reset-opt  exit
    endif
    tos-res lit? if
      opc execute  size +SIZE
      seqflags @ seqprev @ +n[reg]  pop(ws) ,#x>a  use-a
      reset-opt  exit
    endif
    GetNewReg ?dup if
      >r
      pop(ws) r@ P0sdCopy		\ MOV  EAX, nos ; SFP064
      opc execute  seqflags @ seqprev @ +n[reg]  size r> ,sz>a  use-a
      reset-opt  exit
    endif
    tos-res WorkIndMem? if		\ MOV reg, nos
      tos-res res-assoc @ res-refcount @ 1 = if
        tos-res res-assoc @ >r
        pop(ws) r@ P0sdCopy		\ MOV reg, x[reg]
        opc execute  seqflags @ seqprev @ +n[reg]  size r> ,sz>a  use-a
        reset-opt exit
      endif
    endif
\ We have no free registers for the data, so we free one by PUSHing it.
\ We cannot use the register holding the address. If that register is
\ not RBX, we use RBX, otherwise we use RAX
    seqprev @ rRBX =
    if  rRAX  else  rRBX  endif
    >r
    a[ PUSH qword ]a r@ sgx>a  use-a
    pop(ws) r@ P0sdCopy			\ MOV  EAX, nos ; SFP064
    opc execute  seqflags @ seqprev @ +n[reg]  size r@ ,sz>a  use-a
    a[ POP qword ]a  r> dgx>a  use-a
    reset-opt  exit
  endif
\ ...SFP402 SFP051

  2 -DisAmbiguate			\ ignores TOS and NOS

\ SFP401...
  CELLSLITPLUSID opt? if
    nos-res reg/lit-s32? if
      seq-prevbackup			\ remove "cells lit +" code
\      seq-backup
      opc execute  size +SIZE
      seqflags @ pop(ws) MakeInd*8Mem dgx>a  \ convert to MEM dest
      pop(ws) ,sgx>a  use-a
      reset-opt  exit
    endif
    GetNewReg dup -> rtmp1  0= if
      c_shuffle  size opc recurse  exit
    endif
    seq-prevbackup			\ remove "cells lit +" code
\    seq-backup
    NoElim  nos-res rtmp1 sdMove,	\ nos -> temp
    opc execute  size +SIZE
    seqflags @ pop(ws) MakeInd*8Mem dgx>a  \ convert to MEM dest
    rtmp1 ,sgx>a  use-a			\ temp -> mem
    drop(ws)  reset-opt  exit
  endif

  CELLSREGPLUSID opt? if
    nos-res reg/lit-s32? if
      seq-prevbackup			\ remove "cells lit +" code
\      seq-backup
      opc execute  size +SIZE
      0 n>a  seqptr @ i>a  pop(ws) i*8>a  \ convert to base + scaled dest
      pop(ws) ,sgx>a  use-a
      reset-opt  exit
    endif
    seqptr @ -GetNewReg dup -> rtmp1  0= if
      c_shuffle  size opc recurse  exit
    endif
    seq-prevbackup			\ remove "cells lit +" code
\    seq-backup
    NoElim  nos-res rtmp1 sdMove,	\ nos -> temp
    opc execute  size +SIZE
    0 n>a  seqptr @ i>a  pop(ws) i*8>a  \ convert to base + scaled dest
    rtmp1 ,sgx>a  use-a			\ temp -> mem
    drop(ws)  reset-opt  exit
  endif

\ ...SFP401

\ Deal with two literal operands
  tos-res lit?  nos-res lit? and if	\ OPC size [] toslit, # noslit
    opc execute  size +SIZE  pop(ws) []lit  pop(ws) ,#x>a  use-a
    reset-opt  exit
  endif
\ deal with TOS=lit, NOS=reg
  tos-res lit?  nos-res reg? and if	\ OPC [] tos, nosreg
    opc execute  pop(ws) []lit  size pop(ws) ,sz>a  use-a
    reset-opt exit
  endif
\ Deal with TOS=reg, NOS=lit
  tos-res reg?  nos-res lit? and if	\ OPC size 0 [tosreg], # noslit
    ?TOSrem-litplus >r			\ SFP026 ; SFP098
    opc execute  size +SIZE  r> pop(ws) +n[reg]  pop(ws) ,#x>a  use-a  \ SFP026
    reset-opt exit                      \ SFP023
  endif
\ Deal with TOS=reg, NOS=reg
  tos-res reg?  nos-res reg? and if	\ OPC 0 [tosreg], nosreg
    size BSIZE <>  nos-res byte? or if
      ?TOSrem-litplus >r		\ SFP026 SFP098
      opc execute  r> pop(ws) +n[reg]  size pop(ws) ,sz>a  use-a        \ SFP026
      reset-opt  exit                   \ SFP023
    endif
  endif
\ Deal with TOS=lit
  tos-res lit? if			\ MOV reg, nos   OPC [] lit , rsize
    GetNewReg ?dup 0=
    if  c_shuffle  size opc recurse  exit  endif
    -> rtmp1
    a[ MOV ]a  size rtmp1 sz>a  size nos-res ,sz>a  use-a
    opc execute  pop(ws) []lit  size rtmp1 ,sz>a  use-a
    drop(ws)  reset-opt  exit           \ SFP023
  endif
\ Deal with TOS=reg
  tos-res reg? if			\ MOV ACC, nos   OPC 0 [tosreg] , ACC
    GetNewReg ?dup 0=
    if  c_shuffle  size opc recurse  exit  endif
    -> rtmp1
    ?TOSrem-litplus >r			\ SFP026 SFP098
    a[ MOV ]a  size rtmp1 sz>a  size nos-res ,sz>a  use-a                        \ SFP026
    opc execute  r> pop(ws) +n[reg]  size rtmp1 ,sz>a  use-a                  \ SFP026
    drop(ws)  reset-opt  exit           \ SFP023
  endif
\ Deal with NOS=lit
  nos-res lit? if			\ MOV EAX, tos   OPC size 0 [EAX], # noslit
    GetNewReg ?dup 0=
    if  c_shuffle  size opc recurse  exit  endif
    -> rtmp1
    pop(ws) rtmp1 P0sdCopy
    opc execute  size +SIZE  rtmp1 +0[reg]  pop(ws) ,#x>a  use-a
    reset-opt  exit                     \ SFP023
  endif
\ Deal with NOS=reg
  nos-res reg? if			\ MOV EAX, tos   OPC 0 [EAX], nosreg
    GetNewReg ?dup 0=
    if  c_shuffle  size opc recurse  exit  endif
    -> rtmp1
    pop(ws) rtmp1 P0sdCopy
    opc execute  rtmp1 +0[reg]  size pop(ws) ,sz>a  use-a
    reset-opt  exit                     \ SFP023
  endif

\ Now deal with both operands in memory
  GetNewReg ?dup 0=                     \ MOV new, tos   MOV ACC, nos   OPC 0 [new], ACC
  if  c_shuffle  size opc recurse  exit  endif	\ SFP051
  -> rtmp1
  rtmp1 -GetNewReg ?dup 0=
  if  c_shuffle  size opc recurse  exit  endif
  -> rtmp2

  pop(ws) rtmp1 P0sdCopy		\ address
  pop(ws) rtmp2 P0sdCopy		\ data
  opc execute  rtmp1 +0[reg]  size rtmp2 ,sz>a  use-a \ OPC 0 [new], ACC
  reset-opt                             \ SFP023
;

interpreter
: d_storeop       \ size -- ; -- ; xsize STOREOP <name> "instruction"           \ SJC000
  create
    ,                                   \ size
    here  0 ,  lay-asm: drop            \ opcode
  does>
    dup @  swap cell+ @  (storeop)  ;
target

XSIZE d_storeop (c_!)  "MOV   "
LSIZE d_storeop c_l!   "MOV   "
WSIZE d_storeop c_w!   "MOV   "
BSIZE d_storeop c_c!   "MOV   "

XSIZE d_storeop c_+!   "ADD   "
LSIZE d_storeop c_l+!  "ADD   "
WSIZE d_storeop c_w+!  "ADD   "
BSIZE d_storeop c_c+!  "ADD   "

XSIZE d_storeop c_or!  "ORa   "
XSIZE d_storeop c_and! "ANDa  "
XSIZE d_storeop c_xor! "XORa  "

BSIZE d_storeop c_bor!  "ORa  "		\ SFP070
BSIZE d_storeop c_band! "ANDa "		\ SFP070
BSIZE d_storeop c_bxor! "XORa "		\ SFP070

XSIZE d_storeop c_-!   "SUB   "		\ SFP401
LSIZE d_storeop c_l-!  "SUB   "         \ SFP088
WSIZE d_storeop c_w-!  "SUB   "         \ SFP088
BSIZE d_storeop c_c-!  "SUB   "         \ SJC088

: c_!		\ -- ; compile code for ! ; SFP051
  RFROMnREGID opt? if
    seq-backup				\ remove LEA reg1, n [reg2]
    -1 callot(t)			\ remove POP reg

    drop(ws) drop(ws)			\ SFP063
    0 DisAmbiguate

    a[ POP qword ]a seqflags @ seqprev @ +n[reg]  use-a \ POP n [reg]
    reset-opt  exit
  endif

  (c_!)
;

: c_2@          \ -- ; compile code for 2@      \ SFP034
  c_dup cell(t) c_lit c_+ c_@
  c_swap c_@
;

: c_2!          \ -- ; compile code for 2!      \ SFP034
  c_tuck c_!  cell(t) c_lit c_+ c_!
;

: (inc/dec)     { opc | rtmp -- }
\ compile code for INCR or DECR.
\ SFP051...
  nREGID opt?  RFROMnREGID opt? or if
    drop(ws)					\ SFP063
    seq-backup
    0 DisAmbiguate
    opc execute  +QWORD  seqflags @ seqprev @ +n[reg] use-a \ INC n [reg] ; SFP517
    reset-opt  exit
  endif
\ ...SFP051

  1 DisAmbiguate  			\ SFP026 SFP034 SFP051

  tos-res lit? if
    opc execute  +QWORD  pop(ws) []lit  use-a	\ INCR dword [] toslit
    reset-opt  exit                             \ SFP026
  endif

  tos-res reg? if
    ?TOSrem-litplus >r				\ SFP026 SFP098
    opc execute  +QWORD  r> pop(ws) +n[reg]  use-a  \ INCR dword 0 [tosreg]         \ SFP026
    reset-opt  exit
  endif

  GetNewReg ?dup 0=
  if  c_shuffle  opc recurse  exit  endif
  -> rtmp
  ?TOSrem-litplus >r
  pop(ws) rtmp P0sdCopy
  opc execute  +QWORD  r> rtmp +n[reg]  use-a	\ INCR dword 0 [newreg]
  reset-opt
;

interpreter
: inc/dec       \ -- ; -- ; INC/DEC C_INCR "INC   "
  create
    here  0 ,  lay-asm: drop
  does>
    @ (inc/dec)
;
target

inc/dec c_incr  "INC   "        \ -- ; compile code for INCR
inc/dec c_decr  "DEC   "        \ -- ; compile code for DECR


\ **************************************************
\ Comparisons - N.B. these affect the optimiser data
\ **************************************************

: genCMP2       \ -- switched?
\ lay compare code for two operands
  2lits? if				\ MOV EAX, # noslit  CMP EAX, # toslit ; SFP404
    <impolite> @ if
      cr ." Urk! Comparison of two literals "
      .source-line
    endif
    pop(ws)
    GetNewReg dup if
      >r
      a[ MOV ]a  r@ dgx>a  pop(ws) ,#x>a  use-a	\ MOV EAX, # nos
      a[ CMP ]a  r> dgx>a  ,#x>a  use-a		\ CMP EAX, # tos
    else
      drop  >r
      a[ PUSH  RAX ]a use-a
      a[ MOV   RAX ]a  pop(ws) ,#x>a  use-a	\ MOV EAX, # nos
      a[ CMP   RAX ]a  r> ,#x>a  use-a		\ CMP EAX, # tos
      a[ POP   RAX ]a use-a
    endif
    0  exit                                     \ not switched
  endif
  ?tosLit64>reg  ?nosLit64>reg		\ SFP512

  nos-res lit-s32? if	\ CMP tos, # noslit
    a[ CMP qword ]a  pop(ws) dgx>a  pop(ws) ,#x>a  use-a
    1  exit
  endif
  tos-res lit-s32? if	\ CMP nos, # toslit
    a[ CMP qword ]a  pop(ws) >r  pop(ws) dgx>a  r> ,#x>a  use-a
    0  exit
  endif
  nos-res reg? if	\ CMP nosreg, tos
    a[ CMP ]a  pop(ws) >r  pop(ws) dgx>a  r> ,sgx>a  use-a
    0 exit
  endif
  tos-res reg? if	\ CMP tosreg, nos
    a[ CMP ]a  pop(ws) dgx>a  pop(ws) ,sgx>a  use-a
    1  exit
  endif
\ SFP402...
  GetNewReg ?dup if	\ MOV EAX, nos  CMP EAX, tos
    >r  pop(ws)
    pop(ws) r@ P0sdCopy			\ MOV EAX, NOS
    a[ CMP ]a  r> dgx>a  ,sgx>a  use-a	\ CMP EAX, tos
    0 exit
  endif
\ SFP406...
  tos-res IndMem? if	\ MOV reg, tos  CMP reg, nos
    tos-res 1 ReduceIndMem? if		\ will only reduce working registers
      a[ CMP qword ]a  tos-res dgx>a  nos-res ,sgx>a use-a	\ CMP reg, nos
      drop(ws)  drop(ws)
      1 exit
    endif
  endif
  nos-res IndMem? if
    nos-res 1 ReduceIndMem? if		\ will only reduce working registers
      a[ CMP qword ]a  nos-res dgx>a  tos-res ,sgx>a use-a	\ CMP reg, tos
      drop(ws)  drop(ws)
      0 exit
    endif
  endif
\ ...SFP406
  c_shuffle  2 min-depth(ws) recurse
\ ...SFP402
;

: genSETXflag	\ struct --
\ Lay the general flag creation sequence.
\ replace AL by TOS, now in register
  @ execute  BSIZE tos-res sz>a  use-a	\ SETxx AL
  a[ NEG ]a  BSIZE tos-res sz>a  use-a	\ NEG AL
  a[ MOVSX qword ]a  tos-res dgx>a  BSIZE tos-res ,sz>a  use-a \ MOVSX tosreg, AL
;

: genSETBflag	\ --
\ Lay the SETB flag creation sequence.
  a[ SBB ]a  tos-res dgx>a  tos-res ,sgx>a  use-a
;

0 value 'SETBu<	\ -- addr
\ Address of structure for SETB instruction.
0 value 'SETBu>	\ -- addr
\ Address of structure for SETB instruction.

: (cmp2)        \ struct -- ; lays comparision code
                \ struct points to the comparison operator data structure (four strings)
  2 min-depth(ws)			\ SFP051
  2CMPID start-seq                      \ mark start of sequence
  dup      seqptr !
\ For the I386 the first opcode is used after CMP nos, tos
\ and the second opcode is used for CMP tos, nos. The instruction
\ tail after the comparison is SETxx breg  NEG breg  MOVSX lreg, breg
  genCMP2                               \ lay comparison code
  dup seqflags !                        \ set whether comparison is switched
                                        \ -- struct switched? ; see later
  ForceNewTOS                           \ get a free register for the result
\ Select which opcode to lay, according to switched, and lay code
\ -- struct switched?
  if  cell+  endif                      \ step over opcode
  start-bkup                            \ mark start of code to be removed by optimiser
((
\ ******
\ replace AL by TOS, now in register
  @ execute  BSIZE tos-res sz>a  use-a	\ SETxx AL
  a[ NEG ]a  BSIZE tos-res sz>a  use-a	\ NEG AL
  a[ MOVSX ]a  tos-res dgx>a  BSIZE tos-res ,sz>a  use-a \ MOVSX tosreg, AL
\ ******
))
\ SFP419...
  case
    'SETBu< of  genSETBflag  endof
    'SETBu> of  genSETBflag  endof
      genSETXflag
  end-case
\ ...SFP419
  end-seq                               \ mark end of optimisable sequence
;

interpreter
: cmp2          \ -- ; -- ; CMP2 C_< "SETL  "  "SETG  " works for tos nos OP
                \ the first opcode defines the instruction for comparing tos OP nos
                \ the second permits code where nos OP tos is better. The third
                \ and fourth opcodes are used by the branch generators
  create
\    here  4 cells allot
    here 5 cells allot&erase
    lay-asm: lay-asm:                   \ SETxx opcodes for CMP nos, tos  CMP tos, nos
    lay-asm: lay-asm: drop              \ branch opcodes for CMP nos, tos  CMP nos, tos
  does>
    (cmp2)
;

: inverts-cmp	\ xt "<name>" -- ; defines inverted function ; SFP083
  >body  ' >body 4 cells + !
;
target

\          t-n       n-t       t-n       n-t
cmp2 c_=   "SETE  "  "SETE  "  "JNE   "  "JNE   "
cmp2 c_<>  "SETNE "  "SETNE "  "JE    "  "JE    "
cmp2 c_<   "SETL  "  "SETG  "  "JGE   "  "JLE   "
cmp2 c_>   "SETG  "  "SETL  "  "JLE   "  "JGE   "
cmp2 c_<=  "SETLE "  "SETGE "  "JG    "  "JL    "
cmp2 c_>=  "SETGE "  "SETLE "  "JL    "  "JG    "
cmp2 c_u<  "SETB  "  "SETA  "  "JAE   "  "JBE   "
cmp2 c_u>  "SETA  "  "SETB  "  "JBE   "  "JAE   "
cmp2 c_u<= "SETBE "  "SETAE "  "JA    "  "JB    "
cmp2 c_u>= "SETAE "  "SETBE "  "JB    "  "JA    "

' c_u< >body to 'SETBu<
' c_u> >body cell + to 'SETBu>

\ SFP093...
\ Set the inverse functions to optimise "> NOT IF" etc
' c_<>  inverts-cmp c_=
' c_=   inverts-cmp c_<>
' c_>=  inverts-cmp c_<
' c_<=  inverts-cmp c_>
' c_>   inverts-cmp c_<=
' c_<   inverts-cmp c_>=
' c_u>= inverts-cmp c_u<
' c_u<= inverts-cmp c_u>
' c_u>  inverts-cmp c_u<=
' c_u<  inverts-cmp c_u>=
\ ...SFP093

: genCMP1       \ -- ; lay compare code for one operand ; SFP029
                \ does not discard TOS
  ?tosLit64>reg					\ SFP512
  tos-res lit? if
    <impolite> @ if
      cr ." Urk! Conditional tests literal value"
      .source-line
    endif
    GetNewReg dup if
      >r
      a[ MOV ]a  r@ dgx>a  tos-res ,#x>a  use-a	\ MOV EAX, # tos
      a[ ORa ]a  r@ dgx>a  r> ,sgx>a  use-a	\ OR  EAX, EAX
    else
      drop
      a[ PUSH  RAX ]a use-a
      a[ MOV   RAX ]a  tos-res ,#x>a  use-a	\ MOV EAX, # tos
      a[ ORa   RAX, RAX ]a  use-a		\ OR  EAX, EAX
      a[ POP   RAX ]a use-a
    endif
  else
    tos-res reg? if
      a[ TEST ]a  tos-res dgx>a  tos-res ,sgx>a  use-a	\ TEST  tosreg, tosreg
    else
      a[ CMP qword ]a  tos-res dgx>a  a[ ,a # 0 ]a  use-a     \ CMP tos, # 0
    endif
  endif
;

: (cmp1)        \ struct -- ; lays comparision code
                \ struct points to the comparison operator data structure (four strings)
  1 min-depth(ws)
  FLAGSETID opt? >r			\ check for flag already set
  1CMPID start-seq                      \ mark for optimiser
  dup seqptr !                          \ structure holds opcode strings
  0 seqflags !                          \ not switched yet
  r> 0= if  genCMP1  endif              \ lay compare code              \ SFP063
  drop(ws)                              \ SFP029 SFP051
  ForceNewTOS                           \ get a free register for the result
  start-bkup                            \ start of code to be removed
\ TOS in register, replace AL by TOS
  @ execute  BSIZE tos-res sz>a  use-a	\ SETxx AL
  a[ NEG ]a  BSIZE tos-res sz>a  use-a	\ NEG AL
  a[ MOVSX ]a  tos-res dgx>a  BSIZE tos-res ,sz>a  use-a	\ MOVSX tosreg, AL
  end-seq                               \ end of optimisable sequence
;

interpreter
: cmp1          \ -- ; -- ; CMP2 C_< "SETL  "  "SETG  " works for tos nos OP
                \ the first opcode defines the instruction for comparing tos OP nos
                \ the second is a dummy for the optimiser. The third
                \ and fourth opcodes are used by the branch generators.
                \ Note that the data structure is the same as for CMP2
  create
    here  5 cells allot&erase		\ uses same structure as CMP1
    lay-asm: lay-asm:                   \ SETxx opcodes for CMP nos, tos  CMP tos, nos
    lay-asm: lay-asm: drop              \ branch opcodes for CMP nos, tos  CMP nos, tos
  does>
    (cmp1)
;
target

: SETx          \ -- ; error if executed
\  1 abort" SETx should never execute - internal software error"
  opt_setx optThrow
;

cmp1 (c_0=)  "SETE  "  "SETx  "  "JNE   "  "JE    "
cmp1 c_0<>   "SETNE "  "SETx  "  "JE    "  "JNE   "
cmp1 c_0<    "SETL  "  "SETx  "  "JGE   "  "JL    "
cmp1 c_0<=   "SETLE "  "SETx  "  "JG    "  "JLE   "
cmp1 c_0>    "SETG  "  "SETx  "  "JLE   "  "JG    "
cmp1 c_0>=   "SETGE "  "SETx  "  "JL    "  "JGE   "

\ SFP093...
\ Set the inverse functions to optimise "0> NOT IF" etc
' c_0<>  inverts-cmp (c_0=)
' (c_0=) inverts-cmp c_0<>
' c_0>=  inverts-cmp c_0<
' c_0>   inverts-cmp c_0<=
' c_0<=  inverts-cmp c_0>
' c_0<   inverts-cmp c_0>=

: invert-cmp	\ -- ; invert compiled compare
  seqptr @ 4 cells + @ seqptr !		\ invert structure
  [save-opt  (c_invert)  save-opt]	\ lay invert code
  end-seq			        \ mark new end
;

: c_invert	\ -- ; can invert branches ; SFP083
\ <invert-cmp> @ if
  1CMPID opt?
  if  invert-cmp  exit  endif
  2CMPID opt?
  if  invert-cmp  exit  endif
\ endif
  (c_invert)
;

: c_not		\ -- ; can  invert branches ; SFP083
  c_invert
;

: c_0=          \ -- ; magic available with 0= for ?DUP 0= and ?DUP 0= IF
  QDUPID opt? if                        \ SFP032
    seqbeg @  seqbkup @                 \ preserve start info
    (c_0=)                              \ process
    seqbkup !  seqbeg !                 \ restore start info
    QDUPZEID seqid !                    \ mark optimisation type
  else
\ <invert-cmp> @ if
    1CMPID opt?
    if  invert-cmp  exit  endif
    2CMPID opt?
    if  invert-cmp  exit  endif
\ endif
    (c_0=)
  endif
;
\ ...SFP093


\ **********************
\ Return stack operators
\ **********************

: (c_>r)	\ -- ; compile code for >R ; SFP032 SFP094
  1 min-depth(ws)
  DOESID opt? if			\ SFP035
    seq-backup  c_does>_>r  reset-opt
  else
    tos-res >DR1 1insMove? if		\ nearly always
      >RID [opt  tos-res seqres !
      pop(ws) >DR1 P0sdXfer
      opt]
    else				\ Could get new reg and optimise
      GetNewReg ?dup 0=
      if  recurse  exit  endif
      pop(ws) over P0sdXfer
      >DR1 sdmove,
      reset-opt
    endif
  endif
;

: c_>r		\ -- ; compile code for >R
  (c_>r) 1 +rs-count
;

: c_r@          \ -- ; compile code for R@
  >RID opt? if				\ optimise >R R@
    seqres @ push(ws)  reset-opt  exit
  endif
  ForceNewTOS  reset-opt		\ SFP051
  DR1 tos-res sdXfer			\ SFP051
  1 ?inline-rs
;

: c_r>          \ -- ; compile code for R>
  ForceNewTOS  reset-opt
  RFROMID start-seq  start-bkup		\ SFP035
  NoElim  DR1> tos-res sdXfer		\ SFP051
  end-seq                               \ SFP035
  -1 +rs-count
;

: c_2>r         \ -- ; compile code for 2>R ; SFP093
  c_swap c_>r c_>r
;

: c_2r@         \ -- ; compile code for 2R@
  ForceNewTOS+NOS  reset-opt		\ SFP051
  DR1 tos-res sdmove,			\ SFP051
  DR2 nos-res sdmove,			\ SFP051
  2 ?inline-rs
;

: c_2r>         \ -- ; compile code for 2R>
  ForceNewTOS+NOS  reset-opt		\ SFP051
  DR1> tos-res sdCopy			\ SFP051
  DR1> nos-res sdmove,			\ SFP051
  -2 +rs-count
;

: c_rpick       \ -- ; compile code for RPICK ; SFP031, SFP063
  reset-opt
  tos-res lit? if
    pop(ws) res-value @ >r		\ 0, 1, 2 -> 0, 8, 16 ...
    ForceNewTOS
    a[ MOV ]a tos-res dgx>a  ,>a  r@ cells(t) rRSP +n[reg]  use-a
    r> ?inline-rs
    exit
  endif

  c_shuffle
  a[ MOV    RBX, 0 [RSP] [RBX*8] ]a use-a
;


\ -----------------------------------------------
\ low level branches - marking, resolving, laying
\ -----------------------------------------------
\ -----------------------------------------
\ Label handling - CPU specific when hosted
\ -----------------------------------------

((
Branches are handled by generating addresses which are resolved by the assembler.
These addresses are referred to as branch identifiers, denoted by branchid in the
stack comments. and SBRID refers to the location (source) of a branch, and a DBRID
refers to the target (destination) of a branch.

A source is generated before the opcode is executed, and therefore is the address
of the Jxx instruction. This opcode can be inspected by the resolution code. For
a I386 CPU, the offset is 8 bits unless the opcode is 0F, E8, or E9 in which case
the offset is 32 bits. The opcode is one byte long unless it starts with 0F, in
which case the opcode is two bytes long.
))

variable <short-branches>
  <short-branches> off

also forth definitions
: +short-branches	\ --
\ *G Enables the VFX optimiser to produce short forward branches. If
\ ** your code causes a branch limit to be exceeded, you can put
\ ** *\fo{-SHORT-BRANCHES} and *\fo{+SHORT-BRANCHES} around the
\ ** offending words. By default, short branch generation is off
\ ** because it gives better perforance on modern CPUs.
  <short-branches> on
;
: -short-branches	\ --
\ *G Prevents the VFX optimiser producing short forward branches.
\ ** By default, short branch generation is off.
  <short-branches> off
;
: short-branches?	\ -- flag ; true for short branches
\ *G Returns true if the optimiser will produce short forward branches.
  <short-branches> @
;

: [-short-branches	\ -- sys
\ *G Disables short branch optimisation until the previous state is
\ ** restored by *\fo{SHORT-BRANCHES]}.
  short-branches? -short-branches
;

: [+short-branches	\ -- sys
\ *G Enables short branch optimisation until the previous state is
\ ** restored by *\fo{SHORT-BRANCHES]}.
  short-branches? +short-branches
;

: short-branches]	\ sys --
\ *G restores the short branch optimisation previously saved
\ ** by *\fo{+/-SHORT-BRANCHES]}.
  <short-branches> !
;
previous definitions

: ?s/l		\ -- ; select 8 or 32 bit forward branch ; SFP048
  <short-branches> @
  if  a[ SHORT ]a  else  a[ LONG ]a  endif
;

: amrk>         \ -- sbrid ; mark the start (opcode address) of a forward branch
  chere(t)  ;

: >ares32       \ sbrid -- ; fix up forward branch to CHERE(T)
  chere(t) over 4+ - swap l!c(t)  ;

: ?8bit         \ offset -- ; check range of offset
  #-128 #127 within? 0=
  opt_short ?optThrow  ;

: >ares8        \ sbrid -- ; fix up forward branch to CHERE(T)
  chere(t) over 1+ -  dup ?8bit  swap c!c(t)  ;

: >ares         \ sbrid --
  NoElim				\ SFP051
  case  dup c@c(t)
    $0F of  2+ >ares32  endof
    $E8 of  1+ >ares32  endof
    $E9 of  1+ >ares32  endof
            drop  1+ >ares8
  end-case  ;

\ SFP095...
also forth definitions

[undefined] top-mask [if]
variable top-mask	\ -- addr ; moved to KERNEL.FTH
  3 top-mask !
[then]

: LoopAlignment	\ n --
\ *G Set loop starts, e.g. *\fo{BEGIN..XXX} and *\fo{DO..LOOP}
\ ** to be aligned on an n-byte booundary, where n must be a
\ ** power of two. This is useful to force the heads of loops
\ ** onto a cache line boundary. The default is 0.
\ *E   #16 LoopAlignment     \ set to 16 byte boundary
\ **   0   LoopAlignment     \ revert to lowest setting
  1 max  1- top-mask !
;

previous definitions
\ ...SFP095

: toTop		\ -- u
\ Return the number of bytes needed to align the dictionary to
\ loop alignment.
  chere(t) top-mask @ +  top-mask @ 1+ negate and  chere(t) -
;

interpreter
: asm,		\ asm, <name>
  also asm-access  '  previous  ,
;
target

create NopTable	\ -- addr
\ Holds 10 NOP instruction opcodes
also asm-access
  asm, noop
  asm, nop1  asm, nop2  asm, nop3  asm, nop4  asm, nop5
  asm, nop6  asm, nop7  asm, nop8  asm, nop9  asm, nop10
previous

: insert1nop	\ u -- ; insert a single NOP instruction of length u
  cells NopTable + @ execute
;

: insertNops	\ u -- ; insert the given number of NOPs
  begin
    dup
   while				\ -- u
    dup #10 min dup insert1nop		\ -- u u'
    -
  repeat
  drop
;


\ Replace with multi-byte NOPs
: amrk<         \ -- dbrid ; mark the destination of backward branch
  toTop insertNops
  NoElim  chere(t)			\ SFP051
;
((
: amrk<         \ -- dbrid ; mark the destination of backward branch
  begin		\ Align code on a boundary by inserting NOPs
    chere(t) top-mask @ and
   while
    $90 c,c(t)				\ NOP
  repeat
  NoElim  chere(t)			\ SFP051
;
))

: <ares         \ dbrid -- dbrid ; resolve a backward branch - DBRID is consumed by opcode
  NoElim				\ SFP051
  dup chere(t) ( 6 ) 2 + -		\ longest branch is 6 bytes ; SFP063
  #-127 <                               \ always negative, cos its backward
  if  a[ LONG ]a  else  a[ SHORT ]a  endif
;

\ SFP405...
: jge_s,	\ -- sbrid ; short forward JGE/JNL
  amrk>  a[ JGE short ]a  chere(t) 2+  use-a
; doNotSin
\ ...SFP405

: c_?dup        \ -- ; lay code for ?DUP ; here to make author's life easier
  1 min-depth(ws)
  QDUPID start-seq                      \ mark this sequence for the optimiser
  genCMP1                               \ SFP029
  c_GuardedShuffle                      \ flush the working stack
  start-bkup                            \ mark start of optimisable sequence
  ['] c_0<> seqptr !  seqflags off      \ identify Jxx type for later use
  amrk>  a[ JE ]a chere(t)  use-a       \ lay JE xxx code
  a[ LEA    rbp, -8 [rbp] ]a  use-a
  a[ MOV    $000 [rbp],  rbx ]a  use-a
  >ares                                 \ resolve JE xxx
  end-seq                               \ mark end of optimisable sequence
;

: lay-optbranch \ -- ; lays Jxx defined by comparison structure
  seqptr @  seqflags @                  \ if the branch has been switched, use 4th, else 3rd operation in CMPx structure
  if  3  else  2  endif                 \  by skipping 3 or 2 strings
  0 do  cell+  loop                     \ now points to the correct branch opcode
  @ execute                             \ generate the code
;

: optbranch<    \ dbrid -- ; optimise a branch after a conditional
  seq-backup                            \ back up to just after the compare
  drop(ws)                              \ discard TOS, to be replaced by test of flags
  c_GuardedShuffle			\ does not affect the flags ; SFP411
  lay-optbranch  <ares  use-a           \ lay down the branch code
  reset-opt                             \ reset the optimiser
;

: qdupbranch<    \ dbrid -- ; optimise a branch after ?DUP
  seq-backup                            \ back up to just after the compare
  amrk>
  a[ JNE ]a  chere(t)  use-a            \ lay JNE xxx code
  a[ MOV     rbx, 0 [rbp] ]a  use-a     \ lay DROP code for TOS=0 case
  a[ LEA     rbp, 8 [rbp] ]a  use-a
  a[ JMP ]a  swap <ares  use-a          \ lay JMP to BEGIN etc
  >ares                                 \ resolve JNE
  init-opt                              \ reset the optimiser
;

: qdupzebranch< \ dbrid -- ; optimise a branch after ?DUP 0=
  seq-backup                            \ back up to just after the compare
  a[ JNE ]a  <ares  use-a
  init-opt
  c_drop
;

: flagsetbranch< \ dbrid -- ; optimise x ?BRANCH< ; SFP051
\ cr ." got FLAGSETID< in " chere(t) ip>nfa .name
  seq-backup  drop(ws)                  \ backup, discard TOS
  c_GuardedShuffle			\ flags tested later ; SFP411
  a[ JE ]a  <ares  use-a
  init-opt
;

: optbranch>    \ -- sbrid ; optimise a branch after a conditional
  seq-backup                            \ back up to just after the compare
  drop(ws)                              \ discard TOS, to be replaced by test of flags
  c_GuardedShuffle			\ does not affect the flags ; SFP411
  amrk>
  lay-optbranch ?s/l  chere(t)  use-a	\ lay down the branch code ; SFP048
  reset-opt                             \ reset the optimiser
;

: qdupbranch>    \ -- sbrid ; optimise a branch after ?DUP
  seq-backup                            \ back up to just after the compare
  amrk>
  a[ JNE ]a  chere(t)  use-a            \ lay JNE xxx code
  a[ MOV     rbx, 0 [rbp] ]a  use-a     \ lay DROP code for TOS=0 case
  a[ LEA     rbp, 8 [rbp] ]a  use-a
  amrk>
  a[ JMP ]a ?s/l  chere(t)  use-a	\ lay JMP SHORT/LONG to ENDIF/ELSE etc ; SFP048
  swap >ares                            \ resolve JNE
  init-opt                              \ reset the optimiser
;

: qdupzebranch> \ -- sbrid ; optimise a branch after ?DUP 0=
  seq-backup                            \ back up to just after the compare
  amrk>
  a[ JNE ]a ?s/l  chere(t)  use-a	\ SFP048
  init-opt                              \ SFP035
  c_drop
;

: flagsetbranch> \ -- sbrid ; optimise ?BRANCH> ; SFP065
\ cr ." got FLAGSETID> in " chere(t) ip>nfa .name
  seq-backup  drop(ws)                  \ backup, discard TOS
  c_GuardedShuffle			\ flags tested later ; SFP411
  amrk>
  a[ JE ]a ?s/l  chere(t)  use-a
  init-opt
;

: c_mrk_branch< \ -- dbrid ; generate the address to resolve a backward branch
  c_GuardedShuffle  reset-opt		\ flush stack to known state ; SFP411
  amrk<  ;                              \ define it

: c_branch<     \ dbrid -- ; lay BRANCH instructions
  c_shuffle  reset-opt                  \ flush the working stack
  a[ JMP ]a  <ares  use-a  ;

: c_?branch<    \ dbrid -- ; lay ?BRANCH instructions
  1 min-depth(ws)			\ SFP051
  optimisable? if                       \ SFP035
    2CMPID optid?  1CMPID optid? or if	\ try < UNTIL or 0< UNTIL
      optbranch<  exit
    endif
    QDUPID optid? if                    \ try ?DUP UNTIL
      qdupbranch<  exit
    endif
    QDUPZEID optid? if                  \ try ?DUP 0= UNTIL
      qdupzebranch<  exit
    endif
    FLAGSETID optid? if			\ optimise OR UNTIL ; SFP065
      flagsetbranch<  exit
    endif
  endif
  genCMP1  drop(ws)                     \ SFP029
  c_GuardedShuffle  reset-opt		\ flush the working stack ; SFP411
  a[ JE ]a  <ares  use-a
;

: >c_res_branch \ sbrid -- ; resolve a forward branch
  c_shuffle  reset-opt                  \ flush the working stack
  >ares  ;                              \ resolve branch

: c_branch>     \ -- sbrid ; forward referenced branch
  c_shuffle  reset-opt                  \ flush the working stack
  amrk>
  a[ JMP ]a ?s/l  chere(t)  use-a	\ SFP048
;

: c_?branch>    \ -- sbrid ; forward referenced branch
  1 min-depth(ws)			\ SFP051
  optimisable? if                       \ optimisation available        SFP035
    2CMPID optid?  1CMPID optid? or if  \ try < IF  or 0< IF
      optbranch>  exit                  \ optimise the branch
    endif
    QDUPID optid? if                    \ try ?DUP IF
      qdupbranch>  exit                 \ optimise the branch
    endif
    QDUPZEID optid? if                  \ try ?DUP 0= IF
      qdupzebranch>  exit               \ optimise the branch
    endif
    FLAGSETID optid? if			\ optimise OR IF ; SFP051
      flagsetbranch>  exit
    endif
  endif
  genCMP1 drop(ws)			\ SFP029
  c_GuardedShuffle  reset-opt		\ flush the working stack ; SFP411
  amrk>
  a[ JE ]a ?s/l  chere(t)  use-a	\ SFP048
;


\ ---------------
\ case statements
\ ---------------

(( moved to KERNEL.FTH ; del SFP088
variable NextCaseTarg   \ -- addr ; holds address of CASE
))

: c_case        \ -- dbrid
  top-mask @off >r			\ SFP401
  NextCaseTarg @  c_mrk_branch< NextCaseTarg !
  r> top-mask !				\ SFP401
;

: c_of          \ -- sbrid ; MUST match with >C_RES_BRANCH and C_?BRANCH>
                \ lays equivalent of OVER = IF DROP
  2 min-depth(ws)  			\ OF removes 1 item before branch ; SFP051
  nos-res push(ws)  genCMP2 drop	\ leave NOS, lay compare ; SFP406
  c_GuardedShuffle  reset-opt		\ reorder the stack, does not affect flags ; SFP411
  amrk>                                 \ lay the forward branch
  a[ JNE ]a ?s/l  chere(t)  use-a	\ SFP048
  c_drop                                \ lay code for DROP
;

: c_?of         \ -- sbrid
  <short-branches> @off >r		\ SFP057
  c_?branch>
  r> <short-branches> !			\ SFP057
;

: c_endof       \ sbrid1 -- sbrid2
  <short-branches> @off >r		\ SFP048
  c_branch>  swap >c_res_branch
  r> <short-branches> !			\ SFP048
;

(( Unused - see KERNEL.FTH ; del SFP088
: c_dropcase    \ addr --
  NextCaseTarg !
;

: c_endcase     \ addr --
  >c_res_branch
;

: c_nextcasejump \ -- ; lay W. BRA, opcode
  NextCaseTarg @ c_branch<
;

: c_nextcasefix \ addr --
  NextCaseTarg !
;
))


\ -------------------
\ DO LOOP and friends
\ -------------------

\ Return stack:
\ RSP+16: exit address
\ RSP+8:  old index-limit-$8000.0000
\ RSP+0:  old index
\ R15:    new index-limit-xorbit63
\ R14:    new index

: c_unloop      \ -- ; compile code for UNLOOP
  c_shuffle  reset-opt
  a[  pop	r14		\ restore old index
      pop	r15		\ restore old index-limit-xorbit63
      pop	rax		\ discarded
  ]a  use-a
;

: c_do          \ -- mrk> dbrid ; compile code for DO
  2 min-depth(ws)
  ['] (DO) compile-ref           \ SFP01
  MRK>  amrk<
  not-inlined				\ SFP057
;
((
code (do)	\ limit index
  pop		rdx			\ rdx = return address
  mov		rcx, 0 [rdx]		\ exit address
  lea		rdx, 8 [rdx]		\ step over it
  push		rcx			\ rcx = exit address
  push		r15			\ r15 = old index - limit - msb
  push		r14			\ r14 = old index

  mov		r15, rbx		\ rbx = index, r15 = index
  sub           r15, 0 [rbp]		\ r15 = index - limit
  btc		r15, # 63		\ r15 = index - limit - msb
  mov		r14, rbx		\ r14 = index

  mov		rbx, 8 [rbp]		\ restore TOS
  lea		rbp, 8 [rbp]		\ step data stack

  jmp		edx
end-code
))

: c_?do         \ -- mrk> dbrid ; compile code for ?DO
  2 min-depth(ws)
  ['] (?DO) compile-ref		\ SFP019
  MRK>  amrk<
  not-inlined				\ SFP057
;
((
code (?do)	\ limit index --
  cmp		rbx, 0 [rbp]		\ index = limit
  ne, if,
    pop		rdx			\ rdx = return address
    mov		rcx, 0 [rdx]		\ exit address
    lea		rdx, 8 [rdx]		\ step over it
    push	rcx			\ rcx = exit address
    push	r15			\ r15 = old index - limit - msb
    push	r14			\ r14 = old index

    mov		r15, rbx		\ rbx = index, r15 = index
    sub         r15, 0 [rbp]		\ r15 = index - limit
    btc		r15, # 63		\ r15 = index - limit - msb
    mov		r14, rbx		\ r14 = index

    mov		rbx, 8 [rbp]		\ restore TOS
    lea		rbp, 8 [rbp]		\ step data stack

    jmp		rdx
  then,
  pop		rdx			\ rdx = return address
  mov		rcx, 0 [rdx]		\ exit address
  mov		rbx, 8 [rbp]		\ restore TOS
  lea		rbp, 8 [rbp]		\ step data stack
  jmp		rcx
end-code
))

: c_loop        \ mrk> drbid -- ; compile code for LOOP ; SFP094
  c_shuffle reset-opt			\ SFP097
  a[ INC	r14 ]a  use-a		\ update index
  a[ INC	r15 ]a  use-a		\ update limit-index-$8000.0000
  a[ JNO ]a <ares       use-a		\ resolve backward branch
  c_unloop                              \ remove DO ... LOOP state
  >RES                                  \ resolve forward branch
;

\ ...SFP036

: c_+loop       \ mrk> dbrid -- ; compile code for +LOOP
  1 min-depth(ws)
  a[ ADD	r14, ]a  tos-res sgx>a  use-a
  a[ ADD	r15, ]a  tos-res sgx>a  use-a
  drop(ws) c_GuardedShuffle  reset-opt	\ shuffle data stack ; SFP411
  a[ JNO ]a  <ares  use-a               \ resolve backward branch
  c_unloop                              \ remove DO ... LOOP state
  >RES                                  \ resolve forward branch
;

: c_i           \ -- ; compile code for I
\ R14 = index, R15 = index-limit-bit63
  IID opt? if				\ turn I I into I DUP
    c_dup
    IID [opt opt]  exit
  endif

  reset-opt
  ForceNewTOS  rR14 tos-res sdXfer
  IID [opt opt]
;

: c_j           \ -- ; compile code for J
\ (DO) pushed 0 [RSP] = prev R14 = J, 8 [RSP] = prev R15, 16 [RSP] = exit address
  reset-opt
  ForceNewTOS  DR1 tos-res sdXfer
;

: c_leave       \ -- ; compile code for LEAVE
  c_shuffle  reset-opt
  a[ pop	r14  ]a use-a		\ index
  a[ pop	r15  ]a use-a		\ index-limit-$8....
  a[ RET             ]a use-a		\ go to LEAVE address
  not-inlined				\ requires absolute address
;

: c_?leave      \ -- ; compile code for ?LEAVE
  <short-branches> @on >r		\ SFP048
  c_?branch>  c_leave  >c_res_branch
  r> <short-branches> !			\ SFP048
;


\ ************************
\ S>D and other misc words
\ ************************

: c_s>d         \ -- ; compile code for S>D ; SFP048
  1 min-depth(ws)
  tos-res lit? if			\ extend literal at compile time
    tos-res res-value @ 0< c_lit
    exit
  endif
  rRDX free? if				\ use CDQ and rEDX:EAX if available
    tos-res rRAX = if
      a[ CQO ]a  use-a			\ CDQ
      rRDX push(ws)  reset-opt  exit	\ tos is now rRDX
    endif
    rRAX free? if
      tos-res rRAX XferRes
      a[ CQO ]a  use-a			\ CDQ
      rRDX push(ws)  reset-opt  exit	\ tos is now rRDX
    endif
  endif
  c_dup c_0<
;

: MMsetup	\ -- dest src switched?
\ Generate the data moves required for a MIN or MAX operation.
\ Both TOS and NOS are placed in registers to reduce memory
\ traffic. The destination is always a DREG. The return value
\ SWITCHED? is true if TOS is the destination register.

\ See if we can find a DREG.
  nos-res dreg? if
    nos-res  tos-res reg?
    if  tos-res 0  exit  endif
    GetNewReg dup
    if  dup CopyTos 0  exit  endif
    2drop  c_shuffle 2 min-depth(ws)  recurse  exit
  endif
  tos-res dreg? if
    tos-res  nos-res reg?
    if  nos-res 1  exit  endif
    GetNewReg dup
    if  dup ChangeNos 1  exit  endif
    2drop  c_shuffle 2 min-depth(ws)  recurse  exit
  endif

\ Neither are DREGs, so find a new register for the destination.
  GetNewReg ?dup 0=			\ can't find a register
  if  c_shuffle 2 min-depth(ws) recurse  exit  endif
\ -- reg ; If one of TOS/NOS is already a register, use it as the source
  tos-res reg?
  if  dup ChangeNos tos-res 0  exit  endif
  nos-res reg?
  if  dup CopyTos nos-res 1 exit  endif

\ -- reg ; Neither are regs, try to copy both into registers.
  dup -GetNewReg dup
  if  over ChangeNos dup CopyTos 0  exit  endif
  drop
\ -- reg ; We can only get one register, so check for a literal which
\ cannot be the source operand for CMOVcc. Previous code has removed
\ the two literal case.
  tos-res lit?
  if  dup CopyTos  nos-res 1  exit  endif
  dup ChangeNos tos-res 0
;

\ structure layout
\   +0         xt CMOVcc
\   +1 cell    xt Jcc
\   +2 cells   xt interp word
: (MinMax)	\ struct --
\ Code generate for MIN/MAX operations.
  2 min-depth(ws)
  2lits? if				\ can do everything at compile time ; SFP404
    <impolite> @ if
      cr ." Urk! MIN/MAX of two literals"
      .source-line
    endif
    2 cells + @ 2lit1op  reset-opt  exit
  endif
  MMsetup >r				\ -- struct dest src ; R: -- switched?
  2dup >r >r  a[ CMP ]a  r> dgx>a  r> ,sgx>a  use-a	\ CMP dest, src
    \ Lay code using CMOVcc instruction.
    >r >r  @ execute  r> dgx>a  r> ,sgx>a  use-a	\ CMOVcc dest, src
  r> if nip(ws) else drop(ws) endif	\ select which to discard
  reset-opt
;

interpreter
: MinMax        \ -- ; -- ; MinMax C_MIN    "CMOVL "  "JGE "  MIN
\ The first parameter defines the PPro+ instruction for a conditional
\ move. The second parameter is for basic 386 use. The third is
\ is used when both TOS and NOS are literals.
  create
    here  3 cells allot&erase
    lay-asm:  lay-asm:  ' !xt++  drop
  does>
    (MinMax)
;
target

MinMax c_min  "CMOVG "  "JLE "  min
MinMax c_max  "CMOVL "  "JGE "  max
MinMax c_umin "CMOVA "  "JBE "  umin
MinMax c_umax "CMOVB "  "JAE "  umax


\ ***************
\ ABS and friends
\ ***************

\ SFP405...
: ?NEGprim,	\ -- ; primitive for ?NEGATE
  jge_s,				\ -- sbrid ; positive, do nothing
  a[ NEG ]a  tos-res dgx>a  use-a	\ NEG tos
  >ares					\ sbrid -- ; resolve jump
;

: c_abs		\ --
  1 min-depth(ws)  tos-res lit? if	\ do at compile time?
    tos-res res-value @  abs  tos-res res-value !
    reset-opt  exit
  endif
  FlagSetID opt?  TOS>dreg  0= if
    a[ TEST ]a  tos-res dgx>a  tos-res ,sgx>a  use-a	\ TEST  tosreg, tosreg
  endif
  ?NEGprim,
  reset-opt
;

: c_?negate	\ -- ; ?NEGATE ( n1 flag -- n2 )
  2 min-depth(ws)  FlagSetID opt? 0=
  if  genCMP1  endif			\ if flags not set
  drop(ws)
  TOS>dreg ?NEGprim, reset-opt
;
\ ...SFP405


\ ***********************
\ Local variable handling
\ ***********************

variable <fastlvs>
  <fastlvs> on

also forth definitions
: +fastlvs	\ --
\ *G Enables generation of inline local variable entry code. This is
\ ** the default condition, and is strongly recommended.
  <fastlvs> on
;

: -fastlvs	\ --
\ *G Disables generation of inline local variable entry code.
  <fastlvs> off
;
previous definitions

: c_lv@         \ offset -- ; generate code for LVx fetch       SFP051
  reset-opt
  MustLoad? 0= if
    rRDI MakeIndMem push(ws)
  else
    GetNewReg ?dup 0= if
      c_shuffle  recurse  exit
    endif
    \ -- offset reg
    tuck swap >r >r  a[ MOV ]a  r> dgx>a  ,>a  r> n>a  a[ [RDI] ]a   use-a
    push(ws)
  endif
;

: c_lv!         \ offset -- ; generate code for LVx store
\ SFP051...
  RFROMID opt? if
    seq-backup  drop(ws)		\ remove POP reg ; SFP063
    0 DisAmbiguate			\ flush memory refs
    a[ POP qword ]a  n>a a[ [RDI] ]a  use-a	\ POP  n [EDI]
    reset-opt  exit
  endif
\ ...SFP051

  1 DisAmbiguate  reset-opt

  tos-res reg/lit-s32? if
    a[ MOV qword ]a  n>a a[ [RDI] ]a  pop(ws) ,x>a  use-a
    exit
  endif

  GetNewReg ?dup 0= if
    c_shuffle  1 min-depth(ws)  GetNewReg
  endif
  >r
  a[ MOV ]a  r@ dgx>a  pop(ws) ,sgx>a  use-a
  a[ MOV ]a  n>a a[ [RDI], ]a  r> sgx>a  use-a
;

: c_lv          \ offset -- ; generate code for LVx address ; SFP051
  ForceNewTOS				\ SFP063
  RFROMID opt?  reset-opt
  if  RFROMnREGID  else  nREGID  endif
  start-seq  start-bkup
  dup seqflags !  rRDI seqprev !	\ stash offset and register
    >r  a[ LEA ]a  tos-res dgx>a  ,>a  r> n>a a[ [RDI] ]a  use-a
  end-seq
;

: c_flv@         \ offset -- ; generate code for FLVx fetch ; SFP409
  reset-opt
  FPSystem SSE64system = if  >r
    a[  sub	r13, # 8		\ save FTOS
        movsd	0 [r13], xmm8
        movsd	xmm8, r> [rdi]		\ fetch from LV offset
    ]a use-a
  then
  FpSystem HFP387System = if  >r	\ FTOS in deep FP stack
    a[  lea	r13, #-16 [r13]		\ update stack pointer
        fstp tbyte 0 [r13]              \ store and pop
        fld tbyte  r> [rdi]		\ fetch local
    ]a use-a
  then
  FpSystem NDP387System = if  >r	\ FTOS in NDP stack
    a[  fld tbyte  r> [rdi]  ]a use-a	\ fetch local
  then
;

: c_flv!         \ offset -- ; generate code for FLVx store ; SFP409
  reset-opt
  FPSystem SSE64system = if  >r
    a[  movsd	r> n>a [rdi], xmm8	\ store FTOS
        movsd	xmm8, 0 [r13]		\ restore FTOS
        add	r13, # 8
    ]a use-a
  then
  FpSystem HFP387System = if  >r	\ FTOS in deep FP stack
    a[  fstp tbyte   r> n>a [rdi]	\ save FTOS
        fld   tbyte 0 [r13]             \ push FNOS to FPU
        lea   r13, #16 [r13]		\ update stack pointer
    ]a use-a
  then
  FpSystem NDP387System = if  >r	\ FTOS in NDP stack
    a[  fstp tbyte  r> [rdi]  ]a use-a	\ store local
  then
;

: c_flv+!	\ offset -- ; generate code for FLVx ADD ; SFP409
  reset-opt
  FPSystem SSE64system = if  >r
    a[  addsd	xmm8, r@ n>a [rdi]
        movsd	r> n>a [rdi], xmm8
        movsd	xmm8, 0 [r13]		\ restore FTOS
        add	r13, # 8
    ]a use-a
  then
  FpSystem HFP387System = if  >r	\ FTOS in deep FP stack
    a[  fld   tbyte r@ [rdi]		\ get from addr
        faddp st(1), st                 \ do the actual addition
        fstp  tbyte r> [rdi]		\ put back at addr
        fld   tbyte 0 [r13]             \ push FNOS to FPU
        lea   r13, #16 [r13]		\ update stack pointer
    ]a use-a
  then
  FpSystem NDP387System = if  >r	\ FTOS in NDP stack
    a[    fld   tbyte r@ [rdi]		\ get from LV
          faddp st(1), st               \ do the actual addition
          fstp  tbyte r> [rdi]		\ put back at LV
    ]a use-a
  then
;

: c_flv-!	\ offset -- ; generate code for FLVx SUB ; SFP409
  reset-opt
  FPSystem SSE64system = if  >r
    a[  movsd	  xmm9, r@ n>a [rdi]
        subsd     xmm9, xmm8
        movsd	  r> n>a [rdi], xmm9
        movsd	  xmm8, 0 [r13]		\ restore FTOS
        add	  r13, # 8
    ]a use-a
  then
  FpSystem HFP387System = if  >r	\ FTOS in deep FP stack
    a[  fld   tbyte r@ [rdi]		\ get from addr
        fsubrp st(1), st		\ do the actual subtraction
        fstp  tbyte r> [rdi]		\ put back at addr
        fld   tbyte 0 [r13]             \ push FNOS to FPU
        lea   r13, #16 [r13]		\ update stack pointer
    ]a use-a
  then
  FpSystem NDP387System = if  >r	\ FTOS in NDP stack
    a[  fld   tbyte r@ [rdi]		\ get from LV
        fsubrp st(1), st		\ do the actual subtraction
        fstp  tbyte r> [rdi]		\ put back at LV
    ]a use-a
  then
;

: set-FLVarg	\ offset -- ; pop FP stack to FLV
  FPSystem SSE64system = if  >r
    a[  movsd   r> [RSP], xmm8		\ FTOS to memory
        movsd	xmm8, 0 [r13]  		\ restore FTOS
        add	r13, # 8
    ]a use-a
  then
  FpSystem HFP387System = if  >r	\ FTOS in deep FP stack
    a[  fstp tbyte  r> n>a [RSP]	\ save FTOS
        fld   tbyte 0 [r13]		\ push FNOS to FPU
        lea   r13, #16 [r13]		\ update stack pointer
    ]a use-a
  then
  FpSystem NDP387System = if  >r	\ FTOS in NDP stack
    a[  fstp tbyte  r> [RSP]  ]a use-a	\ store local
  then
; doNotSin

also system

: FloatArgs,	\ -- ; SFP409
\ Generate entry code for floating point local arguments.
  #fargs @ if
    #fargs @ /fpl * >r			\ size of float frame
    a[ sub   rsp, # ]a  r@ n>a  use-a	\ make frame
    r> 0
    do  i set-FLVarg  /fpl +loop
  endif
;

: IntArgs,	\ n -- ; SFP409
\ Generate entry code for integer local arguments.
  ?dup if
    dup begin				\ will need #args later
      1- >r				\ push the arguments, deepest first
      a[ PUSH qword ]a  r@ in(ws) @ sgx>a  use-a	\ SFP510
      r> ?dup 0=
    until
    0 do  drop(ws)  loop		\ remove from working stack
  endif
;

variable makelvstyle	\ -- addr ; true for inlined MAKELVS ; SFP045

: c_makelvs     \ #args #frame -- addr(t) ; generate code for MAKELVS ; SFP051
		\ #args and #frame are in target address units
		\ Using PUSH is faster than frame construction by measurement
		\ The performance of this code is VERY sensitive to instruction order.
  { #args #frame | numargs rtmp -- addr(t) }	\ #args and #frame in bytes
  #args 4 cells <=  #fargs @ or		\ must use this for floats
  dup makelvstyle !  if			\ mark style in use
    #args cell(t) / dup min-depth(ws)	\ stops reference count screw up
    dup -> numargs  #fargs @ or if	\ we have some arguments ; SFP409
      GetNewReg ?dup 0= if
        c_shuffle  numargs min-depth(ws)  GetNewReg
      endif
      -> rtmp
      a[ MOV ]a  rtmp dgx>a  a[ ,a RSP ]a  use-a	\ preserve old ESP
      FloatArgs,			\ entry code for FP args ; SFP409
      numargs IntArgs,			\ entry code for int args ; SFP409
      a[ PUSH qword ]a  rtmp sgx>a  use-a	\ add frame control data: previous RSP
    else		\ no parameters to copy
      a[ PUSH   RSP      ]a  use-a	\ preserve old ESP
    endif
    a[ PUSH   RDI      ]a  use-a	\ add frame control data: previous LP
    a[ MOV    RDI, RSP ]a  use-a	\ set new LP
    $48 c,c(t)  $81 c,c(t)  $EC c,c(t)	\ SUB RSP, # bytes ; create frame
    chere(t)  #frame l,c(t)		\ note address to patch later
    c_shuffle  reset-opt		\ flush stack, must do because C_SHUFFLE will not
    					\ reorganise stacks with holes and #CHANGED -ve
    NoElim				\ SFP051
  else
    ['] MAKELVS o_docall,		\ compile local entry word      SFP019, SFP033 SFP043 SFP044
    chere(t)				\ needed when frame is adjusted at compile time
    #args cell / w,c(t)			\ #args
    #frame cell / w,c(t)		\ #frame
  then
;

previous

: o_PatchLocalSize	\ -- ; set #locals to size defined by LVCOUNT
  makelvstyle @ if
    [ also system ]
    lvcount @ negate cell-		\ #locals in bytes
    Patch-Address @ l!
    [ previous ]
  else
    (PatchLocalSize)
  endif
;

also Forth definitions
\ *P Most modern x86 operating systems use task gates for interrupt
\ ** handling, which permits some code generation to be better,
\ ** especially for local variables.

SafeOS? value SafeOS?	\ -- flag
\ *G Returns true if the operating system can be assumed to be
\ ** safe.

: +SafeOS	\ --
\ *G Assume a safe modern operating system.
  -1 -> SafeOS?
;

: -SafeOS	\ --
\ *G Assume an old-fashioned or raw operating system.
  0 -> SafeOS?
;
previous definitions

: c_rellvs      \ -- ; generate code for RELLVS
  c_shuffle  reset-opt		\ SFP051
  SafeOS? if
\ Windows use only or where interrupt protection is not needed
    a[ MOV   RSP, 8 [RDI] ]a  use-a
    a[ MOV   RDI, 0 [RDI] ]a  use-a
  else
\ Generally we MUST restore LP before RSP!
    a[ MOV   RAX, RDI     ]a  use-a
    a[ MOV   RDI, 0 [RDI] ]a  use-a
    a[ MOV   RSP, 8 [RAX] ]a  use-a
  endif
;

: c_lvarg       \ offset -- ; optimising version of S_LVARG,
  case OperatorType @off
    0 of  c_lv@  endof
    1 of  c_lv!  endof
    2 of  c_lv   endof
    3 of  c_lv  c_incr  endof
    4 of  c_lv  c_decr  endof
    5 of  c_lv  c_+!  endof
    6 of  0 c_lit  c_lv  c_!  endof
    7 of  c_negate  c_lv  c_+!  endof
    8 of  drop  cell(t) c_lit  endof	\ SFP049
    9 of  -1 c_lit  c_lv  c_!  endof
                  bad-method
  endcase
;

: c_lvarray     \ offset -- ; optimising version of S_LVARRAY,
  c_lv
;

: c_flvarg	\ offset -- ; action of S_FLVARG,
  case OperatorType @off
    0 of  c_flv@  endof
    1 of  c_flv!  endof
    2 of  c_lv   endof
    5 of  c_flv+!  endof	\ ADD
    7 of  c_flv-!  endof	\ SUB
      bad-method
  endcase
;
also system
assign c_flvarg to-do s_flvarg,
previous

also forth definitions

: lv,           \ offset -- ; used by external compilers
  c_lv  ;

: lv@,          \ offset -- ; used by external compilers
  c_lv@  ;

: lv!,          \ offset -- ; used by external compilers
  c_lv!  ;

previous definitions


\ ************************
\ Forth VM register access
\ ************************

\ SFP037...
interpreter
: c_reg@	\ res -- ; -- addr
  create
    ,
  does>
    reset-opt
    @					\ get src resource
    GetNewReg ?dup 0=			\ get new register
    if  c_shuffle  GetNewReg  endif	\ -- src new
    tuck NoElim sdCopy
    push(ws)
;
target

rRSI c_reg@ c_up@
rRDI c_reg@ c_lp@
rRSP c_reg@ c_rp@
rR13 c_reg@ c_fsp@
rR14 c_reg@ c_RI@			\ index ; SFP519
rR15 c_reg@ c_RIL@			\ index-limit ; SFP519

interpreter
: c_reg!	\ res -- ; addr --
  create
    ,
  does>
    1 min-depth(ws)  reset-opt
    @  pop(ws)				\ -- dest src
    swap sdmove,
;
target

rRSI c_reg! c_up!
rRDI c_reg! c_lp!
rRSP c_reg! (c_rp!)			\ SFP046
rR13 c_reg! c_fsp!
rR14 c_reg! c_RI!			\ SFP519
rR15 c_reg! c_RIL!			\ SFP519

: c_rp!		\ -- ; SFP046
  (c_rp!)  not-inlined
;

: c_sp@         \ -- n
  c_shuffle
  a[    sub     rbp, # 8        ]a use-a
  a[    mov     0 [rbp], rbx    ]a use-a
  a[    mov     rbx, rbp        ]a use-a
;

: c_sp!         \ n --
  c_shuffle
  a[    mov     rbp, rbx        ]a use-a
  a[    mov     rbx, 0 [rbp]    ]a use-a
  a[    add     rbp, # 8        ]a use-a
;
\ ...SFP037


\ *******************
\ EXECUTE and friends
\ *******************

[-short-branches
: c_execute     \ -- ; SFP058
  1 min-depth(ws) reset-opt

  tos-res AbsMem? if			\ SFP093
    pop(ws) >r				\ SFP063
    c_shuffle
    a[ call qword ptr ]a r> sgx>a use-a
    reset-opt  exit
  then

\ SFP093...
  tos-res IndMem? if
    tos-res res-assoc @ rTOS <> if
      pop(ws) >r			\ may not be eliminated
      r@ res-refcount incr		\ so res unused by shuffle
      r@ res-assoc @ res-refcount incr	\ so reg unused by shuffle
      (c_shuffle)
      a[ call qword ]a  r@ sgx>a  use-a	\ DO NOT ADD PTR HERE
      r@ res-assoc @ res-refcount decr
      r> res-refcount decr		\ remove count
      check-stack(ws)
      reset-opt  exit
    endif
  endif
\ ...SFP093

  tos-res lit? if
    pop(ws) res-value @ >r		\ SFP063
    c_shuffle
    a[ call ]a  r> n>a  use-a
    exit
  then

  GetShuffleReg ?dup 0= if		\ if no shuffle registers MUST have shuffle
    a[ push qword ]a  pop(ws) sgx>a  use-a
    c_shuffle
    a[ pop  RAX ]a use-a
    a[ call  RAX ]a use-a
  else
    >r
    pop(ws) r@ P0sdCopy               \ may not be eliminated
    r@ res-refcount incr              \ so reg unused by shuffle
    (c_shuffle)
    a[ call ]a  r@ x>a  use-a
    r> res-refcount decr              \ remove count
    check-stack(ws)
  endif
;
short-branches]


\ ***********************************************
\ Defining word and compilation structure support
\ ***********************************************

\ -----
\ DEFER
\ -----

: c_deferChild	\ xt --
  >body c_lit  c_@  c_execute
;

' c_deferChild ' EMIT 	  >code-len !
' c_deferChild ' EMIT? 	  >code-len !
' c_deferChild ' KEY 	  >code-len !
' c_deferChild ' KEY? 	  >code-len !
' c_deferChild ' EKEY 	  >code-len !
' c_deferChild ' EKEY? 	  >code-len !
' c_deferChild ' CR 	  >code-len !
' c_deferChild ' TYPE 	  >code-len !
' c_deferChild ' ACCEPT	  >code-len !
' c_deferChild ' pause 	  >code-len !
' c_deferChild ' ms 	  >code-len !
' c_deferChild	' allocate	>code-len !
' c_deferChild	' free		>code-len !
' c_deferChild	' resize	>code-len !
[defined] xcall-fault [if]	\ Windows only
' c_deferChild	' xcall-fault	>code-len !
[then]
[defined] idle [if]
' c_deferChild  ' idle          >code-len !
[then]
[defined] WaitIdle [if]
' c_deferChild  ' WaitIdle      >code-len !
[then]
' c_deferChild  ' BusyIdle      >code-len !
' c_deferChild  ' EmptyIdle     >code-len !
' c_deferChild  ' catch-save    >code-len !
' c_deferChild  ' catch-restore >code-len !


\ ----
\ USER
\ ----

: user@         \ xt -- offset ; SFP019
  >body cell+ @
;

: c_userchild   \ xt -- ; SFP051
  ForceNewTOS				\ SFP063
  RFROMID opt?  reset-opt
  if  RFROMnREGID  else  nREGID  endif
  start-seq  start-bkup
  user@ dup seqflags !  rRSI seqprev !	\ stash offset and register
    a[ LEA ]a  tos-res dgx>a
    ,>a n>a a[ [RSI] ]a  use-a		\ LEA newreg, n [ESI]
  end-seq
;

also forth definitions

: user          \ n -- ; -- addr ; optimising form of USER
  user  ['] c_userchild set-compiler
;

: +USER         \ n "<spaces>name" -- ; Exec: -- user-a-addr
  +user  ['] c_userchild set-compiler
;

' c_userchild ' s0 	  >code-len !	\ SFP064
' c_userchild ' r0 	  >code-len !	\ SFP064
' c_userchild ' #tib 	  >code-len !	\ SFP064
' c_userchild ' 'tib 	  >code-len !	\ SFP064
' c_userchild ' >in 	  >code-len !	\ SFP042
' c_userchild ' base      >code-len !	\ SFP042
' c_userchild ' out       >code-len !	\ SFP042
' c_userchild ' op-handle >code-len !	\ SFP042
' c_userchild ' ip-handle >code-len !	\ SFP042
' c_userchild ' fs0	  >code-len !	\ SFP042
' c_userchild ' fsp	  >code-len !	\ SFP042
' c_userchild ' currobj   >code-len !	\ SFP042
' c_userchild ' 'AbortText >code-len !	\ SFP064
' c_userchild ' $s0 	  >code-len !	\ SFP064
' c_userchild ' $sp 	  >code-len !	\ SFP064
' c_userchild ' line# 	  >code-len !	\ SFP064
' c_userchild ' op-line#  >code-len !	\ SFP064
' c_userchild ' pad       >code-len !	\ SFP084
' c_userchild ' ThreadExit? >code-len !	\ SFP087
' c_userchild ' ThreadTCB >code-len !	\ SFP087
' c_userchild ' ThreadSync >code-len !	\ SFP087
' c_userchild ' ExceptStackEnd >code-len !	\ SFP089
' c_userchild ' ExceptStackStart >code-len !	\ SFP089
' c_userchild ' exc0      >code-len !	\ SFP089
' c_userchild ' excp      >code-len !	\ SFP089
' c_userchild ' dpl       >code-len !	\ SFP089
' c_userchild ' ^Syspad   >code-len !	\ SFP425
' c_userchild ' SyspadStart >code-len !	\ SFP425
' c_userchild ' SyspadEnd >code-len !	\ SFP425

previous definitions


\ -------------------------------------
\ VARIABLE 2VARIABLE CONSTANT 2CONSTANT
\ -------------------------------------

: o_varchild    \ xt -- ; compiler for children of VARIABLE/CREATE
\ N.B. this only works in SINGLE-SECTION targets
  >body c_lit
;

: o_conchild    \ xt -- ; compiler for children of CONSTANT
  >body @ c_lit
;

: o_2conchild   \ xt -- ; compiler for children of 2CONSTANT
  >body 2@ swap c_lit c_lit
;

\ external
also forth definitions

: variable      \ -- ; -- addr ; optimising version of VARIABLE
  variable  ['] o_varchild set-compiler
;

[defined] <dp> [if]
' o_conchild ' dp 	>code-len !	\ SFP042
[else]
' o_varchild ' dp 	>code-len !	\ SFP042
[then]
' o_varchild ' state 	>code-len !	\ SFP042
' o_varchild ' wid-link	>code-len !	\ SFP065
' o_varchild ' res-link	>code-len !	\ SFP065
' o_varchild ' lib-link	>code-len !	\ SFP065
' o_varchild ' ovl-link	>code-len !	\ SFP065
' o_varchild ' <id>	>code-len !	\ SFP065
' o_varchild ' import-func-link	>code-len !	\ SFP065
' o_varchild ' current	>code-len !	\ SFP065
' o_varchild ' context	>code-len !	\ SFP065
' o_varchild ' blk	>code-len !	\ SFP065
' o_varchild ' operatortype >code-len !	\ SFP065

: 2variable     \ -- ; -- addr ; optimising version of 2VARIABLE
  2variable  ['] o_varchild set-compiler
;

: buffer:       \ -- ; -- addr ; optimising version of BUFFER:
  buffer:  ['] o_varchild set-compiler
;

: constant      \ n -- ; -- n ; optimising version of CONSTANT
  constant ['] o_conchild set-compiler
;

' o_conchild ' bl >code-len !
' o_conchild ' null >code-len !
' o_conchild ' MAX_PATH >code-len !
' o_conchild ' TRUE >code-len !
' o_conchild ' FALSE >code-len !
' o_conchild ' NDP387SYSTEM >code-len !
' o_conchild ' HFP387SYSTEM >code-len !
' o_conchild ' SSE64SYSTEM >code-len !

: 2constant     \ n -- ; -- n ; optimising version of 2CONSTANT
  2constant  ['] o_2conchild set-compiler
;

previous definitions


\ *******************************************************************
\ Redefinitions of Forth defining words and optimiser sensitive words
\ *******************************************************************

also forth definitions

target-only
: create        \ -- ; -- addr ; optimising version of CREATE
  create ['] o_varchild set-compiler
;
host&target
' o_varchild ' cNull 	>code-len !	\ SFP042
' o_varchild ' zNull 	>code-len !	\ SFP042

: defer		\ --
  defer  ['] c_deferchild set-compiler
;

: no-compiler   \ --
  get-compiler ['] o_varchild = if
\ .source-line  cr ." NO_COMPILER active"
    0 set-compiler
  then
;

: interp>		\ -- ; -- ; for defining words that set their own compilers.
  drop-locals                           \ compiles in stack cleanup ; SFP023
  cleanup-locals                        \ cleans up local dictionary etc. ; SFP023
  last-colon-xt off
  compile (;code)  dodoes,		\ primary action of DOES>
;  immediate

: does>         \ -- ; -- ; magic DOES> to fix optimised CREATEs
  drop-locals                           \ compiles in stack cleanup ; SFP023
  cleanup-locals                        \ cleans up local dictionary etc. ; SFP023
  last-colon-xt off
  postpone no-compiler
  compile (;code)  dodoes,		\ primary action of DOES>
; immediate

(( kernel version
: DOES>         \ C: colon-sys1 -- colon-sys2 ; R: nest-sys --  6.1.1250
  drop-locals                           \ compiles in stack cleanup ; SFP023
  cleanup-locals                        \ cleans up local dictionary etc. ; SFP023
  last-colon-xt off
  COMPILE (;CODE)  dodoes DoLayAbsCall	\ SFP002
; immediate
))
: ;code		\ -- ; -- ; magic ;CODE to fix optimised CREATEs ; SFP046
  postpone no-compiler
  ['] ;code >r			\ *** KLUGE *** CPU specific
  				\ forces JUMP to start of old ;CODE
; immediate

((
: RECURSE       \ Comp: --                                      6.1.2120
  NoInterp  ;
ndcs: ( -- )
  discard-sinline  last-colon-xt @ dup 0= #-259 ?throw  docall,  ;
))
: RECURSE       \ Comp: --                                      6.1.2120
  NoInterp  ;
ndcs: ( -- )
  not-inlined  discard-sinline
  last-colon-xt @ dup 0= #-259 ?throw  docall,
;

previous definitions
\ internal

\ ----------
\ Structures
\ ----------

: o_field       \ xt -- ; compiler for children of FIELD
  >body @ ?dup			\ if offset is non-zero
  if  c_lit  c_+  endif
;

external
also forth definitions

: struct        \ -- addr 0 ; -- n ; optimising version of STRUCT
  struct  ['] o_conchild set-compiler     \ behaves like CONSTANT
;

: field         \ addr n -- addr n' ; optimising version of FIELD
  field  ['] o_field set-compiler
;

: field-type    \ n -- ; second order defining word
  create
    ,
  does>
    @ field
;

: int		\ -- ; kluge for cross compiler
  cell field
;
\ cell field-type int

' o_field ' gen-handle >code-len !	\ SFP060
' o_field ' gen-vector >code-len !	\ SFP060

: begin-structure	\ -- addr 0 ; -- size
  struct  ;

: end-structure		\ addr n --
  end-struct  ;

: +FIELD	\ n <"name"> -- ; Exec: addr -- 'addr
  field  ;

: cfield:       \ n1 <"name"> -- n2 ; Exec: addr -- 'addr
  1 chars +FIELD  ;

: field:        \ n1 <"name"> -- n2 ; Exec: addr -- 'addr
  aligned  1 cells +FIELD  ;

previous definitions


\ ********************
\ Miscellaneous macros
\ ********************

: c_bounds      \ -- ; compile code for BOUNDS
  c_over c_+ c_swap  ;

: c_count       \ -- ; compile code for COUNT
  c_dup c_1+ c_swap c_c@  ;

(( del SFP405
: c_abs         \ compile code for ABS
  <short-branches> @on >r		\ SFP048
  c_dup c_0<
  c_?branch> c_negate >c_res_branch
  r> <short-branches> !			\ SFP048
;
))

: c_cell        \ -- ; compile code for CELL
  cell(t) c_lit  ;

: c_cell+       \ -- ; compile code for CELL+
  c_cell  c_+  ;

: c_cell-       \ -- ; compile code for CELL-
  cell(t) negate c_lit  c_+  ;

: c_cells+      \ -- ; compile code for CELLS+
  c_8* c_+  ;

: c_on          \ -- ; compile code for ON
  -1 c_lit  c_swap  c_!  ;

: c_off         \ -- ; compile code for OFF
  0 c_lit  c_swap  c_!  ;

: c_3drop       \ -- ; compile code for 3DROP
  c_drop c_2drop  ;

: c_4drop       \ -- ; compile code for 3DROP
  c_2drop c_2drop  ;

: c_d+          \ -- ; generate code for D+             \ SFP030
  4 min-depth(ws)
  tos-res lit?  3os-res lit? and
  nos-res lit?  4os-res lit? and  or
  if
    ['] d+ o_docall,					\ SFP043
  else
    c_swap  c_2swap  c_-rot  c_+n  c_-rot  c_+c
  endif
;

: c_d-          \ -- ; generate code for D-             \ SFP030
  4 min-depth(ws)
  tos-res lit?  3os-res lit? and	\ compile time operations
  nos-res lit?  4os-res lit? and  or	\ will prevent flag setting
  if
    ['] d- o_docall,
  else
    c_swap  c_2swap  c_-rot  c_swap c_-n  c_-rot  c_swap c_-c	\ SFP046
  endif
;

: c_m+          \ -- ; generate code for M+             \ SFP030
  c_s>d c_d+
\  c_dup c_0< c_d+
;

: c_within	\ -- ; SFP419
  c_over c_- c_-rot c_- c_swap c_u<
;
((
: within	\ n1|u1 n2|u2 n3|u3 -- flag
  over - -rot			\ -- n3-n2 n1 n2
  -				\ -- n3-n2 n1-n2
  swap u<			\ (n1-n2)-(n3-n2), cy -> -1, ncy -> 0
;
))


\ **************
\ Compiler words
\ **************

: oc:           \ -- ; ?? xt -- ; generate a headerless compiling word
  :noname  ' >code-len !
;
interpreter
: oc:           \ -- ; ?? xt -- ; generate a headerless compiling word
  :noname  ' >code-len !  !csp
;
target

\   "name"      "compiler"
oc: dup         drop c_dup  ;
oc: ?dup        drop c_?dup  ;
oc: 2dup        drop c_2dup  ;
oc: 3dup	drop c_3dup  ;
oc: swap        drop c_swap  ;
oc: 2swap       drop c_2swap  ;
oc: drop        drop c_drop  ;
oc: 2drop       drop c_2drop  ;
oc: 3drop       drop c_3drop  ;
oc: 4drop       drop c_4drop  ;
oc: over        drop c_over  ;
oc: 2over       drop c_2over  ;
oc: rot         drop c_rot  ;
oc: -rot        drop c_-rot  ;
oc: 2rot	drop c_2rot  ;
oc: nip         drop c_nip  ;
oc: tuck        drop c_tuck  ;
oc: pick        drop c_pick  ;
\ oc: roll        drop c_roll  ;

oc: 2@          drop c_2@  ;
oc: @           drop c_@  ;
oc: l@          drop c_l@  ;
oc: w@          drop c_w@  ;
oc: c@          drop c_c@  ;

oc: l@S         drop c_l@s  ;
oc: w@S         drop c_w@s  ;
oc: c@S         drop c_c@s  ;

oc: 2!          drop c_2!  ;
oc: !           drop c_!  ;
oc: l!		drop c_l!  ;
oc: w!          drop c_w!  ;
oc: c!          drop c_c!  ;

oc: +!          drop c_+! ;
oc: w+!		drop c_w+!  ;		\ SFP088
oc: l+!		drop c_l+!  ;
oc: c+!		drop c_c+!  ;		\ SFP088
oc: -!          drop c_-! ;		\ SFP088
oc: l-!		drop c_l-!  ;		\ SFP088
oc: w-!		drop c_w-!  ;		\ SFP088
oc: c-!		drop c_c-!  ;		\ SFP088

oc: on          drop c_on  ;
oc: off         drop c_off  ;
oc: incr        drop c_incr  ;
oc: decr        drop c_decr  ;

oc: i           drop c_i  ;
oc: j           drop c_j  ;
oc: unloop      drop c_unloop  ;

oc: +           drop c_+  ;
\ oc: +c          drop c_+c  ;
oc: -           drop c_-  ;
\ oc: -c          drop c_-c  ;

oc: 1+          drop c_1+  ;
oc: 2+          drop  2 c_lit  c_+  ;
oc: 4+          drop  4 c_lit  c_+  ;
oc: 1-          drop c_1-  ;
oc: 2-          drop c_2-  ;
oc: 4-          drop c_4-  ;
oc: 8-          drop c_8-  ;
oc: 2*          drop c_2*  ;
oc: 4*          drop c_4*  ;
oc: 8*          drop c_8*  ;
oc: 2/          drop c_2/  ;
oc: u2/         drop c_u2/  ;
oc: 4/          drop c_4/  ;
oc: 8/          drop c_8/  ;
oc: u4/         drop c_u4/  ;
oc: u8/         drop c_u8/  ;
oc: negate      drop c_negate  ;
oc: *           drop c_*  ;
oc: m*		drop c_m*  ;		\ SFP041
oc: um*		drop c_um*  ;		\ SFP093
oc: /mod	drop c_/mod  ;		\ SFP401
oc: /		drop c_/  ;		\ SFP041
oc: mod		drop c_mod  ;		\ SFP099
oc: m/		drop c_m/  ;		\ SFP041
oc: sm/rem	drop c_sm/rem  ;	\ SFP041
oc: um/mod	drop c_um/mod  ;	\ SFP092
oc: d+          drop c_d+  ;            \ SFP030
oc: d-          drop c_d-  ;            \ SFP030
oc: m+          drop c_m+  ;            \ SFP030
oc: s>d         drop c_s>d  ;
oc: d>s		drop c_drop  ;

oc: =           drop c_=  ;
oc: <>          drop c_<>  ;
oc: <           drop c_<  ;
oc: >           drop c_>  ;
oc: <=          drop c_<=  ;
oc: >=          drop c_>=  ;
oc: u<          drop c_u<  ;
oc: u>          drop c_u>  ;
oc: u<=         drop c_u<=  ;		\ SFP411
oc: u>=         drop c_u>=  ;		\ SFP411
oc: 0=          drop c_0=  ;
oc: 0<>         drop c_0<>  ;
oc: 0<          drop c_0<  ;
oc: 0>          drop c_0>  ;
oc: false=	drop c_0=  ;
oc: d0=		drop c_or c_0=  ;	\ SFP081

oc: >r          drop c_>r  ;
oc: r@          drop c_r@  ;
oc: r>          drop c_r>  ;
oc: 2>r         drop c_2>r  ;
oc: 2r@         drop c_2r@  ;
oc: 2r>         drop c_2r>  ;
oc: rpick	drop c_rpick  ;		\ SFP086

oc: and         drop c_and  ;
oc: or          drop c_or  ;
oc: xor         drop c_xor  ;
oc: invert      drop c_invert  ;
oc: not         drop c_not  ;

oc: or!		drop c_or!  ;		\ SFP068
oc: and!	drop c_and!  ;		\ SFP068
oc: xor!	drop c_xor!  ;		\ SFP068
oc: bic!	drop c_swap c_invert c_swap c_and!  ;	\ SFP068

oc: set-bit	drop  c_bor!  ;
oc: clear-bit	drop  c_swap c_invert c_swap c_band!  ;
oc: toggle-bit	drop  c_bxor!  ;
oc: test-bit	drop  c_c@ c_and c_0<>  ;

oc: cell        drop c_cell  ;
oc: cell+       drop cell(t) c_lit  c_+  ;
oc: cell-	drop cell(t) c_lit  c_-  ;	\ SFP050
oc: cells       drop c_8*  ;
oc: cells+      drop c_cells+  ;

oc: bounds      drop c_bounds  ;
oc: count       drop c_count ;
oc: /string	drop  c_tuck c_- c_-rot c_+ c_swap  ;		\ SFP055
oc: abs         drop c_abs  ;
oc: ?negate	drop c_?negate  ;	\ SFP405
oc: min         drop c_min  ;
oc: max         drop c_max  ;
oc: umin        drop c_umin  ;		\ SFP404
oc: umax        drop c_umax  ;		\ SFP404

oc: chars       drop  ;                 \ multiply by 1 = noop
oc: char+       drop  1 c_lit  c_+  ;   \ SFP034

oc: >body       drop  #5 c_lit  c_+ ;   \ SJC001
oc: defer@	drop  #5 c_lit  c_+  c_@  ;	\ SFP093
oc: defer!	drop  #5 c_lit  c_+  c_!  ;	\ SFP093

oc: lshift      drop c_lshift  ;
oc: rshift      drop c_rshift  ;
oc: arshift     drop c_arshift  ;	\ SFP084
oc: rol		drop c_rol  ;		\ SFP058
oc: ror		drop c_ror  ;		\ SFP058

oc: nsbwiden    drop c_nsbwiden ;       \ SJC025
oc: nswwiden    drop c_nswwiden ;       \ SJC025
oc: nslwiden    drop c_nslwiden ;
oc: nubwiden    drop c_nubwiden ;       \ SJC025
oc: nuwwiden    drop c_nuwwiden ;       \ SJC025
oc: nulwiden    drop c_nulwiden ;

oc: execute	drop c_execute  ;

oc: here	drop  dp c_lit  c_@  ;						\ SFP054
oc: allot	drop  dp c_lit  c_+!  ;						\ SFP054
oc: c,		drop  dp c_lit c_@ c_tuck c_c!  1 c_lit c_+  dp c_lit  c_!  ;	\ SFP062
oc: w,		drop  dp c_lit c_@ c_tuck c_w!  2 c_lit c_+  dp c_lit  c_!  ;	\ SFP062
oc: ,		drop  dp c_lit c_@ c_tuck c_!  cell c_lit c_+  dp c_lit  c_!  ;	\ SFP062
oc: aligned	drop  7 c_lit c_+  -8 c_lit c_and  ;				\ SFP054
oc: l-aligned	drop  3 c_lit c_+  -4 c_lit c_and  ;				\ SFP088
oc: w-aligned	drop  1 c_lit c_+  -2 c_lit c_and  ;				\ SFP088

oc: up@		drop  c_up@  discard-sinline  ;
oc: fsp@	drop  c_fsp@  ;			\ SFP505
oc: ri@		drop  c_ri@  ;			\ SFP519
oc: ril@	drop  c_ril@  ;			\ SFP519
oc: fsp!	drop  c_fsp!  ;			\ SFP505
oc: ri! 	drop  c_ri!  ;			\ SFP519
oc: ril!	drop  c_ril!  ;			\ SFP519

\ Specifics for VFX Forth
oc: hiword      drop  #16 c_lit  c_rshift  ;
oc: loword      drop  $FFFF c_lit c_and  ;
oc: makelong    drop  #16 c_lit c_lshift   c_swap  $FFFF c_lit c_and  c_or  ;
\ oc: cmove	drop c_cmove  ;

oc: within	drop c_within  ;	\ SFP419

external
only forth  also h-c  also forth definitions


\ ==========================================
\ *S Controlling and Analysing compiled code
\ ==========================================
\ *P These directives control the optimiser

: optimising?   \ -- flag
\ *G Returns true if the optimiser is enabled.
  optimising @
;

: optimised     \ -- ; turn optimisation on
\ *G Enables the optimiser.
  assign c_:         to-do docolon,
  assign c_exit      to-do doexit,
  assign c_;         to-do dosemicolon,
  assign c_lit       to-do clit,
  assign o_docall,   to-do docall,
  assign Compile-Ref to-do compile,
  assign NDCS-Ref    to-do ndcs,
  assign c_do        to-do s_do,
  assign c_?do       to-do s_?do,
  assign c_loop      to-do s_loop,
  assign c_+loop     to-do s_+loop,
  assign c_leave     to-do s_leave,
  assign c_?leave    to-do s_?leave,
  assign c_mrk_branch< to-do s_mrk_br<,
  assign c_branch<   to-do s_br<,
  assign c_?branch<  to-do s_?br<,
  assign >c_res_branch to-do s_res_br>,
  assign c_branch>   to-do s_br>,
  assign c_?branch>  to-do s_?br>,
  assign c_of        to-do s_of,
  assign c_?of       to-do s_?of,
  assign c_endof     to-do s_endof,
  assign c_drop      to-do s_drop,

  assign c_rellvs    to-do s_rellvs,
  assign c_makelvs   to-do s_makelvs,
  assign o_PatchLocalSize to-do PatchLocalSize
  assign c_lvarg     to-do s_lvarg,
  assign c_lvarray   to-do s_lvarray,

  assign (s_val,)    to-do s_val,

  optimising on
  clean-opt				\ SFP410
;

: unoptimised   \ -- ; turn optimisation off
\ *G Disables the optimiser.
  assign (docolon,) to-do docolon,
  assign (doexit,)  to-do doexit,
  assign (doexit,)  to-do dosemicolon,
  assign (clit,)    to-do clit,
  assign doLayAbsCall to-do docall,
  assign (compile,) to-do Compile,
  assign (ndcs,)    to-do ndcs,
  assign (s_do,)    to-do s_do,
  assign (s_?do,)   to-do s_?do,
  assign (s_loop,)  to-do s_loop,
  assign (s_+loop,) to-do s_+loop,
  assign (s_leave,) to-do s_leave,
  assign (s_?leave,) to-do s_?leave,
  assign (s_mrk_br<,) to-do s_mrk_br<,
  assign (s_br<,)   to-do s_br<,
  assign (s_?br<,)  to-do s_?br<,
  assign (s_res_br>,) to-do s_res_br>,
  assign (s_br>,)   to-do s_br>,
  assign (s_?br>,)  to-do s_?br>,
  assign (s_of,)    to-do s_of,
  assign (s_?of,)   to-do s_?of,
  assign (else)      to-do s_endof,
  assign (s_drop,)  to-do s_drop,

  assign (s_rellvs,) to-do s_rellvs,
  assign (s_makelvs,) to-do s_makelvs,
  assign (PatchLocalSize) to-do PatchLocalSize
  assign (s_lvarg,) to-do s_lvarg,
  assign (s_lvarray,) to-do s_lvarray,

  assign (s_val,)   to-do s_val,

  optimising off  discard-sinline
;


\ SFP071...
\ *P The following directives are used to turn optimisation on
\ ** and off around sections of code.

: [opt		\ -- i*x
\ *G Save the current state of optimisation at the start of an
\ ** *\fo{[OPT ... OPT]} structure. You can make no assumptions
\ ** about what the data stack contains.
  optimising?
;

: [-opt		\ -- i*x
\ *G Save the current state of optimisation at the start of an
\ ** *\fo{[-OPT ... OPT]} structure and turn optimisation off.
  optimising? unoptimised
;

: opt]		\ i*x --
\ *G Restore the state of optimisation at the end of an
\ ** *\fo{[OPT ... OPT]} structure to what it was at the start.
  if  optimised  else  unoptimised  endif
;
\ ...SFP071


\ *P The following directives are *\fo{IMMEDIATE} words that you
\ ** can put inside your definitions to obtain an idea of how
\ ** code is being compiled. *\fo{DIS <name>} will disassemble a word.

: []		\ --
\ *G Lay a NOP instruction as a marker, without flushing the
\ ** optimiser.
  $90 c,c(t)
; immediate

: [o/f]         \ --
\ *G Flush the optimiser state, generating the canonical stack
\ ** state again with TOS in the EBX register, and all other
\ ** stack items in the deep (memory) stack.
  flushcode				\ SFP416
; immediate

: [o/s]         \ --
\ *G Show the state of the optimiser's working stack.
  .ws
; immediate

only forth definitions
external


\ *****************
\ *S Hints and Tips
\ *****************
\ *P On x64 CPUs single PUSH and POP instructions
\ ** generated by *\fo{>R} and *\fo{R>} are slow, and the VFX code
\ ** generator is quite conservative in optimising return stack
\ ** manipulations as compared with data stack anipulations.
\ ** Although the code below is convenient, safe and easy to write
\ ** it is slow. The *\fo{rect.xxx} words are fields in a structure.
\ *E : Rect@   \ rect -- l t r b
\ ** \ Retrieve the values x y r b from the RECT[ structure at
\ ** \ the address given.
\ **   >r
\ **   r@ rect.Left @
\ **   r@ rect.Top @
\ **   r@ rect.right @
\ **   r> rect.bottom @
\ ** ;

\ *P The version below generates far better code when performance
\ ** is important.
\ *E : Rect@   \ rect -- l t r b
\ ** \ Retrieve the values x y r b from the RECT[ structure at
\ ** \ the address given.
\ **   dup rect.Left @ swap
\ **   dup rect.Top @ swap
\ **   dup rect.right @ swap
\ **   rect.bottom @
\ ** ;

\ *P Because of the limited number of registers, better code
\ ** may be generated by passing a pointer to a structure
\ ** such as a rectangle rather than passing four items on the
\ ** data stack. Use of words such as *\fo{Rect@} should be
\ ** reseved for preparing parameters for a Windows API call.


\ *****************
\ *S VFX Forth v4.x
\ *****************
\ *P If you have written custom optimisers, the EAX register is
\ ** no longer free for use, but must be requested like any other
\ ** working register. *\fo{CODE} definitions require no changes.


\ ======
\ *> ###
\ ======

